package com.asda.android.adbanner

import android.content.Context
import android.os.Handler
import androidx.recyclerview.widget.RecyclerView
import com.asda.android.abtestingframework.implementation.sitespect.constants.SitespectP13NMetrics
import com.asda.android.admonetization.AdConfig.asdaService
import com.asda.android.admonetization.AdConfig.authentication
import com.asda.android.admonetization.AdConfig.isCriteoEnabled
import com.asda.android.admonetization.AdConfig.sitespectAbTestingEngine
import com.asda.android.admonetization.criteo.*
import com.asda.android.admonetization.criteo.constants.AdConstants
import com.asda.android.admonetization.criteo.constants.AdConstants.DELAYED_CALL_TIME
import com.asda.android.admonetization.criteo.constants.AdConstants.FORWARD_SLASH
import com.asda.android.admonetization.criteo.constants.AdConstants.GREATER_THAN
import com.asda.android.admonetization.formatter.BannerIdsFromZonesFormatter
import com.asda.android.admonetization.helper.view.AdBannerViewHelper
import com.asda.android.admonetization.validator.BannerZoneValidator
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.constants.Anivia
import com.asda.android.app.settings.featureflag.FeatureSettings.isCriteoEnabled
import com.asda.android.base.core.view.ui.ItemQuantityController
import com.asda.android.base.interfaces.IAdManager
import com.asda.android.favourites.features.helpers.SEPARATOR_UNDER_SCORE
import com.asda.android.products.ui.AsdaProductsConfig
import com.asda.android.restapi.ad.model.CriteoResult
import com.asda.android.restapi.app.shop.model.ShelfListItem
import com.asda.android.restapi.cms.model.Configs
import com.asda.android.restapi.cms.model.Zone
import com.asda.android.restapi.service.RestUtils
import com.asda.android.restapi.service.data.Cart
import com.asda.android.restapi.service.data.Merchandising
import com.asda.android.restapi.service.data.catalog.IroProductDetailsPlp
import com.asda.android.utils.CMSUtils
import com.asda.android.utils.Utils
import io.reactivex.subjects.BehaviorSubject
import java.math.BigDecimal
import java.util.*
import kotlin.collections.HashMap
import kotlin.collections.set

const val ABK_DELAY = 1_0000L

object AdBannerManager : IAdManager {
    const val SHARED_PREF_SITESPECT = "sitespect_abtesting_module"
    const val SITESPECT_USER_ID = "DYN_USER_ID"

    override fun onBannerItemClick(context: Context, tag: Any?, shelfListItem: ShelfListItem) {
        // This is for non-banner, very first item (Native tracking) for Criteo
        val preference = context.getSharedPreferences(SHARED_PREF_SITESPECT, Context.MODE_PRIVATE)
        if (isCriteoEnabled && shelfListItem.banner == null) { // native tracking for non-banner items
            AdTrackingServiceQuery().bannerNativeEvent(shelfListItem, tag.toString(), "abp")
        }
        asdaService.sendClickEvent(shelfListItem.clickAddUrl)
        if (shelfListItem.personalizedRecommendation &&
            authentication.isLoggedIn &&
            !preference.getString(SITESPECT_USER_ID, "").isNullOrEmpty()
        ) {
            sitespectAbTestingEngine.trackMetrics(
                SitespectP13NMetrics.VARIATION_GROUP, shelfListItem.metricId, 1,
                RestUtils.poundStringToBigDecimal(shelfListItem.priceToDisplay), false
            ).subscribe()
        }
    }

    override fun bannerAbpEvent(campaign: String?, item: ShelfListItem) {
        var trackingInformation: HashMap<String, String> = HashMap()
        trackingInformation[AdConstants.RNPARAM] = AdConstants.TRACKING_ASDA_APP
        trackingInformation[AdConstants.RSPARAM] = AdMonetization.store
        trackingInformation[AdConstants.RPPARAM] = AdUtils().getRetailPageParam(campaign).toString()
        if (AdMonetization.vuid.isNullOrEmpty())
            trackingInformation[AdConstants.VFPARAM] =
                AdMonetization.account["RETAILIER_VISITOR_ID"].toString() // device id
        else
            trackingInformation[AdConstants.VFPARAM] = AdMonetization.vuid

        trackingInformation[AdConstants.TAPARAM] = AdConstants.TRACKING_ABP
        trackingInformation[AdConstants.PIPARAM] = item.id.toString()
        trackingInformation[AdConstants.PLPARAM] = AdUtils().simplifyName(item.title)
        /* Send the tracking details to Criteo. */
        AdTrackingServiceQuery().getCriteoTracking(trackingInformation)
    }

    override fun bannerAbkEvent(
        existingItems: BigDecimal?,
        newItems: BigDecimal?,
        banner: Merchandising.Banner?,
        context: Context?
    ) {
        if (banner == null)
            return
        if (banner.trackingInformation.size != 0 && existingItems != null && newItems != null && context != null) {
            banner.trackingInformation[AdConstants.TAPARAM] = AdConstants.TRACKING_ABK
            banner.trackingInformation[AdConstants.VFPARAM] = AdMonetization.vuid
            // Deal with respect to the current item being added
            when (newItems.compareTo(existingItems)) {
                1 -> {
                    if (existingItems == BigDecimal.ZERO) {
                        banner.trackingInformation[AdConstants.TEPARAM] =
                            AdConstants.TRACKING_ABK_BTN
                    } else {
                        banner.trackingInformation[AdConstants.TEPARAM] =
                            AdConstants.TRACKING_QTY_CHANGE
                        banner.trackingInformation[AdConstants.TNPARAM] =
                            (newItems - existingItems).toString()
                    }
                    // Sending anivia tracking for adding item from banner.
                    trackCartUpdateFromBanner(
                        (newItems - existingItems).toString(),
                        banner,
                        context
                    )
                }
                -1 -> {
                    banner.trackingInformation[AdConstants.TEPARAM] =
                        AdConstants.TRACKING_QTY_CHANGE
                    banner.trackingInformation[AdConstants.TNPARAM] =
                        (existingItems - newItems).toString() // -ve value
                }
            }
            // Deal with entire cart
            /* Send the tracking details to Criteo. */
            AdCartTrackingServiceQuery().makeCartTrackingApiCall(
                banner.trackingInformation,
                Utils.buildBasketXml()
            )
            Handler().postDelayed(
                { AdTrackingServiceQuery().sendBannerClickEvent(banner) },
                DELAYED_CALL_TIME
            )
        }
    }

    private fun trackCartUpdateFromBanner(
        quantity: String,
        banner: Merchandising.Banner,
        context: Context
    ) {
        if (DuplicateControl.isItemDuplicate(
                banner.trackingCode,
                AdConstants.WAIT_TIME_IN_SEC_BEFORE_PURGE,
                false
            )
        ) {
            return
        }
        val itemList = ArrayList<Cart.CartItem>()
        itemList.add(ItemQuantityController.createCartItem(context, banner.bannerItem, quantity))
        val event = AsdaAnalyticsEvent(Anivia.TROLLEY_ADD_EVENT)
        event.putString(
            Anivia.BANNER_INFO_KEY,
            AdTrackingServiceQuery().buildTrackingStringForAnivia(banner)
        )
        event.putString(Anivia.BANNER_CART_ADD_KEY, banner.trackingCode)
        Tracker.get().trackCartUpdate(
            event,
            Tracker.toCartItemInfoList(itemList),
            null,
            banner.contentid,
            null,
            Anivia.TROLLEY_ADD_EVENT,
            com.asda.android.cxo.CartManager.instance.getBasketId()
        )
    }

    override fun onBannerItemClick(
        context: Context,
        tag: Any?,
        item: IroProductDetailsPlp.Items,
        specialOffersTagIsClickable: Boolean,
        isBigIcon: Boolean
    ) {
        val shelfListItem =
            ShelfListItem.buildShelfObject(item, specialOffersTagIsClickable, isBigIcon)
        onBannerItemClick(context, tag, shelfListItem)
    }

    override fun getBannerIdsFromZones(zone: List<Zone>?): String {
        return getBannerIdFormatter().format(zone)
    }

    private fun getBannerIdFormatter(): BannerIdsFromZonesFormatter {
        return BannerIdsFromZonesFormatter()
    }

    override fun trackBuyEvent(context: Context, basketXml: String?, campaign: String?) {
        if (isCriteoEnabled(context)) {
            AdCartTrackingServiceQuery().buySummeryEvent(basketXml.orEmpty(), campaign)
        }
    }

    override fun sendAddToBasketEvent(
        context: Context,
        banner: Merchandising.Banner?,
        shelfListItem: ShelfListItem,
        qtyInCart: BigDecimal,
        newQuantity: BigDecimal,
        campaign: String?
    ) {
        if (banner != null) {
            Handler().postDelayed(
                {
                    AsdaProductsConfig.adBannerManager.bannerAbkEvent(
                        qtyInCart,
                        newQuantity,
                        banner,
                        context
                    )
                },
                ABK_DELAY
            )
        } else if (qtyInCart == BigDecimal.ZERO) {
            AsdaProductsConfig.adBannerManager.bannerAbpEvent(campaign, shelfListItem)
        }
    }

    override fun getBasicQueryParams(context: Context) = CMSUtils.setBasicQueryParams(context)

    override fun sendHookLogicOrderConfirmation(
        context: Context,
        orderId: String,
        productIds: String,
        quantities: String,
        productPrices: String
    ) {
        AdUtils().sendHookLogicOrderConfirmation(
            context,
            orderId,
            productIds,
            quantities,
            productPrices
        )
    }

    override fun isDwb(type: String) = BannerZoneValidator().isDwb(type)

    override fun makeCriteoCall(
        context: Context,
        sourcePage: String?,
        configs: Configs,
        additionalInfo: MutableMap<String, Any?>,
        adapter: RecyclerView.Adapter<out RecyclerView.ViewHolder>?,
        startPosition: Int?,
        hideOnFailure: Boolean
    ): BehaviorSubject<CriteoResult>? {
        return AdBannerViewHelper.initiateCriteoCall(
            context,
            sourcePage,
            configs,
            additionalInfo,
            adapter,
            startPosition,
            hideOnFailure
        )
    }

    /**
     * input: Food & Meat/abc&dce/xyz > opw
     * Output: food_meat/abc_dce/xyz/opw
     *
     */
    override fun formatItemName(name: String): String {
        val builder = StringBuilder()
        StringBuilder()
        val splChars = StringBuilder()
        name.forEach {
            if (it.isLetterOrDigit()) {
                if (splChars.isNotEmpty()) {
                    builder
                        .append(getReplacementCharacter(splChars.trim().toString()))
                }

                builder.append(it.toLowerCase())
                splChars.setLength(0)
            } else {
                splChars.append(it)
            }
        }
        return builder.toString()
    }

    private fun getReplacementCharacter(chunk: String): String {
        if (chunk.isEmpty()) {
            return SEPARATOR_UNDER_SCORE
        }
        return when (chunk[0]) {
            GREATER_THAN[0] -> {
                FORWARD_SLASH
            }
            FORWARD_SLASH[0] -> {
                FORWARD_SLASH
            }
            else -> {
                SEPARATOR_UNDER_SCORE
            }
        }
    }
}
