package com.asda.android.app.account.parser

import com.asda.android.app.account.singleprofile.view.SPAddEditPaymentCardFragment
import com.asda.android.app.account.view.ChangePasswordFragment
import com.asda.android.app.account.view.FeedbackFragment
import com.asda.android.app.dp.view.ActiveDPFragment
import com.asda.android.app.dp.view.ActiveDPTabbedFragment
import com.asda.android.app.orders.OrderFragment
import com.asda.android.app.recurringslot.RecurringSlotDetailsFragment
import com.asda.android.app.settings.detail.view.SettingsFragment
import com.asda.android.base.core.view.fragments.BaseFragment
import com.asda.android.base.core.web.model.WebViewModel
import com.asda.android.base.core.web.view.WebViewFragment
import com.asda.android.restapi.service.base.AsdaServiceSettings.asdaWebUrl
import com.asda.android.singleprofile.constants.ViewConstants.PERMISSION_CENTER_FRAGMENT_TAG
import com.asda.android.singleprofile.constants.ViewConstants.SIGN_IN_DETAILS_FRAGMENT_TAG
import com.asda.android.singleprofile.features.account.model.FragmentInfoParserModel
import com.asda.android.singleprofile.features.account.view.PersonalDetailsFragment
import com.asda.android.singleprofile.features.account.view.PhoneBookFragment
import com.asda.android.singleprofile.features.account.view.SPAddressesFragment
import com.asda.android.singleprofile.features.account.view.SPPaymentCardsFragment
import com.asda.android.singleprofile.features.account.view.instance
import com.asda.android.singleprofile.interfaces.IAccountFragmentTagParser

/**
 * Creates fragment based on tag from input
 */
class AccountFragmentTagParser : IAccountFragmentTagParser {
    private val platform = "&platform=android"
    private val personalDetailURL = asdaWebUrl + "account?web_view=personal_details" + platform
    private val signInDetailURL = asdaWebUrl + "account?web_view=signin_details" + platform
    private val permissionCenterURL = asdaWebUrl + "account?enable_permission=true&web_view=permission_centre" + platform

    override fun parse(input: FragmentInfoParserModel): BaseFragment? {
        return when (input.tag) {
            OrderFragment.TAG -> OrderFragment()
            ActiveDPFragment.TAG -> ActiveDPFragment()
            ActiveDPTabbedFragment.TAG -> ActiveDPTabbedFragment.newInstance()
            SPAddressesFragment.TAG -> instance()
            ChangePasswordFragment.TAG -> ChangePasswordFragment.newInstance()
            SettingsFragment.TAG -> SettingsFragment()
            RecurringSlotDetailsFragment.TAG -> RecurringSlotDetailsFragment.newInstance()
            PersonalDetailsFragment.TAG -> {
                if (input.phoneNumberLoggedIn) {
                    WebViewFragment.newInstance(WebViewModel(PersonalDetailsFragment.TAG, personalDetailURL, input.title))
                } else PersonalDetailsFragment.newInstance()
            }
            PhoneBookFragment.TAG -> PhoneBookFragment.newInstance()
            SPPaymentCardsFragment.TAG -> SPPaymentCardsFragment.newInstance()
            FeedbackFragment.TAG -> FeedbackFragment.newInstance(FeedbackFragment.SURVEY_TYPE_OPT_INT_KEY, FeedbackFragment.ENTITY_TYPE_OPT_IN_KEY, -1, "")
            SPAddEditPaymentCardFragment.TAG -> SPAddEditPaymentCardFragment.newInstance(input.card, input.card == null, false, SPAddEditPaymentCardFragment.PURCHASE_TYPE_BASKET)
            SIGN_IN_DETAILS_FRAGMENT_TAG -> {
                if (input.phoneNumberLoggedIn) {
                    WebViewFragment.newInstance(WebViewModel(SIGN_IN_DETAILS_FRAGMENT_TAG, signInDetailURL, input.title))
                } else null
            }
            PERMISSION_CENTER_FRAGMENT_TAG -> {
                if (input.permissionCenterEnabled) {
                    WebViewFragment.newInstance(WebViewModel(PERMISSION_CENTER_FRAGMENT_TAG, permissionCenterURL, input.title, input.privacyPolicy))
                } else null
            }
            else -> null
        }
    }
}
