package com.asda.android.app.account.view

import android.content.res.Configuration
import android.os.Bundle
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentManager
import com.asda.android.R
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.events.ErrorEvent
import com.asda.android.app.account.singleprofile.view.SPAddEditPaymentCardFragment
import com.asda.android.app.main.view.AsdaActivity
import com.asda.android.base.core.constants.BottomNavConstants.TAG_HOME
import com.asda.android.base.core.utils.tryCast
import com.asda.android.base.core.view.fragments.BaseEditFragment
import com.asda.android.base.core.view.fragments.BaseFragment
import com.asda.android.base.core.view.fragments.DisplayListFragment
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.base.interfaces.IBaseFragment
import com.asda.android.base.interfaces.TransitionListener
import com.asda.android.restapi.service.RestUtils
import com.asda.android.service.base.Authentication
import com.asda.android.singleprofile.features.account.view.SPAddEditAddressFragment
import com.asda.android.singleprofile.features.account.view.WelcomeFragment

class AccountFragment : FeaturedFragment(), TransitionListener {
    private var mRightView: View? = null
    private var mHomeFragment: AccountHomeFragment? = null
    override fun onStart() {
        super.onStart()
        if (!(Authentication.getInstance() as Authentication).isLoggedIn) {
            activity?.tryCast<AsdaActivity> {
                this.getFragmentSwitcher()?.switchToFragment(TAG_HOME)
            }
        }
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        setupView(view, savedInstanceState)
    }

    override fun isParentFragment(): Boolean {
        return true
    }

    override fun getScreenName(): String {
        return SCREEN_NAME
    }

    public override fun isSplitScreen(): Boolean {
        return RestUtils.isTablet(context)
    }

    private fun setupView(view: View, savedInstanceState: Bundle?) {
        val fragmentManager = childFragmentManager
        if (view.findViewById<View?>(R.id.fragment_left) != null) {
            mRightView = view.findViewById(R.id.fragment_container)
            val width = (resources.getDimension(R.dimen.account_layout_width) + resources.getDimension(R.dimen.account_padding) * 2).toInt()
            mRightView?.layoutParams = LinearLayout.LayoutParams(width, ViewGroup.LayoutParams.MATCH_PARENT)
            mRightView?.requestLayout()

            setHomeFragment(savedInstanceState)

            mHomeFragment?.let {
                it.transitionListener = this
                fragmentManager.addOnBackStackChangedListener {
                    Tracker.get().trackEvent(
                        ErrorEvent(
                            Anivia.TOO_LARGE_PAYLOAD,
                            "AccountFragment " + " Back-stack Count  " + fragmentManager.backStackEntryCount
                        )
                    )
                    val count = fragmentManager.backStackEntryCount
                    if (count > 0) {
                        val entry = fragmentManager.getBackStackEntryAt(count - 1)
                        it.setItemSelected(entry.name)
                    } else {
                        it.setItemSelected("")
                    }
                }
                if (savedInstanceState == null || getTopFragment() is AccountHomeFragment || getTopFragment() == null) {
                    fragmentManager.popBackStackImmediate(null, FragmentManager.POP_BACK_STACK_INCLUSIVE)
                    val fragmentTransaction = getASDAFragmentTransaction(fragmentManager)
                    fragmentTransaction.replace(R.id.fragment_left, it, AccountHomeFragment.TAG)
                    fragmentTransaction.replace(R.id.fragment_container, WelcomeFragment.newInstance(), WelcomeFragment.TAG)
                    fragmentTransaction.commitNow()
                }
            }
        } else if (savedInstanceState == null || getTopFragment() is WelcomeFragment || getTopFragment() == null) {
            push(AccountHomeFragment.newInstance(), false)
        }
    }

    private fun setHomeFragment(savedInstanceState: Bundle?) {
        savedInstanceState?.let {
            mHomeFragment = fragmentManager?.findFragmentByTag(AccountHomeFragment.TAG) as AccountHomeFragment?
            if (mHomeFragment == null) {
                mHomeFragment = AccountHomeFragment.newInstance()
            }
        } ?: run {
            mHomeFragment = AccountHomeFragment.newInstance()
        }
    }

    fun push(fragment: FeaturedFragment, addToBackStack: Boolean) {
        if (shouldDeleteBackHistory(fragment)) {
            childFragmentManager.popBackStackImmediate(null, FragmentManager.POP_BACK_STACK_INCLUSIVE)
        }
        pushNoAnimation(fragment, addToBackStack)
    }

    private fun shouldDeleteBackHistory(newFragment: Fragment): Boolean {
        val topFragment = getTopFragment()
        return topFragment is BaseEditFragment ||
            topFragment is DisplayListFragment && newFragment is DisplayListFragment ||
            topFragment is DisplayListFragment &&
            !(newFragment is SPAddEditPaymentCardFragment || newFragment is SPAddEditAddressFragment)
    }

    fun showDialogIfEditing(): Boolean {
        val fragment = getTopFragment()
        return fragment is BaseEditFragment && fragment.onBackPressed()
    }

    val isEditingTopFragment: Boolean
        get() {
            val fragment = getTopFragment()
            return fragment is BaseEditFragment && fragment.wereFieldsChanged()
        }

    override fun onPopped(tag: String?): Boolean {
        return false
    }

    override fun popChildFragmentsOnViewCreated(): Boolean {
        return false
    }

    override fun onConfigurationChanged(newConfig: Configuration) {
        super.onConfigurationChanged(newConfig)
        mRightView?.let {
            val width = (resources.getDimension(R.dimen.account_layout_width) + resources.getDimension(R.dimen.account_padding) * 2).toInt()
            it.layoutParams = LinearLayout.LayoutParams(width, ViewGroup.LayoutParams.MATCH_PARENT)
            it.requestLayout()
        }
    }

    fun reset() {
        mHomeFragment?.loadRecurringSlot()
    }

    override fun onPushed(fragment: IBaseFragment, addToBackStack: Boolean): Boolean {
        if (fragment is BaseFragment) {
            push(fragment, addToBackStack)
        }

        mHomeFragment?.setItemSelected(fragment.getInternalTag())
        return true
    }

    companion object {
        private const val SCREEN_NAME = "Account"
    }
}
