package com.asda.android.app.account.view

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProvider
import com.asda.android.R
import com.asda.android.app.account.singleprofile.view.SPAddEditPaymentCardFragment
import com.asda.android.app.account.singleprofile.view.SPAddEditPaymentCardFragment.Companion.newInstance
import com.asda.android.app.account.view.BaseAccountActivity.Companion.launchAccountActivity
import com.asda.android.app.account.view.FeedbackFragment.Companion.newInstance
import com.asda.android.app.checkout.SelectCardFragment
import com.asda.android.app.dp.view.ActiveDPFragment
import com.asda.android.app.dp.view.ActiveDPTabbedFragment
import com.asda.android.app.main.view.AsdaActivity
import com.asda.android.app.main.view.SignOutHelper
import com.asda.android.app.orders.OrderFragment
import com.asda.android.app.recurringslot.RecurringSlotDetailsFragment
import com.asda.android.app.settings.detail.view.SettingsFragment
import com.asda.android.app.settings.featureflag.FeatureSettings.isBFFBootstrapEnabled
import com.asda.android.app.settings.featureflag.FeatureSettings.isBottomNavOrderTestEnabled
import com.asda.android.app.settings.featureflag.FeatureSettings.isChatBotModuleEnabled
import com.asda.android.authorization.BootstrapManager.getProfileId
import com.asda.android.base.core.constants.BottomNavConstants
import com.asda.android.base.core.statemanagement.type.State
import com.asda.android.base.core.utils.AsdaBaseCoreConfig.chatBotManager
import com.asda.android.base.core.view.CustomProgressDialog
import com.asda.android.base.core.view.ViewUtil
import com.asda.android.base.core.view.fragments.BaseFragment
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.base.interfaces.IBaseFragment
import com.asda.android.base.interfaces.TransitionListener
import com.asda.android.cxo.view.activities.ApplyVoucherActivity
import com.asda.android.databinding.AccountHomeBinding
import com.asda.android.restapi.service.base.IAuthentication
import com.asda.android.restapi.service.data.PaymentDetailsResponse.PaymentCards
import com.asda.android.restapi.service.data.recurringslots.SubscriptionEligibilityResponse
import com.asda.android.service.base.Authentication
import com.asda.android.singleprofile.base.SingleProfile.getProfileResponse
import com.asda.android.singleprofile.constants.ViewConstants.PERMISSION_CENTER_FRAGMENT_TAG
import com.asda.android.singleprofile.constants.ViewConstants.SIGN_IN_DETAILS_FRAGMENT_TAG
import com.asda.android.singleprofile.features.account.model.FragmentInfoParserModel
import com.asda.android.singleprofile.features.account.view.HostActivity
import com.asda.android.singleprofile.features.account.view.PersonalDetailsFragment
import com.asda.android.singleprofile.features.account.view.PhoneBookFragment
import com.asda.android.singleprofile.features.account.view.SPAddEditAddressFragment
import com.asda.android.singleprofile.features.account.view.SPAddressesFragment
import com.asda.android.singleprofile.features.account.view.SPPaymentCardsFragment
import com.asda.android.singleprofile.features.account.view.SPPaymentCardsFragment.CardListener
import com.asda.android.singleprofile.features.account.view.instance
import com.asda.android.singleprofile.features.account.viewmodel.BaseAccountViewModel
import com.asda.android.slots.recurringslot.repository.SlotSubscriptionRepository
import com.asda.android.utils.Utils
import io.reactivex.observers.ResourceSingleObserver
import java.lang.ref.WeakReference

class AccountHomeFragment : FeaturedFragment(), TransitionListener {
    private var mLastItemSelected: View? = null
    private var mPendingFragment: BaseFragment? = null
    private var mPendingFragmentTag: String? = null
    private var mBinding: AccountHomeBinding? = null
    private var viewModel: BaseAccountViewModel? = null
    override fun getScreenName(): String {
        return SCREEN_NAME
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        viewModel = ViewModelProvider(this).get(BaseAccountViewModel::class.java)
        viewModel?.webEventLiveData?.observe(
            this,
            Observer {
                var result = viewModel?.getWebPageOwner(it)
                if (result != null) {
                    getCurrentFragmentManager()?.popBackStack()
                    it.setState(State.COMPLETE)
                    return@Observer
                }
                result = viewModel?.openUrlParser?.parse(it)
                if (result != null) {
                    ViewUtil.openWebPage(result, context, TAG)
                    it.setState(State.COMPLETE)
                }
            }
        )
    }

    override fun onCreateView(inflater: LayoutInflater, fragmentContainer: ViewGroup?, savedInstanceState: Bundle?): View {
        mBinding = AccountHomeBinding.inflate(inflater, fragmentContainer, false)
        setupView()
        return mBinding!!.root
    }

    private fun setupView() {
        if (activity == null || context == null || mBinding == null) return

        if (!Authentication.getInstance().allowFeedbackOptIn()) {
            mBinding?.appFeedback?.visibility = View.GONE
        }
        if ((Authentication.getInstance() as Authentication).isLoggedIn) {
            mBinding?.addresses?.visibility = View.GONE
            mBinding?.spAddresses?.visibility = View.VISIBLE
            mBinding?.spPhoneBook?.visibility = View.VISIBLE
            if (isBottomNavOrderTestEnabled(requireActivity())) {
                mBinding?.orders?.visibility = View.VISIBLE
                setItemsListener(View.OnClickListener { view: View -> onItemSelected(view) }, mBinding!!.orders)
            } else {
                mBinding?.orders?.visibility = View.GONE
            }
            setItemsListener(
                View.OnClickListener { view: View -> onItemSelected(view) },
                mBinding!!.deliveryPass,
                mBinding!!.personalDetails,
                mBinding!!.changePassword,
                mBinding!!.spAddresses,
                mBinding!!.spPhoneBook,
                mBinding!!.paymentCards,
                mBinding!!.evouchers,
                mBinding!!.settings,
                mBinding!!.logout,
                mBinding!!.recurringSlot,
                mBinding!!.appFeedback,
                mBinding!!.signInDetails,
                mBinding!!.permissionCenter,
                mBinding!!.askUsQuestion
            )
            setUpSingleProfileView()

            (Authentication.getInstance() as Authentication).userViewResponse?.let {
                mBinding!!.costumerId.text = resources.getString(R.string.account_customer_text, it.profileId)
            }
            if (isBFFBootstrapEnabled(requireContext())) {
                getProfileId()?.let {
                    mBinding!!.costumerId.text = resources.getString(R.string.account_customer_text, it)
                }
            }
        } else {
            if (isBottomNavOrderTestEnabled(requireActivity())) {
                disableItems(mBinding!!.orders)
            }
            disableItems(
                mBinding!!.deliveryPass,
                mBinding!!.personalDetails,
                mBinding!!.changePassword,
                mBinding!!.addresses,
                mBinding!!.paymentCards,
                mBinding!!.evouchers,
                mBinding!!.logout,
                mBinding!!.recurringSlot,
                mBinding!!.appFeedback
            )
            mBinding!!.costumerId.text = getString(R.string.you_must_be_signed_in, getString(R.string.fragment_title_account))
        }
        mBinding!!.signInDetails.visibility = if (viewModel!!.isPhoneNumberLoginEnabled()) View.VISIBLE else View.GONE
        mBinding!!.changePassword.visibility = if (viewModel!!.isPhoneNumberLoginEnabled()) View.GONE else View.VISIBLE
        mBinding!!.permissionCenter.visibility = if (viewModel!!.isPermissionCenterEnabled()) View.VISIBLE else View.GONE
        mBinding!!.askUsQuestion.visibility = if (isChatBotModuleEnabled(requireContext())) View.VISIBLE else View.GONE
    }

    private fun setUpSingleProfileView() {
        mBinding?.spPhoneBook?.setOnClickListener { v: View? ->
            launchPhoneBookFragment()
        }
    }

    private fun launchPhoneBookFragment() {
        val intent = Intent(activity, HostActivity::class.java)
        intent.putExtra(HostActivity.OPTION_KEY, PhoneBookFragment.TAG)
        startActivity(intent)
    }

    fun loadRecurringSlot() {
        if (context == null) return

        showProgress(R.string.loading)
        SlotSubscriptionRepository().getSubscriptionEligibility(getProfileResponse()?.profileId.orEmpty())
            ?.subscribe(object : ResourceSingleObserver<SubscriptionEligibilityResponse?>() {
                override fun onSuccess(response: SubscriptionEligibilityResponse) {
                    dismissProgress()
                    if (isAdded) {
                        var visibility = View.GONE
                        response.data?.eligibility_info?.active_subscription?.let {
                            if (it) {
                                visibility = View.VISIBLE
                            }
                        }
                        mBinding?.recurringSlot?.visibility = visibility
                    }
                }

                override fun onError(e: Throwable) {
                    dismissProgress()
                    if (isAdded) {
                        mBinding?.recurringSlot?.visibility = View.GONE
                    }
                    dispose()
                }
            })
    }

    override fun setupActionBar() {
        val actionBar = (activity as AsdaActivity?)!!.supportActionBar
        actionBar?.setTitle(R.string.fragment_title_account)
    }

    override fun onStart() {
        super.onStart()
        setupActionBar()
        loadRecurringSlot()
    }

    private fun setItemsListener(listener: View.OnClickListener, vararg views: View) {
        for (view in views) {
            view.setOnClickListener(listener)
        }
    }

    private fun disableItems(vararg views: View) {
        for (view in views) {
            view.isEnabled = false
        }
    }

    fun setItemSelected(tag: String?) {
        if (mLastItemSelected != null) {
            mLastItemSelected?.isSelected = false
            mLastItemSelected?.tag = null
        }
        when (tag) {
            RecurringSlotDetailsFragment.TAG -> {
                mBinding?.recurringSlot?.isSelected = true
                mLastItemSelected = mBinding?.recurringSlot
                mLastItemSelected?.tag = tag
            }
            OrderFragment.TAG -> {
                mBinding?.orders?.isSelected = true
                mLastItemSelected = mBinding?.orders
                mLastItemSelected?.tag = tag
            }
            SelectCardFragment.TAG, ActiveDPFragment.TAG -> {
                mBinding?.deliveryPass?.isSelected = true
                mLastItemSelected = mBinding?.deliveryPass
                mLastItemSelected?.tag = tag
            }
            ChangePasswordFragment.TAG -> {
                mBinding?.changePassword?.isSelected = true
                mLastItemSelected = mBinding?.changePassword
                mLastItemSelected?.tag = tag
            }
            SPAddEditAddressFragment.TAG -> {
                mBinding?.addresses?.isSelected = true
                mLastItemSelected = mBinding?.addresses
                mLastItemSelected?.tag = tag
            }
            SPPaymentCardsFragment.TAG, SPAddEditPaymentCardFragment.TAG -> {
                mBinding?.paymentCards?.isSelected = true
                mLastItemSelected = mBinding?.paymentCards
                mLastItemSelected?.tag = tag
            }
            SettingsFragment.TAG -> {
                mBinding?.settings?.isSelected = true
                mLastItemSelected = mBinding?.settings
                mLastItemSelected?.tag = tag
            }
            PersonalDetailsFragment.TAG -> {
                setSelectedPersonalDetails()
            }
            FeedbackFragment.TAG -> {
                mBinding?.appFeedback?.isSelected = true
                mLastItemSelected = mBinding?.appFeedback
                mLastItemSelected?.tag = tag
            }
            SIGN_IN_DETAILS_FRAGMENT_TAG -> {
                mBinding?.signInDetails?.isSelected = true
                mLastItemSelected = mBinding?.signInDetails
                mLastItemSelected?.tag = tag
            }
            PERMISSION_CENTER_FRAGMENT_TAG -> {
                mBinding?.permissionCenter?.isSelected = true
                mLastItemSelected = mBinding?.permissionCenter
                mLastItemSelected?.tag = tag
            }
            else -> mLastItemSelected = null
        }
    }

    private fun setSelectedPersonalDetails() {
        mBinding?.personalDetails?.isSelected = true
        mLastItemSelected = mBinding?.personalDetails
        mLastItemSelected?.tag = tag
    }

    private fun onItemSelected(view: View) {
        if (activity == null) return
        when (view.id) {
            R.id.orders -> if (transitionListener != null && mLastItemSelected !== mBinding?.orders) {
                launchFragment(OrderFragment(), OrderFragment.TAG, true)
            } else if (transitionListener == null) {
                launchAccountActivity(activity, OrderFragment.TAG, view)
            }
            R.id.delivery_pass -> if (transitionListener != null && mLastItemSelected !== mBinding?.deliveryPass) {
                launchFragment(ActiveDPTabbedFragment.newInstance(), ActiveDPTabbedFragment.TAG, true)
            } else if (transitionListener == null) {
                launchAccountActivity(activity, ActiveDPTabbedFragment.TAG, view)
            }
            R.id.recurring_slot -> if (transitionListener != null && mLastItemSelected !== mBinding?.recurringSlot) {
                launchFragment(RecurringSlotDetailsFragment.newInstance(), RecurringSlotDetailsFragment.TAG, true)
            } else if (transitionListener == null) {
                launchAccountActivity(activity, RecurringSlotDetailsFragment.TAG, view)
            }
            R.id.personal_details -> if (transitionListener != null && mLastItemSelected !== mBinding?.personalDetails) {
                launchFragment(viewModel!!.parse(FragmentInfoParserModel(PersonalDetailsFragment.TAG)), PersonalDetailsFragment.TAG, true)
            } else if (transitionListener == null) {
                launchAccountActivity(activity, PersonalDetailsFragment.TAG, view)
            }
            R.id.change_password -> if (transitionListener != null && mLastItemSelected !== mBinding?.changePassword) {
                launchFragment(ChangePasswordFragment.newInstance(), ChangePasswordFragment.TAG, true)
            } else if (transitionListener == null) {
                launchAccountActivity(activity, ChangePasswordFragment.TAG, view)
            }
            R.id.addresses, R.id.sp_addresses -> if (transitionListener != null && mLastItemSelected !== mBinding?.addresses) {
                launchFragment(instance(), SPAddressesFragment.TAG, true)
            } else if (transitionListener == null) {
                launchFragment(instance(), SPAddressesFragment.TAG, true)
                launchAccountActivity(activity, SPAddressesFragment.TAG, view)
            }
            R.id.payment_cards -> if (transitionListener != null && mLastItemSelected !== mBinding?.paymentCards) {
                val fragment = SPPaymentCardsFragment.newInstance()
                fragment.cardListener = object : CardListener {
                    override fun onAddCard() {
                        val addFragment = newInstance(null, true, false, SPAddEditPaymentCardFragment.PURCHASE_TYPE_BASKET)
                        push(addFragment)
                    }

                    override fun onEditCard(card: PaymentCards) {
                        val editFragment = newInstance(card, false, false, SPAddEditPaymentCardFragment.PURCHASE_TYPE_BASKET)
                        push(editFragment)
                    }
                }
                launchFragment(fragment, SPPaymentCardsFragment.TAG, true)
            } else if (transitionListener == null) {
                launchAccountActivity(activity, SPPaymentCardsFragment.TAG, view)
            }
            R.id.evouchers -> {
                val intent = Intent(activity, ApplyVoucherActivity::class.java)
                intent.putExtra(ApplyVoucherActivity.IS_CHECKOUT, false)
                startActivity(intent)
            }
            R.id.logout -> if (activity is AsdaActivity) {
                SignOutHelper.showSignOutDialog(activity as AsdaActivity?)
            } else {
                // Just to be safer, handle case of non-AsdaAcitivy
                doSignOut()
            }
            R.id.settings -> if (transitionListener != null && mLastItemSelected !== mBinding?.settings) {
                launchFragment(SettingsFragment(), SettingsFragment.TAG, false)
            } else if (transitionListener == null) {
                launchAccountActivity(activity, SettingsFragment.TAG, view)
            }
            R.id.app_feedback -> if (transitionListener != null && mLastItemSelected !== mBinding?.appFeedback) {
                launchFragment(newInstance(FeedbackFragment.SURVEY_TYPE_OPT_INT_KEY, FeedbackFragment.ENTITY_TYPE_OPT_IN_KEY, -1, ""), FeedbackFragment.TAG, true)
            } else if (transitionListener == null) {
                launchAccountActivity(activity, FeedbackFragment.TAG, view)
            }
            R.id.sign_in_details -> if (transitionListener != null && mLastItemSelected !== mBinding?.signInDetails) {
                launchFragment(viewModel!!.parse(FragmentInfoParserModel(SIGN_IN_DETAILS_FRAGMENT_TAG)), PersonalDetailsFragment.TAG, true)
            } else if (transitionListener == null) {
                launchAccountActivity(activity, SIGN_IN_DETAILS_FRAGMENT_TAG, view)
            }
            R.id.permission_center -> if (transitionListener != null && mLastItemSelected !== mBinding?.permissionCenter) {
                launchFragment(viewModel!!.parse(FragmentInfoParserModel(PERMISSION_CENTER_FRAGMENT_TAG)), PersonalDetailsFragment.TAG, true)
            } else if (transitionListener == null) {
                launchAccountActivity(activity, PERMISSION_CENTER_FRAGMENT_TAG, view)
            }
            R.id.ask_us_question -> chatBotManager.initializeChatBotFragment(WeakReference(requireActivity()), SCREEN_NAME)
        }
    }

    private fun doSignOut() {
        val progressDialog = CustomProgressDialog.create(context)
        progressDialog.setMessage(getString(R.string.signing_out))
        progressDialog.isIndeterminate = true
        progressDialog.show()
        Authentication.getInstance().logout(object : IAuthentication.LogoutCallback {
            override fun onLogoutSuccess() {
                progressDialog.dismiss()
                val activity: Activity? = activity
                if (activity is AsdaActivity) {
                    val fragmentSwitcher = activity.getFragmentSwitcher()
                    fragmentSwitcher?.switchToFragment(BottomNavConstants.TAG_HOME)
                }
            }
        })
    }

    private fun launchFragment(fragment: BaseFragment?, tag: String, addToBackStack: Boolean) {
        if (transitionListener != null) {
            val topFragment = (transitionListener as AccountFragment).getTopFragment()
            if (topFragment != null) {
                if ((topFragment as BaseFragment).onBackPressed()) {
                    mPendingFragment = fragment
                    mPendingFragmentTag = tag
                    mPendingFragment?.transitionListener = transitionListener
                    topFragment.transitionListener = this
                } else {
                    fragment?.let {
                        it.transitionListener = transitionListener
                        transitionListener?.onPushed(it, addToBackStack)
                    }
                }
            }
        }
    }

    override fun onPopped(tag: String?): Boolean {
        if (transitionListener != null) {
            transitionListener?.onPopped(tag)
            if (mPendingFragmentTag?.isEmpty()?.not() == true) {
                val toBackStack = mPendingFragmentTag == ChangePasswordFragment.TAG
                mPendingFragment?.let {
                    transitionListener?.onPushed(it, toBackStack)
                }
            }
        }
        return true
    }

    override fun onSaveInstanceState(outState: Bundle) {
        super.onSaveInstanceState(outState)
        Utils.informAboutTooLargePayload(outState, TAG)
        if (transitionListener != null) {
            val topFragment = (transitionListener as AccountFragment).getTopFragment()
            if (topFragment != null) {
                outState.putString(KEY_LAST_SELECTED_FRAGMENT_TAG, topFragment.tag)
            }
        } else if (mLastItemSelected?.tag is String) {
            outState.putString(KEY_LAST_SELECTED_FRAGMENT_TAG, mLastItemSelected?.tag as String)
        }
    }

    override fun onActivityCreated(savedInstanceState: Bundle?) {
        super.onActivityCreated(savedInstanceState)
        if (savedInstanceState != null) {
            setItemSelected(savedInstanceState.getString(KEY_LAST_SELECTED_FRAGMENT_TAG, ""))
        }
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        // Set chat-bot is active indication
        chatBotManager
            .isChatBotActiveLiveData
            .observe(
                viewLifecycleOwner,
                Observer { isActive: Boolean ->
                    if (isActive) {
                        mBinding?.askUsQuestion?.setCompoundDrawablesWithIntrinsicBounds(0, 0, R.drawable.badge_red, 0)
                    } else {
                        mBinding?.askUsQuestion?.setCompoundDrawablesWithIntrinsicBounds(0, 0, 0, 0)
                    }
                }
            )
    }

    override fun getInternalTag(): String? {
        return TAG
    }

    override fun onPushed(fragment: IBaseFragment, addToBackStack: Boolean): Boolean {
        return false
    }

    companion object {
        const val TAG = "AccountHomeFragment"
        private const val KEY_LAST_SELECTED_FRAGMENT_TAG = "last_selected_tag"
        const val SCREEN_NAME = "Account Home"
        fun newInstance(): AccountHomeFragment {
            return AccountHomeFragment()
        }
    }
}
