package com.asda.android.app.account.view

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.view.View
import androidx.fragment.app.Fragment
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProvider
import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.app.account.singleprofile.view.SPAddEditPaymentCardFragment
import com.asda.android.app.dp.view.ActiveDPFragment
import com.asda.android.app.recurringslot.RecurringSlotDetailsFragment
import com.asda.android.app.settings.detail.view.SettingsFragment
import com.asda.android.base.core.constants.ActivityRequestCodes
import com.asda.android.base.core.statemanagement.type.State
import com.asda.android.base.core.utils.tryCast
import com.asda.android.base.core.view.ViewUtil
import com.asda.android.base.core.view.actitivies.AppCompatActivity
import com.asda.android.base.core.view.fragments.BaseFragment
import com.asda.android.base.core.web.view.IBackable
import com.asda.android.restapi.service.base.IAuthentication
import com.asda.android.restapi.service.data.PaymentDetailsResponse.PaymentCards
import com.asda.android.service.base.Authentication
import com.asda.android.singleprofile.features.account.model.FragmentInfoParserModel
import com.asda.android.singleprofile.features.account.view.SPPaymentCardsFragment
import com.asda.android.singleprofile.features.account.view.SPPaymentCardsFragment.CardListener
import com.asda.android.singleprofile.features.account.viewmodel.BaseAccountViewModel
import com.asda.android.utils.Utils
import com.asda.android.utils.view.ViewUtils

class BaseAccountActivity : AppCompatActivity() {

    private val mAuthListener: IAuthentication.AuthenticationListener = object : IAuthentication.AuthenticationListener {
        override fun onLoginSuccess() {
            // No op
        }

        override fun onAutoLoginFailure() {
            // No op
        }

        override fun onLogoutSuccess() {
            finish()
        }
    }

    private var mOptionKey: String? = null

    private var viewModel: BaseAccountViewModel? = null

    public override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        viewModel = ViewModelProvider(this).get(BaseAccountViewModel::class.java)

        mOptionKey = if (savedInstanceState != null) {
            savedInstanceState.getString(OPTION_KEY)
        } else {
            intent.getStringExtra(OPTION_KEY)
        }

        setContentView(R.layout.container_fragment)

        val actionBar = supportActionBar
        if (actionBar != null) {
            actionBar.setDisplayShowCustomEnabled(true)
            actionBar.setCustomView(R.layout.account_header)
        }

        viewModel?.webEventLiveData?.observe(
            this,
            Observer {
                var result = viewModel?.getWebPageOwner(it)
                if (result != null) {
                    onBackPressed()
                    it.setState(State.COMPLETE)
                    return@Observer
                }
                result = viewModel?.openUrlParser?.parse(it)
                if (result != null) {
                    ViewUtil.openWebPage(result, this, TAG)
                    it.setState(State.COMPLETE)
                }
            }
        )

        if (savedInstanceState == null) {
            mOptionKey?.let {
                val fragment = getFragment(it)
                if (fragment == null) {
                    finish()
                }
                if (fragment is BaseFragment) {
                    push(fragment as BaseFragment?, true)
                } else {
                    push(fragment, true)
                }
            }
        }
        (Authentication.getInstance() as Authentication).addAuthListener(mAuthListener)
        viewModel?.trackPageViewEvent(pageNameForEvent)
    }

    private val pageNameForEvent: String
        get() = when (fragmentName) {
            ActiveDPFragment.TAG -> Anivia.PV_YOUR_ACCOUNT_DELIVERY_PASS
            SettingsFragment.TAG -> Anivia.PV_YOUR_ACCOUNT_SETTINGS
            RecurringSlotDetailsFragment.TAG -> Anivia.PV_YOUR_ACCOUNT_RECURRING_SLOTS
            else -> ""
        }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == ActivityRequestCodes.REQUEST_CODE_REORDER && resultCode == ActivityRequestCodes.RESULT_CODE_SHOW_CART) {
            setResult(ActivityRequestCodes.RESULT_CODE_SHOW_CART)
            finish()
        }
    }

    public override fun onSaveInstanceState(outState: Bundle) {
        super.onSaveInstanceState(outState)
        Utils.informAboutTooLargePayload(outState, TAG)
        outState.putString(OPTION_KEY, mOptionKey)
    }

    public override fun onDestroy() {
        (Authentication.getInstance() as Authentication).removeAuthListener(mAuthListener)
        super.onDestroy()
    }

    override fun getFragmentContainer(): Int {
        return R.id.fragment_container
    }

    private fun getFragment(tag: String): Fragment? {
        val fragment: Fragment? = viewModel?.parse(FragmentInfoParserModel(tag))
        if (SPPaymentCardsFragment.TAG == tag && fragment is SPPaymentCardsFragment) {
            fragment.cardListener = object : CardListener {
                override fun onAddCard() {
                    viewModel?.parse(FragmentInfoParserModel(SPAddEditPaymentCardFragment.TAG))?.tryCast<SPAddEditPaymentCardFragment> {
                        this@BaseAccountActivity.push(this)
                    }
                }

                override fun onEditCard(card: PaymentCards) {
                    val fragmentInfoParserModel = FragmentInfoParserModel(SPAddEditPaymentCardFragment.TAG)
                    fragmentInfoParserModel.card = card
                    viewModel?.parse(fragmentInfoParserModel)?.tryCast<SPAddEditPaymentCardFragment> {
                        this@BaseAccountActivity.push(this)
                    }
                }
            }
        }
        return fragment
    }

    override fun onBackPressed() {
        val fragment = getTopFragment()
        if (fragment is IBackable) {
            if ((fragment as IBackable).onBackPressed()) {
                return
            }
        }
        super.onBackPressed()
    }

    companion object {
        private const val TAG = "BaseAccActivity"
        private const val OPTION_KEY = "activity_tag_key"
        private var fragmentName = ""
        @JvmStatic
        fun launchAccountActivity(activity: Activity?, tag: String, clickedView: View?) {
            activity?.let {
                fragmentName = tag
                val intent = Intent(activity, BaseAccountActivity::class.java)
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                intent.putExtra(OPTION_KEY, tag)
                ViewUtils.startActivity(it, intent, clickedView)
            }
        }
    }
}
