package com.asda.android.app.bookslot

import android.content.Context
import android.text.TextUtils
import android.util.Log
import com.asda.android.R
import com.asda.android.restapi.app.shop.model.ShelfListItem
import com.asda.android.restapi.constants.ShelfListAdapterConstants
import com.asda.android.restapi.service.data.PriceChange
import com.asda.android.utils.view.ViewUtil
import java.util.*
import kotlin.collections.ArrayList

class PreviewBookDataSource {

    fun getShelfItems(context: Context?, changesData: Array<PriceChange>?): ArrayList<ShelfListItem> {
        val itemsToDisplay = ArrayList<ShelfListItem>()
        if (changesData != null && changesData.isNotEmpty()) {
            val promoChangedItems = ArrayList<ShelfListItem>()
            val delistedItems = ArrayList<ShelfListItem>()
            val priceChangedItems = ArrayList<ShelfListItem>()
            val priceChangedItemsIds = HashSet<String>()
            for (priceChange in changesData) {
                if (priceChange.items != null && priceChange.items?.size ?: 0 > 0) {
                    if (PriceChange.TYPE_PROMOS_CHANGED_NAMES.equals(priceChange.type, ignoreCase = true)) {
                        if (promoChangedItems.isEmpty()) {
                            promoChangedItems.add(ShelfListItem(context?.getString(R.string.expired_offers_subtitle), ShelfListAdapterConstants.TYPE_DEPARTMENT_TITLE))
                        }
                        priceChange.items?.let {
                            for (priceChangeItem in it) {
                                if (!TextUtils.isEmpty(priceChangeItem.name)) {
                                    val item = ShelfListItem()
                                    item.title = priceChangeItem.name
                                    item.type = ShelfListAdapterConstants.TYPE_PROMO_CHANGED
                                    promoChangedItems.add(item)
                                }
                            }
                            if (promoChangedItems.size > 1) {
                                itemsToDisplay.addAll(promoChangedItems)
                            }
                        }
                    } else if (priceChange.type != null && priceChange.type?.toLowerCase(Locale.UK)?.contains(PriceChange.DELISTED) == true) {
                        priceChange.items?.let {
                            for (priceChangeItem in it) {
                                val shelfListItem = ShelfListItem(priceChangeItem, false)
                                shelfListItem.priceIncreased = null
                                shelfListItem.imageURL = ViewUtil.pickImageUrl(priceChangeItem.extraLargeImageURL, priceChangeItem.imageURL)
                                delistedItems.add(shelfListItem)
                            }
                            itemsToDisplay.addAll(delistedItems)
                        }
                    } else if ((priceChange.type == PriceChange.PRICE_CHANGED_ITEMS) || (priceChange.type == PriceChange.PRICECHANGEDITEMS)) {
                        priceChange.items?.let {
                            for (priceChangeItem in it) {
                                val listItem = ShelfListItem(priceChangeItem, true)
                                listItem.imageURL = ViewUtil.pickImageUrl(priceChangeItem.extraLargeImageURL, priceChangeItem.imageURL)
                                if (listItem.id == null || priceChangedItemsIds.add(listItem.id)) {
                                    priceChangedItems.add(listItem)
                                } else {
                                    Log.w(TAG, "Don't show " + listItem.title + " because it's already shown, oldPrice " + priceChangeItem.oldPrice + "->" + priceChangeItem.price)
                                }
                            }
                            itemsToDisplay.addAll(priceChangedItems)
                        }
                    }
                }
            }
        }
        return itemsToDisplay
    }

    companion object {
        const val TAG = "PreviewBookSlotPres-er"
    }
}
