package com.asda.android.app.bookslot

import android.app.Activity
import android.app.ProgressDialog
import android.content.Context
import android.os.Bundle
import android.text.SpannableStringBuilder
import android.text.style.ImageSpan
import android.util.Log
import android.view.*
import android.widget.Button
import android.widget.TextView
import android.widget.Toast
import androidx.annotation.VisibleForTesting
import androidx.annotation.VisibleForTesting.PRIVATE
import androidx.appcompat.app.AlertDialog
import com.asda.android.R
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.app.bookslotv3.view.activity.BookSlotActivity
import com.asda.android.app.bookslotv3.view.activity.DeliveryImpactActivity
import com.asda.android.app.recurringslot.RecurringSlotInfoFragment
import com.asda.android.app.settings.FeatureSettingManager
import com.asda.android.base.core.constants.ActivityRequestCodes
import com.asda.android.base.core.constants.DateFormats
import com.asda.android.base.core.constants.TimeZoneFormat
import com.asda.android.base.core.convertToFormat
import com.asda.android.base.core.view.CustomProgressDialog
import com.asda.android.base.core.view.OnSingleClickListener
import com.asda.android.base.core.view.ViewUtil.openWebPage
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.base.core.view.ui.DialogFactory
import com.asda.android.restapi.service.data.BookSlotResponse
import com.asda.android.restapi.service.data.Cart
import com.asda.android.restapi.service.data.bookslotv3.FulfillmentTypes
import com.asda.android.restapi.service.data.bookslotv3.ServiceAddress
import com.asda.android.restapi.service.data.cart.SLOT_TYPE
import com.asda.android.restapi.service.data.recurringslots.*
import com.asda.android.restapi.utils.SubscriptionUtil
import com.asda.android.service.constants.CartManagerConstants
import com.asda.android.singleprofile.base.SingleProfile
import com.asda.android.slots.recurringslot.repository.SlotSubscriptionRepository
import com.asda.android.slots.recurringslot.view.SlotSuggestionFragment
import com.asda.android.slots.utils.SlotSubscriptionManager
import com.asda.android.slots.utils.SlotUtils
import com.asda.android.utils.view.ViewUtil
import com.asda.android.utils.view.ViewUtils
import io.reactivex.observers.ResourceSingleObserver
import java.util.*

const val TWO_WEEK_OPTION = "Every 2 weeks"

class SlotBookedConfirmationPresenter : FeaturedFragment() {

    private var mContext: Activity? = null
    private var mAvailableSlots: BookSlotResponse.RecurringSlotInfo? = null
    private val mAniviaEventShowFrequencies: PageViewEvent = PageViewEvent(Anivia.PV_BOOKSLOT_SELECT_RECURRING_SLOT, Anivia.SECTION_BOOK_SLOT, Anivia.SECTION_BOOK_SLOT)
    private var mBookedSlotType: Int = 0
    private var mView: View? = null
    private var mSuccessSubtitleView: TextView? = null
    private var bookedUntilTextView: TextView? = null
    private var deliveryDetailsHeader: TextView? = null
    private var expressSlotHeader: TextView? = null
    private var bookSlotButtonsHeader: View? = null
    private var deliveryDetailsView: View? = null
    private var bookedRecurringSlotFreqContainer: View? = null
    private var recurringSlotSwitchContainer: View? = null
    private var recurringSlotLoader: View? = null
    private var bookButton: View? = null
    private var rememberSection: View? = null
    private var subscriptionEligibilityInfo: SubscriptionEligibilityInfo? = null
    private val PRIORITY_PASS_RELEASE_DATE = "24 Aug 2020"
    private var slotStartDate = ""
    private var slotEndDate = ""
    private var isFlexSlot = false
    private var isExpressSlot = false

    override fun onStart() {
        super.onStart()
        // book slot page view event
        val pageViewEvent = PageViewEvent(Anivia.BOOK_SLOT_CONFIRMATION, Anivia.SECTION_BOOK_SLOT, Anivia.SECTION_BOOK_SLOT)
        Tracker.get().trackPageView(pageViewEvent)
    }

    override fun loadBundleData(bundle: Bundle?) {
        super.loadBundleData(bundle)
        if (bundle != null) {
            mAvailableSlots = bundle.getParcelable(AVAILABLE_SLOTS)
            mBookedSlotType = bundle.getInt(BOOKED_DELIVERY_METHOD)
            slotStartDate = bundle.getString(EXTRA_START_DATE) ?: ""
            slotEndDate = bundle.getString(EXTRA_END_DATE) ?: ""
            isFlexSlot = bundle.getBoolean(DeliveryImpactActivity.IS_FLEX_SLOT, false)
            isExpressSlot = bundle.getBoolean(DeliveryImpactActivity.IS_EXPRESS_SLOT, false)
            if (bundle.containsKey(DeliveryImpactActivity.SUBSCRIPTION_ELIGIBILITY_INFO)) {
                subscriptionEligibilityInfo = bundle.getParcelable(DeliveryImpactActivity.SUBSCRIPTION_ELIGIBILITY_INFO)
            }
        }
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        mContext = context as Activity
    }

    override fun onCreateView(inflater: LayoutInflater, fragmentContainer: ViewGroup?, savedInstanceState: Bundle?): View {
        if (mView == null) {
            mView = inflater.inflate(R.layout.slot_booked_confirmation, fragmentContainer, false)
            setupViews()
            val customerType = subscriptionEligibilityInfo?.customerType
            if (isFlexSlot || isExpressSlot) {
                hideRecurringSlotView()
            } else {
                when (customerType) {
                    CustomerType.PAYG -> {
                        if (FeatureSettingManager.isRecurringSlotEnabledForPayg(requireContext())) {
                            setupRecurringSlotSection()
                        } else {
                            hideRecurringSlotView()
                        }
                    }
                    else -> setupRecurringSlotSection()
                }
            }
            mView?.findViewById<View>(R.id.recurring_slot_help)?.setOnClickListener { _ -> push(RecurringSlotInfoFragment()) }
            context?.let { context ->
                setupWithCXOCart()
                addCXOCartListener()
                setupRecurringSlotPeak(context)
            }
        }
        return mView!!
    }

    private fun setupRecurringSlotPeak(context: Context) {

        val rsPeakMessage = mView?.findViewById<TextView>(R.id.rs_peak_season_message)
        val rsLearnMorePeak = mView?.findViewById<TextView>(R.id.rs_learn_more_peak_season)

        SlotUtils.getPeakSeasonConfig(context)?.let { _config ->
            _config.message?.let { _message ->
                if (_message.isNotEmpty() && _message.isNotBlank()) {
                    rsPeakMessage?.visibility = View.VISIBLE
                    rsPeakMessage?.text = ViewUtils.fromHtml(_message)

                    _config.faqUrl?.let { _faqUrl ->
                        if (_faqUrl.isNotEmpty() && _faqUrl.isNotBlank()) {
                            rsLearnMorePeak?.visibility = View.VISIBLE
                            rsLearnMorePeak?.setOnClickListener {
                                openWebPage(_faqUrl, context, TAG)
                            }
                        }
                    }
                }
            }
        }
    }

    private fun addCXOCartListener() {
        com.asda.android.cxo.CartManager?.instance.updatedCart.observe(
            viewLifecycleOwner,
            androidx.lifecycle.Observer<com.asda.android.restapi.service.data.cart.Cart> { cart ->
                var bookedTillTime: String?
                if (cart?.fulfillmentDetails?.slotInfo?.isRecurringSlot == true && cart.fulfillmentDetails?.slotInfo?.startDateTime?.isNotEmpty() == true) {
                    cart.fulfillmentDetails?.slotInfo?.startDateTime?.let {
                        bookedTillTime = SubscriptionUtil.getSubscriptionExpiryDate(
                            it,
                            DateFormats.EEE_DD_MMM_YYYY,
                            DateFormats.HH_MM_AA_EEE_DD_MMM,
                            TimeZoneFormat.UK,
                            TimeZoneFormat.UK
                        )
                        bookedUntilTextView?.text = mContext?.getString(getSlotConfirmationMessage(cart, false, SlotUtils.isExpressEnabled(mContext)), bookedTillTime)
                    }
                }
            }
        )
    }

    private fun setupViews() {
        recurringSlotSwitchContainer = mView?.findViewById<View>(R.id.recurring_slot_switch_container)
        bookSlotButtonsHeader = mView?.findViewById<View>(R.id.recurring_slot_frequency_selector_header)
        bookButton = mView?.findViewById<View>(R.id.recurring_slot_book_button)
        recurringSlotLoader = mView?.findViewById<View>(R.id.recurring_slot_loader)
        rememberSection = mView?.findViewById<View>(R.id.remember_section)
        rememberSection?.visibility = View.GONE
        mSuccessSubtitleView = mView?.findViewById(R.id.success_slot_booked)
        ViewUtils.applyAsdaFont(mSuccessSubtitleView)
        bookedUntilTextView = mView?.findViewById<TextView>(R.id.book_slot_confirmation_until)
        deliveryDetailsHeader = mView?.findViewById<TextView>(R.id.delivery_details_header)
        expressSlotHeader = mView?.findViewById<TextView>(R.id.express_slot_header)
        deliveryDetailsView = mView?.findViewById<View>(R.id.delivery_details)
        bookedRecurringSlotFreqContainer = mView?.findViewById<View>(R.id.booked_frequency_info_container)
        bookedRecurringSlotFreqContainer?.visibility = View.GONE
        deliveryDetailsView?.findViewById<View>(R.id.carat)?.visibility = View.GONE
        deliveryDetailsView?.setBackgroundDrawable(null)
        mView?.findViewById<View>(R.id.booked_until_text)?.visibility = View.GONE
        mView?.findViewById<View>(R.id.booked_until_value)?.visibility = View.GONE
    }

    private fun setupWithCXOCart() {
        val cxoCart = com.asda.android.cxo.CartManager.instance.cartResponse?.data?.cart
        val clickAndCollect = cxoCart?.fulfillmentDetails?.slotInfo?.fulfillmentType != FulfillmentTypes.DELIVERY.name

        fun setBookUntilTime(cxoCart: com.asda.android.restapi.service.data.cart.Cart?, clickAndCollect: Boolean) {
            bookedUntilTextView?.text = mContext?.getString(
                getSlotConfirmationMessage(cxoCart, clickAndCollect, SlotUtils.isExpressEnabled(mContext)),
                cxoCart?.fulfillmentDetails?.slotInfo?.bookedUntilTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.H_MM_AA)
            )
            bookedUntilTextView?.visibility = if (cxoCart?.fulfillmentDetails?.slotInfo?.bookedUntilTime.isNullOrEmpty()) View.GONE else View.VISIBLE
        }

        fun setExpressSection(cxoCart: com.asda.android.restapi.service.data.cart.Cart?) {
            if (SlotUtils.isExpressEnabled(mContext) && cxoCart?.fulfillmentDetails?.slotInfo?.slotType == SLOT_TYPE.EXPRESS.type) {
                deliveryDetailsHeader?.visibility = View.GONE
                expressSlotHeader?.visibility = View.VISIBLE
                val spannableStringBuilder = SpannableStringBuilder()
                    .append(mContext?.getString(R.string.single_space))
                mContext?.let {
                    spannableStringBuilder.setSpan(ImageSpan(it, R.drawable.ic_express), spannableStringBuilder.length - 1, spannableStringBuilder.length, 0)
                }
                spannableStringBuilder.append(mContext?.getString(R.string.single_space)).append(mContext?.getString(R.string.slot))
                expressSlotHeader?.text = spannableStringBuilder
            } else {
                deliveryDetailsHeader?.visibility = View.VISIBLE
                expressSlotHeader?.visibility = View.GONE
            }
        }

        fun setAddress(cxoCart: com.asda.android.restapi.service.data.cart.Cart?, clickAndCollect: Boolean) {
            if (!cxoCart?.fulfillmentDetails?.slotInfo?.bookedUntilTime.isNullOrEmpty()) {
                if (deliveryDetailsView == null) return
                deliveryDetailsHeader?.visibility = View.VISIBLE
                deliveryDetailsView?.visibility = View.VISIBLE

                val titleId = if (clickAndCollect) R.string.order_summary_cc_details_header else R.string.order_summary_delivery_details_header
                ViewUtil.setText(R.id.delivery_details_header, mView, mContext?.getString(titleId))
                val deliveryTimeLabelId = if (clickAndCollect) R.string.order_summary_your_cc_slot else R.string.order_summary_your_delivery
                ViewUtil.setText(R.id.delivery_time_text, deliveryDetailsView, mContext?.getString(deliveryTimeLabelId))
                ViewUtil.setText(
                    R.id.delivery_time_value, deliveryDetailsView,
                    getString(
                        R.string.slot_reserved_date_from_to,
                        cxoCart?.fulfillmentDetails?.slotInfo?.startDateTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.EEEE_DD_MMM),
                        cxoCart?.fulfillmentDetails?.slotInfo?.startDateTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.H_MM_AA),
                        cxoCart?.fulfillmentDetails?.slotInfo?.endDateTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.H_MM_AA)
                    )
                )
                val phoneText = deliveryDetailsView?.findViewById<View>(R.id.phone_text)
                val phoneValue = ViewUtil.findViewById<TextView>(deliveryDetailsView!!, R.id.phone_value)
                val address: String?
                address = "${cxoCart?.fulfillmentDetails?.fulfillmentAddress?.address}, ${cxoCart?.fulfillmentDetails?.fulfillmentAddress?.postalCode}"

                if (clickAndCollect) {
                    ViewUtil.setText(R.id.address_fee_text, deliveryDetailsView, mContext?.getString(R.string.order_summary_cc_address))
                    phoneText?.visibility = View.VISIBLE
                    phoneValue.visibility = View.VISIBLE
                    phoneValue.text = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.phoneNumber
                } else {
                    ViewUtil.setText(R.id.address_fee_text, deliveryDetailsView, mContext?.getString(R.string.order_summary_address))
                    phoneText?.visibility = View.GONE
                    phoneValue.visibility = View.GONE
                }
                ViewUtil.setText(R.id.address_fee_value, deliveryDetailsView, address)
            } else {
                deliveryDetailsHeader?.visibility = View.GONE
                deliveryDetailsView?.visibility = View.GONE
            }
        }

        fun setContinueAndCheckoutButtons(cxoCart: com.asda.android.restapi.service.data.cart.Cart?) {
            val fromCheckout = mContext?.intent?.getBooleanExtra(ActivityRequestCodes.EXTRA_FROM_CHECKOUT, false)
            if (mView == null) return
            val continueButton = mView?.findViewById<View>(R.id.continue_button)
            val checkoutButton = ViewUtil.findViewById<Button>(mView!!, R.id.checkout_button)

            // If in "amending" state, the button should say "CHECKOUT AMENDS"
            if (com.asda.android.cxo.CartManager.instance.isOrderBeingAmended() == true) {
                checkoutButton.setText(R.string.cart_checkout_amends)
            } else {
                checkoutButton.setText(R.string.checkout)
            }

            if (fromCheckout == true) {
                continueButton?.visibility = View.GONE
            } else {
                continueButton?.visibility = View.VISIBLE
            }

            checkoutButton.visibility = View.VISIBLE

            continueButton?.setOnClickListener { v ->
                if (mContext is BookSlotActivity || mContext is DeliveryImpactActivity) {
                    mContext?.setResult(ActivityRequestCodes.RESULT_CODE_DO_SHOPPING)
                    mContext?.finish()
                } else {
                    pop()
                }
            }

            checkoutButton.setOnClickListener { v ->
                if (mContext is BookSlotActivity || mContext is DeliveryImpactActivity) {
                    if (fromCheckout == false) {
                        mContext?.setResult(ActivityRequestCodes.RESULT_CODE_DO_CHECKOUT)
                    }
                    mContext?.finish()
                } else {
                    pop()
                }
            }
        }

        setBookUntilTime(cxoCart, clickAndCollect)
        setExpressSection(cxoCart)
        setAddress(cxoCart, clickAndCollect)
        setContinueAndCheckoutButtons(cxoCart)
    }

    private fun setupRecurringSlotSection() {
        mContext?.let {
            val clickAndCollect = mBookedSlotType == CartManagerConstants.SLOT_TYPE_CC
            if (!clickAndCollect) {
                if (subscriptionEligibilityInfo?.eligible == true && subscriptionEligibilityInfo?.active_subscription == false) {
                    bookButton?.visibility = View.VISIBLE
                    bookButton?.setOnClickListener(object : OnSingleClickListener(this@SlotBookedConfirmationPresenter) {
                        override fun onSingleClick(v: View) {
                            mContext?.let { context ->
                                recurringSlotLoader?.visibility = View.VISIBLE
                                bookButton?.visibility = View.GONE

                                val cxoCartSlotBookedDate = com.asda.android.cxo.CartManager.instance.cartResponse?.data?.cart?.fulfillmentDetails?.slotInfo?.startDateTime

                                SlotSubscriptionRepository().getSubscriptionFrequencyOption(
                                    singleProfileId = SingleProfile.getProfileResponse()?.profileId
                                        ?: "",
                                    slotDate = cxoCartSlotBookedDate
                                        ?: ""
                                )?.subscribe(object : ResourceSingleObserver<SubscriptionFreqOptionsResponse>() {
                                    override fun onSuccess(subscriptionFrequencyResponse: SubscriptionFreqOptionsResponse) {
                                        recurringSlotLoader?.visibility = View.GONE
                                        bookButton?.visibility = View.VISIBLE

                                        var labels = arrayOfNulls<String>(
                                            subscriptionFrequencyResponse.data?.frequencies?.size
                                                ?: 0
                                        )
                                        for (i in labels.indices) {
                                            mContext?.let {
                                                labels[i] = SlotUtils.getFrequencyLabel(it, subscriptionFrequencyResponse.data?.frequencies?.get(i)?.frequency)
                                            }
                                        }

                                        if (subscriptionEligibilityInfo?.customerType == CustomerType.PAYG &&
                                            labels.contains(TWO_WEEK_OPTION)
                                        ) {
                                            val list = labels.toMutableList()
                                            list.remove(TWO_WEEK_OPTION)
                                            labels = list.toList().toTypedArray()
                                        }
                                        val builder = AlertDialog.Builder(context)
                                        builder.setTitle(R.string.choose_frequency)
                                        builder.setSingleChoiceItems(labels, -1) { dialog, which ->
                                            dialog.dismiss()
                                            subscriptionFrequencyResponse.data?.frequencies?.let { frequencies ->
                                                bookRecurringSlot(frequencies[which].frequency, labels[which] ?: "")
                                            }
                                        }
                                        builder.setOnDismissListener { }
                                        builder.create().show()
                                    }

                                    override fun onError(e: Throwable) {
                                        recurringSlotLoader?.visibility = View.GONE
                                        bookButton?.visibility = View.VISIBLE
                                        Toast.makeText(mContext, mContext?.getString(R.string.get_subscription_error_msg), Toast.LENGTH_LONG).show()
                                    }
                                })

                                pageViewEvent = mAniviaEventShowFrequencies
                                Tracker.get().trackPageView(mAniviaEventShowFrequencies) // UKAPPSAN109.2.1
                            }
                        }
                    })
                } else {
                    hideRecurringSlotView()
                }
            } else {
                if (mAvailableSlots != null && "true" == mAvailableSlots?.allowNewRecurringSlot && mAvailableSlots?.availableFrequencies != null && !clickAndCollect) {
                    bookSlotButtonsHeader?.visibility = View.VISIBLE
                    val labels = arrayOfNulls<String>(
                        mAvailableSlots?.availableFrequencies?.size
                            ?: 0
                    )
                    for (i in labels.indices) {
                        mContext?.let {
                            labels[i] = SlotUtils.getFrequencyLabel(it, mAvailableSlots?.availableFrequencies?.get(i)?.frequency)
                        }
                    }
                    bookButton?.visibility = View.VISIBLE
                    bookButton?.setOnClickListener(object : OnSingleClickListener(this@SlotBookedConfirmationPresenter) {
                        override fun onSingleClick(v: View) {
                            mContext?.let { context ->
                                val builder = AlertDialog.Builder(context)
                                builder.setTitle(R.string.choose_frequency)
                                builder.setSingleChoiceItems(labels, -1) { dialog, which ->
                                    dialog.dismiss()
                                    mAvailableSlots?.availableFrequencies?.let {
                                        bookRecurringSlot(it[which].frequency, labels[which] ?: "")
                                    }
                                }
                                builder.setOnDismissListener { }
                                builder.create().show()
                                pageViewEvent = mAniviaEventShowFrequencies
                                Tracker.get().trackPageView(mAniviaEventShowFrequencies) // UKAPPSAN109.2.1
                            }
                        }
                    })
                } else {
                    hideRecurringSlotView()
                }
            }
        }
    }

    private fun hideRecurringSlotView() {
        recurringSlotSwitchContainer?.visibility = View.GONE
        bookButton?.visibility = View.GONE
        bookSlotButtonsHeader?.visibility = View.GONE
    }

    @VisibleForTesting(otherwise = PRIVATE)
    fun getSlotConfirmationMessage(cart: Cart, clickAndCollect: Boolean, isConfigExpressEnabled: Boolean): Int {
        var slotConfirmationTextId = R.string.book_slot_confirmation_until
        if (clickAndCollect) {
            slotConfirmationTextId = if (isConfigExpressEnabled && cart.isExpress != null && cart.isExpress?.equals("true", ignoreCase = true) == true) {
                R.string.book_express_slot_confirmation_until
            } else {
                R.string.book_cc_slot_confirmation_until
            }
        } else {
            slotConfirmationTextId = if (isConfigExpressEnabled && cart.isExpress != null && cart.isExpress?.equals("true", ignoreCase = true) == true) {
                R.string.book_express_slot_confirmation_until
            } else {
                R.string.book_slot_confirmation_until
            }
        }
        return slotConfirmationTextId
    }

    // CXO Confirmation Message
    fun getSlotConfirmationMessage(cart: com.asda.android.restapi.service.data.cart.Cart?, clickAndCollect: Boolean, isConfigExpressEnabled: Boolean): Int {
        var slotConfirmationTextId = R.string.book_slot_confirmation_until

        if (clickAndCollect) {
            slotConfirmationTextId = if (isConfigExpressEnabled && cart?.fulfillmentDetails?.slotInfo?.slotType == SLOT_TYPE.EXPRESS.type) {
                R.string.book_express_slot_confirmation_until
            } else {
                R.string.book_cc_slot_confirmation_until
            }
        } else {
            slotConfirmationTextId = if (isConfigExpressEnabled && cart?.fulfillmentDetails?.slotInfo?.slotType == SLOT_TYPE.EXPRESS.type) {
                R.string.book_express_slot_confirmation_until
            } else {
                R.string.book_slot_confirmation_until
            }
        }
        return slotConfirmationTextId
    }

    override fun hasActionBar(): Boolean {
        return true
    }

    override fun getActionBarTitle(): String? {
        return mContext?.getString(R.string.book_slot_header)
    }

    override fun onCreateOptionsMenu(menu: Menu, inflater: MenuInflater) {
        super.onCreateOptionsMenu(menu, inflater)
        menu.clear()
        inflater.inflate(R.menu.done_button, menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == R.id.action_done) {
            mContext?.setResult(ActivityRequestCodes.RESULT_CODE_FINISH_ACTIVITY)
            mContext?.finish()
        }
        return super.onOptionsItemSelected(item)
    }

    override fun onBackPressed(): Boolean {
        if (mContext is DeliveryImpactActivity) {
            mContext?.setResult(ActivityRequestCodes.RESULT_REFRESH_BOOKED_SLOT)
            mContext?.finish()
        }
        return super.onBackPressed()
    }

    private fun bookRecurringSlot(spanId: String?, label: String) {
        context?.let {
            createSubscription(spanId, label)
        }
    }

    private fun createSubscription(spanId: String?, label: String) {
        SingleProfile.getProfileResponse()?.profileId?.let { singleProfileId ->
            val pd = CustomProgressDialog.create(mContext)
            pd.setMessage(mContext?.getString(R.string.booking_slot_progress))
            pd.show()

            SlotSubscriptionRepository().createSubscription(
                singleProfileId,
                getSubscriptionCreationRequest(spanId)
            )?.subscribe(object : ResourceSingleObserver<SubscriptionCreation>() {
                override fun onSuccess(subscriptionCreation: SubscriptionCreation) {
                    validateSuccessSubscription(subscriptionCreation, spanId, label, singleProfileId, pd)
                }

                override fun onError(e: Throwable) {
                    createSubscriptionOnError(pd)
                }
            })
        }
    }

    private fun validateSuccessSubscription(subscriptionCreation: SubscriptionCreation, spanId: String?, label: String, singleProfileId: String, progressDialog: ProgressDialog, suggestionEvent: AsdaAnalyticsEvent? = null) {
        if (subscriptionCreation.data?.suggestedSlotInfo?.slotDays?.isNotEmpty() == true) {
            val fragment = SlotSuggestionFragment.newInstance(
                slotStartDate, slotEndDate,
                spanId, label, subscriptionCreation.data?.suggestedSlotInfo
            )
            fragment.suggestionSelectListener = object : SlotSuggestionFragment.SuggestionSelectListener {
                override fun onSuggestionSelected(subscriptionCreation: SubscriptionCreation, label: String, suggestionEvent: AsdaAnalyticsEvent) {
                    try {
                        progressDialog.show()
                    } catch (ignored: Exception) {
                        // No op
                    }
                    SlotSubscriptionRepository().createSubscription(
                        singleProfileId,
                        subscriptionCreation
                    )?.subscribe(object : ResourceSingleObserver<SubscriptionCreation>() {
                        override fun onSuccess(it: SubscriptionCreation) {
                            validateSuccessSubscription(it, spanId, label, singleProfileId, progressDialog, suggestionEvent)
                        }

                        override fun onError(e: Throwable) {
                            createSubscriptionOnError(progressDialog)
                        }
                    })
                }
            }
            push(fragment)
            progressDialog.dismiss()
        } else {
            createSubscriptionOnSuccess(progressDialog, label, subscriptionCreation, suggestionEvent)
        }
    }

    private fun createSubscriptionOnSuccess(pd: ProgressDialog, label: String, subscriptionCreation: SubscriptionCreation, suggestionEvent: AsdaAnalyticsEvent? = null) {
        try {
            pd.dismiss()
            val aniviaEventShowRecConfirmation = PageViewEvent(Anivia.PV_BOOKSLOT_CONFIRM_RECURRING_SLOT, Anivia.SECTION_BOOK_SLOT, Anivia.SECTION_BOOK_SLOT) // UKAPPSAN109.2.2
            aniviaEventShowRecConfirmation.putString(Anivia.RECURRING_SLOT_KEY, "yes : " + label.toLowerCase(Locale.UK))
            pageViewEvent = aniviaEventShowRecConfirmation
            Tracker.get().trackPageView(aniviaEventShowRecConfirmation)
            rememberSection?.visibility = View.VISIBLE
            bookSlotButtonsHeader?.visibility = View.GONE
            recurringSlotSwitchContainer?.visibility = View.GONE

            val clickAndCollect = mBookedSlotType == CartManagerConstants.SLOT_TYPE_CC
            val recurringSlotDateContainer = mView?.findViewById<ViewGroup>(R.id.next_recurring_slot_date)
            subscriptionCreation.data?.subscription_info
            if (subscriptionCreation.data?.subscription_info != null) {
                (recurringSlotDateContainer?.findViewById<View>(R.id.next_recurring_slot_date_label) as TextView)
                    .setText(if (clickAndCollect) R.string.next_collection_date else R.string.next_delivery_date)
                (recurringSlotDateContainer.findViewById<View>(R.id.next_recurring_slot_date_value) as TextView).text =
                    SubscriptionUtil.getFormattedDeliveryDate(subscriptionCreation.data?.subscription_info)
                recurringSlotDateContainer.visibility = View.VISIBLE
            } else {
                recurringSlotDateContainer?.visibility = View.GONE
            }
            mSuccessSubtitleView?.setText(if (clickAndCollect) R.string.success_recurring_cnc_slot_booked else R.string.success_recurring_slot_booked)
            bookedRecurringSlotFreqContainer?.visibility = View.VISIBLE
            ViewUtil.setText(R.id.frequency_value, bookedRecurringSlotFreqContainer, label)
            context?.let {
                com.asda.android.cxo.CartManager?.instance.refreshCart()
                if (subscriptionCreation.data?.subscription_info?.reservation_expiry_time != null) {
                    bookedUntilTextView?.text = mContext?.getString(
                        R.string.book_slot_confirmation_until,
                        subscriptionCreation.data?.subscription_info?.reservation_expiry_time?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.HH_MM_AA_EEE_DD_MMM)
                    )
                } else {
                    bookedUntilTextView?.text = mContext?.getString(
                        R.string.book_slot_confirmation_until,
                        subscriptionCreation.data?.subscription_info?.subscription_bookings?.first()?.expiry_time?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.HH_MM_AA_EEE_DD_MMM)
                    )
                }
            }

            suggestionEvent?.let {
                Tracker.get().trackEvent(it)
            } ?: run {
                val recurringSlotEvent = AsdaAnalyticsEvent(Anivia.RECURRING_SLOT)
                recurringSlotEvent.putString(Anivia.RS_CUSTOMER_JOURNEY_STEPS, Anivia.RS_HAPPY_PATH)
                recurringSlotEvent.putString(Anivia.RS_OVERLAY_TITLE, Anivia.SECTION_RECURRING_SLOT)
                recurringSlotEvent.putString(Anivia.RS_END_TIME, subscriptionCreation.data?.subscription_info?.end_time)
                recurringSlotEvent.putString(Anivia.RS_START_TIME, subscriptionCreation.data?.subscription_info?.start_time)
                recurringSlotEvent.putString(Anivia.RS_FREQUENCY, label)
                recurringSlotEvent.putString(Anivia.RS_START_DATE, subscriptionCreation.data?.subscription_info?.start_date)
                Tracker.get().trackEvent(recurringSlotEvent)
            }
            SlotSubscriptionManager.refreshSubscriptionEligibility()
        } catch (ignored: Exception) {
            Log.e(TAG, "Something went wrong while creating subscriptions.")
        }
    }

    private fun createSubscriptionOnError(pd: ProgressDialog) {
        try {
            pd.dismiss()
            DialogFactory.createAlertDialog(context?.getString(R.string.oops_error), context?.getString(R.string.ok), null, "", context).show()
            Tracker.get().trackPageView(
                PageViewEvent(Anivia.PV_BOOK_SLOT_ERROR, Anivia.SECTION_BOOK_SLOT, Anivia.SECTION_BOOK_SLOT)
                    .putString(Anivia.ERROR_EXPERIENCED_KEY, "book slot : ")
            )
        } catch (ignored: Exception) {
            Log.e(TAG, "Something went wrong while creating subscriptions.")
        }
    }

    private fun getSubscriptionCreationRequest(spanId: String?): SubscriptionCreation {
        val subscriptionCreation = SubscriptionCreation()
        subscriptionCreation.data = SubscriptionCreationData()
        val cxoCart = com.asda.android.cxo.CartManager.instance.cartResponse?.data?.cart
        val serviceAddress = ServiceAddress(
            postcode = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.postalCode,
            latitude = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.latitude,
            longitude = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.longitude,
            address_1 = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.address,
            address_2 = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.address2,
            city = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.city,
            country = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.country
        )

        subscriptionCreation.data?.subscription_info = SubscriptionInfo(
            frequency = spanId,
            group = "PT2H",
            start_date = cxoCart?.fulfillmentDetails?.slotInfo?.startDateTime,
            end_date = cxoCart?.fulfillmentDetails?.slotInfo?.endDateTime,
            start_time = cxoCart?.fulfillmentDetails?.slotInfo?.startDateTime,
            end_time = cxoCart?.fulfillmentDetails?.slotInfo?.endDateTime,
            postcode = cxoCart?.fulfillmentDetails?.fulfillmentAddress?.postalCode,
            fulfillment_type = cxoCart?.fulfillmentDetails?.slotInfo?.fulfillmentType,
            reservation_id = cxoCart?.fulfillmentDetails?.slotInfo?.reservationId,
            service_address = serviceAddress,
            cart_id = cxoCart?.cartMetadata?.cartId
        )

        return subscriptionCreation
    }

    companion object {
        const val TAG = "SlotBookedConfPresenter"
        private const val BOOKED_DELIVERY_METHOD = "booked_delivery_method"
        private const val AVAILABLE_SLOTS = "available_slots"
        private const val EXTRA_START_DATE = "extra_start_date"
        private const val EXTRA_END_DATE = "extra_end_date"
        const val PRIORITY_PASS = "priority pass"

        fun newInstance(
            availableSlots: BookSlotResponse.RecurringSlotInfo?,
            bookedDeliveryMethod: Int,
            startDate: String?,
            endDate: String?,
            subscriptionEligibilityInfo: SubscriptionEligibilityInfo? = null,
            isFlexSlot: Boolean,
            isExpressSlot: Boolean
        ): SlotBookedConfirmationPresenter {
            val fragment = SlotBookedConfirmationPresenter()
            val args = Bundle()
            args.putInt(BOOKED_DELIVERY_METHOD, bookedDeliveryMethod)
            args.putParcelable(AVAILABLE_SLOTS, availableSlots)
            args.putString(EXTRA_START_DATE, startDate)
            args.putString(EXTRA_END_DATE, endDate)
            args.putBoolean(DeliveryImpactActivity.IS_FLEX_SLOT, isFlexSlot)
            args.putBoolean(DeliveryImpactActivity.IS_EXPRESS_SLOT, isExpressSlot)
            subscriptionEligibilityInfo?.let {
                args.putParcelable(DeliveryImpactActivity.SUBSCRIPTION_ELIGIBILITY_INFO, subscriptionEligibilityInfo)
            }
            fragment.arguments = args
            return fragment
        }
    }
}
