package com.asda.android.app.bookslotv3.view.fragment

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.text.SpannableStringBuilder
import android.text.style.ImageSpan
import android.view.View
import androidx.core.content.ContextCompat
import androidx.lifecycle.Observer
import com.asda.android.R
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.events.LinkButtonClickEvent
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.app.bookslot.SlotBookedConfirmationPresenter
import com.asda.android.app.bookslotv3.view.activity.DeliveryImpactActivity
import com.asda.android.app.dp.view.DPOfferCardViewHandler
import com.asda.android.app.dp.view.DeliveryPassPurchaseActivity
import com.asda.android.app.recurringslot.RecurringSlotDetailsFragment
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.base.core.constants.ActivityRequestCodes
import com.asda.android.base.core.constants.DateFormats
import com.asda.android.base.core.constants.TimeZoneFormat
import com.asda.android.base.core.convertToFormat
import com.asda.android.base.core.getFormattedDateWithDay
import com.asda.android.base.core.isEarlier
import com.asda.android.base.core.utils.tryCast
import com.asda.android.base.core.view.ui.gone
import com.asda.android.base.core.view.ui.visible
import com.asda.android.base.interfaces.ISlotEvents
import com.asda.android.restapi.service.data.AddressBookResponse
import com.asda.android.restapi.service.data.DeliveryPass
import com.asda.android.restapi.service.data.bookslotv3.FulfillmentTypes
import com.asda.android.restapi.service.data.bookslotv3.ServiceAddress
import com.asda.android.restapi.service.data.cart.SLOT_TYPE
import com.asda.android.restapi.service.data.recurringslots.SubscriptionEligibilityInfo
import com.asda.android.restapi.utils.SubscriptionUtil
import com.asda.android.service.base.Authentication
import com.asda.android.service.constants.CartManagerConstants
import com.asda.android.singleprofile.base.SingleProfile
import com.asda.android.singleprofile.features.account.view.AddAddressActivity
import com.asda.android.singleprofile.features.account.view.SelectAddressActivity
import com.asda.android.slots.bookslot.bookslotv3.constants.BookSlotType.CLICK_AND_COLLECT
import com.asda.android.slots.bookslot.bookslotv3.constants.BookSlotType.HOME_DELIVERY
import com.asda.android.slots.deliverypass.model.ShowDPCardValidatorModel
import com.asda.android.slots.deliverypass.validator.ShowDPOfferValidator
import com.asda.android.slots.subscription.model.PlanTypeEnum
import com.asda.android.slots.utils.SlotSubscriptionManager
import com.asda.android.slots.utils.SlotUtils
import com.asda.android.utils.view.ViewUtils
import kotlinx.android.synthetic.main.book_slot_delivery_pass.*
import kotlinx.android.synthetic.main.book_slot_header_dp_booked_slot.*
import kotlinx.android.synthetic.main.fragment_book_slot.*
import kotlinx.android.synthetic.main.home_card_express.*
import kotlinx.android.synthetic.main.order_summary_express_details.*
import com.asda.android.cxo.CartManager as CxoCartManager

class BookSlotHomeDeliveryFragment : BookSlotFragment(HOME_DELIVERY) {
    private var ADDRESS_SELECTION_REQ_CODE: Int = 1001

    private var dpOfferCardViewHandler: DPOfferCardViewHandler? = null
    private val showDPOfferValidator: ShowDPOfferValidator by lazy { ShowDPOfferValidator() }
    private var subscriptionEligibility: SubscriptionEligibilityInfo? = null

    companion object {
        const val TAG = "BookSlot-HomeDelivery"
        fun createInstance(slotEvents: ISlotEvents): BookSlotFragment {
            val fragment = BookSlotHomeDeliveryFragment()
            fragment.slotEvents = slotEvents
            return fragment
        }
    }

    override fun onStart() {
        super.onStart()
        Tracker.get().trackPageView(PageViewEvent(Anivia.PV_BOOK_SLOT_HD, Anivia.SECTION_BOOK_SLOT, Anivia.SECTION_BOOK_SLOT))
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        context?.let {
            if (!viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.cartSlotId.isNullOrBlank() &&
                !viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.fulfillmentType.isNullOrBlank() &&
                FulfillmentTypes.DELIVERY.type != viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.fulfillmentType
            ) {
                slotEvents?.switchTabsTo(CLICK_AND_COLLECT)
            }
        }
        setupRecurringSlotObservers()
    }

    override fun inflateHeaderView(view: View) {
        context?.let {
            if (FeatureSettings.isDPOrderSummaryBannerEnabled(it)) {
                val activeDp = (Authentication.getInstance() as Authentication).getActiveDeliveryPass()
                activeDp?.let {
                    book_slot_delivery_pass_details_stub?.gone()
                } ?: run {
                    book_slot_delivery_pass_details_stub?.visible()
                }
            } else {
                book_slot_delivery_pass_details_stub?.visible()
            }
        } ?: run {
            book_slot_delivery_pass_details_stub?.visible()
        }
    }

    override fun getCurrentFragmentTitle(context: Context): String {
        return context.getString(R.string.home_delivery)
    }

    override fun setDeliveryHeaderText() {
        if (FeatureSettings.isDPBannerForSlotsEnabled(requireContext())) {
            updateDeliveryPassView()
        } else {
            booked_slot_details?.gone()
        } // Hidden under ticket - https://jira.walmart.com/browse/UKGRS-3944
    }

    override fun setAddress() {

        // ServiceAddress might be populated from onActivityResult, if that is the case it takes
        // priority over the rest of the addresses found.

        if (viewModel.serviceAddress == null)
            viewModel.serviceAddress = viewModel.getHomeDeliveryServiceAddress(
                if (viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.fulfillmentType == FulfillmentTypes.DELIVERY.type && viewModel.cxoCart?.fulfillmentDetails?.slotInfo?.slotExpired == false)
                    viewModel.cxoCart?.fulfillmentDetails?.fulfillmentAddress else null,
                SingleProfile.getProfileResponse()?.addresses
            )

        if (viewModel.serviceAddress != null) {
            setAddress(viewModel.serviceAddress)
        } else {
            showAddAddressScreen()
        }

        viewModel.fulfillmentType = FulfillmentTypes.DELIVERY.type
    }

    override fun setExpressInfo() {
        val cxoCart = com.asda.android.cxo.CartManager.instance.cartResponse?.data?.cart
        val showExpressInfo = cxoCart?.cartItemSummary?.lineItemCount ?: 0 <= 0 && cxoCart?.cartItemSummary?.totalQuantity ?: 0 <= 0 && SlotUtils.isExpressEnabled(context) &&
            SlotUtils.expressFlagHD
        if (showExpressInfo) {
            express_info.visibility = View.VISIBLE
            val spannableStringBuilder = SpannableStringBuilder()
                .append(context?.getString(R.string.add_items_to_cart))
                .append(context?.getString(R.string.double_spaces))
            spannableStringBuilder.setSpan(ImageSpan(requireContext(), R.drawable.ic_express), spannableStringBuilder.length - 1, spannableStringBuilder.length, 0)
            spannableStringBuilder.append(context?.getString(R.string.single_space)).append(context?.getString(R.string.slots_available))
            express_text.text = spannableStringBuilder
        } else {
            express_info.visibility = View.GONE
        }
    }

    private fun showAddAddressScreen() {
        AddAddressActivity.launchForResult(
            fragment = this,
            title = getString(R.string.select_address_add),
            onlyAddAddress = true,
            requestCode = ADDRESS_SELECTION_REQ_CODE
        )
    }

    private fun showSelectAddressScreen() {
        SelectAddressActivity.launchForResult(
            this,
            getString(R.string.choose_address),
            book_slot_location_type_address.text.toString(), // Currently selected address (formatted address)
            true,
            ADDRESS_SELECTION_REQ_CODE
        )
    }

    private fun updateDeliveryPassView() {
        val userViewResponse = (Authentication.getInstance() as Authentication).userViewResponse
        val activeDeliveryPass = (Authentication.getInstance() as Authentication).getActiveDeliveryPass()
        val cart = CxoCartManager.instance.cartResponse?.data?.cart
        cart?.let {
            if (activeDeliveryPass != null) {
                showDPView(activeDeliveryPass)
            } else {
                showNoDpView(activeDeliveryPass)
            }
        }
        delivery_pass_holder.setOnClickListener { v ->
            val event = LinkButtonClickEvent(
                linkButtonText = Anivia.DP_BANNER_LEARN_MORE,
                channel = Anivia.SECTION_CHECKOUT,
                screenName = Anivia.BOOK_SLOT_PAGE_CHECKOUT
            )
            viewModel.triggerLearnMoreClickEvent(userViewResponse, Tracker.get(), event)
            DeliveryPassPurchaseActivity.start(
                (activity as Activity),
                activeDeliveryPass != null,
                v
            )
        }
    }

    private fun showDPView(activeDp: DeliveryPass) {
        var dpBannerText: String? = ""
        val poundSymbol = "£"
        val cxoCart = CxoCartManager.instance.cartResponse?.data?.cart
        when {
            activeDp.passType?.equals(PlanTypeEnum.GOODWILL.value, ignoreCase = true) ?: false -> hideDeliveryPassBanner()
            activeDp.isMidWeekDP == "true" -> dpBannerText = getString(R.string.dp_midweek_free_delivery)
            else -> {
                val totalCost = cxoCart?.cartPriceSummary?.itemCost?.totalCost ?: 0.0
                val diff = activeDp.dpMinSpendAmt?.replace(poundSymbol, "")?.toDouble()?.minus(totalCost)
                dpBannerText = if (diff != null && diff > 0) {
                    String.format(getString(R.string.spend_amount), diff)
                } else {
                    getString(R.string.home_dp_active)
                }
            }
        }
        save_with_delivery_pass_text.text = dpBannerText
    }

    private fun hideDeliveryPassBanner() {
        delivery_pass_holder?.let {
            it.visibility = View.GONE
            it.parent?.tryCast<View> { this.visibility = View.GONE }
        }
    }

    private fun showNoDpView(deliveryPass: DeliveryPass?) {
        val shouldShowDPOfferCard = showDPOfferValidator.validate(
            ShowDPCardValidatorModel(
                Authentication.getInstance() as Authentication,
                deliveryPass
            )
        )
        if (shouldShowDPOfferCard) {
            dpOfferCardViewHandler?.load()
            book_slot_delivery_pass_holder?.visibility = View.GONE
        } else {
            save_with_delivery_pass_text.text = ViewUtils.fromHtml(getString(R.string.delivery_pass_free_delivery))
        }
        context?.let {
            if (FeatureSettings.isDeliveryPassBlocked(it)) {
                this.book_slot_delivery_pass_details_stub.visibility = View.GONE
            } else {
                this.book_slot_delivery_pass_details_stub.visibility = View.VISIBLE
            }
        }
    }

    override fun onLocationTypeAddressClicked() {
        showSelectAddressScreen()
    }

    override fun getServiceAddressForTab(): ServiceAddress? {
        return viewModel.serviceAddress
    }

    override fun openBookSlotConfirmationPage() {
        val slotDates = viewModel.getBookedStartAndEndTime()
        val intent = DeliveryImpactActivity.newIntent(
            activity, CartManagerConstants.SLOT_TYPE_DELIVERY,
            slotDates.first, slotDates.second,
            SlotBookedConfirmationPresenter.TAG, subscriptionEligibility,
            isFlexSlot = standardSlotSelected.not(),
            isExpressSlot = viewModel.slotInfoOfReserveCallRequest?.is_express_slot ?: false
        )
        startActivityForResult(intent, DeliveryImpactActivity.REQUEST_CODE)
    }

    private fun updateRecurringSlotsView() {
        if (hasRecurringSlot()) {
            book_slot_recurring_slot_divider.visibility = View.VISIBLE
            book_slot_recurring_slot_text.visibility = View.VISIBLE
            setupRecurringSlotClickListener()
        } else {
            book_slot_recurring_slot_divider.visibility = View.GONE
            book_slot_recurring_slot_text.visibility = View.GONE
        }
    }

    private fun setupRecurringSlotClickListener() {
        book_slot_recurring_slot_text.setOnClickListener {
            val intent = DeliveryImpactActivity.newIntent(activity, RecurringSlotDetailsFragment.TAG)
            startActivityForResult(intent, DeliveryImpactActivity.REQUEST_CODE)
        }
    }

    private fun hasRecurringSlot(): Boolean {
        return subscriptionEligibility?.active_subscription == true
    }

    private fun refreshRecurringSlots() {
        context?.let { context ->
            SlotSubscriptionManager.refreshSubscriptionEligibility()
        }
    }

    private fun setupSubscriptionChangedObserver() {
        SlotSubscriptionManager.subscriptionEligibilityChanged.observe(
            viewLifecycleOwner,
            Observer {
                subscriptionEligibility = it?.data?.eligibility_info
                updateRecurringSlotsView()
            }
        )
    }

    private fun setupRecurringSlotObservers() {
        context?.let {
            setupSubscriptionChangedObserver()
        }
        refreshRecurringSlots()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == ADDRESS_SELECTION_REQ_CODE && resultCode == Activity.RESULT_OK) {
            data?.let {
                val selectedAddress: AddressBookResponse.Address = data.getParcelableExtra(SelectAddressActivity.SELECTED_ADDRESS)
                viewModel.serviceAddress = ServiceAddress(
                    selectedAddress.postCode, selectedAddress.latitude, selectedAddress.longitude,
                    selectedAddress.city, selectedAddress.country, selectedAddress.line1, selectedAddress.line2, selectedAddress.addressId, selectedAddress.deliveryInstructions
                )
                setAddress()
                standardSlotSelected = true
                resetGrid()
                viewSlots(true)
            }
        } else if (requestCode == DeliveryImpactActivity.REQUEST_CODE && resultCode == ActivityRequestCodes.RESULT_CODE_RECURRING_SLOT_SUCCESS) {
            updateRecurringSlotsView()
        }
    }

    override fun setAdditionalCostText() {
        additional_cost_text.text = getString(R.string.the_asda_same_day_has_additional_cost, getString(R.string.delivery))
    }

    override fun handleSlotData() {
        context?.let {
            val slotInfo = com.asda.android.cxo.CartManager.instance.cartResponse?.data?.cart?.fulfillmentDetails
            slotInfo?.let { _slotInfo ->
                if (_slotInfo.slotInfo?.fulfillmentType == FulfillmentTypes.DELIVERY.type && _slotInfo.slotInfo?.cartSlotId?.isNotEmpty() == true &&
                    _slotInfo.slotInfo?.cartSlotId?.isNotBlank() == true && _slotInfo.slotInfo?.slotExpired != true
                ) {
                    booked_slot_details_express?.visible()
                    booked_slot_details_stub?.gone()
                    val fulFillmentAddress = _slotInfo.fulfillmentAddress
                    fulFillmentAddress?.let { _fulFillmentAddress ->
                        express_store_name?.text = getString(R.string.express_delivery_address, _fulFillmentAddress.address, _fulFillmentAddress.city)
                        express_store_address?.text = _fulFillmentAddress.postalCode

                        express_slot_date?.text = _slotInfo.slotInfo?.startDateTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.EEEE_DD_MMMM_YYYY)
                        express_slot_time?.text = getString(R.string.express_slot_time, _slotInfo.slotInfo?.startDateTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.H_MM_AA), _slotInfo.slotInfo?.endDateTime?.convertToFormat(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.H_MM_AA))

                        val isSlotExpired = _slotInfo.slotInfo?.bookedUntilTime?.isEarlier(DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z)
                        if (isSlotExpired == true) {
                            context?.let { _context ->
                                express_reserved_until.setTextColor(ContextCompat.getColor(_context, R.color.asda_red))
                                express_reserved_until.text = getString(R.string.home_slot_expired)
                            }
                        } else {
                            val reservedUntil = if (_slotInfo.slotInfo?.isRecurringSlot == true) {
                                SubscriptionUtil.getSubscriptionExpiryDate(_slotInfo.slotInfo?.startDateTime, DateFormats.YYYY_MM_DD_T_HH_MM_SS_Z, DateFormats.HH_MM_AA_EEE_DD_MMM, TimeZoneFormat.UTC, TimeZoneFormat.UTC)
                            } else {
                                _slotInfo.slotInfo?.bookedUntilTime?.getFormattedDateWithDay(addOffSet = false)
                            }
                            express_reserved_until?.text = getString(R.string.slot_reserved_until, reservedUntil)
                        }

                        if (_slotInfo.slotInfo?.slotType == SLOT_TYPE.EXPRESS.type) {
                            context?.let { _context ->
                                express_reserved_until.setTextColor(ContextCompat.getColor(_context, R.color.asda_red))
                            }
                            express_span?.visible()
                            express_span_bookslot?.gone()
                        } else {
                            express_span?.gone()
                        }
                    } ?: run {
                        booked_slot_details_express?.gone()
                    }
                } else {
                    booked_slot_details_express?.gone()
                }
            } ?: run {
                booked_slot_details_express?.gone()
            }
        }
    }

    override fun setExpressAvailable(expressAvailable: Boolean) {
        if (SlotUtils.isExpressEnabled(context) && expressAvailable) {
            setExpressInfo()

            context?.let {
                val cartItemList = com.asda.android.cxo.CartManager.instance.getCartItems()
                val cart = com.asda.android.cxo.CartManager.instance.cartResponse?.data?.cart
                cart?.let { _cart ->
                    if (_cart.fulfillmentDetails?.slotInfo?.fulfillmentType == FulfillmentTypes.DELIVERY.type && _cart.fulfillmentDetails?.slotInfo?.cartSlotId?.isNotEmpty() == true &&
                        _cart.fulfillmentDetails?.slotInfo?.cartSlotId?.isNotBlank() == true
                    ) {
                        express_info.gone()
                        express_span_bookslot?.gone()
                    } else {
                        if (cartItemList?.isEmpty() == true) {
                            express_info.visibility = View.VISIBLE
                            express_span_bookslot?.gone()
                        } else {
                            express_info.visibility = View.GONE
                            express_span_bookslot?.visible()
                        }
                    }
                }
            }
        } else {
            express_span_bookslot?.gone()
        }
    }
}
