package com.asda.android.app.chatbot.utils

import android.content.Context
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.base.core.constants.DeepLinkConstants
import okhttp3.Cookie
import pangea.isQAServer

const val ORDER_DETAILS_SCREEN = "Order Details"
const val ACCOUNT_HOME_SCREEN = "Account Home"

class ChatBotUtils {

    fun isFragmentRedirection(type: String) =
        type == DeepLinkConstants.TYPE_ORDER_DETAILS || type == DeepLinkConstants.TYPE_ORDER_LIST ||
            type == DeepLinkConstants.TYPE_ACCOUNT_HOME || type == DeepLinkConstants.TYPE_REFUND_STATUS

    /**
     * Function tracks chat-bot page view events
     *
     * @param source Chat-bot entry point page name
     */
    fun trackPageView(source: String) {
        val pageViewEvent =
            PageViewEvent(Anivia.PV_CHATBOT, Anivia.SECTION_CHATBOT, Anivia.SECTION_CHATBOT)
        when (source) {
            ORDER_DETAILS_SCREEN -> {
                pageViewEvent.putString(
                    Anivia.CHAT_BOT_ENTRY_POINT_KEY,
                    Anivia.CHAT_BOT_ORDER_DETAILS_ENTRY_VALUE
                )
            }
            ACCOUNT_HOME_SCREEN -> {
                pageViewEvent.putString(
                    Anivia.CHAT_BOT_ENTRY_POINT_KEY,
                    Anivia.CHAT_BOT_ACCOUNT_ENTRY_VALUE
                )
            }
        }
        Tracker.get().trackPageView(pageViewEvent)
    }

    /**
     * Function constructs entry point cookie for chat-bot
     */
    fun getChatBotEntryPointCookie(sourceName: String, context: Context): Cookie {
        return when (sourceName) {
            ORDER_DETAILS_SCREEN -> {
                getCookie("Android_Order_Details", context)
            }
            ACCOUNT_HOME_SCREEN -> {
                getCookie("Android_Account_Tab", context)
            }
            else -> {
                getCookie("Android_Account_Tab", context)
            }
        }
    }

    private fun getCookie(pageName: String, context: Context): Cookie {
        return Cookie.Builder()
            .expiresAt(System.currentTimeMillis() + 24 * 60 * 60 * 1000)
            .name("CHATBOT_ENTRY_POINT").value(pageName)
            .domain(if (isQAServer(context)) "groceries-qa.asda.com" else "groceries.asda.com")
            .path(
                "/"
            ).build()
    }
}
