package com.asda.android.app.chatbot.view

import android.app.Activity
import android.os.Build
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.webkit.CookieManager
import android.webkit.JavascriptInterface
import android.webkit.WebSettings
import android.webkit.WebView
import android.webkit.WebView.setWebContentsDebuggingEnabled
import android.webkit.WebViewClient
import android.widget.FrameLayout
import com.asda.android.R
import com.asda.android.app.chatbot.utils.ChatBotUtils
import com.asda.android.base.core.utils.AsdaBaseCoreConfig
import com.asda.android.base.core.view.fragments.BaseFragment
import com.asda.android.base.core.web.constants.WebConstants
import com.asda.android.base.core.web.constants.WebConstants.CHAT_BOT_JAVASCRIPT
import com.asda.android.base.core.web.model.WebViewModel
import com.asda.android.network.AsdaNetwork
import com.asda.android.orders.constants.OrderConstants.EMPTY
import kotlinx.android.synthetic.main.chat_bot_fragment.*
import java.lang.ref.WeakReference

class ChatBotWebFragment : BaseFragment() {
    override fun getInternalTag(): String? {
        var result: String? = null
        model?.pageName?.let {
            result = it
        }
        if (result == null) {
            result = tempTag
        }
        return result
    }

    private var tempTag: String? = null
    private var model: WebViewModel? = null
    private val chatBotUtils = ChatBotUtils()
    private val chatBotManager = AsdaBaseCoreConfig.chatBotManager
    private var sourceName: String = EMPTY
    override fun onCreate(savedInstanceState: Bundle?) {
        arguments?.let {
            model = it.getParcelable(WebConstants.ARG_PARAM1)
            sourceName = it.getString(WebConstants.SOURCE_NAME, EMPTY)
        }
        super.onCreate(savedInstanceState)
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        return inflater.inflate(R.layout.chat_bot_fragment, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        init()
        super.onViewCreated(view, savedInstanceState)
    }

    override fun onDestroyView() {
        webView.removeJavascriptInterface(CHAT_BOT_JAVASCRIPT)
        super.onDestroyView()
    }

    private fun init() {
        val cookieManager = CookieManager.getInstance()
        cookieManager.setAcceptCookie(true)
        configureWebView()
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            webView.settings.mixedContentMode = WebSettings.MIXED_CONTENT_ALWAYS_ALLOW
            cookieManager.setAcceptThirdPartyCookies(webView, true)
        }
        // Flush asda cookies to webview
        val cookies = AsdaNetwork.getCookies().also {
            it?.add(chatBotUtils.getChatBotEntryPointCookie(sourceName, requireContext()))
        }
        cookies?.let {
            for (cookie in it) {
                val cookieString = cookie.name() + "=" + cookie.value() + "; domain=" + cookie.domain()
                cookieManager.setCookie(cookie.domain(), cookieString)
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                    cookieManager.flush()
                }
            }
        }
        val layoutParams = FrameLayout.LayoutParams(
            FrameLayout.LayoutParams.MATCH_PARENT,
            FrameLayout.LayoutParams.MATCH_PARENT
        )
        webView.layoutParams = layoutParams
        webView.loadUrl(model?.url)
        showProgress(R.string.please_wait)
    }

    private fun configureWebView() {
        model?.allowJavascript?.let {
            webView.settings.javaScriptEnabled = it
        }
        webView.apply {
            settings.apply {
                setSupportMultipleWindows(true)
                javaScriptCanOpenWindowsAutomatically = true
                domStorageEnabled = true
            }
            setWebContentsDebuggingEnabled(true)
            addJavascriptInterface(NativeBridge(), CHAT_BOT_JAVASCRIPT)
            webViewClient = webClient
        }
    }

    override fun onBackPressed() = false

    override fun hasActionBar(): Boolean = true

    private inner class NativeBridge {
        @JavascriptInterface
        fun handleChatBotRedirection(url: String) {
            activity?.apply {
                if (isFinishing.not() && isAdded) {
                    runOnUiThread {
                        chatBotManager.handleRedirectionFromChatBot(
                            WeakReference(this),
                            url
                        )
                    }
                }
            }
        }

        @JavascriptInterface
        fun handleMinimizeChatBot() {
            activity?.apply {
                if (isFinishing.not() && isAdded) {
                    runOnUiThread {
                        chatBotManager.hideChatBot(WeakReference(this as Activity))
                    }
                }
            }
        }

        @JavascriptInterface
        fun handleCloseChatBot() {
            activity?.apply {
                if (isFinishing.not() && isAdded) {
                    runOnUiThread {
                        chatBotManager.closeChatBotSession(WeakReference(this as Activity))
                    }
                }
            }
        }
    }

    private val webClient = object : WebViewClient() {
        override fun onPageFinished(view: WebView?, url: String?) {
            dismissProgress()
            super.onPageFinished(view, url)
        }
    }

    companion object {
        const val TAG = "ChatBotFragment"
        fun newInstance(model: WebViewModel, sourceName: String): ChatBotWebFragment {
            val fragment = ChatBotWebFragment()
            fragment.tempTag = model.pageName
            val bundle = Bundle()
            bundle.putParcelable(WebConstants.ARG_PARAM1, model)
            bundle.putString(WebConstants.SOURCE_NAME, sourceName)
            fragment.arguments = bundle
            return fragment
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        activity?.apply { chatBotManager.closeChatBotSession(WeakReference(this)) }
    }
}
