package com.asda.android.app.checkout

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import com.asda.android.R
import com.asda.android.app.account.view.FeedbackFragment.Companion.newInstance
import com.asda.android.app.checkout.constants.CheckoutConstants.CXO_ADD_CARD_PRESENTER
import com.asda.android.app.checkout.constants.CheckoutConstants.CXO_FEEDBACK_FRAGMENT
import com.asda.android.app.checkout.constants.CheckoutConstants.CXO_ORDER_DETAILS_PRESENTER
import com.asda.android.app.checkout.constants.CheckoutConstants.CXO_SELECT_CARD_PRESENTER
import com.asda.android.app.checkout.view.CheckoutController
import com.asda.android.app.orders.OrderDetailsPresenter
import com.asda.android.app.view.AsdaFullScreenActivity
import com.asda.android.app.view.WrapperPresenter
import com.asda.android.base.core.constants.ActivityRequestCodes
import com.asda.android.base.core.view.ui.Presenter
import com.asda.android.restapi.service.data.checkout.CheckoutResponse
import com.asda.android.service.base.OrderManager
import com.asda.android.utils.Utils

class OrderApplyItemsActivity : AsdaFullScreenActivity() {
    private var mStartPage = 0
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        showHeaderWithBackCarat()

        when (mPresenterType) {
            CXO_ADD_CARD_PRESENTER -> {
                val presenter = CheckoutController.createCXOPresenter(this, null, CXO_ADD_CARD_PRESENTER)
                pushPresenter(presenter, false)
            }
            CXO_SELECT_CARD_PRESENTER -> {
                val presenter = CheckoutController.createCXOPresenter(
                    this,
                    intent.getParcelableExtra(EXTRA_CHECKOUT_DATA),
                    CXO_SELECT_CARD_PRESENTER
                )
                pushPresenter(presenter, false)
            }
            CXO_ORDER_DETAILS_PRESENTER -> {
                val orderId = intent.getStringExtra(EXTRA_ORDER_ID)
                val fragment = OrderDetailsPresenter.newInstance(
                    orderId, OrderManager.get().isEligibleForCNC(orderId)
                )
                fragment.setTitleText(this.getString(R.string.review_your_order))
                pushAndReplacePresenter(WrapperPresenter(this, fragment), true)
            }
            CXO_FEEDBACK_FRAGMENT -> {
                val orderId = intent.getStringExtra(EXTRA_ORDER_ID)
                val tagId = intent.getIntExtra(EXTRA_TAG_ID, -1)
                val fragment = newInstance(
                    "order-confirmation",
                    "asda_uk_grocery_mobile_order_confirmation",
                    tagId,
                    orderId
                )
                pushAndReplacePresenter(WrapperPresenter(this, fragment), true)
            }
        }
    }

    override fun onSaveInstanceState(outState: Bundle) {
        super.onSaveInstanceState(outState)
        Utils.informAboutTooLargePayload(outState, TAG)
        outState.putInt(EXTRA_START_PAGE, mStartPage)
    }

    fun onAbortCheckout() {
        setResult(ActivityRequestCodes.RESULT_CODE_CHECKOUT_COMPLETE_CART)
        close()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == ActivityRequestCodes.REQUEST_CODE_PREVIEW_AMEND) {
            onAbortCheckout()
        } else if (requestCode == ActivityRequestCodes.REQUEST_CODE_REORDER &&
            resultCode == ActivityRequestCodes.RESULT_CODE_SHOW_CART
        ) {
            setResult(ActivityRequestCodes.RESULT_CODE_CHECKOUT_COMPLETE_CART)
            close()
        } else if (requestCode == ActivityRequestCodes.REQUEST_CODE_REORDER &&
            resultCode == ActivityRequestCodes.RESULT_CODE_SHOW_LIST
        ) {
            setResult(resultCode, data)
            close()
        }
    }

    private fun pushPresenter(presenter: Presenter?, animate: Boolean) {
        if (mPresenterStack != null) {
            mPresenterStack?.pushPresenter(presenter, animate)
        }
    }

    private fun pushAndReplacePresenter(presenter: Presenter?, animate: Boolean) {
        if (mPresenterStack != null) {
            mPresenterStack?.pushAndReplacePresenter(presenter, animate)
        }
    }

    fun popPresenter() {
        if (mPresenterStack != null) {
            mPresenterStack?.popPresenter()
        }
    }

    companion object {
        private const val TAG = "CheckoutActivity"
        private const val EXTRA_START_PAGE = "EXTRA_START_PAGE"
        private const val EXTRA_CHECKOUT_DATA = "EXTRA_CHECKOUT_DATA"
        private const val EXTRA_ORDER_ID = "EXTRA_ORDER_ID"
        private const val EXTRA_TAG_ID = "EXTRA_TAG_ID"
        var mPresenterType = 0

        fun startCXOPresenter(
            activity: Activity?,
            type: Int,
            orderId: String?,
            tagId: Int,
            checkoutResponse: CheckoutResponse?
        ) {
            mPresenterType = type
            val intent = Intent(activity, OrderApplyItemsActivity::class.java)
            when (type) {
                CXO_ADD_CARD_PRESENTER -> {
                    mPresenterType = type
                    activity?.startActivityForResult(intent, ActivityRequestCodes.REQUEST_CODE_ADD_CARD)
                }
                CXO_SELECT_CARD_PRESENTER -> {
                    mPresenterType = type
                    intent.putExtra(EXTRA_CHECKOUT_DATA, checkoutResponse)
                    activity?.startActivityForResult(intent, ActivityRequestCodes.REQUEST_CODE_SELECT_CARD)
                }
                CXO_ORDER_DETAILS_PRESENTER -> {
                    mPresenterType = type
                    intent.putExtra(EXTRA_ORDER_ID, orderId)
                    activity?.startActivityForResult(intent, ActivityRequestCodes.REQUEST_CODE_ORDER_DETAILS)
                }
                CXO_FEEDBACK_FRAGMENT -> {
                    mPresenterType = type
                    intent.putExtra(EXTRA_ORDER_ID, orderId)
                    intent.putExtra(EXTRA_TAG_ID, tagId)
                    activity?.startActivityForResult(intent, ActivityRequestCodes.REQUEST_CODE_FEEDBACK)
                }
            }
        }
    }
}
