package com.asda.android.app.checkout

import android.app.Dialog
import android.content.Context
import android.os.Bundle
import android.os.Handler
import android.text.InputFilter
import android.text.InputFilter.LengthFilter
import android.text.TextUtils
import android.util.Log
import android.view.LayoutInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.widget.CompoundButton
import android.widget.LinearLayout
import androidx.appcompat.widget.AppCompatRadioButton
import androidx.core.content.ContextCompat
import androidx.core.util.Pair
import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.app.account.singleprofile.view.SPAddEditPaymentCardFragment
import com.asda.android.app.account.singleprofile.view.SPAddEditPaymentCardFragment.Companion.newInstance
import com.asda.android.app.account.singleprofile.view.SPAddEditPaymentCardFragment.OnCardAddedListener
import com.asda.android.app.model.CardCheckoutModel
import com.asda.android.base.core.view.CustomProgressDialog
import com.asda.android.base.core.view.WrappedDrawable
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.base.core.view.ui.BaseAsdaDialogHelper
import com.asda.android.base.core.view.ui.DialogHelper
import com.asda.android.databinding.SelectCardFragmentBinding
import com.asda.android.restapi.deliverypass.GenericCardCheckoutModel
import com.asda.android.restapi.service.constants.CreditCardTypeConstants
import com.asda.android.restapi.service.data.AsdaResponse
import com.asda.android.restapi.service.data.PaymentDetailsResponse
import com.asda.android.restapi.service.data.PaymentDetailsResponse.PaymentCards
import com.asda.android.singleprofile.features.account.datasource.UserAccountDetails.Companion.instance
import com.asda.android.slots.deliverypass.model.DPCardCheckoutModel
import com.asda.android.utils.Utils
import com.asda.android.utils.view.ViewUtil
import com.google.android.material.textfield.TextInputEditText
import com.google.android.material.textfield.TextInputLayout
import com.jakewharton.rxbinding2.widget.RxTextView
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import io.reactivex.observers.DisposableObserver
import java.util.*
import java.util.concurrent.TimeUnit

class SelectCardFragment : FeaturedFragment() {
    private var mBinding: SelectCardFragmentBinding? = null
    private var mCardsGroup: LinearLayout? = null
    private val mRadioButtons: ArrayList<Pair<LinearLayout?, AppCompatRadioButton?>> = ArrayList()
    private var mPaymentCardsList: MutableList<PaymentCards>? = null
    private var mCardNumber: String? = null
    private var mSelectedCardType: String? = null
    private var mPaymentCard: PaymentCards? = null
    private var mEdtCvvLabel: TextInputLayout? = null
    private var mEdtCvv: TextInputEditText? = null
    private var mCardCvv: String? = null
    private var mPurchaseType: String? = null
    private var mPurchaseDpSku: String? = null
    private var mPurchaseDpTrial = false
    private var mEnableSingleUse = false
    private var mIsDPDetails = false

    interface OnSPCardSelectedListener {
        fun onCardSelected(
            paymentCard: PaymentCards?,
            cardNumber: String?,
            cvv: String?
        )
    }

    private var mContext: Context? = null
    private var mSelectedCardNickName: String? = null
    private var preSelectedLast4DigitsForDp: String? = null
    private var mOnSPCardSelectedListener: OnSPCardSelectedListener? = null
    override fun onAttach(context: Context) {
        super.onAttach(context)
        mContext = context
        pageViewEvent = PageViewEvent(Anivia.PV_SELECT_CARD, Anivia.SECTION_CHECKOUT, Anivia.SECTION_CHECKOUT)
    }

    override fun loadBundleData(bundle: Bundle?) {
        super.loadBundleData(bundle)
        if (bundle != null) {
            mSelectedCardNickName = bundle.getString(SELECTED_CARD_NICKNAME, "")
            preSelectedLast4DigitsForDp = bundle.getString(PRE_SELECTED_LAST_4_DIGITS_FOR_DP, null)
            mPurchaseType = bundle.getString(PURCHASE_TYPE)
            mPurchaseDpSku = bundle.getString(PURCHASE_DP_SKU, "")
            mPurchaseDpTrial = bundle.getBoolean(PURCHASE_DP_TRIAL, false)
            mEnableSingleUse = bundle.getBoolean(ENABLE_SINGLE_USE, false)
            mIsDPDetails = bundle.getBoolean(PURCHASE_IS_DP_DETAIL, false)
        }
    }

    override fun hasActionBar(): Boolean {
        return true
    }

    override fun getActionBarTitle(): String? {
        return getString(R.string.select_card_title)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == android.R.id.home) {
            pop()
            return true
        }
        return false
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        fragmentContainer: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        mBinding = SelectCardFragmentBinding.inflate(inflater, fragmentContainer, false)
        mCardsGroup = mBinding?.group
        mBinding?.saveButton?.isEnabled = false
        mBinding?.saveButton?.setText(R.string.select_card_done)
        mBinding?.addLabel?.setText(R.string.select_card_add)
        wireListeners()
        return mBinding!!.root
    }

    override fun onResume() {
        super.onResume()
        getCards()
    }

    fun setOnSPCardSelectedListener(listener: OnSPCardSelectedListener) {
        mOnSPCardSelectedListener = listener
    }

    private fun wireListeners() {
        mBinding?.saveButton?.setOnClickListener { v: View? ->
            if (mPaymentCardsList != null) {
                if (mPaymentCard?.expired == true) {
                    DialogHelper.displayErrorDialog(
                        mContext?.getString(R.string.selected_card_expired), mContext, SCREEN_NAME
                    )
                } else if (mSelectedCardNickName == mPaymentCard?.nickName || mSelectedCardNickName == mPaymentCard?.cardId) {
                    pop()
                } else {
                    changeCreditCardSP(mPaymentCard, "", mCardCvv)
                }
            } else {
                pop()
            }
        }

        mBinding?.addLabel?.setOnClickListener { v: View? ->
            val addEditPaymentCardFragment = newInstance(
                null, true, mEnableSingleUse, mPurchaseType
            )
            addEditPaymentCardFragment.setOnCardAddedListener(
                object : OnCardAddedListener {
                    override fun onCardAdded(creditCardResponse: AsdaResponse, isSingleUse: Boolean, cardNumber: String?) {
                        if (SPAddEditPaymentCardFragment.PURCHASE_TYPE_DP_RENEW == mPurchaseType && isSingleUse) {
                            changeCreditCardSP(null, "", "")
                        } else {
                            if (creditCardResponse.isSuccess) {
                                if (creditCardResponse is PaymentDetailsResponse &&
                                    creditCardResponse.cards != null && creditCardResponse.cards!!.size > 0 && creditCardResponse.cards!![0] != null
                                ) {
                                    if (cardNumber != null) {
                                        changeCreditCardSP(creditCardResponse.cards!![0], cardNumber, "")
                                    }
                                } else {
                                    BaseAsdaDialogHelper.displayErrorDialog(null, -1, mContext, SCREEN_NAME)
                                }
                            } else if (isSingleUse) {
                                if (context == null) return
                                val disposable: Disposable = instance
                                    .getUserCreditCardsDP(
                                        requireContext(), mPurchaseDpSku, mPurchaseDpTrial
                                    )
                                    .observeOn(AndroidSchedulers.mainThread())
                                    .subscribe { cardCheckoutModel: GenericCardCheckoutModel? ->
                                        val paymentCards = (cardCheckoutModel as CardCheckoutModel).paymentCardsList
                                        if (paymentCards.isNotEmpty()) {
                                            changeCreditCardSP(paymentCards[0], "", "")
                                        }
                                    }
                                safeAdd(disposable)
                                mBinding?.loading?.loadingPage?.visibility = View.GONE
                            } else {
                                BaseAsdaDialogHelper.displayErrorDialog(
                                    creditCardResponse, -1, mContext, SCREEN_NAME
                                )
                            }
                        }
                    }
                })
            push(addEditPaymentCardFragment)
        }
    }

    private fun changeCreditCardSP(paymentCard: PaymentCards?, cardNumber: String, cvv: String?) {
        if (mOnSPCardSelectedListener != null) {
            finish(paymentCard, cardNumber, cvv)
        }
    }

    private fun finish(
        paymentCard: PaymentCards?,
        cardNumber: String,
        cvv: String?
    ) {
        Handler()
            .postDelayed(
                {
                    pop() // pops SelectCardFragment (this)
                    mOnSPCardSelectedListener?.onCardSelected(paymentCard, cardNumber, cvv)
                },
                200
            )
    }

    override fun getView(): View? {
        return mBinding?.root
    }

    private fun populateCreditCards(paymentCards: Array<PaymentCards>, view: View?) {
        mCardsGroup?.removeAllViews()
        mRadioButtons.clear()
        val primaryCards: MutableList<PaymentCards> = ArrayList()
        val secondaryCards: MutableList<PaymentCards> = ArrayList()
        for (paymentCard in paymentCards) {
            if (paymentCard.associatedToOrder) {
                primaryCards.add(paymentCard)
            } else {
                secondaryCards.add(paymentCard)
            }
        }
        primaryCards.addAll(secondaryCards)
        mPaymentCardsList = primaryCards
        var preSelectedCardIndex = 0
        if (mContext != null) {
            for (card in primaryCards) {
                if (!TextUtils.isEmpty(card.getCreditCardNumber())) {
                    val container = ViewUtil.inflate<LinearLayout>(
                        getView()?.context, R.layout.custom_radio_button, view as ViewGroup?, false
                    )
                    val radioButton = ViewUtil.findViewById<AppCompatRadioButton>(container, R.id.radio)
                    val cardText: CharSequence = card.getCreditCardNumber()
                    mCardNumber = cardText.toString()
                    val id = primaryCards.indexOf(card)
                    radioButton.id = id
                    if (isValidForDPCardPreSelection()) {
                        preSelectedCardIndex = id
                    }
                    radioButton.tag = card
                    radioButton.isEnabled = !card.expired
                    val type = Utils.getCardIssuer(card.creditCardType)
                    mSelectedCardType = card.creditCardType
                    val cardIcon = ContextCompat.getDrawable(mContext!!, type.cardIcon)
                    if (cardIcon != null) {
                        val wrappedDrawable = WrappedDrawable(cardIcon)
                        wrappedDrawable.setBounds(
                            0,
                            0,
                            mContext!!.resources.getDimensionPixelSize(R.dimen.credit_card_width),
                            mContext!!.resources.getDimensionPixelSize(R.dimen.credit_card_height)
                        )
                        radioButton.setCompoundDrawablesWithIntrinsicBounds(wrappedDrawable, null, null, null)
                    }
                    radioButton.text = cardText.toString()
                    val res = resources
                    val radioButtonCompoundDrawable = res.getDimension(R.dimen.radio_button_compound_drawable).toInt()
                    val radioButtonLeftPadding = res.getDimension(R.dimen.radio_button_left).toInt()
                    val radioButtonTopPadding = res.getDimension(R.dimen.radio_button_top).toInt()
                    val radioButtonRightPadding = res.getDimension(R.dimen.radio_button_right).toInt()
                    val radioButtonBottomPadding = res.getDimension(R.dimen.radio_button_bottom).toInt()
                    val containerPaddingLeft = res.getDimension(R.dimen.container_left).toInt()
                    val containerPaddingTop = res.getDimension(R.dimen.container_top).toInt()
                    val containerPaddingRight = res.getDimension(R.dimen.container_right).toInt()
                    val containerPaddingBottom = res.getDimension(R.dimen.container_bottom).toInt()
                    radioButton.compoundDrawablePadding = radioButtonCompoundDrawable
                    radioButton.setPadding(
                        radioButtonLeftPadding,
                        radioButtonTopPadding,
                        radioButtonRightPadding,
                        radioButtonBottomPadding
                    )
                    container.setPadding(
                        containerPaddingLeft,
                        containerPaddingTop,
                        containerPaddingRight,
                        containerPaddingBottom
                    )
                    mRadioButtons.add(Pair(container, radioButton))
                    mCardsGroup?.addView(
                        container,
                        LinearLayout.LayoutParams.MATCH_PARENT,
                        LinearLayout.LayoutParams.WRAP_CONTENT
                    )
                }
            }
            for (pair in mRadioButtons) {
                if (pair.second != null && mPaymentCardsList != null) {
                    pair.second!!.setOnCheckedChangeListener { buttonView: CompoundButton, isChecked: Boolean ->
                        if (isChecked) {
                            for (rb in mRadioButtons) {
                                rb.second!!.isChecked = false
                                if (rb.first != null) {
                                    ViewUtil.findViewById<View>(rb.first!!, R.id.edtCvv_label).visibility = View.GONE
                                    ViewUtil.findViewById<View>(rb.first!!, R.id.edtCvv_text).visibility = View.GONE
                                }
                            }
                            buttonView.isChecked = true
                            mPaymentCard = pair.second!!.tag as PaymentCards
                            mSelectedCardType = mPaymentCardsList?.get(pair.second!!.id)?.creditCardType
                            if (mPaymentCardsList != null) {
                                mCardNumber = getString(
                                    R.string.edit_payment_card_number,
                                    mPaymentCardsList!![pair.second!!.id]
                                        .getCreditCardNumber()
                                        .substring(
                                            mPaymentCardsList!!
                                                .get(pair.second!!.id)
                                                .getCreditCardNumber()
                                                .length -
                                                4
                                        )
                                )
                                if (pair.first != null) {
                                    mEdtCvvLabel = ViewUtil.findViewById(pair.first!!, R.id.edtCvv_label)
                                    mEdtCvv = ViewUtil.findViewById(pair.first!!, R.id.edtCvv)
                                    setupCvvObservable(view, mEdtCvv, mEdtCvvLabel)
                                }
                            }
                            if (mIsDPDetails) {
                                mEdtCvvLabel?.setVisibility(View.VISIBLE)
                                mEdtCvv?.requestFocus()
                                if (mEdtCvv != null) {
                                    ViewUtil.showKeyboard(mEdtCvv!!)
                                }
                            } else {
                                mBinding?.saveButton?.isEnabled = true
                            }
                        }
                    }
                }
            }
        }
        if (mRadioButtons.isNotEmpty()) {
            try {
                if (mRadioButtons[preSelectedCardIndex].second != null) {
                    mRadioButtons[preSelectedCardIndex].second?.isChecked = true
                }
                if (mRadioButtons[preSelectedCardIndex].first != null) {
                    ViewUtil.findViewById<View>(mRadioButtons[preSelectedCardIndex].first!!, R.id.edtCvv_text).visibility = View.GONE
                    ViewUtil.findViewById<View>(mRadioButtons[preSelectedCardIndex].first!!, R.id.edtCvv_label).visibility = View.GONE
                }
            } catch (e: IndexOutOfBoundsException) {
                // no op
            }

            getView()?.let {
                ViewUtil.hideKeyboard(it)
            }
        }
        mCardsGroup?.visibility = View.VISIBLE
    }

    private fun isValidForDPCardPreSelection(): Boolean {
        return (
            mIsDPDetails &&
                mCardNumber != null && preSelectedLast4DigitsForDp != null && (
                mCardNumber.equals(preSelectedLast4DigitsForDp, ignoreCase = true) ||
                    mCardNumber!!.length > 4 &&
                    mCardNumber!!
                        .substring(mCardNumber!!.length - 4)
                        .equals(preSelectedLast4DigitsForDp, ignoreCase = true)
                )
            )
    }

    private fun setupCvvObservable(view: View?, cvvEdt: TextInputEditText?, cvvLabel: TextInputLayout?) {
        if (cvvEdt != null) {
            val mEditCvvObservable = RxTextView.textChanges(cvvEdt)
                .skip(3)
                .debounce(300, TimeUnit.MILLISECONDS, AndroidSchedulers.mainThread())
            mEditCvvObservable.subscribe(
                object : DisposableObserver<CharSequence?>() {
                    override fun onNext(cvv: CharSequence) {
                        if (Utils.validateCsc(cvv.toString(), mSelectedCardType)) {
                            mBinding?.saveButton?.isEnabled = true
                            view?.let { ViewUtil.hideKeyboard(it) }
                            cvvLabel?.error = null
                            mCardCvv = cvv.toString()
                        } else {
                            mBinding?.saveButton?.isEnabled = false
                            cvvLabel?.error = getString(R.string.enter_valid_csc)
                            view?.let {
                                ViewUtil.findViewById<View>(it, R.id.edtCvv_text).visibility = View.GONE
                            }
                        }
                        if (CreditCardTypeConstants.TYPE_AMEX == mSelectedCardType) {
                            cvvEdt.filters = arrayOf<InputFilter>(LengthFilter(4))
                            cvvEdt.filters = arrayOf<InputFilter>(LengthFilter(4))
                        } else {
                            cvvEdt.filters = arrayOf<InputFilter>(LengthFilter(3))
                        }
                    }

                    override fun onError(throwable: Throwable) {
                        // No op
                    }

                    override fun onComplete() {
                        // No op
                    }
                })
        }
    }

    private fun getCards() {
        showProgress(R.string.loading)
        mBinding?.loading?.loadingPage?.visibility = View.VISIBLE
        var disposable: Disposable? = null
        if (SPAddEditPaymentCardFragment.PURCHASE_TYPE_DP == mPurchaseType && !mIsDPDetails) {
            if (context == null) return
            disposable = instance
                .getUserCreditCardsDP(requireContext(), mPurchaseDpSku, mPurchaseDpTrial)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    { model: GenericCardCheckoutModel? -> fetchAndPopulateCards(model) },
                    { throwable: Throwable? ->
                        if (Utils.isDebug) {
                            Log.e(TAG, "Unable to fetch credit cards")
                        }
                    }
                )
        } else {
            val observable = instance.userCreditCards
            if (observable != null) {
                disposable = observable
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(
                        { creditCardResponse: PaymentDetailsResponse? ->
                            if (creditCardResponse != null) {
                                val model = CardCheckoutModel(creditCardResponse, null, null)
                                fetchAndPopulateCards(model)
                            }
                        }
                    ) { throwable: Throwable? ->
                        if (Utils.isDebug) {
                            Log.e(TAG, "Unable to fetch credit cards")
                        }
                    }
            }
        }
        if (disposable != null) safeAdd(disposable) else {
            Log.e(TAG, "disposable is null")
            mBinding?.loading?.loadingPage?.visibility = View.GONE
        }
    }

    private fun fetchAndPopulateCards(model: GenericCardCheckoutModel?) {
        val response: PaymentDetailsResponse?
        if (model != null) {
            response = if (model is DPCardCheckoutModel) model.creditCardResponse else (model as CardCheckoutModel).creditCardResponse
            dismissProgress()
            if (!isAdded) {
                return
            }
            if (response?.cards != null) {
                populateCreditCards(response.cards!!, view)
            }
            mBinding?.loading?.loadingPage?.visibility = View.GONE
        }
    }

    override fun onCreateDialog(dialog: Int): Dialog? {
        val d: Dialog?
        when (dialog) {
            DIALOG_SAVING -> {
                val pd = CustomProgressDialog.create(mContext)
                pd.setCancelable(false)
                pd.setMessage(getString(R.string.saving_in_progress))
                pd.isIndeterminate = true
                d = pd
            }
            else -> d = super.onCreateDialog(dialog)
        }
        return d
    }

    override fun getInternalTag(): String? {
        return TAG
    }

    companion object {
        const val TAG = "SelectCardFragment"
        private const val SCREEN_NAME = "Payment Card Select"
        private const val SELECTED_CARD_NICKNAME = "selected_card_arg"
        private const val CHANGE_MAIN_CARD = "change_main_card"
        private const val ENABLE_SINGLE_USE = "enable_single_use"
        private const val PURCHASE_TYPE = "purchase_type"
        private const val PURCHASE_DP_SKU = "purchase_dp_sku"
        private const val PURCHASE_DP_TRIAL = "purchase_dp_trial"
        private const val PURCHASE_IS_DP_DETAIL = "purchase_dp_detail"
        private const val PRE_SELECTED_LAST_4_DIGITS_FOR_DP = "preSelectedLast4Digits"
        private const val DIALOG_SAVING = 1
        fun newInstance(
            selectedCardNickName: String?,
            purchaseType: String?
        ): SelectCardFragment {
            val args = Bundle()
            args.putString(SELECTED_CARD_NICKNAME, selectedCardNickName)
            args.putString(PURCHASE_TYPE, purchaseType)
            val fragment = SelectCardFragment()
            fragment.arguments = args
            return fragment
        }

        fun newInstance(
            changeMainCard: Boolean,
            purchaseType: String?,
            preSelectedLast4DigitsForDP: String?,
            isDPDetails: Boolean
        ): SelectCardFragment {
            val args = Bundle()
            args.putBoolean(CHANGE_MAIN_CARD, changeMainCard)
            args.putString(PURCHASE_TYPE, purchaseType)
            args.putBoolean(ENABLE_SINGLE_USE, false)
            args.putBoolean(PURCHASE_IS_DP_DETAIL, isDPDetails)
            args.putString(PRE_SELECTED_LAST_4_DIGITS_FOR_DP, preSelectedLast4DigitsForDP)
            val fragment = SelectCardFragment()
            fragment.arguments = args
            return fragment
        }

        fun newInstance(
            selectedCardNickName: String?,
            purchaseType: String?,
            dpSkuId: String?,
            isFreeTrial: Boolean
        ): SelectCardFragment {
            val args = Bundle()
            args.putString(SELECTED_CARD_NICKNAME, selectedCardNickName)
            args.putString(PURCHASE_TYPE, purchaseType)
            args.putString(PURCHASE_DP_SKU, dpSkuId)
            args.putBoolean(PURCHASE_DP_TRIAL, isFreeTrial)
            val fragment = SelectCardFragment()
            fragment.arguments = args
            return fragment
        }
    }
}
