package com.asda.android.app.checkout.view

import android.app.Activity
import android.content.Intent
import com.asda.android.app.account.singleprofile.view.SPAddEditPaymentCardFragment.OnCXOCardAddedListener
import com.asda.android.app.checkout.AddCardWrapperPresenter
import com.asda.android.app.checkout.OrderApplyItemsActivity
import com.asda.android.app.checkout.SelectCardPresenter
import com.asda.android.app.checkout.constants.CheckoutConstants
import com.asda.android.base.core.view.ui.Presenter
import com.asda.android.cxo.view.fragments.CREDITCARD
import com.asda.android.cxo.viewmodel.EXTRA_CHECKOUT_RESPONSE

class CheckoutController {
    var deliveryInstructions: String? = null

    companion object {
        private fun createAddCardPresenter(activity: OrderApplyItemsActivity): Presenter {
            val listener = object : OnCXOCardAddedListener {
                override fun onCXOCardAdded(checkoutResponse: com.asda.android.restapi.service.data.checkout.CheckoutResponse?, cardNickName: String?) {
                    val intent = Intent()
                    intent.putExtra(EXTRA_CHECKOUT_RESPONSE, checkoutResponse)
                    activity.setResult(Activity.RESULT_OK, intent)
                    activity.finish()
                }
            }
            return AddCardWrapperPresenter(activity, listener)
        }

        private fun createSelectCardPresenter(
            activity: OrderApplyItemsActivity,
            response: com.asda.android.restapi.service.data.checkout.CheckoutResponse?
        ): Presenter {
            var selectedCard = ""
            if (response?.data != null && response.data?.purchaseOrder != null && response.data?.purchaseOrder?.paymentDetails != null && response.data?.purchaseOrder?.paymentDetails?.isNotEmpty() == true) {
                for ((_, _, _, cardDisplayNumber, _, _, _, _, _, _, _, _, paymentType) in response.data?.purchaseOrder?.paymentDetails!!) {
                    if (CREDITCARD.equals(paymentType, ignoreCase = true)) {
                        if (cardDisplayNumber != null) {
                            selectedCard = cardDisplayNumber
                        }
                        break
                    }
                }
            }
            val presenter = SelectCardPresenter(activity, selectedCard, true)
            presenter.setOnCXOCardChangedListener(object : SelectCardPresenter.OnCXOCardChangedListener {
                override fun onCardChanged(checkoutResponse: com.asda.android.restapi.service.data.checkout.CheckoutResponse?) {
                    val intent = Intent()
                    intent.putExtra(EXTRA_CHECKOUT_RESPONSE, checkoutResponse)
                    activity.setResult(Activity.RESULT_OK, intent)
                    activity.finish()
                }
            })
            return presenter
        }

        fun createCXOPresenter(
            activity: OrderApplyItemsActivity,
            checkoutResponse: com.asda.android.restapi.service.data.checkout.CheckoutResponse?,
            presenterId: Int
        ): Presenter? {
            return when (presenterId) {
                CheckoutConstants.CXO_ADD_CARD_PRESENTER -> createAddCardPresenter(activity)
                CheckoutConstants.CXO_SELECT_CARD_PRESENTER -> createSelectCardPresenter(activity, checkoutResponse)
                else -> null
            }
        }
    }
}
