package com.asda.android.app.dp.view

import android.app.Dialog
import android.content.Context
import android.os.Bundle
import android.os.Handler
import android.text.TextUtils
import android.util.Log
import android.view.LayoutInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import androidx.appcompat.app.AlertDialog
import androidx.core.text.HtmlCompat
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProvider
import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.app.account.singleprofile.view.SPAddEditPaymentCardFragment
import com.asda.android.app.account.view.AccountFragment
import com.asda.android.app.account.view.BaseAccountActivity
import com.asda.android.app.main.view.AsdaActivity
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.base.core.utils.AsdaBaseCoreConfig
import com.asda.android.base.core.utils.orZero
import com.asda.android.base.core.utils.tryCast
import com.asda.android.base.core.view.DialogFactory
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.base.core.view.ui.AsdaDialogHelper
import com.asda.android.base.core.view.ui.showAlertDialog
import com.asda.android.base.interfaces.IAsdaLogger
import com.asda.android.databinding.DpDetailsBinding
import com.asda.android.payment.threeds.ThreeDsDataSource
import com.asda.android.restapi.service.NetworkCallback
import com.asda.android.restapi.service.base.RxFailure
import com.asda.android.restapi.service.constants.AsdaServiceSettingsConstants
import com.asda.android.restapi.service.data.AsdaResponse
import com.asda.android.restapi.service.data.EditDpResponse
import com.asda.android.restapi.service.data.PaymentDetailsResponse
import com.asda.android.restapi.service.data.ViewDPResponse
import com.asda.android.restapi.service.data.ViewDPResponse.DeliveryPassItem
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.service.base.Authentication
import com.asda.android.singleprofile.features.account.view.isValidCardAdded
import com.asda.android.slots.AsdaSlotsConfig
import com.asda.android.slots.deliverypass.model.actionstates.ActiveDPState
import com.asda.android.slots.deliverypass.model.actionstates.CancelDPState
import com.asda.android.slots.deliverypass.model.actionstates.DPToggleAutoRenewState
import com.asda.android.slots.deliverypass.viewmodel.ActiveDPViewModel
import com.asda.android.utils.Utils

class ActiveDPFragment : FeaturedFragment() {

    private lateinit var activeDpViewModel: ActiveDPViewModel
    private var mActiveDP: DeliveryPassItem? = null
    private lateinit var mBinding: DpDetailsBinding
    var purchaseType: String? = null
        private set
    private var mOpenWarningDialog: Dialog? = null
    private var mOpenWarningListener: MutableMap<Int, View.OnClickListener>? = null
    private lateinit var threeDsDataSource: ThreeDsDataSource
    private val mHandler = Handler()

    private val mCloseListener = View.OnClickListener {
        try {
            mOpenWarningDialog?.dismiss()
            mBinding.toggleAutoRenew.isChecked = false
        } catch (e: Exception) {
            Log.w(TAG, Log.getStackTraceString(e))
        }
    }

    private val mAddCardListener = View.OnClickListener {
        try {
            mOpenWarningDialog?.dismiss()
        } catch (e: Exception) {
            Log.w(TAG, Log.getStackTraceString(e))
        }

        val addEditPaymentCardFragment =
            SPAddEditPaymentCardFragment.newInstance(null, true, false, purchaseType)
        addEditPaymentCardFragment.setOnCardAddedListener(object :
                SPAddEditPaymentCardFragment.OnCardAddedListener {
                override fun onCardAdded(
                    creditCardResponse: AsdaResponse,
                    isSingleUser: Boolean,
                    cardNumber: String?
                ) {
                    if (isValidCC(creditCardResponse)) {
                        activeDpViewModel.dataCollection(
                            (creditCardResponse as PaymentDetailsResponse).cards!![0],
                            cardNumber,
                            ""
                        )
                    }
                }
            })

        addEditPaymentCardFragment.setOnSaveStateChange(object :
                SPAddEditPaymentCardFragment.OnSaveStateChange {
                override fun onChange(codeChange: Int) {
                    if (CODE_NO_CHANGE == codeChange) {
                        mBinding.toggleAutoRenew.isChecked = false
                    }
                }
            })
        push(addEditPaymentCardFragment)
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        setUpListeners()
        setUpViewModels()
    }

    private fun setUpListeners() {
        mOpenWarningListener = mutableMapOf()
        mOpenWarningListener!![R.id.close] = mCloseListener
        mOpenWarningListener!![R.id.add_card] = mAddCardListener
    }

    private fun setUpViewModels() {
        threeDsDataSource = ThreeDsDataSource()
        activeDpViewModel = ViewModelProvider(this).get(ActiveDPViewModel::class.java).apply {
            initialize()
        }
    }

    private fun setUpObservers() {
        setUpActiveSubscriptionObserver()
        setUpAutoRenewToggleObserver()
        setUpReloadDataObserver()
        setUpPerformAuthEditDpObserver()
        setUpIsCardChangeInProgressObserver()
        setUpShowErrorLiveData()
        setUpCancelDPLiveData()
        setUpShowDismissLoadingLiveData()
    }

    private fun setUpShowDismissLoadingLiveData() {
        activeDpViewModel.showDismissProgressLiveData.observe(
            viewLifecycleOwner,
            Observer {
                val content = it.getContentIfNotHandled()
                if (content?.not() == true) {
                    dismissProgress()
                }
            }
        )
    }

    private fun setUpCancelDPLiveData() {
        activeDpViewModel.cancelDPLiveData.observe(
            viewLifecycleOwner,
            Observer {
                val content = it.getContentIfNotHandled()
                if (!isAdded || !isVisible) return@Observer
                content?.let { cancelDPState ->
                    when (cancelDPState) {
                        is CancelDPState.Loading -> {
                            showProgress(R.string.please_wait)
                        }
                        is CancelDPState.Loaded -> {
                            dismissProgress()
                            loadDataFromActiveDP()
                        }
                        is CancelDPState.Error -> {
                            val rxFailure = RxFailure(cancelDPState.error.errorCode?.toIntOrNull().orZero(), cancelDPState.error)
                            showDialog(rxFailure)
                        }
                    }
                }
            }
        )
    }

    private fun setUpShowErrorLiveData() {
        activeDpViewModel.showErrorLiveData.observe(
            viewLifecycleOwner,
            Observer { throwable ->
                val content = throwable.getContentIfNotHandled()
                content?.let {
                    if (!isAdded) return@Observer
                    showDialog(it)
                }
            }
        )
    }

    private fun setUpIsCardChangeInProgressObserver() {
        activeDpViewModel.isCardChangeInProgressLiveData.observe(
            viewLifecycleOwner,
            Observer { inProgress ->
                val content = inProgress.getContentIfNotHandled()
                content?.let {
                    if (it) showProgress(R.string.please_wait) else dismissProgress()
                }
            }
        )
    }

    private fun setUpPerformAuthEditDpObserver() {
        activeDpViewModel.performSecureAuthEditDPLiveData.observe(
            viewLifecycleOwner,
            Observer { performSecureAuthCheckoutDPOneTimeLiveDataEvent ->
                val content =
                    performSecureAuthCheckoutDPOneTimeLiveDataEvent.getContentIfNotHandled()
                content?.let {
                    performSecureAuthenticationAndEditDP(
                        it.editDpResponse,
                        it.paymentCards,
                        it.cardNumber,
                        it.cvv
                    )
                }
            }
        )
    }

    private fun setUpReloadDataObserver() {
        activeDpViewModel.reloadDPDetailsLiveData.observe(
            viewLifecycleOwner,
            Observer { isReloadingRequired ->
                if (!isAdded || !isVisible) return@Observer
                val content = isReloadingRequired.getContentIfNotHandled()
                content?.let {
                    if (it) {
                        loadDataFromActiveDP()
                    } else {
                        DialogFactory.createAlertDialog(
                            context?.getString(R.string.error_card_auth),
                            context,
                            SCREEN_NAME
                        ).show()
                    }
                }
            }
        )
    }

    private fun setUpAutoRenewToggleObserver() {
        activeDpViewModel.toggleAutoRenewLiveData.observe(
            viewLifecycleOwner,
            Observer { toggleState ->
                if (!isAdded || !isVisible) return@Observer
                val content = toggleState.getContentIfNotHandled()
                content?.let {
                    handleToggleDPResponse(it)
                }
            }
        )
    }

    private fun setUpActiveSubscriptionObserver() {
        activeDpViewModel.activeSubscriptionLiveData.observe(
            viewLifecycleOwner,
            Observer { activeSubscriptionState ->
                val content = activeSubscriptionState.getContentIfNotHandled()
                content?.let {
                    if (isAdded.not() || isVisible.not()) return@Observer
                    when (it) {
                        is ActiveDPState.Loading -> {
                            activeDpViewModel.mIsLoading.set(true)
                        }

                        is ActiveDPState.Loaded -> {
                            handleActiveDPSuccess(it.response, it.validateToggleDP)
                        }

                        is ActiveDPState.Error -> {
                            val rxFailure = RxFailure(it.error.errorCode?.toIntOrNull().orZero(), it.error)
                            if (it.error.firstError.errorCode == NO_ACTIVE_DP_ERROR_CODE.toString()) {
                                handleNoActiveDP()
                            } else {
                                AsdaBaseCoreConfig.asdaLogger.log(
                                    error = IAsdaLogger.AsdaError(
                                        exception = rxFailure
                                    )
                                )
                                showDialog(rxFailure)
                            }
                        }
                    }
                }
            }
        )
    }

    private fun handleToggleDPResponse(dpToggleAutoRenewState: DPToggleAutoRenewState) {
        when (dpToggleAutoRenewState) {
            is DPToggleAutoRenewState.Loading -> {
                showProgress(R.string.please_wait)
            }

            is DPToggleAutoRenewState.Loaded -> {
                dismissProgress()
                loadDataFromActiveDP(validateToggleDP = true)
                activeDpViewModel.updateInitSessionSingleProfileResponse()
            }

            is DPToggleAutoRenewState.Error -> {
                dismissProgress()
                dpToggleAutoRenewState.error.tryCast<RxFailure> {
                    AsdaDialogHelper.displayErrorDialog(
                        this.asdaResponse,
                        this.integer,
                        context,
                        Anivia.PV_DP_ORDER_SUMMARY
                    )
                }
                mBinding.toggleAutoRenew.isChecked = false
                loadDataFromActiveDP()
            }
        }
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        mBinding = DpDetailsBinding.inflate(inflater, container, false)
        mBinding.model = activeDpViewModel
        mBinding.handler = ActiveDPViewHandler()
        mBinding.fragment = this
        return mBinding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        setUpObservers()
    }

    override fun onResume() {
        super.onResume()
        loadDataFromActiveDP()
    }

    private fun bindViewFromModel() {
        mBinding.model = activeDpViewModel
        mBinding.toggleAutoRenew.setOnCheckedChangeListener { buttonView, isChecked ->
            if (activeDpViewModel.mIsAutoRenewActive.get() != isChecked) {
                showWarningPopUpBeforeEditingDP(isChecked)
            }
        }
    }

    private fun showWarningPopUpBeforeEditingDP(checked: Boolean) {
        context?.let {
            if (checked)
                showDialog(DIALOG_TOGGLE_AUTO_RENEW_ENABLE)
            else
                showDialog(DIALOG_TOGGLE_AUTO_RENEW_DISABLE)
        }
    }

    private fun openWarningDialog() {
        context?.let {
            mOpenWarningDialog =
                AsdaDialogHelper.displayCustomDialog(
                    R.layout.dp_renew_no_card,
                    mOpenWarningListener,
                    null,
                    false,
                    it
                )
        }
    }

    private fun loadDataFromActiveDP(validateToggleDP: Boolean = false) {
        activeDpViewModel.mIsLoading.set(true)

        if (AsdaSlotsConfig.featureSettingsManager.isSubscriptionAPIEnabled(requireContext())) {
            activeDpViewModel.getActiveSubscription(validateToggleDP)
        } else {
            AsdaServiceFactory.get().viewUserDP(object : NetworkCallback<ViewDPResponse?>() {

                override fun onFailure(error: Int?, data: ViewDPResponse?) {
                    if (isAdded) {
                        activeDpViewModel.mIsLoading.set(false)
                    }
                }

                override fun onSuccess(data: ViewDPResponse?) {
                    handleActiveDPSuccess(data, validateToggleDP)
                }
            })

            val activeDp = (Authentication.getInstance() as Authentication).getActiveDeliveryPass()
            if (activeDp != null) {
                mBinding.title.text = ActiveDPViewModel.stripPriceFromName(activeDp.name)
            }
        }
    }

    private fun handleActiveDPSuccess(data: ViewDPResponse?, validateToggleDP: Boolean = false) {
        if (!isAdded) return
        mActiveDP = null
        if (!data?.activeDpPromotions.isNullOrEmpty()) {
            handleActiveDPExists(data, validateToggleDP)
        } else {
            handleNoActiveDP()
        }
    }

    private fun handleActiveDPExists(data: ViewDPResponse?, validateToggleDP: Boolean = false) {
        if (data?.activeDpPromotions.isNullOrEmpty()) return

        data?.activeDpPromotions?.get(0)?.let {
            mActiveDP = it
            AsdaSlotsConfig.activeDeliveryPass = it
            activeDpViewModel.initDPData(it)
            purchaseType = getPurchaseType(it)
            bindViewFromModel()
            activeDpViewModel.loadRecurringSlots()
            activeDpViewModel.loadCardData()

            if (validateToggleDP) validateDP(true, data)
        }
    }

    private fun handleNoActiveDP() {
        mActiveDP = null
        AsdaSlotsConfig.activeDeliveryPass = null
        activeDpViewModel.mIsLoading.set(false)
        activeDpViewModel.resetToNoActiveDP()
        context?.let {
            activeDpViewModel.mIsDeliveryPassBlocked.set(FeatureSettings.isDeliveryPassBlocked(it))
            val dpUnavailableText =
                SharedPreferencesUtil.getDeliveryPassUnavailableText(it).orEmpty()
            val slotUnavailableText =
                SharedPreferencesUtil.getSlotUnavailableTextWithoutDp(it).orEmpty()
            if (activeDpViewModel.mIsDeliveryPassBlocked.get()) {
                activeDpViewModel.mActiveDPTitletext.set(
                    HtmlCompat.fromHtml(
                        dpUnavailableText, HtmlCompat.FROM_HTML_MODE_LEGACY
                    ).toString() + System.lineSeparator() + System.lineSeparator() +
                        HtmlCompat.fromHtml(
                            slotUnavailableText,
                            HtmlCompat.FROM_HTML_MODE_LEGACY
                        )
                )
            } else {
                activeDpViewModel.mActiveDPTitletext.set(it.getString(R.string.active_delivery_pass))
            }
        }
    }

    private fun getPurchaseType(it: DeliveryPassItem): String? {
        return if (STATUS_RENEW_PENDING == it.status || STATUS_TRIAL_RENEW_PENDING == it.status)
            SPAddEditPaymentCardFragment.PURCHASE_TYPE_DP_RENEW
        else
            SPAddEditPaymentCardFragment.PURCHASE_TYPE_DP
    }

    private fun validateDP(isChecked: Boolean, data: ViewDPResponse?) {
        if (isChecked && !isValidCardAdded() && !isCardDetailsPresent(data)) {
            openWarningDialog()
        }
    }

    private fun isCardDetailsPresent(data: ViewDPResponse?): Boolean {
        return (
            data?.activeDpPromotions != null && data.activeDpPromotions!!.isNotEmpty() && !TextUtils.isEmpty(
                data.activeDpPromotions!![0].dpCardType
            ) &&
                !TextUtils.isEmpty(data.activeDpPromotions!![0].dpCardNumber)
            )
    }

    override fun hasActionBar(): Boolean {
        val activity = activity
        return activity is AsdaActivity || activity is DeliveryPassPurchaseActivity || activity is BaseAccountActivity || parentFragment is AccountFragment
    }

    override fun getActionBarTitle(): String? {
        return if (activity != null) getString(R.string.delivery_pass) else ""
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == android.R.id.home) {
            pop()
            return true
        }
        return false
    }

    override fun getInternalTag(): String? {
        return TAG
    }

    override fun onCreateDialog(dialog: Int): Dialog? {
        activity?.let { activity ->
            when (dialog) {
                DIALOG_CANCEL_DP -> {
                    val builder = AlertDialog.Builder(activity)
                    return builder
                        .setTitle(R.string.are_you_sure)
                        .setMessage(activeDpViewModel.getCancellationPopUpMessage(mActiveDP))
                        .setPositiveButton(R.string.no, null)
                        .setNegativeButton(R.string.yes) { dialog1, whichButton ->
                            cancelDP()
                        }
                        .create()
                }

                DIALOG_VOUCHER_INFO -> {
                    val builder = AlertDialog.Builder(activity)
                    return builder.setMessage(
                        getString(
                            R.string.dp_voucher_info,
                            Utils.addPoundIfNecessary(activeDpViewModel.rewardValue)
                        )
                    ).setPositiveButton(R.string.ok, null)
                        .create()
                }

                DIALOG_AUTO_RENEW_INFO -> {
                    val builder = AlertDialog.Builder(activity)
                    return builder.setMessage(R.string.dp_autorenew_info)
                        .setPositiveButton(R.string.ok, null)
                        .create()
                }

                DIALOG_TOGGLE_AUTO_RENEW_ENABLE -> {
                    showAlertDialog(
                        getString(R.string.dp_auto_renew_to_be_enabled),
                        getString(R.string.dp_dialog_continue),
                        getString(R.string.dp_dialog_cancel),
                        positiveCallback = {
                            activeDpViewModel.toggleAutoRenew(activeDpViewModel.orderId, true, Authentication.getInstance().csrfToken)
                        },
                        negativeCallback = {
                            mBinding.toggleAutoRenew.toggle()
                        },
                        isCancelable = false
                    )
                }

                DIALOG_TOGGLE_AUTO_RENEW_DISABLE -> {
                    showAlertDialog(
                        getString(R.string.dp_auto_renew_to_be_disabled, activeDpViewModel.autoRenewEndDate),
                        getString(R.string.dp_dialog_continue),
                        getString(R.string.dp_dialog_cancel),
                        positiveCallback = {
                            activeDpViewModel.toggleAutoRenew(activeDpViewModel.orderId, false, Authentication.getInstance().csrfToken)
                        },
                        negativeCallback = {
                            mBinding.toggleAutoRenew.toggle()
                        },
                        isCancelable = false
                    )
                }
            }
        }
        return super.onCreateDialog(dialog)
    }

    private fun cancelDP() {
        if (context == null) return
        if (AsdaSlotsConfig.featureSettingsManager.isSubscriptionAPIEnabled(requireContext())) {
            activeDpViewModel.cancelSubscription()
        } else {
            showProgress(R.string.please_wait)
            safeAdd(
                AsdaServiceFactory.get().cancelDP(Authentication.getInstance().csrfToken)
                    ?.subscribe(
                        {
                            dismissProgress()
                            if (isAdded) {
                                loadDataFromActiveDP()
                            }
                        },
                        { throwable ->
                            dismissProgress()
                            if (isAdded) {
                                showDialog(throwable)
                            }
                        }
                    )
            )
        }
    }

    private fun isValidCC(creditCardResponse: AsdaResponse?): Boolean {
        return if (creditCardResponse is PaymentDetailsResponse) {
            creditCardResponse.cards?.let {
                !it.isNullOrEmpty() && !it[0]?.cardId.isNullOrEmpty()
            } ?: run {
                false
            }
        } else false
    }

    private fun performSecureAuthenticationAndEditDP(
        editDpResponse: EditDpResponse,
        paymentCard: PaymentDetailsResponse.PaymentCards?,
        cardNumber: String?,
        cvv: String?
    ) {

        context?.let {
            threeDsDataSource.doThreeDsSecureCheckout(it, editDpResponse, "", FeatureSettings.isThreedsEnabledForDp(it) && FeatureSettings.isThreedsEnabledForEditDp(it))
            threeDsDataSource.getThreeDsLiveData().observe(
                this,
                Observer { threeDsResponse ->
                    if (AsdaServiceSettingsConstants.debug) {
                        Log.i(TAG, "editdp()" + threeDsResponse.status)
                    }
                    handleThreeDsResponse(threeDsResponse, paymentCard, cardNumber, cvv)
                }
            )
        }
    }

    private fun handleThreeDsResponse(
        threeDsResponse: ThreeDsDataSource.ThreeDsResult?,
        paymentCard: PaymentDetailsResponse.PaymentCards?,
        cardNumber: String?,
        cvv: String?
    ) {
        threeDsResponse?.let {
            if (it.status == context?.getString(R.string.threeDsSuccess)) {
                handleThreeDsSuccess(it, paymentCard, cardNumber, cvv)
            } else if (it.status == context?.getString(R.string.threeDsFailed) && context != null) {
                handleThreeDsFailed(it)
            }
        }
    }

    private fun handleThreeDsSuccess(
        it: ThreeDsDataSource.ThreeDsResult,
        paymentCard: PaymentDetailsResponse.PaymentCards?,
        cardNumber: String?,
        cvv: String?
    ) {
        if (!it.jwt.isNullOrEmpty()) {
            activeDpViewModel.newUpdateDP(paymentCard, cardNumber, cvv, it.jwt)
        }
        if (!it.paRes.isNullOrEmpty()) {
            activeDpViewModel.newUpdateDP(paymentCard, cardNumber, cvv, it.paRes)
        }
    }

    private fun handleThreeDsFailed(it: ThreeDsDataSource.ThreeDsResult) {
        if (it.errorNumber != null && it.errorDescription != null) {
            mHandler.postDelayed(
                {
                    DialogFactory.createAlertDialog(
                        context?.getString(R.string.error_card_auth),
                        context,
                        SCREEN_NAME
                    ).show()
                },
                DELAY_SECONDS.toLong()
            )
        }
        if (it.errorResponse?.firstErrorMessage != null) {
            AsdaDialogHelper.displayErrorDialog(it.errorResponse, 1000, context, SCREEN_NAME)
        } else {
            DialogFactory.createAlertDialog(
                context?.getString(R.string.error_card_auth),
                context,
                SCREEN_NAME
            ).show()
        }
    }

    companion object {
        const val SCREEN_NAME = "Delivery Pass Details"
        const val TAG = "DPDetailsFragment"

        private const val STATUS_RENEW_PENDING = "RENEW_PENDING"
        private const val STATUS_TRIAL_RENEW_PENDING = "TRIAL_RENEW_PENDING"
        private const val CODE_NO_CHANGE = -1

        const val DIALOG_VOUCHER_INFO = 2
        const val DIALOG_AUTO_RENEW_INFO = 3
        const val DIALOG_CANCEL_DP = 4
        private const val DIALOG_TOGGLE_AUTO_RENEW_ENABLE = 6
        private const val DIALOG_TOGGLE_AUTO_RENEW_DISABLE = 7
        private const val DELAY_SECONDS = 50
        private const val NO_ACTIVE_DP_ERROR_CODE = 403
    }
}
