package com.asda.android.app.dp.view

import android.view.View
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.constants.Anivia
import com.asda.android.app.checkout.SelectCardFragment
import com.asda.android.app.recurringslot.RecurringSlotDetailsFragment
import com.asda.android.app.recurringslot.RecurringSlotInfoFragment
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.base.core.view.ViewUtil.openWebPage
import com.asda.android.restapi.service.data.PaymentDetailsResponse
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.slots.deliverypass.viewmodel.ActiveDPViewModel

class ActiveDPViewHandler {

    companion object {
        const val TAG = "DPViewModel"
    }

    fun onClickVoucherInfo(v: View, fragment: ActiveDPFragment?) {
        fragment?.let {
            it.showDialog(ActiveDPFragment.DIALOG_VOUCHER_INFO)
        }
    }

    fun onClickViewDPs(v: View, fragment: ActiveDPFragment?) {
        fragment?.let {
            DeliveryPassPurchaseActivity.startDPList(it.activity, v)
        }
    }

    fun onClickRecurringSlotInfo(v: View, fragment: ActiveDPFragment?) {
        fragment?.push(RecurringSlotInfoFragment.newInstance())
    }

    fun onClickAutoRenewInfo(v: View, mActiveDPFragment: ActiveDPFragment?) {
        mActiveDPFragment?.showDialog(ActiveDPFragment.DIALOG_AUTO_RENEW_INFO)
    }

    fun onClickLearnMore(v: View, mActiveDPFragment: ActiveDPFragment?) {
        if (mActiveDPFragment != null) {
            DeliveryPassIntroActivity.launch(mActiveDPFragment.activity!!)
        }
    }

    fun onClickViewRecurringSlot(v: View, mActiveDPFragment: ActiveDPFragment?) {
        mActiveDPFragment?.push(RecurringSlotDetailsFragment.newInstance())
    }

    fun onClickFAQ(v: View) {
        if (v.context != null) {
            openWebPage(SharedPreferencesUtil.getDPFAQUrl(v.context), v.context, TAG)
            Tracker.get().trackEvent(
                AsdaAnalyticsEvent(Anivia.DP_FAQ_EVENT)
                    .putString(Anivia.ENGAGEMENT_REPORT_KEY, "Delivery Pass : FAQ Views")
            )
        }
    }

    fun onClickTC(v: View) {
        if (v.context != null) {
            openWebPage(
                SharedPreferencesUtil.getDPTermsAndConditionsUrl(v.context),
                v.context,
                TAG
            )
        }
    }

    fun onClickChangeCard(
        v: View,
        mActiveDPFragment: ActiveDPFragment?,
        activeDpViewModel: ActiveDPViewModel
    ) {
        if (mActiveDPFragment != null) {
            val selectCardFragment =
                SelectCardFragment.newInstance(true, mActiveDPFragment.purchaseType, activeDpViewModel.mCreditCardNumber, true)

            selectCardFragment.setOnSPCardSelectedListener(object : SelectCardFragment.OnSPCardSelectedListener {
                override fun onCardSelected(creditCard: PaymentDetailsResponse.PaymentCards?, cardNumber: String?, cvv: String?) {
                    activeDpViewModel.setCardChangingStatus(true)
                    if (creditCard != null && FeatureSettings.isThreedsEnabledForDp(v.context) && FeatureSettings.isThreedsEnabledForEditDp(v.context)) {
                        activeDpViewModel.dataCollection(creditCard, cardNumber, cvv)
                    } else {
                        activeDpViewModel.updateDP(creditCard, cardNumber, cvv)
                    }
                }
            })
            mActiveDPFragment.push(selectCardFragment)
        }
    }

    fun onClickCancelDP(
        v: View,
        mActiveDPFragment: ActiveDPFragment?,
        mIsFreeTrialActive: Boolean
    ) {
        mActiveDPFragment?.showDialog(ActiveDPFragment.DIALOG_CANCEL_DP)
    }
}
