package com.asda.android.app.dp.view

import android.app.Dialog
import android.content.Context
import android.content.DialogInterface
import android.os.Bundle
import android.text.TextUtils
import android.util.Log
import android.view.LayoutInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import androidx.appcompat.app.AlertDialog
import androidx.fragment.app.DialogFragment
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProvider
import com.asda.android.R
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.app.account.singleprofile.view.SPAddEditPaymentCardFragment
import com.asda.android.app.account.singleprofile.view.SPAddEditPaymentCardFragment.Companion.CODE_NO_CHANGE
import com.asda.android.app.checkout.SelectCardFragment
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.base.core.view.DialogFactory
import com.asda.android.base.core.view.actitivies.AsdaAppCompatActivity
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.base.core.view.ui.AsdaDialogHelper
import com.asda.android.base.core.view.ui.DialogHelper
import com.asda.android.databinding.DeliveryPassCheckoutBinding
import com.asda.android.payment.threeds.ThreeDsDataSource
import com.asda.android.restapi.service.base.RxFailure
import com.asda.android.restapi.service.constants.AsdaServiceSettingsConstants
import com.asda.android.restapi.service.data.AsdaResponse
import com.asda.android.restapi.service.data.CheckoutDeliveryPassResponse
import com.asda.android.restapi.service.data.ConfirmDeliveryPassResponse
import com.asda.android.restapi.service.data.PaymentDetailsResponse
import com.asda.android.restapi.service.data.checkout.ThreeDsVerification
import com.asda.android.service.base.Authentication
import com.asda.android.slots.deliverypass.model.DeliveryPassItemModel
import com.asda.android.slots.deliverypass.model.actionstates.DPCheckoutLoadContentState
import com.asda.android.slots.deliverypass.model.actionstates.DPCheckoutState
import com.asda.android.slots.deliverypass.model.actionstates.DPConfirmOrderState
import com.asda.android.slots.deliverypass.parser.DPItemModelPriceParser
import com.asda.android.slots.deliverypass.viewmodel.DPCheckoutViewModel
import com.asda.android.slots.subscription.constants.SubscriptionErrorCodes
import com.asda.android.utils.Utils
import com.asda.android.utils.view.ViewUtil

class DeliveryPassCheckoutFragment : FeaturedFragment() {
    private var mSelectedCardNickname: String? = null
    private var mRetrying: Boolean = false
    private var mDPModel: DeliveryPassItemModel? = null
    private var mSelectedDPId: String? = null
    private var mPassPlainCvv = true
    private var mSelectedCard: PaymentDetailsResponse.PaymentCards? = null
    private var mCheckoutDeliveryResponse: CheckoutDeliveryPassResponse? = null
    private var mIsAddingCard: Boolean = false
    private var mLoadData = true
    private var dpItemModelPriceParser: DPItemModelPriceParser? = null

    private var mContext: AsdaAppCompatActivity? = null
    private lateinit var mBinding: DeliveryPassCheckoutBinding
    private lateinit var mViewModel: DPCheckoutViewModel

    private val onDismissFinishListener = (
        DialogInterface.OnDismissListener {
            activity?.finish()
        }
        )

    override fun loadBundleData(bundle: Bundle?) {
        super.loadBundleData(bundle)
        bundle?.let {
            mSelectedDPId = it.getString(DP_SELECTED_ID)
            mDPModel = it.getParcelable(DP_MODEL)
            mSelectedCardNickname = it.getString(DP_SELECTED_CARD)
        }
    }

    override fun onStart() {
        super.onStart()
        Tracker.get().trackPageView(PageViewEvent(Anivia.PV_DP_ORDER_SUMMARY, Anivia.SECTION_DP, Anivia.SECTION_DP))
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        mContext = context as AsdaAppCompatActivity
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == android.R.id.home) {
            ViewUtil.hideKeyboard(mBinding.securityCode)
            pop()
            return true
        }
        return false
    }

    override fun getActionBarTitle(): String? {
        return mContext?.getString(R.string.buy_delivery_pass_title)
    }

    override fun getScreenName(): String {
        return SCREEN_NAME
    }

    override fun hasActionBar(): Boolean {
        return true
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        fragmentContainer: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        mBinding = DeliveryPassCheckoutBinding.inflate(inflater, fragmentContainer, false)
        setUpViewModel()
        setUpBindingVariables()
        setUpObservables()
        dpItemModelPriceParser = DPItemModelPriceParser()
        return mBinding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        getData()
    }

    private fun getData() {
        if (!mIsAddingCard) {
            if (mLoadData) {
                mViewModel.loadContentForDPId(
                    mSelectedDPId!!,
                    (Authentication.getInstance() as Authentication).isDPFreeTrialEligible &&
                        (mDPModel?.isMonthlyFreeTrial == true || mDPModel?.isOneTimeFreeTrial == true)
                )
            } else {

                mCheckoutDeliveryResponse?.let {
                    mPassPlainCvv = it.passPlainCvv
                    dismissProgress()
                    mViewModel.setFreeTrialData(it)
                    if (!it.dpAutoRenew.isNullOrEmpty()) mViewModel.mIsAutoRenew.set(
                        java.lang.Boolean.valueOf(
                            it.dpAutoRenew
                        )
                    )
                    setSelectedCard(mSelectedCard)
                }
            }
        } else {
            pop()
        }
    }

    private fun setUpObservables() {
        mViewModel.dpCheckoutLoadContentStateLiveData.observe(
            viewLifecycleOwner,
            Observer { dpCheckoutLoadContentStateOneTimeLiveDataEvent ->
                val content =
                    dpCheckoutLoadContentStateOneTimeLiveDataEvent.getContentIfNotHandled()

                if (!isAdded) return@Observer

                content?.let {
                    when (it) {
                        is DPCheckoutLoadContentState.Loading -> showProgress(R.string.loading)
                        is DPCheckoutLoadContentState.Loaded -> handleInitialContentLoaded(it)
                        is DPCheckoutLoadContentState.Error -> handleInitialContentLoadError(it)
                    }
                }
            }
        )

        mViewModel.dpConfirmOrderStateLiveData.observe(
            viewLifecycleOwner,
            Observer { dpConfirmOrderStateOneTimeLiveDataEvent ->

                val content = dpConfirmOrderStateOneTimeLiveDataEvent.getContentIfNotHandled()

                if (!isAdded) return@Observer

                content?.let {
                    when (it) {
                        is DPConfirmOrderState.Loading -> showProgress(R.string.placing_order)
                        is DPConfirmOrderState.Loaded -> handleConfirmOrderLoaded(it)
                        is DPConfirmOrderState.Error -> handleConfirmOrderError(it)
                    }
                }
            }
        )

        mViewModel.dpCheckoutStateLiveData.observe(
            viewLifecycleOwner,
            Observer { dpCheckoutStateOneTimeLiveDataEvent ->
                val content = dpCheckoutStateOneTimeLiveDataEvent.getContentIfNotHandled()
                if (!isAdded) return@Observer

                content?.let {
                    when (it) {
                        is DPCheckoutState.Loading -> showProgress(R.string.placing_order)
                        is DPCheckoutState.Loaded -> handleCheckoutLoaded(it)
                        is DPCheckoutState.Error -> handleCheckoutError(it)
                    }
                }
            }
        )
    }

    private fun setUpBindingVariables() {
        mBinding.model = mViewModel
        mBinding.handler = DeliveryPassCheckoutViewHandler()
        mBinding.fragment = this
    }

    private fun setUpViewModel() {
        activity?.let {
            mViewModel = ViewModelProvider(it).get(DPCheckoutViewModel::class.java)
            mViewModel.init(mSelectedDPId!!, mDPModel!!)
        }
    }

    private fun handleInitialContentLoadError(content: DPCheckoutLoadContentState.Error) {

        val throwable = content.error

        if (throwable is RxFailure) {
            val response = throwable.asdaResponse
            val checkoutDeliveryPassResponse: CheckoutDeliveryPassResponse
            val error = throwable.integer

            if (response is CheckoutDeliveryPassResponse) {
                checkoutDeliveryPassResponse = response
                mPassPlainCvv = checkoutDeliveryPassResponse.passPlainCvv
                dismissProgress()
                if (!isAdded) return
                if (checkoutDeliveryPassResponse.hasErrorCode(ConfirmDeliveryPassResponse.ERROR_FREETRIAL_NOT_ELIGIBLE)) {
                    DialogFactory.createAlertDialog(
                        getString(R.string.dp_freetrial_ineligible),
                        getString(R.string.continue_button),
                        { dialog, which ->
                            if (isAdded) {
                                showProgress(R.string.loading)
                                mViewModel.loadContentForDPId(
                                    content.selectedDPId!!,
                                    content.isFreeTrial
                                )
                            }
                        },
                        getString(R.string.cancel),
                        context,
                        SCREEN_NAME
                    )
                        .show()
                } else {
                    showDialog(error, checkoutDeliveryPassResponse)
                    pop()
                }
            }
        }
    }

    private fun handleInitialContentLoaded(content: DPCheckoutLoadContentState.Loaded) {
        val model = content.response
        if (model != null) {
            val paymentCards = model.paymentCardsList
            if (paymentCards.isEmpty()) {
                openAddCard()
            } else {
                mCheckoutDeliveryResponse = model.checkoutDeliveryPassResponse
                mCheckoutDeliveryResponse?.let {
                    mPassPlainCvv = it.passPlainCvv
                    dismissProgress()
                    if (paymentCards.isNotEmpty()) {
                        setSelectedCard(paymentCards[0])
                    }
                    mViewModel.setFreeTrialData(it)
                    if (!it.dpAutoRenew.isNullOrEmpty()) {
                        mViewModel.mIsAutoRenew.set(java.lang.Boolean.valueOf(it.dpAutoRenew))
                    }
                }
            }
        }
    }

    fun doCheckout() {
        val csc = mBinding.securityCode.text.toString().trim { it <= ' ' }
        mSelectedCard?.let {
            if (Utils.validateCsc(csc, it.creditCardType)) {
                mViewModel.confirmOrder(
                    viewLifecycleOwner,
                    mSelectedDPId!!,
                    mSelectedCardNickname!!,
                    csc,
                    null,
                    mSelectedDPId == mDPModel?.monthlyId,
                    mPassPlainCvv, null, null
                )
                ViewUtil.hideKeyboard(mBinding.securityCode)
            } else {
                if (mContext != null) {
                    DialogHelper.displayErrorDialog(
                        mContext?.getString(R.string.enter_valid_csc),
                        mContext,
                        SCREEN_NAME
                    )
                }
            }
        }
    }

    override fun getView(): View {
        return mBinding.root
    }

    private fun setSelectedCard(paymentCard: PaymentDetailsResponse.PaymentCards?) {

        paymentCard?.let {
            if (!it.nickName.isNullOrEmpty() || !it.cardId.isNullOrEmpty() || !it.orderCardNickName.isNullOrEmpty()) {
                mSelectedCard = it
                val cardNickname: String?
                if (!it.orderCardNickName.isNullOrEmpty() &&
                    it.orderCardNickName!!.toLowerCase().contains("temp")
                ) {
                    cardNickname = it.orderCardNickName
                    mSelectedCardNickname = it.orderCardNickName
                } else {
                    mSelectedCardNickname = it.cardId
                    if (mSelectedCardNickname == null) mSelectedCardNickname = it.nickName
                    cardNickname = mContext?.getString(
                        R.string.formatted_card_nickname,
                        it.mCreditCardType,
                        it.card
                    )
                }

                val cardNicknameView = mBinding.cardNickname

                val cscField = mBinding.securityCode
                cscField.setText("")
                cardNicknameView.text = cardNickname
            } else {
                openAddCard()
            }
        } ?: kotlin.run {
            openAddCard()
        }
    }

    fun openAddCard() {
        val addFragment =
            SPAddEditPaymentCardFragment.newInstance(
                null,
                true,
                false,
                SPAddEditPaymentCardFragment.PURCHASE_TYPE_DP
            )

        addFragment.setOnCardAddedListener(
            object : SPAddEditPaymentCardFragment.OnCardAddedListener {
                override fun onCardAdded(
                    creditCardResponse: AsdaResponse,
                    isSingleUser: Boolean,
                    cardNumber: String?
                ) {
                    onChangeCard()
                }
            })

        addFragment.setOnSaveStateChange(object : SPAddEditPaymentCardFragment.OnSaveStateChange {
            override fun onChange(codeChange: Int) {
                if (CODE_NO_CHANGE == codeChange) {
                    mIsAddingCard = true
                }
            }
        })
        push(addFragment)
    }

    private fun handleConfirmOrderError(content: DPConfirmOrderState.Error) {
        dismissProgress()
        if (!isAdded || mContext == null) {
            return
        }
        val throwable = content.error
        if (throwable is RxFailure) {
            val response = throwable.asdaResponse
            if (response is ConfirmDeliveryPassResponse) {
                val data = response

                if (data.hasErrorCode(ConfirmDeliveryPassResponse.ERROR_DP_INCOMPLETE) && !mRetrying) {
                    mRetrying = true
                    mViewModel.checkout(
                        mSelectedDPId!!,
                        content.cardNickname!!,
                        content.csc!!,
                        content.paRes,
                        content.autoRenew
                    )
                } else if (data.hasErrorCode(ConfirmDeliveryPassResponse.ERROR_FREETRIAL_NOT_ELIGIBLE)) {
                    showFreeTrialNotEligibleDialog(content)
                } else {
                    AsdaDialogHelper.displayErrorDialog(
                        data,
                        throwable.integer,
                        mContext,
                        SCREEN_NAME
                    )
                }
            } else {
                AsdaDialogHelper.displayErrorDialog(
                    response,
                    throwable.integer,
                    mContext,
                    SCREEN_NAME
                )
            }
        }
    }

    private fun showFreeTrialNotEligibleDialog(content: DPConfirmOrderState.Error) {
        DialogFactory.createAlertDialog(
            getString(R.string.dp_freetrial_ineligible),
            getString(R.string.continue_button),
            { dialog, which ->
                if (isAdded) {
                    showProgress(R.string.placing_order)
                    mViewModel.confirmOrder(
                        viewLifecycleOwner,
                        mSelectedDPId!!,
                        content.cardNickname!!,
                        content.csc!!,
                        content.paRes,
                        content.autoRenew,
                        mPassPlainCvv,
                        content.threedsPayHandle,
                        content.payOrderThreedsTxId
                    )
                }
            },
            getString(R.string.cancel),
            context,
            SCREEN_NAME
        ).show()
    }

    private fun handleConfirmOrderLoaded(content: DPConfirmOrderState.Loaded) {
        dismissProgress()
        val confirmDeliveryPassResponse = content.response
        if (!isAdded || confirmDeliveryPassResponse == null) {
            return
        }

        if (confirmDeliveryPassResponse.threeDSAuthRequired) {
            if (AsdaServiceSettingsConstants.debug) {
                Log.i(TAG, "submitdp() 3DS auth needed")
            }
            mContext?.let {
                mViewModel.trackThreeDsEvent(confirmDeliveryPassResponse, it.getString(R.string.submitdp_page_name))
                performSecureAuthenticationAndCheckoutDP(
                    confirmDeliveryPassResponse,
                    content.cardNickname,
                    content.csc,
                    content.paRes,
                    content.autoRenew
                )
            }
        } else {
            mViewModel.trackTransaction(
                mSelectedDPId,
                dpItemModelPriceParser!!.parse(Pair(mDPModel!!, mSelectedDPId!!)),
                content.autoRenew
            )

            mContext?.let {
                try {
                    val bundle = Bundle()
                    bundle.putBoolean(
                        SuccessResultFragment.AUTO_RENEW,
                        confirmDeliveryPassResponse.dpAutoRenew
                    )
                    val fragment = SuccessResultFragment()
                    fragment.arguments = bundle
                    fragment.show(parentFragmentManager, "refund request dialog")
                } catch (t: Throwable) {
                    Log.e(TAG, "Error showing success dialog", t)
                    it.finish()
                }
                mViewModel.trackDPCheckoutPageViewEvent(confirmDeliveryPassResponse)
            }
        }
    }

    private fun handleCheckoutError(content: DPCheckoutState.Error) {
        dismissProgress()

        val data = content.response

        if (data != null && (
            data.hasErrorCode(CheckoutDeliveryPassResponse.ERROR_USER_HAVE_ACTIVE_DELIVERY_PASS) ||
                data.hasErrorCode(SubscriptionErrorCodes.ERROR_SUBSCRIPTION_ALREADY_EXISTS.errorCode.toString())
            )
        ) {
            val msg = getString(SubscriptionErrorCodes.ERROR_SUBSCRIPTION_ALREADY_EXISTS.customMessageResource)
            val errorStr = "delivery pass : already have active delivery pass"
            val dialog = DialogFactory.createAlertDialog(msg, mContext, SCREEN_NAME)
            dialog.setOnDismissListener(onDismissFinishListener)
            dialog.show()
            Tracker.get().trackPageView(
                PageViewEvent(Anivia.PV_DP_ERROR, Anivia.SECTION_DP, Anivia.SECTION_DP)
                    .putString(Anivia.ERROR_EXPERIENCED_KEY, errorStr)
            )
        } else {
            val error = content.error
            AsdaDialogHelper.displayErrorDialog(
                data,
                error,
                mContext,
                false,
                onDismissFinishListener,
                SCREEN_NAME
            )
            Tracker.get().trackPageView(
                PageViewEvent(Anivia.PV_DP_ERROR, Anivia.SECTION_DP, Anivia.SECTION_DP)
                    .putString(
                        Anivia.ERROR_EXPERIENCED_KEY,
                        if (data != null && !TextUtils.isEmpty(data.firstErrorMessage)) data.firstErrorMessage else "general error"
                    )
            )
        }
    }

    private fun handleCheckoutLoaded(content: DPCheckoutState.Loaded) {
        dismissProgress()
        mViewModel.confirmOrder(
            viewLifecycleOwner, mSelectedDPId!!, content.cardNickname!!, content.csc!!,
            content.paRes, content.autoRenew, mPassPlainCvv, null, null
        )
    }

    fun onChangeCard() {
        val cvvField = mBinding.securityCode
        cvvField.text.clear()
        val isFreeTrial =
            (Authentication.getInstance() as Authentication).isDPFreeTrialEligible && (mDPModel!!.isMonthlyFreeTrial || mDPModel!!.isOneTimeFreeTrial)
        val selectCardFragment = SelectCardFragment.newInstance(
            mSelectedCardNickname,
            SPAddEditPaymentCardFragment.PURCHASE_TYPE_DP,
            mSelectedDPId,
            isFreeTrial
        )
        selectCardFragment.setOnSPCardSelectedListener(object : SelectCardFragment.OnSPCardSelectedListener {
            override fun onCardSelected(creditCard: PaymentDetailsResponse.PaymentCards?, cardNumber: String?, cvv: String?) {
                mLoadData = false
                mSelectedCard = creditCard
                mContext?.let {
                    if (creditCard != null && FeatureSettings.isThreedsEnabledForDp(it))
                        mViewModel.initThreeDSForDP(creditCard, isFreeTrial, mSelectedDPId!!)
                }
            }
        })
        push(selectCardFragment)
    }

    private fun performSecureAuthenticationAndCheckoutDP(
        confirmDeliveryPassResponse: ConfirmDeliveryPassResponse,
        cardNickname: String?,
        csc: String?,
        paRes: String?,
        autoRenew: Boolean
    ) {
        mContext?.let {
            val threeDsDataSource = ThreeDsDataSource()

            val threeDSVerification = ThreeDsVerification(
                confirmDeliveryPassResponse.acsUrl,
                confirmDeliveryPassResponse.stepupUrl,
                confirmDeliveryPassResponse.termUrl,
                confirmDeliveryPassResponse.PAResStatus,
                confirmDeliveryPassResponse.stepupjwt,
                confirmDeliveryPassResponse.PaReq,
                "true",
                confirmDeliveryPassResponse.threeDSTransId,
                confirmDeliveryPassResponse.enrolled,
                confirmDeliveryPassResponse.threedsversion
            )
            threeDsDataSource.doThreeDsSecureCXOCheckout(it, threeDSVerification, "", FeatureSettings.isThreedsEnabledForDp(it))

            threeDsDataSource.getThreeDsLiveData().observe(
                it,
                Observer { threeDsResponse ->
                    if (threeDsResponse.status == it.getString(R.string.threeDsSuccess)) {
                        handleThreeDsSuccess(threeDsResponse, confirmDeliveryPassResponse, cardNickname, csc, paRes, autoRenew)
                    } else if (threeDsResponse.status == it.getString(R.string.threeDsFailed)) {
                        handleThreeDsFailed(threeDsResponse)
                    }
                }
            )
        }
    }

    private fun handleThreeDsFailed(threeDsResponse: ThreeDsDataSource.ThreeDsResult) {
        mContext?.let {
            if (threeDsResponse.errorNumber != null && threeDsResponse.errorDescription != null) {
                it.runOnUiThread {
                    DialogFactory.createAlertDialog(
                        it.getString(R.string.error_card_auth),
                        it,
                        SCREEN_NAME
                    ).show()
                }
            }
            if (!threeDsResponse.errorResponse?.firstErrorMessage.isNullOrEmpty()) {
                AsdaDialogHelper.displayErrorDialog(
                    threeDsResponse.errorResponse,
                    1000,
                    it,
                    SCREEN_NAME
                )
            } else {
                DialogFactory.createAlertDialog(
                    it.getString(R.string.error_card_auth),
                    it,
                    SCREEN_NAME
                ).show()
            }
        }
    }

    private fun handleThreeDsSuccess(
        threeDsResponse: ThreeDsDataSource.ThreeDsResult,
        confirmDeliveryPassResponse: ConfirmDeliveryPassResponse,
        cardNickname: String?,
        csc: String?,
        paRes: String?,
        autoRenew: Boolean
    ) {
        if (!threeDsResponse.jwt.isNullOrEmpty()) {
            mContext?.runOnUiThread {
                mViewModel.confirmOrder(
                    viewLifecycleOwner,
                    mSelectedDPId!!,
                    cardNickname!!,
                    csc!!,
                    threeDsResponse.jwt,
                    autoRenew,
                    mPassPlainCvv,
                    confirmDeliveryPassResponse.threedsPayHandle,
                    confirmDeliveryPassResponse.payOrderThreedsTxId
                )
            }
        }
        if (!threeDsResponse.paRes.isNullOrEmpty()) {
            mViewModel.confirmOrder(
                viewLifecycleOwner,
                mSelectedDPId!!,
                cardNickname!!, csc!!, threeDsResponse.paRes, autoRenew,
                mPassPlainCvv, confirmDeliveryPassResponse.threedsPayHandle, confirmDeliveryPassResponse.payOrderThreedsTxId
            )
        }
    }

    class SuccessResultFragment : DialogFragment() {

        override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
            val builder = AlertDialog.Builder(requireActivity())
            return builder.setMessage(
                if (arguments != null && requireArguments().getBoolean(
                        AUTO_RENEW,
                        false
                    )
                ) R.string.dp_buy_success_auto_renew else R.string.dp_buy_success
            )
                .setCancelable(false)
                .setPositiveButton(R.string.cheers) { dialog, whichButton -> dismissAllowingStateLoss() }
                .create()
        }

        override fun onDismiss(dialog: DialogInterface) {
            super.onDismiss(dialog)
            activity?.finish()
        }

        companion object {
            internal val AUTO_RENEW = "auto_renew"
        }
    }

    companion object {
        private const val SCREEN_NAME = "Delivery Pass Landing Page"
        private const val TAG = "DPCheckoutPresenter"
        private const val DP_SELECTED_ID = "DP_SELECTED_ID"
        private const val DP_SELECTED_CARD = "DP_SELECTED_CARD"
        private const val DP_MODEL = "DP_MODEL"
        private const val EXTRA_ADD_STATE = "DP_ADD_STATE"

        fun newInstance(
            deliveryPassPromoId: String,
            cardNickName: String?,
            dpModel: DeliveryPassItemModel
        ): DeliveryPassCheckoutFragment {
            val bundle = Bundle()
            bundle.putString(DP_SELECTED_ID, deliveryPassPromoId)
            bundle.putString(DP_SELECTED_CARD, cardNickName)
            bundle.putParcelable(DP_MODEL, dpModel)
            bundle.putBoolean(EXTRA_ADD_STATE, false)
            val fragment = DeliveryPassCheckoutFragment()
            fragment.arguments = bundle
            return fragment
        }
    }
}
