package com.asda.android.app.dp.view

import android.app.Activity
import android.content.Context
import android.content.DialogInterface
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.text.Html
import android.text.SpannableStringBuilder
import android.text.Spanned
import android.text.style.ForegroundColorSpan
import android.view.LayoutInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProvider
import androidx.recyclerview.widget.LinearLayoutManager
import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.constants.Anivia.PV_DP_LANDING
import com.asda.android.analytics.constants.Anivia.SECTION_DP
import com.asda.android.analytics.events.AsdaEngineeringAnalyticsEvent
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.app.account.singleprofile.view.SPAddEditPaymentCardFragment
import com.asda.android.app.checkout.SelectCardFragment
import com.asda.android.app.dp.view.adapters.DeliveryPassListAdapter
import com.asda.android.app.settings.FeatureSettingManager
import com.asda.android.base.core.constants.*
import com.asda.android.base.core.utils.orZero
import com.asda.android.base.core.utils.tryCast
import com.asda.android.base.core.view.ViewUtil.openWebPage
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.base.core.view.ui.AsdaDialogHelper
import com.asda.android.base.interfaces.IBaseFragment
import com.asda.android.base.interfaces.TransitionListener
import com.asda.android.restapi.constants.ErrorCodes
import com.asda.android.restapi.service.NetworkCallback
import com.asda.android.restapi.service.base.RxFailure
import com.asda.android.restapi.service.data.AsdaResponse
import com.asda.android.restapi.service.data.DeliveryPassList
import com.asda.android.restapi.service.data.PaymentDetailsResponse
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.service.base.Authentication
import com.asda.android.singleprofile.base.SingleProfile
import com.asda.android.singleprofile.features.account.view.BaseAddEditAddressFragment
import com.asda.android.singleprofile.features.account.view.PersonalDetailsFragment
import com.asda.android.singleprofile.features.account.view.SPAddEditAddressFragment
import com.asda.android.singleprofile.features.login.view.LoginSpActivity
import com.asda.android.slots.AsdaSlotsConfig
import com.asda.android.slots.deliverypass.interfaces.OnDeliveryPassListLoadedListener
import com.asda.android.slots.deliverypass.model.DeliveryPassItemModel
import com.asda.android.slots.deliverypass.model.actionstates.CreditCardExistsActionState
import com.asda.android.slots.deliverypass.model.actionstates.SubscriptionPlansState
import com.asda.android.slots.deliverypass.viewmodel.DPListViewModel
import com.asda.android.slots.subscription.constants.SubscriptionErrorCodes
import com.asda.android.utils.view.ViewUtil
import kotlinx.android.synthetic.main.delivery_pass_list.*

class DeliveryPassListFragment : FeaturedFragment() {

    private lateinit var dpListViewModel: DPListViewModel

    private var mContext: AppCompatActivity? = null

    private var mView: View? = null

    private var mListLoadCallback: NetworkCallback<DeliveryPassList>? = null

    private val headerView: View by lazy {
        LayoutInflater.from(mContext)
            .inflate(R.layout.delivery_pass_list_subheader, recycler_view, false)
    }

    private val mOnDismissFinishListener = DialogInterface.OnDismissListener { mContext?.finish() }

    companion object {
        private const val SCREEN_NAME = "Delivery Pass Landing Page"
    }

    private val deliveryPassListAdapter: DeliveryPassListAdapter by lazy {
        DeliveryPassListAdapter { deliveryPassId, dpItemModel ->
            checkout(deliveryPassId, dpItemModel)
        }
    }

    private val footerView: View by lazy {

        val footer =
            LayoutInflater.from(mContext).inflate(R.layout.dp_list_footer, recycler_view, false)
        ViewUtil.setText(
            R.id.faq,
            footer,
            Html.fromHtml(mContext!!.getString(R.string.dp_footer_faq))
        )
        footer.findViewById<View>(R.id.faq).setOnClickListener {
            openWebPage(
                SharedPreferencesUtil.getDPFAQUrl(mContext),
                mContext,
                SCREEN_NAME
            )
        }

        footer.findViewById<View>(R.id.terms).setOnClickListener {
            openWebPage(
                SharedPreferencesUtil.getDPTermsAndConditionsUrl(mContext),
                mContext,
                SCREEN_NAME
            )
        }

        val sb = SpannableStringBuilder(mContext!!.getString(R.string.needs_help_deciding))
        sb.append(' ')
        val start = sb.length
        sb.append(mContext!!.getString(R.string.learn_more))
        sb.setSpan(
            ForegroundColorSpan(ContextCompat.getColor(mContext!!, R.color.asda_dark_green)),
            start,
            sb.length,
            Spanned.SPAN_EXCLUSIVE_INCLUSIVE
        )
        val learnMore = footer.findViewById<View>(R.id.needs_help) as TextView
        learnMore.text = sb
        learnMore.setOnClickListener {
            activity?.let { DeliveryPassIntroActivity.launch(it) }
        }
        footer
    }

    override fun getScreenName(): String {
        return SCREEN_NAME
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        mContext = context as AppCompatActivity
    }

    override fun onShown() {
        super.onShown()
        setHomeAsUp(false)
    }

    override fun onHidden() {
        super.onHidden()
        setHomeAsUp(true)
    }

    private fun setHomeAsUp(show: Boolean) {
        if (mContext != null) {
            val actionBar = mContext!!.supportActionBar
            actionBar?.setDisplayHomeAsUpEnabled(show)
        }
    }

    override fun hasActionBar(): Boolean {
        return true
    }

    override fun getActionBarTitle(): String? {
        return mContext!!.getString(R.string.delivery_pass_title)
    }

    override fun onStart() {
        super.onStart()
        setHomeAsUp(false)
        dpListViewModel.trackPageViewEvent(PageViewEvent(PV_DP_LANDING, SECTION_DP, SECTION_DP))
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        fragmentContainer: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {

        mView?.let {
            return it
        } ?: run {
            mView = inflater.inflate(R.layout.delivery_pass_list, fragmentContainer, false)
            recycler_view.apply {
                adapter = deliveryPassListAdapter
                layoutManager = LinearLayoutManager(this.context)
            }
            return mView!!
        }
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        setUpViewModels()
        setUpObservables()
    }

    private fun setUpObservables() {
        setUpPlansObservable()
        setUpCreditCardExistsListener()
    }

    private fun setUpPlansObservable() {
        dpListViewModel.subscriptionPlansLiveData.observe(
            viewLifecycleOwner,
            Observer {
                it.getContentIfNotHandled()?.let { subscriptionPlansState ->
                    if (isAdded.not()) return@Observer
                    when (subscriptionPlansState) {

                        is SubscriptionPlansState.Loading -> {
                            // no operation required
                        }
                        is SubscriptionPlansState.Loaded -> {
                            if (subscriptionPlansState.response.errors.isNullOrEmpty().not()) {
                                handleLoadError(subscriptionPlansState.response.errors?.first()?.errorCode?.toIntOrNull().orZero(), subscriptionPlansState.response)
                            } else {
                                handleLoadSuccess(subscriptionPlansState.loadListener, subscriptionPlansState.response)
                            }
                        }
                        is SubscriptionPlansState.Error -> {
                            handleLoadError(subscriptionPlansState.error.errorCode?.toIntOrNull().orZero(), subscriptionPlansState.error)
                        }
                    }
                }
            }
        )
    }

    private fun setUpCreditCardExistsListener() {
        dpListViewModel.getCreditCardExistsLiveData().observe(
            viewLifecycleOwner,
            Observer {

                it.getContentIfNotHandled()?.let { creditCardExistsActionState ->

                    when (creditCardExistsActionState) {
                        is CreditCardExistsActionState.Loading -> {

                            showProgress(R.string.loading)
                        }
                        is CreditCardExistsActionState.Error -> {

                            handleCreditCardExistsError(creditCardExistsActionState)
                        }
                        is CreditCardExistsActionState.Loaded -> {

                            dismissProgress()
                            if (!isAdded) return@Observer
                            handleCreditCardExistsSuccess(creditCardExistsActionState)
                        }
                    }
                }
            }
        )
    }

    private fun handleCreditCardExistsError(creditCardExistsActionState: CreditCardExistsActionState.Error) {
        creditCardExistsActionState.error.tryCast<RxFailure> {
            val response = (creditCardExistsActionState.error as RxFailure).asdaResponse
            AsdaDialogHelper.displayErrorDialog(
                response,
                ErrorCodes.ERROR_CODE_UNKNOWN_ERROR,
                mContext,
                SCREEN_NAME
            )
        }
        dismissProgress()
    }

    private fun handleCreditCardExistsSuccess(creditCardExistsActionState: CreditCardExistsActionState.Loaded) {
        val checkoutDeliveryPassResponse = creditCardExistsActionState.response.checkoutDeliveryPassResponse
        if (!checkoutDeliveryPassResponse?.errors.isNullOrEmpty()) {
            // handle the error and return
            val error = checkoutDeliveryPassResponse?.firstError
            if (SubscriptionErrorCodes.fromCode(error?.errorCode?.toInt().orZero()) == SubscriptionErrorCodes.ERROR_PAYMENT_CARD_MISSING) {
                pushAddEditPaymentCardFragment(
                    creditCardExistsActionState.deliveryPassPromoId,
                    creditCardExistsActionState.dpModel
                )
            } else {
                AsdaDialogHelper.displayErrorDialog(
                    checkoutDeliveryPassResponse,
                    ErrorCodes.ERROR_CODE_UNKNOWN_ERROR,
                    mContext,
                    SCREEN_NAME
                )
            }
            return
        }
        val paymentCards = creditCardExistsActionState.response.paymentCardsList
        val selectedCredit = if (paymentCards.isNotEmpty()) paymentCards[0] else null

        selectedCredit?.let { cards ->
            if (cards.expired) {
                pushSelectCardFragment(
                    cards.nickName,
                    creditCardExistsActionState.deliveryPassPromoId,
                    creditCardExistsActionState.isFreeTrial
                )
            } else if (cards.orderCardNickName == null && cards.nickName == null && cards.cardId == null) {
                pushAddEditPaymentCardFragment(
                    creditCardExistsActionState.deliveryPassPromoId,
                    creditCardExistsActionState.dpModel
                )
            } else {
                pushDeliveryPassCheckoutFragment(
                    creditCardExistsActionState.deliveryPassPromoId,
                    cards,
                    creditCardExistsActionState.dpModel
                )
            }
        }
    }

    private fun setUpViewModels() {
        dpListViewModel = ViewModelProvider(this).get(DPListViewModel::class.java)
    }

    override fun onResume() {
        super.onResume()
        loadList(null)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == android.R.id.home) {
            pop()
            return true
        }
        return false
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == ActivityRequestCodes.REQUEST_CODE_LOGIN && resultCode == Activity.RESULT_OK) {
            val bypassData = data?.getBundleExtra(EXTRA_DATA)
            val deliveryPassId = bypassData?.getString(EXTRA_DELIVERY_PASS_PROMO_ID)
            val dpModel =
                bypassData?.getParcelable<DeliveryPassItemModel>(EXTRA_DELIVERY_PASS_MODEL)
            loadList(object : OnDeliveryPassListLoadedListener {
                override fun onDeliveryPassListLoaded(list: DeliveryPassList) {
                    dpModel?.let {
                        if (!checkStillExists(
                                deliveryPassId,
                                list
                            ) && !checkStillExists(dpModel.oneTimeId, list)
                        ) {
                            checkStillExists(dpModel.monthlyId, list)
                        }
                    }
                }

                private fun checkStillExists(dpId: String?, list: DeliveryPassList): Boolean {
                    val deliveryPassStillExist =
                        !list.deliveryPassPromotions?.filter { it.dpSkuId == dpId }.isNullOrEmpty()
                    if (deliveryPassStillExist)
                        checkout(deliveryPassId, dpModel)
                    return deliveryPassStillExist
                }
            })
        }
    }

    private fun loadList(listener: OnDeliveryPassListLoadedListener?) {
        if (mListLoadCallback == null && context != null) {
            loading_view.visibility = View.VISIBLE

            context?.let {
                if (AsdaSlotsConfig.featureSettingsManager.isSubscriptionAPIEnabled(it)) {
                    dpListViewModel.getSubscriptionPlans(listener)
                } else {
                    mListLoadCallback = object : NetworkCallback<DeliveryPassList>() {

                        override fun onFailure(error: Int?, data: DeliveryPassList) {
                            handleLoadError(error, data)
                        }

                        override fun onSuccess(data: DeliveryPassList) {
                            handleLoadSuccess(listener, data)
                        }
                    }
                    AsdaServiceFactory.get().getDeliveryPassList(
                        FeatureSettingManager.isStoreIdForListDpEnabled(it),
                        mListLoadCallback as NetworkCallback<DeliveryPassList?>
                    )
                }
            }
        }
    }

    private fun handleLoadSuccess(listener: OnDeliveryPassListLoadedListener?, data: DeliveryPassList) {
        mListLoadCallback = null
        if (isAdded) {
            loading_view.visibility = View.GONE

            deliveryPassListAdapter.updateData(data, headerView, footerView)
            empty_view.visibility =
                if (deliveryPassListAdapter.itemCount == 0) View.VISIBLE else View.GONE
            listener?.onDeliveryPassListLoaded(data)
        }
    }

    private fun handleLoadError(errorCode: Int?, data: AsdaResponse) {
        if (isAdded) {
            loading_view.visibility = View.GONE
            empty_view.visibility = View.VISIBLE
            AsdaDialogHelper.displayErrorDialog(
                data,
                errorCode,
                mContext,
                false,
                mOnDismissFinishListener,
                SCREEN_NAME
            )
        }
    }

    override fun getView(): View? {
        return mView
    }

    private fun checkout(deliveryPassPromoId: String?, dpModel: DeliveryPassItemModel?) {
        if (!(Authentication.getInstance() as Authentication).isLoggedIn) {
            val intent = Intent(mContext, LoginSpActivity::class.java)
            intent.putExtra(EXTRA_MESSAGE, getString(R.string.login_screen_dp_message))
            intent.putExtra(EXTRA_ORIGIN, "dp")
            val dataToPass = Bundle()
            dataToPass.putString(EXTRA_DELIVERY_PASS_PROMO_ID, deliveryPassPromoId)
            dataToPass.putParcelable(EXTRA_DELIVERY_PASS_MODEL, dpModel)
            intent.putExtra(EXTRA_DATA, dataToPass)
            startActivityForResult(intent, ActivityRequestCodes.REQUEST_CODE_LOGIN)
            dpListViewModel.trackSignInEvent(
                AsdaEngineeringAnalyticsEvent(Anivia.SIGNIN_EVENT)
                    .putString(Anivia.PAGE_KEY, "DeliveryPassList")
            )
        } else {
            checkPrerequisites(deliveryPassPromoId, dpModel, null)
        }
    }

    private fun pushDeliveryPassCheckoutFragment(
        deliveryPassPromoId: String?,
        paymentCards: PaymentDetailsResponse.PaymentCards,
        dpModel: DeliveryPassItemModel?
    ) {
        deliveryPassPromoId?.let {
            push(
                DeliveryPassCheckoutFragment.newInstance(
                    it,
                    if (paymentCards.cardId != null)
                        paymentCards.cardId else paymentCards.orderCardNickName,
                    dpModel!!
                )
            )
        }
    }

    private fun pushAddEditPaymentCardFragment(
        deliveryPassPromoId: String?,
        dpModel: DeliveryPassItemModel?
    ) {
        val addEditPaymentCardFragment = SPAddEditPaymentCardFragment
            .newInstance(
                null,
                true,
                false,
                SPAddEditPaymentCardFragment.PURCHASE_TYPE_DP
            )

        addEditPaymentCardFragment.setOnCardAddedListener(
            object : SPAddEditPaymentCardFragment.OnCardAddedListener {
                override fun onCardAdded(
                    creditCardResponse: AsdaResponse,
                    isSingleUser: Boolean,
                    cardNumber: String?
                ) {

                    if (creditCardResponse is PaymentDetailsResponse) {
                        creditCardResponse.cards?.let { cards ->
                            if (cards.isNotEmpty() && cards[0] != null) {
                                val card = cards[0]
                                deliveryPassPromoId?.let {
                                    push(
                                        DeliveryPassCheckoutFragment.newInstance(
                                            it,
                                            card.cardId,
                                            dpModel!!
                                        )
                                    )
                                }
                            }
                        }
                    }
                }
            }
        )
        push(addEditPaymentCardFragment)
    }

    private fun pushSelectCardFragment(
        nickName: String?,
        deliveryPassPromoId: String?,
        isFreeTrial: Boolean
    ) {
        val selectCardFragment = SelectCardFragment.newInstance(
            nickName,
            SPAddEditPaymentCardFragment.PURCHASE_TYPE_DP,
            deliveryPassPromoId,
            isFreeTrial
        )
        push(selectCardFragment)
    }

    private fun checkPrerequisites(
        deliveryPassPromoId: String?,
        dpModel: DeliveryPassItemModel?,
        originTag: String?
    ) {

        val listener = object : TransitionListener {
            override fun onPushed(fragment: IBaseFragment, addToBackStack: Boolean): Boolean {
                return false
            }

            override fun onPopped(tag: String?): Boolean {
                Handler().post {
                    val extraInfo = SingleProfile.getProfileResponse()?.addresses
                    extraInfo?.let {
                        if (SPAddEditAddressFragment.TAG == tag && !dpListViewModel.addressIsEmpty(it) || !dpListViewModel.isUserNameEmpty()
                        ) {
                            // If user did not press back without entering required info, go ahead
                            checkPrerequisites(deliveryPassPromoId, dpModel, tag)
                        }
                    }
                }
                return false
            }
        }

        val extraInfo = SingleProfile.getProfileResponse()?.addresses

        if (
            dpListViewModel.addressIsEmpty(extraInfo) && SPAddEditAddressFragment.TAG != originTag
        ) {

            val featuredFragment = BaseAddEditAddressFragment.getMainAddressEditor(false)
            featuredFragment.transitionListener = listener
            push(featuredFragment)
        } else if (dpListViewModel.isUserNameEmpty()) {

            val fragment = PersonalDetailsFragment.newInstance()
            fragment.transitionListener = listener
            push(fragment)
        } else {
            dpListViewModel.checkCreditCardExists(deliveryPassPromoId, dpModel)
        }
    }
}
