package com.asda.android.app.home.view

import android.app.Activity
import android.app.Dialog
import android.content.Context
import android.content.Intent
import android.graphics.Typeface
import android.os.Bundle
import android.text.*
import android.text.method.LinkMovementMethod
import android.text.style.ImageSpan
import android.text.style.StyleSpan
import android.text.style.URLSpan
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.widget.TextView
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.content.res.AppCompatResources
import androidx.appcompat.widget.AppCompatButton
import androidx.appcompat.widget.AppCompatImageView
import androidx.appcompat.widget.AppCompatTextView
import androidx.core.text.HtmlCompat
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentActivity
import androidx.fragment.app.FragmentManager
import androidx.lifecycle.Observer
import com.asda.android.R
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.events.LinkButtonClickEvent
import com.asda.android.app.bookslot.LearnAboutCNCActivity
import com.asda.android.app.bookslotv3.view.activity.BookSlotActivity
import com.asda.android.app.bookslotv3.view.fragment.BookSlotParentFragment
import com.asda.android.app.dp.view.DPOfferCardViewHandler
import com.asda.android.app.main.view.AsdaActivity
import com.asda.android.app.settings.featureflag.FeatureSettings.isExpressHDEnabled
import com.asda.android.app.shop.FREQUENTLY_BOUGHT
import com.asda.android.app.shop.view.MerchandisingHandler
import com.asda.android.app.view.AsdaFragmentSwitcher
import com.asda.android.base.core.constants.ActivityRequestCodes
import com.asda.android.base.core.constants.BottomNavConstants.TAG_FAVORITES
import com.asda.android.base.core.constants.BottomNavConstants.TAG_ORDERS
import com.asda.android.base.core.constants.BottomNavConstants.TAG_SHOP
import com.asda.android.base.core.constants.BottomNavConstants.TAG_SHOP_OLD
import com.asda.android.base.core.constants.DeepLinkConstants.TYPE_MESSAGES
import com.asda.android.base.core.constants.DeepLinkConstants.TYPE_RECIPES
import com.asda.android.base.core.constants.DeepLinkConstants.TYPE_SHOPPING_LISTS
import com.asda.android.base.core.constants.DeepLinkConstants.TYPE_TOP_OFFERS
import com.asda.android.base.core.constants.FragmentConfigConstants
import com.asda.android.base.core.constants.FragmentConfigConstants.TAG_MSG_CENTER
import com.asda.android.base.core.constants.HOME_PAGE
import com.asda.android.base.core.utils.AsdaBaseCoreConfig
import com.asda.android.base.core.utils.tryCast
import com.asda.android.base.core.view.DialogFactory
import com.asda.android.base.core.view.ViewUtil.openWebPage
import com.asda.android.base.core.view.fragments.BaseFragment
import com.asda.android.base.core.view.ui.AsdaDialogHelper
import com.asda.android.base.core.view.ui.gone
import com.asda.android.base.core.view.ui.setSafeOnClickListener
import com.asda.android.base.core.view.ui.visible
import com.asda.android.base.interfaces.IHomeManager
import com.asda.android.home.AsdaHomeConfig
import com.asda.android.home.constants.ServiceEligibilityType
import com.asda.android.home.model.PostCodeDialogModel
import com.asda.android.home.view.HomeFragmentNew
import com.asda.android.home.view.HomeFragmentNew.Companion.KEY_POST_CODE_DIALOG_MODEL
import com.asda.android.home.view.RDPostCodeView
import com.asda.android.restapi.utils.SharedPreferencesUtil
import com.asda.android.service.base.Authentication
import com.asda.android.service.constants.CartManagerConstants.SLOT_TYPE_CC
import com.asda.android.service.constants.CartManagerConstants.SLOT_TYPE_DELIVERY
import com.asda.android.slots.deliverypass.model.ShowDPCardValidatorModel
import com.asda.android.slots.deliverypass.validator.ShowDPOfferValidator
import com.asda.android.slots.utils.ExpressUtils.isExpressSlotBooked
import com.asda.android.slots.utils.SlotUtils
import com.asda.android.slots.utils.view.SlotsViewUtils
import com.asda.android.utils.view.ViewUtils
import com.google.android.material.textfield.TextInputEditText
import com.google.android.material.textfield.TextInputLayout
import java.lang.ref.WeakReference

object HomeManagerImpl : IHomeManager {

    const val STORE_LOCATION_TAG = "StoreLocationViewModel"

    override fun onClickStartShopping(asdaActivity: Activity) {
        if (AsdaBaseCoreConfig.featureSettingManager.isNewTaxonomyEnabled(asdaActivity)) {
            switchSection(asdaActivity, TAG_SHOP, null)
        } else {
            switchSection(asdaActivity, TAG_SHOP_OLD, null)
        }
    }

    override fun onClickViewOrders(asdaActivity: Activity) {
        switchSection(asdaActivity, TAG_ORDERS, null)
    }

    override fun launchShoppingList(sourceFragment: Fragment, context: Context) {
        sourceFragment.tryCast<BaseFragment> {
            MerchandisingHandler.handleCmsMerchandisingEvent(context as Activity, this.getCurrentFragmentManager(), this, TYPE_SHOPPING_LISTS, null, null, false, null)
            Tracker.get().trackEvent(AsdaAnalyticsEvent(Anivia.SECTION_TAP).putString(Anivia.LOCATION_KEY, HOME_PAGE).putString(Anivia.SECTION_NAME_KEY, FragmentConfigConstants.TAG_LIST))
        }
    }

    override fun launchRecipe(sourceFragment: Fragment, context: Context) {
        context.tryCast<Activity> {
            MerchandisingHandler.handleCmsMerchandisingEvent(this, null, sourceFragment, TYPE_RECIPES, null, null, false, null)
        }
    }

    override fun launchMessages(sourceFragment: Fragment, context: Context) {
        sourceFragment.tryCast<BaseFragment> {
            MerchandisingHandler.handleCmsMerchandisingEvent(context as Activity, this.getCurrentFragmentManager(), this, TYPE_MESSAGES, null, null, false, null)
            Tracker.get().trackEvent(AsdaAnalyticsEvent(Anivia.SECTION_TAP).putString(Anivia.LOCATION_KEY, HOME_PAGE).putString(Anivia.SECTION_NAME_KEY, TAG_MSG_CENTER))
        }
    }

    override fun launchStoreFinder(sourceFragment: Fragment, context: Context) {

        var mOpenPetrolPopup: Dialog? = null
        val mOpenPetrolPopupListener: Map<Int, View.OnClickListener>? = mutableMapOf(
            R.id.btn_ok_webpage to View.OnClickListener {
                mOpenPetrolPopup?.dismiss()
                openWebPage(SharedPreferencesUtil.getStoreLocatorUrl(context), context, STORE_LOCATION_TAG)
            },
            R.id.btn_cancel_webpage to View.OnClickListener {
                mOpenPetrolPopup?.dismiss()
            }
        )
        mOpenPetrolPopup = AsdaDialogHelper.displayCustomDialog(R.layout.alert_petrol_store_locator, mOpenPetrolPopupListener, null, false, context)
    }

    override fun handlePastelCmsLinkDestination(sourceFragment: Fragment, context: Context) {
        sourceFragment.tryCast<BaseFragment> {
            MerchandisingHandler.handleCmsMerchandisingEvent(context as Activity, this.getCurrentFragmentManager(), this, TYPE_TOP_OFFERS, null, null, false, null)
        }
    }

    override fun launchBookSlotCC(context: Context, fromView: View?) {
        launchBookSlot(context, fromView, SLOT_TYPE_CC)
    }

    override fun launchBookSlotDelivery(context: Context, fromView: View?) {
        launchBookSlot(context, fromView, SLOT_TYPE_DELIVERY)
    }

    override fun launchMessagesScreen(context: Context) {
        switchSection(context, TAG_MSG_CENTER)
    }

    override fun launchFavorites(context: Context) {
        val args = Bundle()
        args.putBoolean(FREQUENTLY_BOUGHT, true)
        switchSection(context, TAG_FAVORITES, args)
    }

    private fun launchBookSlot(context: Context, fromView: View?, slotType: Int) {
        if (context !is AsdaActivity) {
            Log.d(javaClass.name, "context is not of type AsdaActivity, returning without launching book slot with tab $slotType")
            return
        }
        val intent = Intent(context, BookSlotActivity::class.java)
        intent.putExtra(BookSlotActivity.EXTRA_TARGET_TAB, slotType)
        if (slotType == SLOT_TYPE_CC)
            intent.putExtra(BookSlotParentFragment.SWITCH_TO_CNC_TAB, true)
        ViewUtils.startActivityForResult(context, intent, ActivityRequestCodes.REQUEST_CODE_BOOK_SLOT, fromView)

        val linkButtonTextEvent = if (slotType == SLOT_TYPE_CC) Anivia.BOOK_A_COLLECTION else Anivia.BOOK_A_DELIVERY
        triggerHomePageAnalyticsEvent(linkButtonTextEvent)
    }

    private fun switchSection(context: Context, tag: String, bundle: Bundle? = null) {

        if (context !is AsdaActivity) {
            Log.d(javaClass.name, "context is not of type AsdaActivity, returning without launching $tag")
            return
        }

        context.getFragmentSwitcher()?.let {
            it.switchToFragment(tag, bundle)
            triggerHomePageAnalyticsEvent(Anivia.VIEW_MY_ORDERS)
        } ?: run {
            Log.d(javaClass.name, "fragmentSwitcher is null, returning without launching $tag")
        }
    }
    private fun triggerHomePageAnalyticsEvent(linkButtonTextEvent: String) {
        val event = LinkButtonClickEvent(
            linkButtonText = linkButtonTextEvent,
            screenName = Anivia.SECTION_HOMEPAGE
        )
        AsdaHomeConfig.tracker.trackEvent(event)
    }
    override fun favButtonClick(context: Context) {
        context.tryCast<AsdaActivity> {
            getFragmentSwitcher()?.let {
                switchSection(it, TAG_FAVORITES, null)
            }
        }
    }

    override fun reorderButtonClick(context: Context) {
        context.tryCast<AsdaActivity> {
            getFragmentSwitcher()?.let {
                switchSection(it, TAG_ORDERS, null)
            }
        }
    }

    override fun onCNCBannerClick(view: View) {
        view.context.tryCast<Activity> {
            ViewUtils.startActivityForResult(this, Intent(this, LearnAboutCNCActivity::class.java), ActivityRequestCodes.REQUEST_CODE_LEARN_ABOUT_CNC, view)
        }
    }

    private fun switchSection(fragmentSwitcher: AsdaFragmentSwitcher, tag: String, bundle: Bundle?) {
        fragmentSwitcher.switchToFragment(tag, bundle)
        Tracker.get().trackEvent(AsdaAnalyticsEvent(Anivia.SECTION_TAP).putString(Anivia.LOCATION_KEY, HOME_PAGE).putString(Anivia.SECTION_NAME_KEY, tag))
    }

    override fun getExpressDeliveryInfo(context: Context): CharSequence? {
        val cxoCart = com.asda.android.cxo.CartManager.instance.cartResponse?.data?.cart
        val showExpressInfo = (
            cxoCart?.cartItemSummary?.lineItemCount ?: 0 <= 0 && cxoCart?.cartItemSummary?.totalQuantity ?: 0 <= 0 && SlotUtils.isExpressEnabled(context) &&
                SlotUtils.expressFlagCNC
            )
        if (showExpressInfo) {
            val spannableStringBuilder = SpannableStringBuilder()
                .append(context.getString(R.string.add_items_to_cart))
                .append(context.getString(R.string.double_spaces))
            spannableStringBuilder.setSpan(ImageSpan(context, R.drawable.ic_express), spannableStringBuilder.length - 1, spannableStringBuilder.length, 0)
            spannableStringBuilder.append(context.getString(R.string.single_space)).append(context.getString(R.string.slots_available))
            return spannableStringBuilder
        }

        return null
    }

    override fun setUpExpressBanner(sourceFragment: Fragment?, tvBanner: WeakReference<TextView>) {
        sourceFragment?.context?.let { context ->
            // normal express section
            tvBanner.get()?.let {
                setUpExpressSection(sourceFragment, it)
            }
            // handling the cxo
            com.asda.android.cxo.CartManager.instance.let { cxoCartInstance ->
                cxoCartInstance.setExpressSection.observe(
                    sourceFragment,
                    Observer { isExpressEnabled ->
                        if (isExpressEnabled) {
                            cxoCartInstance.cartResponse?.data?.cart?.let { cxoCart ->
                                cxoCart.fulfillmentDetails?.let { cxoFulfillMentDetails ->
                                    if (isExpressSlotBooked(cxoCart)) {
                                        tvBanner.get()?.apply {
                                            text = SlotsViewUtils.getTextForReserveUntilExpressBanner(cxoFulfillMentDetails.slotInfo?.bookedUntilTime, context, R.drawable.ic_express)
                                            visible()
                                        }
                                    } else {
                                        tvBanner.get()?.let {
                                            handleFallBackExpressBanner(sourceFragment, context, it)
                                        }
                                    }
                                }
                            } ?: kotlin.run {
                                tvBanner.get()?.let {
                                    handleFallBackExpressBanner(sourceFragment, context, it)
                                }
                            }
                        } else {
                            tvBanner.get()?.let {
                                handleFallBackExpressBanner(sourceFragment, context, it)
                            }
                        }
                    }
                )
            }
        }
    }

    private fun setUpExpressSection(sourceFragment: Fragment, tvBanner: TextView) {
        sourceFragment.context?.let { context ->
            com.asda.android.cxo.CartManager.instance.getCxoCart()?.let { cart ->
                if (SlotUtils.isExpressEnabled(sourceFragment.context) && isExpressSlotBooked(cart)) {
                    tvBanner.apply {
                        text = SlotsViewUtils.getTextForReserveUntilExpressBanner(cart.fulfillmentDetails?.slotInfo?.bookedUntilTime, context, R.drawable.ic_express)
                        visible()
                    }
                } else {
                    handleFallBackExpressBanner(sourceFragment, context, tvBanner)
                }
            } ?: kotlin.run {
                handleFallBackExpressBanner(sourceFragment, context, tvBanner)
            }
        }
    }

    private fun handleFallBackExpressBanner(fragment: Fragment, context: Context, tvBanner: TextView) {
        if (SlotUtils.isExpressEnabled(context) && SlotUtils.expressFlag) {
            val spannableStringBuilder = SpannableStringBuilder().apply {
                append(context.getString(R.string.express_select)).append(" ")
                setSpan(ImageSpan(context, R.drawable.ic_express), length - 1, length, 0)
                append(ViewUtils.fromHtml(context.getString(R.string.express_checkout_desc)))
            }
            tvBanner.apply {
                text = spannableStringBuilder
                visible()
                setOnClickListener {
                    fragment.tryCast<BaseFragment> {
                        showDialog(HomeFragmentNew.DIALOG_ABOUT_EXPRESS)
                    }
                }
            }
        } else {
            tvBanner.gone()
        }
    }

    override fun handleDialog(code: Int, sourceFragment: Fragment, context: Context, fallback: () -> Dialog?): Dialog? = when (code) {
        HomeFragmentNew.DIALOG_CHECK_POST_CODE -> {
            getCheckPostCodeDialog(sourceFragment, context)
        }
        HomeFragmentNew.DIALOG_ABOUT_EXPRESS -> {
            DialogFactory.createCustomDialog(
                R.layout.alert_dialog_express, R.id.btn_view_stores,
                {
                    val intent = Intent(context, BookSlotActivity::class.java)
                    intent.putExtra(BookSlotActivity.EXTRA_IS_EXPRESS_BANNER_CLICKED, true)
                    sourceFragment.startActivityForResult(intent, ActivityRequestCodes.REQUEST_CODE_BOOK_SLOT)
                    sourceFragment.tryCast<BaseFragment> {
                        dismissDialog(HomeFragmentNew.DIALOG_ABOUT_EXPRESS)
                    }
                },
                R.id.btn_ok,
                {
                    sourceFragment.tryCast<BaseFragment> {
                        dismissDialog(HomeFragmentNew.DIALOG_ABOUT_EXPRESS)
                    }
                },
                context
            ) { view: View? ->
                if (view != null) {
                    val expressDesc = view.findViewById<TextView>(R.id.express_about)
                    if (expressDesc != null) {
                        if (SlotUtils.expressFlag && SlotUtils.expressFlagCNC && SlotUtils.expressFlagHD && isExpressHDEnabled(context)) {
                            expressDesc.text = context.getString(R.string.about_express_conditional_desc, context.getString(R.string.about_express_condition_hd_cnc))
                        } else if (SlotUtils.expressFlag && SlotUtils.expressFlagCNC && !SlotUtils.expressFlagHD) {
                            expressDesc.text = context.getString(R.string.about_express_conditional_desc, context.getString(R.string.about_express_condition_cnc))
                        } else if (SlotUtils.expressFlag && isExpressHDEnabled(context) && !SlotUtils.expressFlagCNC && SlotUtils.expressFlagHD) {
                            expressDesc.text = context.getString(R.string.about_express_conditional_desc, context.getString(R.string.about_express_condition_hd))
                        } else {
                            expressDesc.text = context.getString(R.string.about_express_conditional_desc, context.getString(R.string.about_express_condition_hd_cnc))
                        }
                    }
                }
            }
        }
        else -> {
            fallback()
        }
    }

    override fun handleDpCardHomePage(containerDpActive: View, tvOfferText: TextView, tvExpiresTime: TextView, offerCard: View, containerDpNoOffer: View): Boolean {
        Authentication.getInstance().tryCast<Authentication> {
            val shouldShowDPOfferCard: Boolean = ShowDPOfferValidator().validate(
                ShowDPCardValidatorModel(this, (Authentication.getInstance() as Authentication).getActiveDeliveryPass())
            )
            if (shouldShowDPOfferCard) {
                val dpOfferCardViewHandler = DPOfferCardViewHandler().apply {
                    load()
                    containerDpNoOffer.context?.tryCast<Activity> {
                        attach(this)
                    }
                }
                containerDpNoOffer.apply {
                    if (dpOfferCardViewHandler.mIsVisible.get()) {
                        visible()
                        tvOfferText.text = dpOfferCardViewHandler.mDPOfferText.get()
                        tvExpiresTime.text = dpOfferCardViewHandler.mDPOfferExpire.get()
                        offerCard.setSafeOnClickListener {
                            dpOfferCardViewHandler.onClickBuyDP(it)
                        }
                        return true
                    }
                }
            } else if (getActiveDeliveryPass() != null) {
                containerDpActive.apply {
                    visible()
                    setSafeOnClickListener {
                        Intent(context, BookSlotActivity::class.java).apply intent@{
                            putExtra(BookSlotActivity.EXTRA_TARGET_TAB, SLOT_TYPE_DELIVERY)
                            context.tryCast<AsdaActivity> {
                                ViewUtils.startActivityForResult(this, this@intent, ActivityRequestCodes.REQUEST_CODE_BOOK_SLOT, it)
                            }
                        }
                    }
                }
                return true
            }
        }
        return false
    }

    override fun handleCMSMerchHandlerEvent(activity: Activity, fragmentManager: FragmentManager?, sourceFragment: Fragment?, type: String?, id: String?, bannerCampaignId: String?, isTempoUrl: Boolean, url: String?) {
        sourceFragment.tryCast<BaseFragment> {
            MerchandisingHandler.handleCmsMerchandisingEvent(context as Activity, this.getCurrentFragmentManager(), this, type, id, bannerCampaignId, isTempoUrl, url)
        }
    }

    private fun getCheckPostCodeDialog(sourceFragment: Fragment, context: Context): Dialog? {
        val dialogModel: PostCodeDialogModel? = sourceFragment.arguments?.getParcelable(KEY_POST_CODE_DIALOG_MODEL)
        if (sourceFragment.activity == null || dialogModel == null) {
            return null
        }
        return if (dialogModel.serviceEligibilityType != ServiceEligibilityType.NONE) {
            getCheckPostCodeInfoDialog(sourceFragment, context, dialogModel)
        } else {
            getCheckPostCodeErrorDialog(sourceFragment, context, dialogModel)
        }
    }

    private fun getCheckPostCodeInfoDialog(sourceFragment: Fragment, context: Context, dialogModel: PostCodeDialogModel): Dialog? {
        val dialogView = LayoutInflater.from(context).inflate(R.layout.rd_intro_check_postcode_info_dialog, null)
        dialogView.findViewById<AppCompatTextView>(R.id.postCodeInfoTitleTV).text = dialogModel.title
        dialogView.findViewById<AppCompatTextView>(R.id.postCodeInfoMessageTV).text = dialogModel.message
        dialogView.findViewById<AppCompatButton>(R.id.postCodeInfoCreateAccountBtn).setOnClickListener {
            dismissCheckPostCodeDialog(sourceFragment)
            launchSignIn(sourceFragment.requireActivity())
        }
        dialogView.findViewById<AppCompatTextView>(R.id.postCodeInfoSignInTV).setOnClickListener {
            dismissCheckPostCodeDialog(sourceFragment)
            launchSignIn(sourceFragment.requireActivity())
        }
        dialogView.findViewById<AppCompatImageView>(R.id.postCodeInfoCloseIV).setOnClickListener {
            dismissCheckPostCodeDialog(sourceFragment)
        }
        return AlertDialog.Builder(context).setCancelable(true).setView(dialogView).create()
    }

    private fun getCheckPostCodeErrorDialog(sourceFragment: Fragment, context: Context, dialogModel: PostCodeDialogModel): Dialog? {
        val dialogView = LayoutInflater.from(context).inflate(R.layout.rd_intro_check_postcode_error_dialog, null)
        dialogView.findViewById<AppCompatTextView>(R.id.postCodeErrorMessageTV).text = dialogModel.message
        val checkPostCodeError = dialogView.findViewById<TextInputLayout>(R.id.tlCheckPostCodeError)
        val enterPostCodeET = dialogView.findViewById<TextInputEditText>(R.id.enterPostCodeET)
        enterPostCodeET.addTextChangedListener(object : TextWatcher {
            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
                // no op
            }
            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
                checkPostCodeError.errorIconDrawable = null
                checkPostCodeError.error = null
            }
            override fun afterTextChanged(s: Editable?) {
                // no op
            }
        })
        dialogView.findViewById<AppCompatImageView>(R.id.postCodeErrorCloseIV).setOnClickListener {
            dismissCheckPostCodeDialog(sourceFragment)
        }
        dialogView.findViewById<AppCompatButton>(R.id.checkPostCodeBtn).setOnClickListener {
            if (AsdaHomeConfig.bookSlotHelper.isValidPostCode(enterPostCodeET.text.toString())) {
                checkPostCodeError.errorIconDrawable = null
                checkPostCodeError.error = null
                sourceFragment.tryCast<HomeFragmentNew> {
                    this.getView(RDPostCodeView.TAG).tryCast<RDPostCodeView> {
                        this.onPostCodeClick(enterPostCodeET.text.toString(), true)
                        dismissCheckPostCodeDialog(sourceFragment)
                    }
                }
            } else {
                val drawable = AppCompatResources.getDrawable(context, com.asda.android.home.R.drawable.ic_system_alert)
                drawable?.let {
                    it.setBounds(0, 0, it.intrinsicWidth, it.intrinsicHeight)
                    enterPostCodeET.error = null
                    checkPostCodeError.errorIconDrawable = drawable
                }
                checkPostCodeError.error = context.getString(com.asda.android.home.R.string.rd_check_postcode_error)
            }
        }
        val contactUsTV = dialogView.findViewById<AppCompatTextView>(R.id.postCodeErrorAdditionalInfo3TV)
        contactUsTV.text = HtmlCompat.fromHtml(context.getString(R.string.contact_us_message), HtmlCompat.FROM_HTML_MODE_LEGACY)
        val ss = SpannableString(contactUsTV.text)
        val matchStartIndex = contactUsTV.text.indexOf(context.resources.getString(R.string.contact_us))
        val matchEndIndex = matchStartIndex + context.resources.getString(R.string.contact_us).length
        ss.setSpan(StyleSpan(Typeface.BOLD), matchStartIndex, matchEndIndex, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
        for (u in ss.getSpans(0, ss.length, URLSpan::class.java)) {
            ss.setSpan(
                object : URLSpan(u.url) {
                    override fun updateDrawState(ds: TextPaint) {
                        super.updateDrawState(ds)
                        ds.isUnderlineText = false
                    }
                },
                ss.getSpanStart(u), ss.getSpanEnd(u), 0
            )
        }
        contactUsTV.text = ss
        contactUsTV.movementMethod = LinkMovementMethod.getInstance()

        return AlertDialog.Builder(context).setCancelable(true).setView(dialogView).create()
    }

    private fun dismissCheckPostCodeDialog(sourceFragment: Fragment) {
        val bundle = Bundle()
        bundle.putParcelable(KEY_POST_CODE_DIALOG_MODEL, null)
        sourceFragment.arguments = bundle
        sourceFragment.tryCast<BaseFragment> {
            dismissDialog(HomeFragmentNew.DIALOG_CHECK_POST_CODE, true)
        }
    }

    private fun launchSignIn(activity: FragmentActivity) {
        AsdaHomeConfig.authenticationManager.launchLoginActivityForResult(
            activity,
            "",
            ActivityRequestCodes.REQUEST_CODE_LOGIN_SP
        )
    }
}
