package com.asda.android.app.home.view

import android.content.Context
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.databinding.DataBindingUtil
import androidx.databinding.ViewDataBinding
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.PagerSnapHelper
import androidx.recyclerview.widget.RecyclerView
import com.asda.android.R
import com.asda.android.app.model.WismoCardModel
import com.asda.android.app.settings.featureflag.FeatureSettings
import com.asda.android.base.core.utils.tryCast
import com.asda.android.base.interfaces.IWismoManager
import com.asda.android.databinding.HomeCardWismoNewBinding
import com.asda.android.home.constants.HomeConstant
import com.asda.android.orders.wismo.view.WismoAdapter
import com.asda.android.orders.wismo.view.indicator.PagerIndicatorDecoration
import com.asda.android.restapi.service.data.OdsOrderModel
import com.asda.android.restapi.service.data.ViewOrderResponse
import com.asda.android.restapi.utils.SharedPreferencesUtil

class WismoManager : IWismoManager {
    override fun setUpWismo(order: OdsOrderModel?, oldOrder: ViewOrderResponse.Order?, container: ViewGroup) {
        if (FeatureSettings.isNewOrdersModuleEnabled(container.context)) {
            val adapter = WismoAdapter(container.context, order?.payload)
            val decorator = PagerIndicatorDecoration()
            val pagerSnapHelper = PagerSnapHelper()
            container.tryCast<RecyclerView> {
                this.layoutManager = LinearLayoutManager(container.context, LinearLayoutManager.HORIZONTAL, false)
                this.adapter = adapter
                pagerSnapHelper.attachToRecyclerView(this)
                this.addItemDecoration(decorator)
            }
        } else {
            val storeIdsList = SharedPreferencesUtil.getEligibleStores(container.context).split(",").map { it.toLowerCase().trim() }
            if (storeIdsList.contains(oldOrder?.storeID) || storeIdsList.contains(HomeConstant.ELIGIBLE_STORE_FOR_ALL.toLowerCase())) {
                DataBindingUtil.inflate<ViewDataBinding>(LayoutInflater.from(container.context), R.layout.home_card_wismo_new, container, true).apply {
                    tryCast<HomeCardWismoNewBinding> {
                        viewModel = WismoCardModel(container.context, oldOrder)
                    }
                }
            }
            val list: List<String> = ArrayList()
        }
    }

    override fun isNewOrderModuleFlag(context: Context): Boolean {
        return FeatureSettings.isNewOrdersModuleEnabled(context)
    }
}
