package com.asda.android.app.messagecenter

import android.content.Context
import android.util.Log
import com.asda.android.R
import com.asda.android.base.core.constants.DateFormats.MMM_DD
import com.asda.android.base.core.constants.DateFormats.YYYY_MM_dd
import com.asda.android.base.core.constants.TimeZoneFormat
import com.asda.android.base.core.convertToFormat
import com.asda.android.base.core.isToday
import com.asda.android.base.core.isYesterday
import java.text.ParseException
import java.text.SimpleDateFormat
import java.util.Date
import java.util.Locale
import java.util.TimeZone

fun formatOSMMessageDate(context: Context, date: Long): String? {
    val dateTime = Date(date)
    val formattedDate = SimpleDateFormat(YYYY_MM_dd, Locale.UK).format(dateTime)
    val isToday = formattedDate.isToday(YYYY_MM_dd, true)
    val isYesterday = formattedDate.isYesterday(YYYY_MM_dd, true)
    return when {
        isToday -> context.getString(R.string.txt_today)
        isYesterday -> context.getString(R.string.txt_yesterday)
        else -> {
            formattedDate.convertToFormat(YYYY_MM_dd, MMM_DD)
        }
    }
}

/**
 * Function converts date string to timestamp and returns time
 *
 * @param dateString input date string
 * @return time in long
 */
fun convertToTimeStamp(dateString: String, inputFormat: String): Long? {
    val sdf = SimpleDateFormat(
        inputFormat,
        Locale.UK
    )
    sdf.timeZone = TimeZone.getTimeZone(TimeZoneFormat.UK)
    var date: Date? = null
    try {
        date = sdf.parse(dateString)
    } catch (e: ParseException) {
        Log.w("MessageCenterUtils", e)
    }
    return date?.time
}
