package com.asda.android.app.messagecenter.view

import androidx.lifecycle.LiveData
import androidx.lifecycle.MediatorLiveData
import com.asda.android.app.main.AsdaApplication
import com.asda.android.app.messagecenter.MessageCenterController
import com.asda.android.app.messagecenter.MessageCenterController.OnInboxUpdateListener
import com.asda.android.base.core.utils.orZero
import com.asda.android.base.interfaces.IMessageCenterController
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.launch

object MessageCenterControllerImpl : IMessageCenterController {

    private val msgCountUpdateListener = MediatorLiveData<Int>()

    private val onInboxUpdateListener = object : OnInboxUpdateListener {
        override fun onInboxUpdate() {
            updateMessageCount()
        }
    }

    override fun getMessageCountUpdateListener(): LiveData<Int> {
        if (msgCountUpdateListener.value == null) {
            MessageCenterController.getInstance(AsdaApplication.getInstance())?.apply {
                removeOnInboxUpdateListener(onInboxUpdateListener)
                addOnInboxUpdateListener(onInboxUpdateListener)
            }
        }
        updateMessageCount()
        return msgCountUpdateListener
    }

    private fun updateMessageCount() {
        // Since one of the UAirship function calls it from sdk which doesnt have any scope to launch the coroutine, So launching it as a global scope
        // More over it observed as live data which has lifecycle owner scope.
        GlobalScope.launch {
            val count = MessageCenterController.getInstance(AsdaApplication.getInstance())?.getUnreadMessageCount()
            msgCountUpdateListener.postValue(count?.orZero())
        }
    }
}
