/*
Copyright 2009-2014 Urban Airship Inc. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE URBAN AIRSHIP INC ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL URBAN AIRSHIP INC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.asda.android.app.messagecenter.view

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.view.MenuItem
import androidx.appcompat.app.AppCompatActivity
import androidx.fragment.app.Fragment
import androidx.lifecycle.lifecycleScope
import com.asda.android.R
import com.asda.android.app.messagecenter.MessageCenterController
import com.asda.android.base.core.view.fragments.BaseFragment.Companion.getASDAFragmentTransaction
import com.asda.android.utils.Utils
import kotlinx.coroutines.launch

class MessageDetailsActivity : AppCompatActivity() {
    private var mMessageId: String? = null
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_message_details)
        val ab = supportActionBar
        if (ab != null) {
            ab.setDisplayHomeAsUpEnabled(true)
            ab.setHomeButtonEnabled(true)
        }
        setMessageId(savedInstanceState)
        val controller = MessageCenterController.getInstance(this) ?: return

        mMessageId?.let { messageId ->
            lifecycleScope.launch {
                val message = controller.findMessageById(messageId)
                if (isFinishing) {
                    return@launch
                }
                var fragment: Fragment? = null
                message?.let {
                    val ab1 = supportActionBar
                    ab1?.title = it.title
                    fragment = it.createMessageDetailFragment()
                    if (it.readStatus.not()) {
                        controller.markRead(messageId)
                    }
                }
                fragment?.let {
                    getASDAFragmentTransaction(supportFragmentManager)
                        .add(R.id.fragment_container, it, "msg_fragment")
                        .commitAllowingStateLoss()
                } ?: run {
                    finish()
                }
            }
        }
    }

    private fun setMessageId(savedInstanceState: Bundle?) {
        mMessageId = if (savedInstanceState == null) {
            intent.getStringExtra(EXTRA_MESSAGE_ID_KEY)
        } else {
            savedInstanceState.getString(EXTRA_MESSAGE_ID_KEY)
        }
    }

    override fun onSaveInstanceState(outState: Bundle) {
        super.onSaveInstanceState(outState)
        Utils.informAboutTooLargePayload(
            outState,
            TAG
        )
        outState.putString(EXTRA_MESSAGE_ID_KEY, mMessageId)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == android.R.id.home) {
            finish()
            return true
        }
        return false
    }

    companion object {
        private const val TAG = "MessageDetailsActivity"
        private const val EXTRA_MESSAGE_ID_KEY = "message_id"
        fun startActivity(parentActivity: Activity, messageId: String?) {
            val messageDetailsIntent =
                Intent(parentActivity, MessageDetailsActivity::class.java)
            messageDetailsIntent.putExtra(
                EXTRA_MESSAGE_ID_KEY,
                messageId
            )
            parentActivity.startActivity(messageDetailsIntent)
        }
    }
}
