package com.asda.android.app.mobileAppPreview.view

import android.os.Bundle
import androidx.fragment.app.Fragment
import com.asda.android.R
import com.asda.android.app.shop.taxonomy.view.TaxonomyManager
import com.asda.android.app.shop.view.CXOHelper
import com.asda.android.base.core.constants.*
import com.asda.android.base.core.parser.MobileAppPreviewParser
import com.asda.android.base.core.processor.LinkTypeProcessor
import com.asda.android.base.core.view.actitivies.BaseAppCompatActivity
import com.asda.android.base.core.view.fragments.BaseFragment
import com.asda.android.cxo.view.fragments.BeforeYouGoFragment
import com.asda.android.cxo.view.fragments.CXOCartPagerFragment
import com.asda.android.cxo.view.fragments.PAGE_BEFORE_YOU_GO
import com.asda.android.favourites.features.favorites.view.FavoritesFragmentNew
import com.asda.android.home.view.HomeFragmentNew
import com.asda.android.products.ui.detail.model.ProductDetailsExtraInfo
import com.asda.android.products.ui.detail.view.ProductDetailsFragment
import com.asda.android.restapi.cms.model.Refinement
import com.asda.android.restapi.constants.EventConstants
import com.asda.android.restapi.search.model.AsdaSearchData
import com.asda.android.restapi.search.model.SearchData
import com.asda.android.search.constants.Constants.SEARCH_INFO
import com.asda.android.search.model.SearchResultsInfoModel
import com.asda.android.search.view.SearchResultsFragment
import com.asda.android.service.base.Authentication
import com.asda.android.taxonomy.AsdaTaxonomyConfig
import com.asda.android.taxonomy.features.eventpage.view.EventPageFragment
import com.asda.android.taxonomy.features.linksave.view.LinkSaveFragment
import com.asda.android.taxonomy.features.taxonomy.constants.TaxonomyConstants.EXTRA_INFO
import com.asda.android.taxonomy.features.taxonomy.model.TaxonomyViewInfoModel
import com.asda.android.taxonomy.features.topOffers.view.TopOffersFragment

class MobileAppPreviewActivity : BaseAppCompatActivity() {
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_mobile_app_preview)

        supportActionBar?.title = getString(R.string.mobile_app_preview)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        val pageType = intent?.extras?.getString(MobileAppPreviewParser.PAGE_TYPE)
        getFragmentBasedOnType(pageType.orEmpty())?.let {
            push(it, false)
        }
    }

    override fun getFragmentContainer(): Int = R.id.fragment_container

    private fun getFragmentBasedOnType(pageType: String): Fragment? {
        return when (pageType) {
            PAGE_TYPE_HOME -> {
                HomeFragmentNew().apply {
                    arguments = getArgs()
                }
            }
            PAGE_TYPE_CATEGORY -> {
                val pageName = LinkTypeProcessor()[LinkTypeProcessor.REGEX_CATEGORY_FULL]
                val details = getDetails(pageName)
                details?.second?.apply {
                    arguments?.putAll(getArgs())
                }
            }
            PAGE_TYPE_DEPARTMENT -> {
                val pageName = LinkTypeProcessor()[LinkTypeProcessor.REGEX_DEPT_FULL]
                val details = getDetails(pageName)
                details?.second?.apply {
                    arguments?.putAll(getArgs())
                }
            }
            PAGE_TYPE_AISLE -> {
                val pageName = LinkTypeProcessor()[LinkTypeProcessor.REGEX_AISLE_FULL]
                val details = getDetails(pageName)
                details?.second?.apply {
                    arguments?.putAll(getArgs())
                }
            }
            PAGE_TYPE_SHELF -> {
                val pageName = LinkTypeProcessor()[LinkTypeProcessor.REGEX_SHELF_FULL]
                val details = getDetails(pageName)
                details?.second?.apply {
                    arguments?.putAll(getArgs())
                }
            }
            PAGE_TYPE_SEARCH -> {
                val bndle = getArgs()
                val searchData = AsdaSearchData(
                    bndle.getString(MobileAppPreviewParser.PAGE_ID),
                    SearchData.SearchType.SEARCH_TYPE_SUGGESTION,
                    null
                )
                val searchResultsInfo = SearchResultsInfoModel(searchData, false, false)
                val fragment = SearchResultsFragment()
                val bundle = Bundle().apply {
                    putParcelable(SEARCH_INFO, searchResultsInfo)
                }
                fragment.apply {
                    arguments = bndle.apply {
                        putAll(bundle)
                    }
                }
            }
            PAGE_TYPE_PDP -> {
                val bndle = getArgs()
                val itemId = bndle.getString(MobileAppPreviewParser.PAGE_ID)
                val info = ProductDetailsExtraInfo(itemId.orEmpty())
                ProductDetailsFragment.newInstance(info)
            }
            PAGE_TYPE_TOP_OFFERS -> {
                val bndle = getArgs()
                val bundle = Bundle().apply {
                    putAll(bndle)
                    putParcelable(
                        EXTRA_INFO,
                        TaxonomyViewInfoModel(storeId = AsdaTaxonomyConfig.asdaService.lastStoreIdFromSessionMetadata)
                    )
                }
                TopOffersFragment().apply {
                    arguments = bundle
                }
            }
            PAGE_TYPE_LINK_SAVE -> {
                LinkSaveFragment.newInstance(
                    getArgs().getString(MobileAppPreviewParser.PAGE_ID),
                    ""
                ).apply {
                    arguments?.putAll(getArgs())
                }
            }
            PAGE_TYPE_EVENT -> {
                val bndle = getArgs()
                val info = TaxonomyViewInfoModel(
                    storeId = AsdaTaxonomyConfig.asdaService.lastStoreIdFromSessionMetadata,
                    eventPageUrl = "/event/${bndle.getString(MobileAppPreviewParser.PAGE_ID)}"
                )
                EventPageFragment().apply {
                    arguments = bndle.apply {
                        putParcelable(EXTRA_INFO, info)
                    }
                }
            }
            PAGE_TYPE_APP_FAVOURITES -> {
                if (Authentication.getInstance().isLoggedIn) {
                    FavoritesFragmentNew().apply {
                        arguments = getArgs()
                    }
                } else null
            }
            PAGE_BEFORE_YOU_GO -> {
                if (Authentication.getInstance().isLoggedIn) {
                    BeforeYouGoFragment().apply {
                        arguments = getArgs()
                    }
                } else null
            }
            PAGE_TYPE_FULL_TROLLEY -> {
                if (Authentication.getInstance().isLoggedIn) {
                    CXOHelper().toggleCartFragment(true)
                    CXOCartPagerFragment().apply {
                        arguments = getArgs()
                    }
                } else null
            }
            else -> null
        }
    }

    private fun getDetails(pageName: String): Pair<String?, BaseFragment?>? {
        return TaxonomyManager.getTaxonomyFragmentDetails(
            this,
            arrayListOf(
                Refinement(
                    null,
                    null,
                    null,
                    null,
                    null,
                    pageName,
                    intent.getStringExtra(MobileAppPreviewParser.PAGE_ID),
                    null,
                    null,
                    null
                )
            ),
            null
        )
    }

    private fun getArgs() = Bundle().apply {
        val previewDate = intent?.getStringExtra(MobileAppPreviewParser.PREVIEW_DATE)
        val pageId = intent?.getStringExtra(MobileAppPreviewParser.PAGE_ID)
        putBoolean(EventConstants.IS_MOBILE_APP_PREVIEW, true)
        putString(MobileAppPreviewParser.PREVIEW_DATE, previewDate)
        putString(MobileAppPreviewParser.PAGE_ID, pageId)
    }
}
