package com.asda.android.app.orders

import android.text.TextUtils
import androidx.lifecycle.ViewModel
import com.asda.android.base.core.scheduler.SchedulerProvider
import com.asda.android.cxo.datasource.OrderSummaryDataSource
import com.asda.android.orders.constants.OrderConstants.ADDRESS_ERROR
import com.asda.android.orders.constants.OrderConstants.ERROR_MSG_MODIFY_ERROR
import com.asda.android.orders.constants.OrderConstants.GENERATE_JWT_FAILURE
import com.asda.android.orders.constants.OrderConstants.INVALID_BILLING_ADDRESS
import com.asda.android.orders.constants.OrderConstants.MODIFY_ORDER_ADD_CARD
import com.asda.android.payment.threeds.ThreeDsUtils
import com.asda.android.restapi.exception.AsdaException
import com.asda.android.restapi.service.base.RxFailure
import com.asda.android.restapi.service.data.*
import com.asda.android.restapi.service.data.PaymentDetailsResponse.PaymentCards
import com.asda.android.restapi.service.data.checkout.BrowserInfo
import com.asda.android.restapi.service.data.checkout.CheckoutResponse
import com.asda.android.service.base.Authentication
import com.asda.android.singleprofile.base.SingleProfile.getProfileResponse
import com.asda.android.singleprofile.base.SingleProfile.getService
import com.asda.android.singleprofile.viewmodel.AddressViewModel
import com.asda.android.singleprofile.viewmodel.PaymentCardsViewModel
import com.asda.android.utils.Utils.isThreeDs2XEnabled
import io.reactivex.Observable
import io.reactivex.ObservableSource
import io.reactivex.Single
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.functions.Function
import io.reactivex.schedulers.Schedulers
import pangea.EncryptCreditCardService
import pangea.EncryptCreditCardService.EncryptedCc
import pangea.getEncryptCreditCardService
import java.util.*

class AddEditPaymentCardViewModel : ViewModel() {
    var dataSource = OrderSummaryDataSource(SchedulerProvider())
    val paymentHelper = PaymentHelper()
    fun addCardModifyOrderCXO(data: ModifyOrderCXOData): Observable<PaymentDetailsResponse>? {
        val paymentCardsViewModel = PaymentCardsViewModel(getService())
        val paymentCard = data.paymentCard
        return if (data.addAddress) {
            addAddressForCard(data)
                .flatMap { response: AddressBookResponse ->
                    getEncryptCreditCardService(data.context)
                        ?.flatMap { encryptCreditCardService: EncryptCreditCardService ->
                            paymentCard.card?.let {
                                encryptCreditCardService.rxEncrypt(
                                    it, data.cvv
                                )
                            }
                        }
                        ?.doOnNext { encryptedCc: EncryptedCc ->
                            paymentHelper.setCardEncyptedValues(
                                paymentCard,
                                encryptedCc
                            )
                        }
                        ?.flatMap {
                            // Cannot be null as it is being checked above
                            paymentCard.billingAddressId =
                                response.addressBook?.get(0)?.addressId
                            paymentCard.address = AddressBookResponse.Address()
                            paymentCard.address?.addressId =
                                response.addressBook?.get(0)?.addressId
                            val profileId =
                                getProfileResponse()?.profileId
                            val paymentDetailsResponse =
                                PaymentDetailsResponse()
                            paymentDetailsResponse.profileId = profileId
                            paymentDetailsResponse.cards = arrayOfNulls(1)
                            (paymentDetailsResponse.cards as Array<PaymentCards?>)[0] = paymentCard
                            paymentCardsViewModel.addCreditCards(paymentDetailsResponse)
                        }
                }
                .doOnNext { creditCardResponse: PaymentDetailsResponse ->
                    handleV1CardResponse(
                        creditCardResponse,
                        data
                    )
                }
        } else {
            getEncryptCreditCardService(data.context)
                ?.flatMap { encryptCreditCardService: EncryptCreditCardService ->
                    encryptCreditCardService.rxEncryptCc(
                        paymentCard.card
                    )
                }
                ?.doOnNext { encryptedCc: EncryptedCc ->
                    paymentHelper.setCardEncyptedValues(
                        paymentCard,
                        encryptedCc
                    )
                }
                ?.flatMap {
                    // Cannot be null as it is being checked above
                    paymentCard.address = AddressBookResponse.Address()
                    paymentCard.billingAddressId = data.addressId
                    paymentCard.address?.addressId = data.addressId
                    val profileId = getProfileResponse()?.profileId
                    val paymentDetailsResponse = PaymentDetailsResponse()
                    paymentDetailsResponse.profileId = profileId
                    paymentDetailsResponse.cards = arrayOfNulls(1)
                    (paymentDetailsResponse.cards as Array<PaymentCards?>)[0] = paymentCard
                    paymentCardsViewModel.addCreditCards(paymentDetailsResponse)
                }
                ?.doOnNext { creditCardResponse: PaymentDetailsResponse ->
                    handleV1CardResponse(
                        creditCardResponse,
                        data
                    )
                }
        }
    }

    fun useOnceCardModifyOrderCXO(
        data: ModifyOrderCXOData,
        cardType: String
    ): Observable<GenerateJwtResponse>? {
        return if (data.paymentCard.card != null) {
            if (data.addAddress) {
                addAddressForCard(data)
                    .flatMap { response: AddressBookResponse ->
                        getEncryptCreditCardService(data.context)
                            ?.flatMap { encryptCreditCardService: EncryptCreditCardService ->
                                encryptCreditCardService.rxEncrypt(
                                    data.paymentCard.card!!, data.cvv
                                )
                            }
                            ?.flatMap { encryptedCc: EncryptedCc ->
                                data.paymentCard.billingAddressId =
                                    response.addressBook?.get(0)?.addressId
                                data.paymentCard.address = AddressBookResponse.Address()
                                data.paymentCard.address?.addressId =
                                    response.addressBook?.get(0)?.addressId
                                val createJwtDataRequest =
                                    CreateJwtDataRequest(
                                        data.orderId,
                                        data.orderTotal,
                                        paymentHelper.setCardJwtEncyptedValues(encryptedCc, cardType)
                                    )
                                generateJwtCXO(GenerateJwtRequest(createJwtDataRequest))?.toObservable()
                            }
                    }
            } else {
                getEncryptCreditCardService(data.context)
                    ?.flatMap { encryptCreditCardService: EncryptCreditCardService ->
                        encryptCreditCardService.rxEncrypt(
                            data.paymentCard.card!!, data.cvv
                        )
                    }
                    ?.flatMap { encryptedCc: EncryptedCc ->
                        data.paymentCard.address = AddressBookResponse.Address()
                        data.paymentCard.billingAddressId = data.addressId
                        data.paymentCard.address?.addressId = data.addressId
                        if (isThreeDs2XEnabled(
                                data.context,
                                cardType
                            )
                        ) {
                            val createJwtDataRequest = CreateJwtDataRequest(
                                data.orderId,
                                data.orderTotal,
                                paymentHelper.setCardJwtEncyptedValues(encryptedCc, cardType)
                            )
                            return@flatMap generateJwtCXO(GenerateJwtRequest(createJwtDataRequest))
                                ?.toObservable()
                        } else {
                            val error = RxFailure(
                                GENERATE_JWT_FAILURE,
                                null
                            )
                            return@flatMap Observable.error<GenerateJwtResponse>(error)
                        }
                    }
            }
        } else null
    }

    private fun addAddressForCard(data: ModifyOrderCXOData): Observable<out AddressBookResponse> {
        val addressViewModel = AddressViewModel(getService())
        // Addresss
        val addressResponse = AddressBookResponse()
        addressResponse.profileId = getProfileResponse()?.profileId
        val addresses = arrayOfNulls<AddressBookResponse.Address>(1)
        val formattedAddress = data.address + ", " + data.townOrCity + ", " + data.postCode
        val newAddress = AddressBookResponse.Address(
            data.address,
            data.address2,
            " ",
            data.townOrCity,
            " ",
            AddressBookResponse.DEFAULT_COUNTRY,
            data.postCode,
            AddressBookResponse.Type.ShipTo.toString(),
            false,
            AddressBookResponse.AddressType.Flat.toString(),
            "",
            formattedAddress
        )
        addresses[0] = newAddress
        addressResponse.addressBook = addresses
        addressResponse.statusCodeNumber = 0
        return addressViewModel
            .addOrUpdateAddressBasic(addressResponse, true)
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())
            .doOnNext { response: AddressBookResponse ->
                val addressBook = response.addressBook
                requireNotNull(addressBook) { ADDRESS_ERROR }
                require(!(addressBook.isEmpty() || addressBook[0] == null || addressBook[0]?.addressId == null)) { INVALID_BILLING_ADDRESS }
            }
    }

    @Throws(AsdaException::class)
    private fun handleV1CardResponse(
        creditCardResponse: PaymentDetailsResponse,
        data: ModifyOrderCXOData
    ) {
        if (!creditCardResponse.isSuccess) {
            if (TextUtils.isEmpty(data.paRes)) {
                throw RxFailure(-1, creditCardResponse)
            } else {
                if (paymentHelper.hasProfileCard(getProfileResponse())) {
                    val profileCards: Array<PaymentCards>? = getProfileResponse()?.cards
                    val cardIndex = paymentHelper.getCardIndex(data.paymentCard, profileCards)
                    if (paymentHelper.isProfileCardsInValid(profileCards, cardIndex)) {
                        throw AsdaException(ERROR_MSG_MODIFY_ERROR)
                    }
                } else {
                    throw AsdaException(ERROR_MSG_MODIFY_ERROR)
                }
            }
        } else {
            val cardIndex = paymentHelper.getCardIndex(data.paymentCard, creditCardResponse)
            if (paymentHelper.isProfileCardsInValid(creditCardResponse.cards, cardIndex)) {
                throw AsdaException(ERROR_MSG_MODIFY_ERROR)
            }
        }
    }

    fun makeCXOModifyOrderCall(
        data: ModifyOrderCXOData,
        confirm3ds: Boolean?
    ): Single<CheckoutResponse?> {
        return getModifyOrderRequest(data)
            .singleOrError()
            .flatMap { request: CXOModifyOrderRequest? ->
                modifyOrderCXO(
                    data.orderId,
                    confirm3ds,
                    request
                )
            }
    }

    fun getModifyOrderRequest(data: ModifyOrderCXOData): Observable<CXOModifyOrderRequest> {
        val cxoModifyOrderRequestData = CXOModifyOrderRequestData()
        val paymentInfo = PaymentInfo()
        cxoModifyOrderRequestData.browserInfo = BrowserInfo(ThreeDsUtils.DEVICE_CHANNEL)
        cxoModifyOrderRequestData.purchaseOrderId = data.purchaseOrderId
        if (!TextUtils.isEmpty(data.paymentCard.cardHolderName)) {
            paymentInfo.cardHolderName = data.paymentCard.cardHolderName
        } else {
            paymentInfo.cardHolderName = Authentication.getInstance().userViewResponse?.name
        }
        paymentInfo.cardType = data.paymentCard.cardBrand
        paymentInfo.expiryMonth = data.paymentCard.expiryMonth.toString()
        paymentInfo.expiryYear = data.paymentCard.expiryYear.toString()
        paymentInfo.startMonth = data.paymentCard.startMonth.toString()
        paymentInfo.startYear = data.paymentCard.startYear.toString()
        paymentInfo.last4Digits = data.paymentCard.last4Digits.toString()
        if (MODIFY_ORDER_ADD_CARD == data.cardOperation) {
            if (data.saveToProfile) {
                paymentInfo.creditCardNickName = data.creditCardNickName
            } else {
                paymentInfo.creditCardNickName = ""
            }
            cxoModifyOrderRequestData.address = BillingAddressInfo(
                data.address,
                data.address2,
                data.townOrCity,
                "UK",
                "",
                data.postCode
            )
        } else {
            paymentInfo.creditCardNickName = data.creditCardNickName
            cxoModifyOrderRequestData.address = BillingAddressInfo("", "", "", "UK", "", "")
        }
        return getEncryptCreditCardService(data.context)?.flatMap(
            Function<EncryptCreditCardService, ObservableSource<out EncryptedCc>> { encryptCreditCardService: EncryptCreditCardService ->
                if (MODIFY_ORDER_ADD_CARD == data.cardOperation && data.paymentCard.card != null) {
                    return@Function encryptCreditCardService.rxEncrypt(
                        data.paymentCard.card!!, data.cvv
                    )
                } else {
                    return@Function encryptCreditCardService.rxEncryptCvv(data.cvv)
                }
            }
        )
            ?.map { encryptedCc: EncryptedCc ->
                cxoModifyOrderRequestData.payment =
                    paymentHelper.updatePaymentCXO(paymentInfo, encryptedCc, data.paRes)
                CXOModifyOrderRequest(cxoModifyOrderRequestData)
            } as Observable<CXOModifyOrderRequest>
    }

    fun generateJwtCXO(request: GenerateJwtRequest?): Single<GenerateJwtResponse>? {
        return request?.let { dataSource.generateJwt(it) }
    }
    fun modifyOrderCXO(
        orderId: String?,
        confirm3ds: Boolean?,
        cxoModifyOrderRequest: CXOModifyOrderRequest?
    ): Single<CheckoutResponse>? {
        return orderId?.let {
            confirm3ds?.let { it1 ->
                cxoModifyOrderRequest?.let { it2 ->
                    dataSource.modifyOrder(
                        it,
                        it1, it2
                    )
                }
            }
        }
    }
}
