package com.asda.android.app.orders

import android.Manifest
import android.app.Dialog
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.graphics.drawable.Drawable
import android.os.Bundle
import android.os.Handler
import android.text.InputFilter
import android.text.InputFilter.LengthFilter
import android.text.TextUtils
import android.util.Log
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import android.widget.AdapterView.OnItemSelectedListener
import androidx.annotation.StringRes
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.widget.AppCompatRadioButton
import androidx.core.content.ContextCompat
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProviders
import com.asda.android.R
import com.asda.android.app.home.view.RegisterHelper
import com.asda.android.base.core.view.CustomProgressDialog
import com.asda.android.base.core.view.GroupFourDigitsEditText
import com.asda.android.base.core.view.WrappedDrawable
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.base.core.view.ui.AsdaDialogHelper
import com.asda.android.base.core.view.ui.BaseAsdaDialogHelper
import com.asda.android.cxo.CartManager
import com.asda.android.orders.constants.OrderConstants.GENERATE_JWT_FAILURE
import com.asda.android.orders.constants.OrderConstants.MODIFY_OPERATION_EXISTING
import com.asda.android.orders.constants.OrderConstants.MODIFY_OPERATION_NEW
import com.asda.android.orders.constants.OrderConstants.MODIFY_ORDER_ADD_CARD
import com.asda.android.orders.constants.OrderConstants.MODIFY_ORDER_CXO_ID1
import com.asda.android.orders.constants.OrderConstants.MODIFY_ORDER_CXO_ID2
import com.asda.android.payment.threeds.ThreeDsDataSource
import com.asda.android.payment.threeds.ThreeDsDataSource.ThreeDsResult
import com.asda.android.payment.threeds.ThreeDsUtils.trackThreeDSEventCXO
import com.asda.android.restapi.service.base.RxFailure
import com.asda.android.restapi.service.constants.AsdaServiceSettingsConstants.debug
import com.asda.android.restapi.service.constants.CreditCardTypeConstants
import com.asda.android.restapi.service.constants.CreditCardTypeConstants.maestro
import com.asda.android.restapi.service.data.*
import com.asda.android.restapi.service.data.PostcodeCheckResponse.AddressSuggestion
import com.asda.android.restapi.service.data.checkout.CheckoutResponse
import com.asda.android.service.base.Authentication
import com.asda.android.service.base.OrderManager.Companion.get
import com.asda.android.singleprofile.base.SingleProfile.getService
import com.asda.android.singleprofile.features.account.datasource.UserAccountDetails
import com.asda.android.singleprofile.features.account.datasource.UserAccountDetails.AddressBookLoadListener
import com.asda.android.singleprofile.features.account.view.BaseAddEditAddressFragment
import com.asda.android.singleprofile.features.account.view.CheckPostcodeDialogFragment
import com.asda.android.singleprofile.features.account.view.SPCheckPostCodeDialogFragment.Companion.newInstance
import com.asda.android.singleprofile.interfaces.AddressBookChangedListener
import com.asda.android.singleprofile.interfaces.PostcodeCheckListener
import com.asda.android.singleprofile.model.AddressSuggestUiState
import com.asda.android.singleprofile.model.CreateCardInitializationHolder
import com.asda.android.singleprofile.viewmodel.AddressViewModel
import com.asda.android.singleprofile.viewmodel.PaymentCardsViewModel.Companion.createCreditCard
import com.asda.android.utils.Utils.currentTimeInUK
import com.asda.android.utils.Utils.getCardType
import com.asda.android.utils.Utils.isDevBuild
import com.asda.android.utils.Utils.isThreeDs2XEnabled
import com.asda.android.utils.Utils.isValidCardNumber
import com.asda.android.utils.Utils.parseNewCreditCardDate
import com.asda.android.utils.Utils.validateCsc
import com.asda.android.utils.view.Permissions
import com.asda.android.utils.view.Permissions.isCameraPermissionGranted
import com.asda.android.utils.view.ViewUtil.findViewById
import com.asda.android.utils.view.ViewUtil.hideKeyboard
import com.google.android.material.textfield.TextInputEditText
import com.google.android.material.textfield.TextInputLayout
import com.jakewharton.rxbinding2.widget.RxTextView
import io.card.payment.CardIOActivity
import io.card.payment.CreditCard
import io.reactivex.*
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.exceptions.CompositeException
import io.reactivex.functions.Function5
import io.reactivex.observers.ResourceObserver
import io.reactivex.observers.ResourceSingleObserver
import io.reactivex.processors.PublishProcessor
import io.reactivex.subscribers.DefaultSubscriber
import io.reactivex.subscribers.DisposableSubscriber
import java.text.ParseException
import java.util.*

class AddPaymentCardFragment :
    FeaturedFragment(),
    PostcodeCheckListener,
    AddressBookChangedListener {
    private val threeDsDataSource = ThreeDsDataSource()
    private val mHandler = Handler()
    private lateinit var mContext: Context
    private lateinit var mCardHolderNameLabel: TextInputLayout
    private lateinit var mCardHolderName: TextInputEditText
    private lateinit var mExpiryDateLabel: TextInputLayout
    private lateinit var mStartDateLabel: TextInputLayout
    private lateinit var mStartDate: TextInputEditText
    private lateinit var mExpiryDate: TextInputEditText
    private var mExpiryCvvContainer: View? = null
    private lateinit var mMaestroCvvLabel: TextInputLayout
    private lateinit var mMaestroCvv: TextInputEditText
    private lateinit var mCardCvvLabel: TextInputLayout
    private lateinit var mCardCvv: TextInputEditText
    private lateinit var mCardNumberLabel: TextInputLayout
    lateinit var cardNumber: GroupFourDigitsEditText
        private set
    private lateinit var mAddressSpinner: Spinner
    private lateinit var mSaveCardRadioGroup: RadioGroup
    private var mSaveCardYesRadioButton: AppCompatRadioButton? = null
    private lateinit var mSaveCardNoRadioButton: AppCompatRadioButton
    private lateinit var mAddressLine1Label: TextInputLayout
    lateinit var addressLine1: TextInputEditText
        private set
    private lateinit var mAddressLine2Label: TextInputLayout
    lateinit var addressLine2: TextInputEditText
        private set
    private lateinit var mPostCodeContainer: ViewGroup
    private lateinit var mPostcodeLabel: TextInputLayout
    lateinit var postcode: TextInputEditText
        private set
    private lateinit var mFindPostCode: Button
    private lateinit var mCityLabel: TextInputLayout
    lateinit var city: TextInputEditText
        private set
    private lateinit var mNewAddressContainer: ViewGroup
    private lateinit var mSaveButton: Button
    private var mCancelButton: Button? = null
    private var mCardIcon: Drawable? = null
    private var mScanButton: Button? = null
    private lateinit var mChooseAddressText: TextView
    private lateinit var mChooseAddressSpinner: Spinner
    private var mTitleText: String? = null
    private var mHasError = false
    private var mCreditCardNickName: String? = ""
    private var mCardHolderNameObservable: Flowable<CharSequence>? = null
    private var mCardNumberObservable: Flowable<CharSequence>? = null
    private var mMaestroCardStartDateObservable: Flowable<CharSequence>? = null
    private var mCardExpiryObservable: Flowable<CharSequence>? = null
    private var mCardCvvObservable: Flowable<CharSequence>? = null
    private var mMaestroCardCvvObservable: Flowable<CharSequence>? = null
    private var mAddress1Observable: Flowable<CharSequence>? = null
    private var mAddress2Observable: Flowable<CharSequence>? = null
    private var mPostcodeObservable: Flowable<CharSequence>? = null
    private var mCityObservable: Flowable<CharSequence>? = null
    private lateinit var mViewModel: AddEditPaymentCardViewModel
    var orderId: String? = null
        private set
    private var mPurchaseOrderId: String? = null
    private var mOrderTotal: Double? = null
    private var mYesOrNo = false
    private var mPaResReceived = false
    private var mPaRes: String? = null
    private var mAddress: String? = null
    private val mAddress2: String? = null
    private var mPostCode: String? = null
    private var mTownOrCity: String? = null
    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        val view = inflater.inflate(R.layout.order_add_payment_card, container, false)
        setupView(view)
        combineLatestEvents(view)
        return view
    }

    override fun loadBundleData(bundle: Bundle?) {
        super.loadBundleData(bundle)
        if (bundle != null) {
            orderId = bundle.getString(KEY_ORDER_ID)
            if (bundle.containsKey(ORDER_TOTAL)) {
                mOrderTotal = bundle.getDouble(ORDER_TOTAL)
            }
        }
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        mContext = context
        mTitleText = mContext.getString(R.string.add_new_card)
        mViewModel = ViewModelProviders.of(requireActivity())[AddEditPaymentCardViewModel::class.java]
    }

    fun setTitleText(titleText: String?) {
        mTitleText = titleText
    }

    private fun setupView(view: View) {
        mCardHolderNameLabel = findViewById<TextInputLayout>(view, R.id.card_holder_name_text)
        mCardHolderName = findViewById<TextInputEditText>(view, R.id.card_holder_name)
        mCardNumberLabel = findViewById<TextInputLayout>(view, R.id.card_number_text)
        cardNumber = findViewById<GroupFourDigitsEditText>(view, R.id.card_number)
        cardNumber.setDontGroupAfterDigitsCount(16)
        mScanButton = findViewById<Button>(view, R.id.scan_button)
        mStartDateLabel = findViewById<TextInputLayout>(view, R.id.start_date_text)
        mStartDate = findViewById<TextInputEditText>(view, R.id.start_date)
        mExpiryCvvContainer = findViewById<View>(view, R.id.expiry_cvv_container)
        mMaestroCvvLabel = findViewById<TextInputLayout>(view, R.id.card_cvv_maestro_text)
        mMaestroCvv = findViewById<TextInputEditText>(view, R.id.card_cvv_maestro)
        mExpiryDateLabel = findViewById<TextInputLayout>(view, R.id.expiry_date_text)
        mExpiryDate = findViewById<TextInputEditText>(view, R.id.expiry_date)
        mCardCvvLabel = findViewById<TextInputLayout>(view, R.id.card_cvv_text)
        mCardCvv = findViewById<TextInputEditText>(view, R.id.card_cvv)
        mSaveCardRadioGroup = findViewById<RadioGroup>(view, R.id.save_card_radio_group)
        mSaveCardYesRadioButton =
            findViewById<AppCompatRadioButton>(view, R.id.save_card_yes_button)
        mSaveCardNoRadioButton = findViewById<AppCompatRadioButton>(view, R.id.save_card_no_button)
        mAddressSpinner = findViewById<Spinner>(view, R.id.address_spinner)
        mAddressLine1Label = findViewById<TextInputLayout>(view, R.id.address_line1_text)
        addressLine1 = findViewById<TextInputEditText>(view, R.id.address_line1_value)
        mAddressLine2Label = findViewById<TextInputLayout>(view, R.id.address_line2_text)
        addressLine2 = findViewById<TextInputEditText>(view, R.id.address_line2_value)
        mPostCodeContainer = findViewById<ViewGroup>(view, R.id.postcode_container)
        mPostcodeLabel = findViewById<TextInputLayout>(view, R.id.postcode_text)
        postcode = findViewById<TextInputEditText>(view, R.id.postcode_value)
        mFindPostCode = findViewById<Button>(view, R.id.find_postcode)
        mCityLabel = findViewById<TextInputLayout>(view, R.id.city_text)
        city = findViewById<TextInputEditText>(view, R.id.city_value)
        mChooseAddressText = findViewById<TextView>(view, R.id.choose_address_text)
        mNewAddressContainer = findViewById<ViewGroup>(view, R.id.new_address_container)
        mSaveButton = findViewById<Button>(view, R.id.save_card)
        mCancelButton = findViewById<Button>(view, R.id.cancel_card)
        mSaveButton.isEnabled = false
        mCancelButton?.setOnClickListener({ v: View? -> pop() })
        mScanButton?.setOnClickListener { v: View? ->
            if (isCameraPermissionGranted(mContext)) {
                startCardIOActivity()
            } else if (isAdded()) {
                requestPermissions(
                    arrayOf(Manifest.permission.CAMERA),
                    Permissions.REQUEST_CODE_PERMISSIONS_REQUEST_CAMERA
                )
            }
        }
        mChooseAddressSpinner = findViewById<Spinner>(view, R.id.choose_address_spinner)
        mCardIcon = ContextCompat.getDrawable(mContext, R.drawable.ic_blank)
        val wrappedDrawable = mCardIcon?.let { WrappedDrawable(it) }
        wrappedDrawable?.setBounds(
            0, 0, mContext.resources.getDimensionPixelSize(R.dimen.credit_card_width),
            mContext.resources.getDimensionPixelSize(R.dimen.credit_card_height)
        )
        val scanIcon = ContextCompat.getDrawable(mContext, R.drawable.ic_scan)
        scanIcon?.setBounds(
            0,
            0,
            mContext.resources.getDimensionPixelSize(R.dimen.camera_icon_width),
            mContext.resources.getDimensionPixelSize(R.dimen.camera_icon_height)
        )
        cardNumber.setCompoundDrawables(mCardIcon, null, scanIcon, null)
        cardNumber.compoundDrawablePadding = 16
        mCardNumberObservable = RxTextView.textChanges(cardNumber).skip(1).toFlowable(
            BackpressureStrategy.LATEST
        )
        mCardHolderNameObservable = RxTextView.textChanges(mCardHolderName).toFlowable(
            BackpressureStrategy.LATEST
        )
        mCardExpiryObservable = RxTextView.textChanges(mExpiryDate).skip(2).toFlowable(
            BackpressureStrategy.LATEST
        )
        mCardCvvObservable = RxTextView.textChanges(mCardCvv).skip(3).toFlowable(
            BackpressureStrategy.LATEST
        )
        mMaestroCardStartDateObservable = Flowable.fromCallable({ DUMMY_DATE })
        mMaestroCardCvvObservable = RxTextView.textChanges(mMaestroCvv).skip(3).toFlowable(
            BackpressureStrategy.LATEST
        )
        mAddress1Observable = RxTextView.textChanges(addressLine1).skip(1).toFlowable(
            BackpressureStrategy.LATEST
        )
        mAddress2Observable = RxTextView.textChanges(addressLine2).skip(1).toFlowable(
            BackpressureStrategy.LATEST
        )
        mPostcodeObservable = RxTextView.textChanges(postcode).skip(1).toFlowable(
            BackpressureStrategy.LATEST
        )
        mCityObservable =
            RxTextView.textChanges(city).skip(1).toFlowable(BackpressureStrategy.LATEST)
        if (mNewAddressContainer.visibility == View.VISIBLE) {
            if (mStartDate.visibility == View.VISIBLE) {
                clearErrorLabels(
                    mCardHolderNameLabel,
                    mCardNumberLabel,
                    mStartDateLabel,
                    mExpiryDateLabel,
                    mMaestroCvvLabel,
                    mAddressLine1Label,
                    mAddressLine2Label,
                    mPostcodeLabel,
                    mCityLabel
                )
            } else {
                clearErrorLabels(
                    mCardHolderNameLabel,
                    mCardNumberLabel,
                    mExpiryDateLabel,
                    mCardCvvLabel,
                    mAddressLine1Label,
                    mAddressLine2Label,
                    mPostcodeLabel,
                    mCityLabel
                )
            }
        } else {
            if (mStartDate.visibility == View.VISIBLE) {
                clearErrorLabels(
                    mCardHolderNameLabel,
                    mCardNumberLabel,
                    mStartDateLabel,
                    mExpiryDateLabel,
                    mMaestroCvvLabel
                )
            } else {
                clearErrorLabels(
                    mCardHolderNameLabel,
                    mCardNumberLabel, mExpiryDateLabel, mCardCvvLabel
                )
            }
        }
        loadBillingAddresses()
        mCardNumberObservable?.subscribe(
            object : DefaultSubscriber<CharSequence?>() {
                override fun onNext(charSequence: CharSequence?) {
                    Log.d(TAG, "Card Number Observable OnNext")
                    mCardNumberLabel.isHintEnabled = false
                    val type = getCardType(charSequence.toString().replace("\\s".toRegex(), ""))
                    mCardIcon = ContextCompat.getDrawable(mContext, type.cardIcon)
                    if (mCardIcon != null) {
                        val wrappedDrawable = WrappedDrawable(
                            mCardIcon!!
                        )
                        wrappedDrawable.setBounds(
                            0,
                            0,
                            mContext.resources.getDimensionPixelSize(R.dimen.credit_card_width),
                            mContext.resources.getDimensionPixelSize(R.dimen.credit_card_height)
                        )
                    }
                    val scanIcon = ContextCompat.getDrawable(mContext, R.drawable.ic_scan)
                    scanIcon?.setBounds(
                        0,
                        0,
                        mContext.resources.getDimensionPixelSize(R.dimen.camera_icon_width),
                        mContext.resources.getDimensionPixelSize(R.dimen.camera_icon_height)
                    )
                    cardNumber.setCompoundDrawables(mCardIcon, null, scanIcon, null)
                    cardNumber.compoundDrawablePadding = 16
                    if (type === maestro) {
                        mStartDateLabel.visibility = View.VISIBLE
                        mMaestroCvvLabel.visibility = View.VISIBLE
                        mCardCvvLabel.visibility = View.GONE
                        maestroSource.onNext(mMaestroCvvLabel.visibility == View.VISIBLE)
                        mMaestroCardStartDateObservable = RxTextView.textChanges(mStartDate)
                            .skip(1)
                            .toFlowable(BackpressureStrategy.LATEST)
                        combineLatestEvents(view)
                        mMaestroCardStartDateObservable?.subscribe(
                            object : DefaultSubscriber<CharSequence?>() {
                                override fun onNext(charSequence: CharSequence?) {
                                    if (charSequence != null) {
                                        if (charSequence.length == 2 && charSequence[0] != '/' && charSequence[1] != '/') {
                                            val date = "$charSequence/"
                                            mStartDate.setText(date)
                                            mStartDate.setSelection(date.length)
                                        }
                                        if (charSequence.length == 1 && charSequence[0] == '/') {
                                            mStartDate.setText("")
                                            mStartDate.setSelection(0)
                                        }
                                        if (charSequence.length == 5) {
                                            mExpiryDate.requestFocus()
                                        }
                                    }
                                }

                                override fun onError(throwable: Throwable) {
                                    // No op
                                }

                                override fun onComplete() {
                                    // No op
                                }
                            })
                    } else {
                        mStartDateLabel.visibility = View.GONE
                        mMaestroCvvLabel.visibility = View.GONE
                        mCardCvvLabel.visibility = View.VISIBLE
                    }
                    if (charSequence != null) {
                        when (type.cardtype) {
                            CreditCardTypeConstants.TYPE_AMEX -> if (charSequence.length == 18) {
                                mExpiryDate.requestFocus()
                                mCardCvv.filters = arrayOf<InputFilter>(LengthFilter(4))
                            }
                            CreditCardTypeConstants.TYPE_VISA, CreditCardTypeConstants.TYPE_MASTER_CARD, CreditCardTypeConstants.TYPE_ASDA_MONEY_CREDIT -> if (charSequence.length == 19) {
                                mExpiryDate.requestFocus()
                                mCardCvv.filters = arrayOf<InputFilter>(LengthFilter(3))
                            }
                            CreditCardTypeConstants.TYPE_MAESTRO -> if (charSequence.length == 19) {
                                mStartDate.requestFocus()
                                mCardCvv.filters = arrayOf<InputFilter>(LengthFilter(3))
                            }
                        }
                    }
                }

                override fun onError(t: Throwable) {
                    Log.d(TAG, "Card Number Observable onError" + t.message)
                }

                override fun onComplete() {
                    Log.d(TAG, "Card Number Observable onComplete")
                }
            })
        mCardExpiryObservable?.subscribe(
            object : DefaultSubscriber<CharSequence?>() {

                override fun onNext(charSequence: CharSequence?) {
                    if (charSequence != null) {
                        if (charSequence.length == 2 && charSequence[0] != '/' && charSequence[1] != '/') {
                            val date = "$charSequence/"
                            mExpiryDate.setText(date)
                            mExpiryDate.setSelection(date.length)
                        }
                        if (charSequence.length == 1 && charSequence[0] == '/') {
                            mExpiryDate.setText("")
                            mExpiryDate.setSelection(0)
                        }
                        if (charSequence.length == 5) {
                            if (mMaestroCvvLabel.visibility == View.VISIBLE) {
                                mMaestroCvv.requestFocus()
                            } else {
                                mCardCvv.requestFocus()
                            }
                        }
                    }
                }

                override fun onError(t: Throwable) {
                    Log.d(TAG, "Expiry date Observable onError" + t.message)
                }

                override fun onComplete() {
                    Log.d(TAG, "Expiry date Observable onComplete")
                }
            })
        mPostcodeObservable?.subscribe(
            object : DefaultSubscriber<CharSequence?>() {

                override fun onNext(charSequence: CharSequence?) {
                    clearErrorLabels(mPostcodeLabel)
                    if (charSequence != null) {
                        if (charSequence.length == 6) {
                            mFindPostCode.isEnabled = true
                            mFindPostCode.setBackgroundColor(
                                ContextCompat.getColor(mContext, R.color.asda_green)
                            )
                            mFindPostCode.setTextColor(
                                ContextCompat.getColor(
                                    mContext, R.color.black
                                )
                            )
                            mFindPostCode.setOnClickListener { v: View? ->
                                hideKeyboard(mFindPostCode)
                                showDialog(DIALOG_CHECK_POSTCODE)
                                val model =
                                    AddressViewModel(getService())
                                model
                                    .getSuggestAddressState(charSequence.toString())
                                    .observeOn(AndroidSchedulers.mainThread())
                                    .subscribe(
                                        object :
                                            ResourceObserver<AddressSuggestUiState?>() {
                                            override fun onNext(addressSuggestUiState: AddressSuggestUiState) {
                                                try {
                                                    dismissDialog(DIALOG_CHECK_POSTCODE)
                                                } catch (ignored: Exception) {
                                                    // No op
                                                }
                                                if ((addressSuggestUiState.addressResponse != null)
                                                ) {
                                                    if (addressSuggestUiState.addressResponse?.isSuccess() == true) {
                                                        validateCheckPostCode(
                                                            addressSuggestUiState.addressResponse,
                                                            view
                                                        )
                                                    } else {
                                                        BaseAsdaDialogHelper.displayErrorDialog(
                                                            addressSuggestUiState.addressResponse,
                                                            -1,
                                                            mContext,
                                                            SCREEN_NAME
                                                        )
                                                    }
                                                }
                                            }

                                            override fun onError(e: Throwable) {
                                                if (!isDisposed()) {
                                                    dispose()
                                                }
                                                try {
                                                    dismissDialog(DIALOG_CHECK_POSTCODE)
                                                    showNewAddressView(view, true)
                                                    clearErrorLabels(mPostcodeLabel)
                                                } catch (ignored: Exception) {
                                                    // No op
                                                }
                                                setPostCodeError(getString(R.string.error_postcode_lookup))
                                            }

                                            override fun onComplete() {
                                                if (!isDisposed()) {
                                                    dispose()
                                                }
                                            }
                                        })
                            }
                        }
                    }
                }

                override fun onError(t: Throwable) {
                    Log.d(TAG, "Post Code Observable onError" + t.message)
                }

                override fun onComplete() {
                    Log.d(TAG, "Post Code Observable onComplete")
                }
            })
        mSaveCardNoRadioButton.isChecked = true
        mSaveCardRadioGroup.setOnCheckedChangeListener { radioGroup: RadioGroup?, id: Int ->
            when (id) {
                R.id.save_card_yes_button -> mYesOrNo = true
                R.id.save_card_no_button -> mYesOrNo = false
                else -> mYesOrNo = false
            }
        }
        mSaveButton.setOnClickListener { view1: View? ->
            addNewPaymentCardCxo(
                view1,
                false,
                MODIFY_ORDER_CXO_ID1
            )
        }
    }

    private fun addNewPaymentCardCxo(view: View?, confirm3ds: Boolean, type: String) {
        // Call v1/card to save the Card and Address(if needed) OR call useOnceCardAPI then if 3DS2.0
        // enabled call generateJWT method with the nickname of newly added card./
        showDialog(DIALOG_AUTHORISING)
        try {
            var expiryMonthNum = -1
            var expiryYearNum = -1
            if (mExpiryDate.text.toString().length >= 5) {
                expiryMonthNum = mExpiryDate.text.toString().substring(0, 2).toInt()
                expiryYearNum = ("20" + mExpiryDate.text.toString().substring(3, 5)).toInt()
            }
            val cardBrand = getCardType(cardNumber.text.toString()).name
            var addAddress = false
            var selectedAddressId: String? = null
            if (MODIFY_OPERATION_NEW == addressOperation) {
                addAddress = true
            } else {
                if (mChooseAddressSpinner.visibility == View.VISIBLE) {
                    addAddress = true
                } else {
                    selectedAddressId = addressIdFromSelectedAddress
                }
            }
            val initializationHolder = CreateCardInitializationHolder()
            initializationHolder.cardNumber = cardNumber.rawNumberString
            initializationHolder.cardHolderName = mCardHolderName.text.toString()
            initializationHolder.billingAddressId = selectedAddressId
            initializationHolder.expiryMonth = expiryMonthNum
            initializationHolder.expiryYear = expiryYearNum
            initializationHolder.expiryDate = mExpiryDate.text.toString()
            initializationHolder.cardBrand = cardBrand
            initializationHolder.isMainCard = true
            initializationHolder.editCard = null
            val card = createCreditCard(initializationHolder)
            val data = ModifyOrderCXOData(
                card,
                if (mMaestroCvvLabel.visibility == View.VISIBLE) mMaestroCvv.text.toString() else mCardCvv.text.toString(),
                addAddress,
                mYesOrNo,
                orderId,
                mPurchaseOrderId,
                mAddress,
                mAddress2,
                mPostCode,
                mTownOrCity,
                selectedAddressId,
                Authentication.getInstance().csrfToken,
                mPaRes,
                mOrderTotal,
                mContext,
                mCreditCardNickName,
                MODIFY_ORDER_ADD_CARD,
                type
            )
            if (MODIFY_ORDER_CXO_ID1 == type) {
                if (data.saveToProfile) {
                    safeAdd(
                        mViewModel
                            .addCardModifyOrderCXO(data)
                            ?.singleOrError()
                            ?.flatMap { creditCardResponse: PaymentDetailsResponse? ->
                                if (isThreeDs2XEnabled(
                                        (mContext), cardBrand
                                    )
                                ) {
                                    return@flatMap callGenerateJWT(creditCardResponse, data)
                                } else {
                                    val error = RxFailure(
                                        GENERATE_JWT_FAILURE,
                                        null
                                    )
                                    return@flatMap Single.error<GenerateJwtResponse>(error)
                                }
                            }
                            ?.subscribe(
                                { generateJwtResponse: GenerateJwtResponse? ->
                                    handleGenerateJwtSuccess(
                                        generateJwtResponse, cardBrand, data, view, confirm3ds
                                    )
                                },
                                { throwable: Throwable? ->
                                    callModifyOrderCXO(
                                        mViewModel.getModifyOrderRequest(data), view, confirm3ds
                                    )
                                }
                            )
                    )
                } else {
                    safeAdd(
                        mViewModel
                            .useOnceCardModifyOrderCXO(data, cardBrand)
                            ?.singleOrError()
                            ?.subscribe(
                                { generateJwtResponse: GenerateJwtResponse? ->
                                    handleGenerateJwtSuccess(
                                        generateJwtResponse, cardBrand, data, view, confirm3ds
                                    )
                                },
                                { throwable: Throwable? ->
                                    callModifyOrderCXO(
                                        mViewModel.getModifyOrderRequest(data), view, confirm3ds
                                    )
                                }
                            )
                    )
                }
            } else {
                callModifyOrderCXO(mViewModel.getModifyOrderRequest(data), view, confirm3ds)
            }
        } catch (e: Exception) {
            try {
                dismissDialog(DIALOG_AUTHORISING, true)
            } catch (ignored: Exception) {
                // No op
            }
            BaseAsdaDialogHelper.displayErrorDialog(null, -1, activity, SCREEN_NAME)
        }
    }

    private fun callModifyOrderCXO(
        modifyOrderRequest: Observable<CXOModifyOrderRequest>,
        view: View?,
        confirm3ds: Boolean
    ) {
        modifyOrderRequest
            .singleOrError()
            .flatMap { reqBody: CXOModifyOrderRequest? ->
                mViewModel.modifyOrderCXO(
                    orderId, confirm3ds, reqBody
                )
            }.subscribe(getModifyOrderObserver(view))
    }

    private fun getModifyOrderObserver(view: View?): ResourceSingleObserver<CheckoutResponse?> {
        return object : ResourceSingleObserver<CheckoutResponse?>() {
            override fun onSuccess(checkoutResponse: CheckoutResponse) {
                if (mViewModel.paymentHelper.is3DSRequired(checkoutResponse) == true) {
                    dismissDialog(DIALOG_AUTHORISING, true)
                    val cardType: String?
                    if (checkoutResponse.data != null && checkoutResponse.data?.purchaseOrder != null && checkoutResponse.data?.purchaseOrder?.paymentDetails != null &&
                        checkoutResponse.data?.purchaseOrder?.threeDsVerification != null
                    ) {
                        val cxoCartManager: CartManager = CartManager.instance
                        if (cxoCartManager.orderId != null && cxoCartManager.authentication.singleProfileId != null && threeDsDataSource.isThreeDsSDKVersion(
                                checkoutResponse
                                    .data!!
                                    .purchaseOrder!!
                                    .threeDsVerification!!
                                    .version
                            )
                        ) {
                            trackThreeDSEventCXO(
                                checkoutResponse
                                    .data!!
                                    .purchaseOrder!!
                                    .threeDsVerification!!
                                    .version,
                                TAG,
                                cxoCartManager.orderId,
                                cxoCartManager.authentication.singleProfileId!!,
                                cxoCartManager.cartId
                            )
                        }
                        mPurchaseOrderId = checkoutResponse.data?.purchaseOrder?.purchaseOrderId
                        cardType = checkoutResponse
                            .data!!
                            .purchaseOrder!!
                            .paymentDetails!![0]
                            .cardType
                        threeDsDataSource.doThreeDsSecureCXOCheckout(
                            mContext,
                            checkoutResponse.data?.purchaseOrder?.threeDsVerification,
                            cardType,
                            isThreeDs2XEnabled(mContext, cardType!!)
                        )
                    }
                    threeDsDataSource
                        .getThreeDsLiveData()
                        .observe(
                            viewLifecycleOwner,
                            Observer { threeDsResponse: ThreeDsResult ->
                                if (debug) {
                                    Log.i(
                                        TAG,
                                        "Modify Order CXO()" + threeDsResponse.status
                                    )
                                }
                                if ((
                                    threeDsResponse
                                        .status
                                        == mContext.getString(R.string.threeDsSuccess)
                                    )
                                ) {
                                    if ((
                                        threeDsResponse.jwt != null &&
                                            !TextUtils.isEmpty(threeDsResponse.jwt)
                                        )
                                    ) {
                                        mPaRes = threeDsResponse.jwt
                                    }
                                    if ((
                                        threeDsResponse.paRes != null &&
                                            !TextUtils.isEmpty(threeDsResponse.paRes)
                                        )
                                    ) {
                                        showDialog(DIALOG_AUTHORISING)
                                        if (debug) {
                                            Log.i(
                                                TAG,
                                                "Add payment card PaRes received"
                                            )
                                        }
                                        mPaRes = threeDsResponse.paRes
                                    }
                                    mPaResReceived = true
                                } else if ((
                                    threeDsResponse
                                        .status
                                        == mContext.getString(R.string.threeDsFailed)
                                    )
                                ) {
                                    if ((
                                        debug &&
                                            (
                                                threeDsResponse.errorNumber != null
                                                ) && (threeDsResponse.errorDescription != null)
                                        )
                                    ) {
                                        Log.i(
                                            TAG,
                                            (
                                                "ErrorNumber: " +
                                                    threeDsResponse.errorNumber +
                                                    " errorDescription: " +
                                                    threeDsResponse.errorDescription
                                                )
                                        )
                                    }
                                    mHandler.postDelayed(
                                        {
                                            if (view != null) {
                                                hideKeyboard(view)
                                            }
                                            showDialog(DIALOG_AUTHORISING_FAILED)
                                        },
                                        DELAY_SECONDS.toLong()
                                    )
                                    if ((
                                        threeDsResponse.errorResponse != null
                                        ) && (
                                            threeDsResponse.errorResponse?.firstErrorMessage != null
                                            )
                                    ) {
                                        AsdaDialogHelper.displayErrorDialog(
                                            threeDsResponse.errorResponse,
                                            1000,
                                            mContext,
                                            SCREEN_NAME
                                        )
                                    } else {
                                        showDialog(DIALOG_AUTHORISING_FAILED)
                                    }
                                }
                            }
                        )
                } else {
                    try {
                        dismissDialog(DIALOG_AUTHORISING, true)
                        showDialog(DIALOG_AUTHORISING_SUCCESS)
                    } catch (ignored: Exception) {
                        // Do nothing
                    }
                }
            }

            override fun onError(throwable: Throwable) {
                dispose()
                try {
                    dismissDialog(DIALOG_AUTHORISING, true)
                } catch (ignored: Exception) {
                    // No op
                }
                if (throwable is RxFailure) {
                    val error = throwable
                    if (error.integer != -2) {
                        AsdaDialogHelper.displayErrorDialog(
                            error.asdaResponse, error.integer, activity, SCREEN_NAME
                        )
                    }
                } else if (throwable is CompositeException &&
                    throwable.cause.cause is RxFailure
                ) {
                    val error = throwable.cause.cause as RxFailure?
                    if (error != null && error.integer != -2) {
                        AsdaDialogHelper.displayErrorDialog(
                            error.asdaResponse, error.integer, activity, SCREEN_NAME
                        )
                    }
                } else {
                    try {
                        showDialog(DIALOG_AUTHORISING_FAILED)
                    } catch (ignore: Exception) {
                        // No op
                    }
                }
            }
        }
    }

    private fun callGenerateJWT(
        creditCardResponse: PaymentDetailsResponse?,
        data: ModifyOrderCXOData
    ): Single<GenerateJwtResponse>? {
        var cardIndex = 0
        var cardId: String? = ""
        if (TextUtils.isEmpty(data.paRes) &&
            creditCardResponse != null && creditCardResponse.cards != null
        ) {
            cardIndex = mViewModel.paymentHelper.getCardIndex(data.paymentCard, creditCardResponse)
            cardId = creditCardResponse.cards?.get(cardIndex)?.cardId
        }
        mCreditCardNickName = cardId
        data.creditCardNickName = mCreditCardNickName
        val createJwtPaymentDetails = CreateJwtPaymentDetails()
        createJwtPaymentDetails.creditCardNickName = cardId
        val generateJwtRequest = GenerateJwtRequest(
            CreateJwtDataRequest(
                data.orderId, data.orderTotal, createJwtPaymentDetails
            )
        )
        return mViewModel.generateJwtCXO(generateJwtRequest)
    }

    private fun handleGenerateJwtSuccess(
        generateJwtResponse: GenerateJwtResponse?,
        cardBrand: String,
        data: ModifyOrderCXOData,
        view: View?,
        confirm3ds: Boolean
    ) {
        if (isThreeDs2XEnabled(mContext, cardBrand) &&
            generateJwtResponse != null && generateJwtResponse.data.jwt != null && generateJwtResponse.data.jwt?.isNotEmpty() == true
        ) {
            threeDsDataSource.initializeCardinal(
                generateJwtResponse.data.jwt!!, isDevBuild
            )
        }
        threeDsDataSource
            .getInitCardinalLiveData()
            .observe(
                viewLifecycleOwner,
                Observer { value: Boolean? ->
                    callModifyOrderCXO(
                        mViewModel.getModifyOrderRequest(
                            data
                        ),
                        view, confirm3ds
                    )
                }
            )
    }

    private val addressIdFromSelectedAddress: String?
        private get() {
            val address: AddressBookResponse.Address =
                mAddressSpinner.selectedItem as AddressBookResponse.Address
            if (address != null) {
                if (address.address != null && address.address?.contentEquals("ADD NEW ADDRESS") == true) {
                    val chooseAddress: AddressBookResponse.Address =
                        mAddressSpinner.selectedItem as AddressBookResponse.Address
                    if (chooseAddress != null) {
                        return chooseAddress.addressId
                    }
                } else {
                    return address.addressId
                }
            }
            return null
        }

    fun validateCheckPostCode(response: PostcodeCheckResponse?, view: View?) {
        if (RegisterHelper.dontDeliver(response)) {
            showNewAddressView(view, false)
            setPostCodeError(response?.postcodeDeliveryMessage)
        } else if (response?.checkPostcodeValidation == true ||
            response?.addresses != null && response.addresses?.size!! > 0
        ) {
            showNewAddressView(view, false)
            setChooseAddressSpinner(response.addresses)
        } else {
            showNewAddressView(view, true)
            setPostCodeError(getString(R.string.error_postcode_lookup))
        }
    }

    private val addressOperation: String
        private get() {
            return if ((
                mNewAddressContainer.visibility == View.VISIBLE &&
                    mChooseAddressSpinner.selectedItemPosition == 0
                )
            ) {
                MODIFY_OPERATION_NEW
            } else MODIFY_OPERATION_EXISTING
        }

    private fun startCardIOActivity() {
        val scanIntent = Intent(mContext, CardIOActivity::class.java)
        scanIntent.putExtra(CardIOActivity.EXTRA_REQUIRE_EXPIRY, true)
        scanIntent.putExtra(CardIOActivity.EXTRA_SCAN_EXPIRY, true)
        scanIntent.putExtra(CardIOActivity.EXTRA_SUPPRESS_CONFIRMATION, true)
        scanIntent.putExtra(CardIOActivity.EXTRA_SUPPRESS_MANUAL_ENTRY, true)
        scanIntent.putExtra(CardIOActivity.EXTRA_HIDE_CARDIO_LOGO, true)
        scanIntent.putExtra(CardIOActivity.EXTRA_KEEP_APPLICATION_THEME, true)
        startActivityForResult(scanIntent, SCAN_REQUEST_CODE)
    }

    override fun hasActionBar(): Boolean {
        return true
    }

    override fun getActionBarTitle(): String? {
        return mTitleText
    }

    override fun onCreateDialog(type: Int): Dialog? {
        when (type) {
            DIALOG_CHECK_POSTCODE -> {
                val pd = CustomProgressDialog.create(mContext)
                pd.setCancelable(false)
                pd.setMessage(getMessage(POST_CODE_LOOKUP_MESSAGE))
                pd.isIndeterminate = true
                return pd
            }
            DIALOG_AUTHORISING -> {
                val builder = AlertDialog.Builder(
                    mContext
                )
                builder.setTitle(R.string.authorising_card)
                builder.setMessage(getMessage(AUTHORISING_MESSAGE))
                builder.setCancelable(false)
                return builder.create()
            }
            DIALOG_AUTHORISING_SUCCESS -> {
                val successBuilder = AlertDialog.Builder(
                    mContext
                )
                successBuilder.setCancelable(false)
                successBuilder.setTitle(R.string.authorising_success)
                successBuilder.setMessage(getMessage(AUTHORISATION_SUCCESS_MESSAGE))
                successBuilder.setPositiveButton(
                    R.string.done
                ) { dialogInterface, i ->
                    dismissDialog(DIALOG_AUTHORISING_SUCCESS)
                    val isEligibleForCNC = get().isEligibleForCNC(orderId)
                    val fragment = OrderDetailsPresenter.newInstance(orderId, isEligibleForCNC)
                    fragment.setOnOrderCancelledListener { canceled: Boolean ->
                        if (canceled) {
                            fragment.pop()
                        }
                    }
                    if (getCurrentFragmentManager() != null &&
                        getCurrentFragmentManager()?.backStackEntryCount!! > 1
                    ) {
                        getCurrentFragmentManager()?.popBackStack()
                        getCurrentFragmentManager()?.popBackStack()
                    }
                }
                return successBuilder.create()
            }
            DIALOG_AUTHORISING_FAILED -> {
                val failedBuilder = AlertDialog.Builder(
                    (mContext)
                )
                val customTitle = TextView(mContext)
                customTitle.setText(R.string.authorising_failed)
                customTitle.textSize = 20f
                customTitle.setPadding(55, 42, 0, 0)
                customTitle.gravity = Gravity.LEFT
                customTitle.setTextColor(
                    ContextCompat.getColor(
                        (mContext),
                        R.color.sa_savings_red
                    )
                )
                failedBuilder.setCustomTitle(customTitle)
                failedBuilder.setCancelable(false)
                failedBuilder.setMessage(getMessage(AUTHORISATION_FAILED_MESSAGE))
                failedBuilder.setPositiveButton(
                    R.string.edit_details
                ) { dialogInterface, i -> dismissDialog(DIALOG_AUTHORISING_FAILED) }
                failedBuilder.setNegativeButton(
                    R.string.close,
                    (
                        { dialogInterface, i ->
                            dismissDialog(DIALOG_AUTHORISING_FAILED)
                            val isEligibleForCNC = get().isEligibleForCNC(orderId)
                            val fragment = OrderDetailsPresenter.newInstance(orderId, isEligibleForCNC)
                            fragment.setOnOrderCancelledListener { canceled: Boolean ->
                                if (canceled) {
                                    fragment.pop()
                                }
                            }
                            if ((
                                getCurrentFragmentManager() != null &&
                                    getCurrentFragmentManager()?.backStackEntryCount!! > 1
                                )
                            ) {
                                getCurrentFragmentManager()?.popBackStack()
                                getCurrentFragmentManager()?.popBackStack()
                            }
                        }
                        )
                )
                return failedBuilder.create()
            }
            else -> return super.onCreateDialog(type)
        }
    }

    protected fun getMessage(type: Int): String {
        when (type) {
            POST_CODE_LOOKUP_MESSAGE -> return getString(R.string.looking_up_postcode)
            AUTHORISING_MESSAGE -> return getString(
                R.string.please_wait_while_card,
                cardNumber.rawNumberString.substring(cardNumber.rawNumberString.length - 4)
            )
            AUTHORISATION_SUCCESS_MESSAGE -> return getString(
                R.string.success_authorize_text,
                cardNumber.rawNumberString.substring(cardNumber.rawNumberString.length - 4)
            )
            AUTHORISATION_FAILED_MESSAGE -> return getString(
                R.string.authorise_failed_text,
                cardNumber.rawNumberString.substring(cardNumber.rawNumberString.length - 4)
            )
            else -> return getString(R.string.oops_error)
        }
    }

    private fun loadBillingAddresses() {
        UserAccountDetails.instance
            .loadAddressBook(
                object : AddressBookLoadListener {
                    override fun onAddressBookLoadSuccess(
                        addressBookResponse: AddressBookResponse?
                    ) {
                        if (addressBookResponse != null) {
                            populateAddresses(addressBookResponse.addressBook)
                        }
                    }

                    override fun onAddressBookLoadFailure(error: Int?, data: AddressBookResponse?) {
                        AsdaDialogHelper.displayErrorDialog(data, error, mContext, SCREEN_NAME)
                    }
                })
    }

    private fun populateAddresses(addresses: Array<AddressBookResponse.Address?>?) {
        var addresses = addresses
        if (addresses == null) {
            addresses = arrayOfNulls(0)
        }
        val newAddressList = arrayOfNulls<AddressBookResponse.Address>(addresses.size + 1)
        val addAddress = AddressBookResponse.Address()
        addAddress.address = resources.getString(R.string.add_new_address)
        addAddress.city = null
        addAddress.postalCode = null
        System.arraycopy(addresses, 0, newAddressList, 0, addresses.size)
        newAddressList[addresses.size] = addAddress
        val adapter = ArrayAdapter(
            (mContext), R.layout.address_simple_spinner_item, newAddressList
        )
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        mAddressSpinner.adapter = adapter
        mAddressSpinner.visibility = View.VISIBLE
        mAddressSpinner.setSelection(0)
        mAddressSpinner.onItemSelectedListener = object : OnItemSelectedListener {
            override fun onItemSelected(
                parent: AdapterView<*>?,
                view: View,
                position: Int,
                id: Long
            ) {
                if (adapter.count == position + 1) {
                    showDialog(DIALOG_CHECK_POSTCODE)
                } else {
                    postcode.text?.clear()
                    mPostCodeContainer.visibility = View.GONE
                    mChooseAddressSpinner.visibility = View.INVISIBLE
                    mChooseAddressText.visibility = View.INVISIBLE
                    mNewAddressContainer.visibility = View.INVISIBLE
                }
            }

            override fun onNothingSelected(parent: AdapterView<*>?) {
                // No op
            }
        }
    }

    var source = PublishProcessor.create<Boolean>()
    var maestroSource = PublishProcessor.create<Boolean>()

    /**
     * Validate credit card form input, Combine 6 different Observables that track input changes at
     * Credit Card Number, Card Holder Name, Expiry Date, and Billing Address. Once an event change is
     * triggered from all 4 inputs, the result is combined and form is evaluated for validity.
     */
    private fun combineLatestEvents(view: View?) {
        val disposableObserver: DisposableSubscriber<Boolean?> =
            object : DisposableSubscriber<Boolean?>() {
                override fun onNext(p0: Boolean?) {
                    Log.d(
                        TAG,
                        "Form is Valid: $p0"
                    )
                }

                override fun onError(t: Throwable) {
                    Log.d(TAG, "Form has Errors", t)
                }

                override fun onComplete() {
                    Log.d(TAG, " Form onComplete")
                }
            }
        Flowable.combineLatest(
            mCardHolderNameObservable,
            mCardNumberObservable,
            mMaestroCardStartDateObservable,
            mCardExpiryObservable,
            if (mMaestroCvvLabel.visibility == View.VISIBLE) mMaestroCardCvvObservable else mCardCvvObservable,
            Function5 { cardHolderName: CharSequence, cardNumber: CharSequence, startDate: CharSequence, expiryDate: CharSequence, cvv: CharSequence ->
                formValid(
                    cardHolderName,
                    cardNumber,
                    startDate,
                    expiryDate,
                    cvv
                )
            }
        )
            .doOnNext { aBoolean: Boolean ->
                if (aBoolean) {
                    clearErrorLabels(
                        (mCardHolderNameLabel),
                        (mCardNumberLabel),
                        (mStartDateLabel),
                        (mExpiryDateLabel),
                        (mCardCvvLabel),
                        (mMaestroCvvLabel)
                    )
                    mHasError = false
                    Log.d(TAG, "Form is " + aBoolean)
                    mSaveButton.isEnabled = true
                    mSaveButton.setTextColor(
                        ContextCompat.getColor(
                            (mContext),
                            R.color.asda_blue
                        )
                    )
                    if (view != null) {
                        hideKeyboard(view)
                    }
                } else {
                    Log.d(
                        TAG,
                        "Form Valid " + aBoolean
                    )
                    mHasError = true
                    mSaveButton.setEnabled(false)
                    mSaveButton.setTextColor(
                        ContextCompat.getColor(
                            (mContext),
                            R.color.grey_53
                        )
                    )
                }
            }
            .flatMap { isValidCard: Boolean ->
                Flowable.combineLatest(
                    source,
                    Flowable.just(isValidCard),
                    mAddress1Observable,
                    mAddress2Observable,
                    mCityObservable,
                    Function5 { visibility: Boolean, cardValidity: Boolean?, address1: CharSequence?, address2: CharSequence?, city: CharSequence? ->
                        val address1Valid: Boolean = !TextUtils.isEmpty(address1)
                        if (!address1Valid) {
                            displayErrorMessage(
                                addressLine1,
                                mAddressLine1Label,
                                R.string.error_invalid_address,
                                true
                            )
                            mHasError = true
                        }
                        val address2Valid: Boolean = !TextUtils.isEmpty(address2)
                        if (!address2Valid) {
                            displayErrorMessage(
                                addressLine2,
                                mAddressLine2Label,
                                R.string.error_invalid_address,
                                true
                            )
                            mHasError = true
                        }
                        val cityValid: Boolean = !TextUtils.isEmpty(city)
                        if (!cityValid) {
                            displayErrorMessage(
                                this.city,
                                mCityLabel,
                                R.string.register_ERROR_NO_TOWN_OR_CITY,
                                true
                            )
                            mHasError = true
                        }
                        (
                            visibility &&
                                formValid(
                                    mCardHolderName.text.toString(),
                                    cardNumber.rawNumberString,
                                    mStartDate.text.toString(),
                                    mExpiryDate.text.toString(),
                                    mCardCvv.text.toString()
                                ) &&
                                address1Valid &&
                                address2Valid &&
                                cityValid
                            )
                    }
                )
            }
            .doOnNext { aBoolean: Boolean ->
                if (aBoolean) {
                    clearErrorLabels(
                        (mAddressLine1Label),
                        (mAddressLine2Label),
                        (mPostcodeLabel),
                        (mCityLabel)
                    )
                    mHasError = false
                    Log.d(TAG, "Form is " + aBoolean)
                    mSaveButton.isEnabled = true
                    mSaveButton.setTextColor(
                        ContextCompat.getColor(
                            (mContext),
                            R.color.asda_blue
                        )
                    )
                    if (view != null) {
                        hideKeyboard(view)
                    }
                } else {
                    Log.d(
                        TAG,
                        "Form Valid " + aBoolean
                    )
                    mHasError = true
                    mSaveButton.setEnabled(false)
                    mSaveButton.setTextColor(
                        ContextCompat.getColor(
                            (mContext),
                            R.color.grey_53
                        )
                    )
                }
            }
            .subscribe(disposableObserver)
        safeAdd(disposableObserver)
    }

    private fun formValid(
        cardHolderName: CharSequence,
        cardNumber: CharSequence,
        startDate: CharSequence,
        expiryDate: CharSequence,
        cvv: CharSequence
    ): Boolean {
        val cardHolderNameValid = !TextUtils.isEmpty(cardHolderName) || cardHolderName.toString()
            .trim { it <= ' ' }.isEmpty()
        if (!cardHolderNameValid) {
            displayErrorMessage(
                mCardHolderName, mCardHolderNameLabel, R.string.error_no_cardholder_name, true
            )
            mHasError = true
        }
        if (TextUtils.isEmpty(cardNumber)) {
            displayErrorMessage(
                this.cardNumber,
                mCardNumberLabel,
                R.string.error_no_card_number,
                true
            )
            mHasError = true
        }
        val cardNumberValid = isValidCardNumber(cardNumber.toString())
        if (!cardNumberValid) {
            displayErrorMessage(
                this.cardNumber,
                mCardNumberLabel,
                R.string.error_invalid_card_number,
                true
            )
            mHasError = true
        }
        if (TextUtils.isEmpty(startDate)) {
            displayErrorMessage(mStartDate, mStartDateLabel, R.string.error_no_start_date, true)
            mHasError = true
        }
        val startDateValid = (startDate == DUMMY_DATE) || isValidStartDate(startDate.toString())
        if (!startDateValid) {
            displayErrorMessage(
                mStartDate,
                mStartDateLabel,
                R.string.error_invalid_start_date,
                true
            )
            mHasError = true
        }
        if (TextUtils.isEmpty(expiryDate)) {
            displayErrorMessage(mExpiryDate, mExpiryDateLabel, R.string.error_no_expire_date, true)
            mHasError = true
        }
        val expirationDateValid = isValidDate(expiryDate.toString())
        if (!expirationDateValid) {
            displayErrorMessage(
                mExpiryDate, mExpiryDateLabel, R.string.error_invalid_expiration_date, true
            )
            mHasError = true
        }
        val cvvValid = validateCsc(cvv.toString(), getCardType(cardNumber.toString()).cardtype)
        if (mMaestroCvvLabel.visibility == View.VISIBLE) {
            if (TextUtils.isEmpty(cvv)) {
                displayErrorMessage(mMaestroCvv, mMaestroCvvLabel, R.string.error_no_cvv, true)
                mHasError = true
            }
            if (!cvvValid) {
                displayErrorMessage(mMaestroCvv, mMaestroCvvLabel, R.string.enter_valid_csc, true)
                mHasError = true
            }
        } else {
            if (TextUtils.isEmpty(cvv)) {
                displayErrorMessage(mCardCvv, mCardCvvLabel, R.string.error_no_cvv, true)
                mHasError = true
            }
            if (!cvvValid) {
                displayErrorMessage(mCardCvv, mCardCvvLabel, R.string.enter_valid_csc, true)
                mHasError = true
            }
        }
        return (
            cardHolderNameValid &&
                cardNumberValid &&
                startDateValid &&
                expirationDateValid &&
                cvvValid
            )
    }

    protected fun isValidDate(expiryDate: String?): Boolean {
        if (expiryDate != null && expiryDate.length > 2) {
            val currentYear = currentTimeInUK[Calendar.YEAR]
            val currentMonth = currentTimeInUK[Calendar.MONTH] + 1
            try {
                val expiryDateParsed = parseNewCreditCardDate(expiryDate)
                val month = expiryDateParsed.month
                val year = expiryDateParsed.year
                if (month < 1 || month > 12) {
                    return false
                }
                if (year == currentYear && month < currentMonth) {
                    return false
                }
                if (currentYear > expiryDateParsed.year) {
                    return false
                }
            } catch (e: ParseException) {
                Log.e(
                    TAG,
                    "Error parsing card expiry date $expiryDate"
                )
                return false
            }
            return true
        }
        return false
    }

    protected fun isValidStartDate(startDate: String?): Boolean {
        if (startDate != null && startDate.length > 2) {
            val currentYear = currentTimeInUK[Calendar.YEAR]
            try {
                val startDateParsed = parseNewCreditCardDate(startDate)
                if (currentYear >= startDateParsed.year) {
                    return true
                }
            } catch (e: ParseException) {
                Log.e(
                    TAG,
                    "Error parsing card start date $startDate"
                )
            }
        }
        return false
    }

    private fun displayErrorMessage(
        toRequestFocus: View?,
        errorLabel: TextInputLayout?,
        @StringRes errorMsgId: Int,
        requestFocus: Boolean
    ) {
        errorLabel?.error = getString(errorMsgId)
        if (requestFocus) {
            if (view != null) {
                toRequestFocus?.let { view?.scrollTo(it.left, toRequestFocus.top) }
            }
        }
    }

    private fun clearErrorLabels(vararg textInputLayouts: TextInputLayout) {
        for (layout: TextInputLayout in textInputLayouts) {
            layout.error = null
        }
    }

    fun setPostCodeError(message: String?) {
        mPostcodeLabel.error = message
        mHasError = true
    }

    fun setChooseAddressSpinner(
        addressSuggestions: Array<AddressSuggestion>?
    ) {
        mChooseAddressSpinner.adapter =
            RegisterHelper.createAddressSpinnerAdapter(mContext, addressSuggestions)
        mChooseAddressSpinner.setSelection(1)
        mChooseAddressSpinner.onItemSelectedListener = object : OnItemSelectedListener {
            override fun onItemSelected(
                arg0: AdapterView<*>?,
                view: View,
                position: Int,
                id: Long
            ) {
                mNewAddressContainer.visibility = if (position == 0) View.VISIBLE else View.GONE
                if (position == 0) {
                    mSaveButton.isEnabled = false
                    mSaveButton.setTextColor(
                        ContextCompat.getColor(
                            (mContext),
                            R.color.grey_53
                        )
                    )
                    source.onNext(mNewAddressContainer.visibility == View.VISIBLE)
                } else if ((position - 1) >= 1 && (position - 1) < addressSuggestions?.size!!) {
                    // -1 As it is using the addressSuggestions Response data not from adapter.
                    // Adapter is only to display string.
                    fillAddressSuggestions(addressSuggestions.get(position - 1))
                }
            }

            override fun onNothingSelected(arg0: AdapterView<*>?) {
                // No op
            }
        }
    }

    private fun fillAddressSuggestions(addressSuggestion: AddressSuggestion) {
        mAddress = addressSuggestion.address
        mTownOrCity = addressSuggestion.town
        mPostCode = postcode.text.toString()
    }

    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<String>,
        grantResults: IntArray
    ) {
        if (grantResults.size != 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
            startCardIOActivity()
        } else {
            AlertDialog.Builder((mContext))
                .setMessage(R.string.no_permission_for_camera)
                .setPositiveButton(R.string.ok, null)
                .show()
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == SCAN_REQUEST_CODE && data != null) {
            val scanResult: CreditCard = data.getParcelableExtra(CardIOActivity.EXTRA_SCAN_RESULT)
            populateScannedCardFields(scanResult)
        }
    }

    private fun populateScannedCardFields(scanResult: CreditCard?) {
        if (scanResult != null) {
            cardNumber.setText(scanResult.cardNumber)
            if (!TextUtils.isEmpty(scanResult.cardholderName)) {
                mCardHolderName.setText(scanResult.cardholderName)
            }
            if (scanResult.isExpiryValid) {
                val currentYear = currentTimeInUK[Calendar.YEAR]
                val expiryDate =
                    (scanResult.expiryMonth - 1).toString() + "/" + (scanResult.expiryYear - currentYear)
                mExpiryDate.setText(expiryDate)
            } else if (scanResult.expiryMonth != 0 && scanResult.expiryYear != 0) {
                // Expiry date detected but card is expired
                displayErrorMessage(
                    mExpiryDate,
                    mExpiryDateLabel,
                    R.string.add_card_error_expired,
                    true
                )
            }
        }
    }

    override fun onResume() {
        super.onResume()
        mHandler.postDelayed(
            {
                if (mPaResReceived) {
                    view?.let { hideKeyboard(it) }
                    addNewPaymentCardCxo(
                        view,
                        true,
                        MODIFY_ORDER_CXO_ID2
                    )
                }
            },
            DELAY_SECONDS.toLong()
        )
    }

    fun showNewAddressView(view: View?, hideChooseAddressSpinner: Boolean) {
        if (hideChooseAddressSpinner) {
            mChooseAddressText.visibility = View.GONE
            mChooseAddressSpinner.visibility = View.GONE
            mNewAddressContainer.visibility = View.VISIBLE
        } else {
            mChooseAddressText.visibility = View.VISIBLE
            mChooseAddressSpinner.visibility = View.VISIBLE
            mNewAddressContainer.visibility = View.VISIBLE
        }
        combineLatestEvents(view)
    }

    override fun showDialog(type: Int) {
        val ft = activity?.supportFragmentManager?.let {
            getASDAFragmentTransaction(
                it
            )
        }
        val previousDialog = requireActivity().supportFragmentManager.findFragmentByTag("dialog")
        if (previousDialog != null) {
            ft?.remove(previousDialog)
        }
        ft?.addToBackStack(null)
        when (type) {
            DIALOG_CHECK_POSTCODE -> {
                var postcodeDialogFrag: CheckPostcodeDialogFragment? = null
                postcodeDialogFrag = newInstance()
                postcodeDialogFrag.setPostcodeCheckedListener(this, SCREEN_NAME)
                if (ft != null) {
                    postcodeDialogFrag.show(ft, "dialog")
                }
            }
            else -> super.showDialog(type)
        }
    }

    override fun onPostcodeCheckSuccess(
        addressSuggestions: Array<AddressSuggestion?>?,
        postcode: String?
    ) {
        mCardCvv.setText("")
        mMaestroCvv.setText("")
        mExpiryDate.setText("")
        val addressFragment = BaseAddEditAddressFragment.getAddAddressFragment(
            (postcode.toString()),
            true,
            null,
            addressSuggestions,
            null,
            null,
            null,
            false,
            true,
            AddressBookResponse.Type.ShipTo.toString(),
            false
        )
        addressFragment.setAddressBookChangedListener(this)
        push(addressFragment)
    }

    override fun onAddressBookChanged(
        addressBookResponse: AddressBookResponse?,
        addedAddress: AddressBookResponse.Address?
    ) {
        if (isAdded) {
            UserAccountDetails.instance.setAddressBookResponse(addressBookResponse)
            if (addressBookResponse != null) {
                populateAddresses(addressBookResponse.addressBook)
            }
            mAddressSpinner.setSelection(0)
        }
    }

    companion object {
        val TAG = AddPaymentCardFragment::class.java.simpleName
        private val SCREEN_NAME = "Payment : Add Card"
        private val KEY_ORDER_ID = "order_id"
        private val DUMMY_DATE = "11/2017"
        private val DIALOG_CHECK_POSTCODE = 1
        val DIALOG_AUTHORISING = 2
        val DIALOG_AUTHORISING_SUCCESS = 3
        val DIALOG_AUTHORISING_FAILED = 4
        protected val SCAN_REQUEST_CODE = 1001
        private val DELAY_SECONDS = 100
        private val ORDER_TOTAL = "order_total"
        val POST_CODE_LOOKUP_MESSAGE = 1
        val AUTHORISING_MESSAGE = 2
        val AUTHORISATION_SUCCESS_MESSAGE = 3
        val AUTHORISATION_FAILED_MESSAGE = 4
        fun newInstance(orderId: String?): AddPaymentCardFragment {
            val fragment = AddPaymentCardFragment()
            val args = Bundle()
            args.putString(KEY_ORDER_ID, orderId)
            fragment.arguments = args
            return fragment
        }

        fun newInstanceCXO(orderId: String?, orderTotal: Double?): AddPaymentCardFragment {
            val fragment = AddPaymentCardFragment()
            val args = Bundle()
            args.putString(KEY_ORDER_ID, orderId)
            if (orderTotal != null) {
                args.putDouble(ORDER_TOTAL, orderTotal)
            }
            fragment.arguments = args
            return fragment
        }
    }
}
