package com.asda.android.app.orders

import android.content.Context
import android.os.Bundle
import android.os.Parcel
import android.os.Parcelable
import android.text.TextUtils
import android.text.format.DateUtils
import android.view.LayoutInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import androidx.databinding.Bindable
import androidx.databinding.Observable
import androidx.databinding.Observable.OnPropertyChangedCallback
import androidx.fragment.app.FragmentActivity
import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.app.account.view.AccountFragment
import com.asda.android.app.account.view.BaseAccountActivity
import com.asda.android.app.dp.view.ActiveDPTabbedFragment.Companion.newInstance
import com.asda.android.app.dp.view.DeliveryPassPurchaseActivity
import com.asda.android.app.main.view.AsdaActivity
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.databinding.DpOrderDetailsBinding
import com.asda.android.restapi.service.NetworkCallback
import com.asda.android.restapi.service.RestUtils
import com.asda.android.restapi.service.data.DeliveryPass
import com.asda.android.restapi.service.data.ViewOrderResponse
import com.asda.android.service.base.AsdaServiceFactory.get
import com.asda.android.service.base.Authentication
import com.asda.android.utils.Utils
import com.asda.android.utils.Utils.addPoundIfNecessary
import com.asda.android.utils.Utils.getTimeInMillisFromIso
import com.asda.android.utils.Utils.informAboutTooLargePayload
import com.asda.android.utils.Utils.removeTrailingZeros
import java.text.DateFormat
import java.text.ParseException
import java.text.SimpleDateFormat
import java.util.*

class DPOrderFragment() : FeaturedFragment() {
    private var mDPModel: DPOrderViewModel? = null
    override fun loadBundleData(bundle: Bundle?) {
        if (bundle != null && bundle.containsKey(DP_MODEL_ARG)) {
            mDPModel = bundle.getParcelable(DP_MODEL_ARG)
            if (mDPModel != null) {
                mDPModel?.attach(context, this)
            }
        }
    }

    override fun onSaveInstanceState(outState: Bundle) {
        super.onSaveInstanceState(outState)
        informAboutTooLargePayload(outState, TAG)
        if (mDPModel != null) {
            outState.putParcelable(DP_MODEL_ARG, mDPModel)
        }
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        pageViewEvent =
            PageViewEvent("dp order details", Anivia.SECTION_ACCOUNT, Anivia.SECTION_ACCOUNT)
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        val binding: DpOrderDetailsBinding =
            DpOrderDetailsBinding.inflate(inflater, container, false)
        if (RestUtils.isTablet(context) && parentFragment is AccountFragment) {
            val params: ViewGroup.LayoutParams = binding.content.layoutParams
            params.width = ViewGroup.LayoutParams.WRAP_CONTENT
            binding.content.layoutParams = params
        }
        binding.model = mDPModel
        get()
            .viewOrder(
                mDPModel?.orderId,
                object : NetworkCallback<ViewOrderResponse?>() {
                    override fun onFailure(error: Int, data: ViewOrderResponse?) {
                        if (isAdded) {
                            showDialog(error, data)
                        }
                    }

                    override fun onSuccess(data: ViewOrderResponse?) {
                        if (isAdded && (data?.orders != null) && (data.orders?.isNotEmpty() == true)) {
                            mDPModel = data.orders?.get(0)?.let { DPOrderViewModel(it) }
                            mDPModel?.attach(context, this@DPOrderFragment)
                            binding.model = mDPModel
                        }
                    }
                }
            )
        return binding.root
    }

    override fun hasActionBar(): Boolean {
        val activity: FragmentActivity? = activity
        return (
            activity is AsdaActivity ||
                activity is DeliveryPassPurchaseActivity ||
                activity is BaseAccountActivity ||
                parentFragment is AccountFragment
            )
    }

    override fun getActionBarTitle(): String {
        return if (activity != null) getString(R.string.delivery_pass) else ""
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == android.R.id.home) {
            pop()
            return true
        }
        return false
    }

    override fun getInternalTag(): String {
        return TAG
    }

    class DPOrderViewModel : Parcelable, Observable {
        private val name: String?
        private val startDate: String?
        private val endDate: String?
        private val isAutoRenew: String?
        private var autoRenewDate: String? = null
        val creditCardType: String?
        val creditCardNumber: String?

        @get:Bindable
        val orderId: String?
        var price: String?
        val durationInMonths: Int
        private var mContext: Context? = null
        private var mFragment: DPOrderFragment? = null

        internal constructor(`in`: Parcel) {
            name = `in`.readString()
            startDate = `in`.readString()
            endDate = `in`.readString()
            isAutoRenew = `in`.readString()
            autoRenewDate = `in`.readString()
            creditCardType = `in`.readString()
            creditCardNumber = `in`.readString()
            orderId = `in`.readString()
            price = `in`.readString()
            durationInMonths = `in`.readInt()
        }

        internal constructor(order: ViewOrderResponse.Order) {
            creditCardNumber = order.creditCardNumber
            creditCardType = order.creditCardType
            name = order.dpPromotionName
            endDate = order.expirationDate
            startDate = order.orderedDate
            isAutoRenew = order.isAutoRenewOrder
            orderId = order.orderId
            price = order.dpPromotionPrice
            durationInMonths = (
                (
                    (
                        getTimeInMillisFromIso(order.expirationDateISO) -
                            getTimeInMillisFromIso(order.orderedDateISO)
                        ) /
                        (DateUtils.DAY_IN_MILLIS * 30)
                    )
                ).toInt()
        }

        fun attach(context: Context?, fragment: DPOrderFragment?) {
            mContext = context
            mFragment = fragment
        }

        override fun describeContents(): Int {
            return 0
        }

        override fun writeToParcel(dest: Parcel, flags: Int) {
            dest.writeString(name)
            dest.writeString(startDate)
            dest.writeString(endDate)
            dest.writeString(isAutoRenew)
            dest.writeString(autoRenewDate)
            dest.writeString(creditCardType)
            dest.writeString(creditCardNumber)
            dest.writeString(orderId)
            dest.writeString(price)
            dest.writeInt(durationInMonths)
        }

        @get:Bindable
        val title: String
            get() = stripPriceFromName(name)

        @get:Bindable
        val validityText: String
            get() = mContext?.getString(
                R.string.dp_order_validity, parseDate(startDate), parseDate(endDate)
            ).toString()

        @get:Bindable
        val subTitle: String?
            get() {
                val priceStr: String? = addPoundIfNecessary(removeTrailingZeros(price))
                return mContext
                    ?.resources
                    ?.getQuantityString(
                        if (isAutoRenew()) R.plurals.dp_order_price_monthly else R.plurals.dp_order_price_one_payment,
                        durationInMonths,
                        priceStr,
                        durationInMonths
                    )
            }

        @get:Bindable
        val cardDetails: String
            get() {
                var cardDetails: String = mContext?.getString(R.string.unavailable).toString()
                if (!TextUtils.isEmpty(creditCardType)) {
                    if (mContext != null) {
                        cardDetails = mContext?.getString(
                            R.string.dp_card_details, creditCardType,
                            getCardEnding(
                                (mContext!!)
                            )
                        ).toString()
                    }
                }
                return cardDetails
            }

        private fun getCardEnding(context: Context): String {
            if (!TextUtils.isEmpty(creditCardNumber)) {
                return creditCardNumber?.substring(
                    if (creditCardNumber.length >= 4) creditCardNumber.length - 4 else 0
                ).toString()
            }
            return context.getString(R.string.unavailable)
        }

        private fun isAutoRenew(): Boolean {
            return (
                "true".equals(isAutoRenew, ignoreCase = true) &&
                    !TextUtils.isEmpty(autoRenewDate) &&
                    (getTime(autoRenewDate) > System.currentTimeMillis())
                )
        }

        @get:Bindable
        val orderedDate: String
            get() {
                val fullDateFormat = SimpleDateFormat("EEE dd MMM yyyy", Locale.UK)
                fullDateFormat.timeZone = Utils.UK.TIME_ZONE
                return fullDateFormat.format(Date(getTime(startDate)))
            }

        @get:Bindable
        val viewDetailsVisibility: Int
            get() {
                return if (isActiveDP) View.VISIBLE else View.GONE
            }
        val isActiveDP: Boolean
            get() {
                val deliveryPass: DeliveryPass? =
                    (Authentication.getInstance() as Authentication).getActiveDeliveryPass()
                if (deliveryPass != null) {
                    if (deliveryPass.name != null && (deliveryPass.name == name)) {
                        return getTime(endDate) > System.currentTimeMillis()
                    }
                }
                return false
            }

        fun onClickViewDetails(view: View?) {
            mFragment?.push(newInstance())
        }

        override fun addOnPropertyChangedCallback(onPropertyChangedCallback: OnPropertyChangedCallback) {
            // No op
        }

        override fun removeOnPropertyChangedCallback(
            onPropertyChangedCallback: OnPropertyChangedCallback
        ) {
            // No op
        }

        companion object {
            private val sPossibleDateFormats: Array<SimpleDateFormat> = arrayOf(
                SimpleDateFormat("EEEE dd MMM yyyy", Locale.UK),
                SimpleDateFormat("EEE dd MMM yyyy", Locale.UK),
                SimpleDateFormat("yyyy-MM-dd HH:mm:ss.S", Locale.UK)
            )
            val sInputFormatter: DateFormat = SimpleDateFormat("dd-MMM-yyyy", Locale.UK)
            @JvmField val CREATOR: Parcelable.Creator<DPOrderViewModel?> =
                object : Parcelable.Creator<DPOrderViewModel?> {
                    override fun createFromParcel(source: Parcel?): DPOrderViewModel? {
                        return source?.let { DPOrderViewModel(it) }
                    }

                    override fun newArray(size: Int): Array<DPOrderViewModel?> {
                        return arrayOfNulls(size)
                    }
                }

            private fun stripPriceFromName(name: String?): String {
                return if (name == null) "" else name.replace("\\(£.*\\)".toRegex(), "")
                    .trim { it <= ' ' }
            }

            private fun parseDate(date: String?): String {
                return if (date != null) sInputFormatter.format(Date(getTime(date))) else " "
            }

            private fun getTime(date: String?): Long {
                var time: Long = 0
                for (format: SimpleDateFormat in sPossibleDateFormats) {
                    try {
                        time = format.parse(date).time
                        break
                    } catch (ignore: ParseException) {
                        // No op
                    }
                }
                return time
            }
        }
    }

    companion object {
        private val DP_MODEL_ARG: String = "dp_model_arg"
        val TAG: String = "DPOrderFragment"
        fun newInstance(order: ViewOrderResponse.Order): DPOrderFragment {
            val fragment = DPOrderFragment()
            val bundle = Bundle()
            bundle.putParcelable(DP_MODEL_ARG, DPOrderViewModel(order))
            fragment.arguments = bundle
            return fragment
        }
    }
}
