package com.asda.android.app.orders

import android.app.Dialog
import android.content.Context
import android.os.Bundle
import android.os.Handler
import android.text.InputFilter
import android.text.InputFilter.LengthFilter
import android.text.TextUtils
import android.util.Log
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.CompoundButton
import android.widget.LinearLayout
import android.widget.TextView
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.widget.AppCompatRadioButton
import androidx.core.content.ContextCompat
import androidx.core.util.Pair
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProviders
import com.asda.android.R
import com.asda.android.base.core.view.OnSingleClickListener
import com.asda.android.base.core.view.WrappedDrawable
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.base.core.view.ui.AsdaDialogHelper
import com.asda.android.base.core.view.ui.BaseAsdaDialogHelper
import com.asda.android.base.core.view.ui.DialogHelper
import com.asda.android.cxo.CartManager
import com.asda.android.cxo.CartManager.Companion.instance
import com.asda.android.orders.constants.OrderConstants.MODIFY_ORDER_CXO_ID1
import com.asda.android.orders.constants.OrderConstants.MODIFY_ORDER_CXO_ID2
import com.asda.android.orders.constants.OrderConstants.MODIFY_ORDER_EDIT_CARD
import com.asda.android.payment.threeds.ThreeDsDataSource
import com.asda.android.payment.threeds.ThreeDsDataSource.ThreeDsResult
import com.asda.android.payment.threeds.ThreeDsUtils
import com.asda.android.payment.threeds.ThreeDsUtils.trackThreeDSEventCXO
import com.asda.android.restapi.service.base.RxFailure
import com.asda.android.restapi.service.constants.AsdaServiceSettingsConstants.debug
import com.asda.android.restapi.service.constants.CreditCardTypeConstants
import com.asda.android.restapi.service.data.*
import com.asda.android.restapi.service.data.PaymentDetailsResponse.PaymentCards
import com.asda.android.restapi.service.data.checkout.CheckoutResponse
import com.asda.android.service.base.Authentication
import com.asda.android.singleprofile.base.SingleProfile
import com.asda.android.singleprofile.viewmodel.PaymentCardsViewModel
import com.asda.android.utils.Utils
import com.asda.android.utils.Utils.isThreeDs2XEnabled
import com.asda.android.utils.view.ViewUtil
import com.asda.android.utils.view.ViewUtil.hideKeyboard
import com.google.android.material.textfield.TextInputEditText
import com.google.android.material.textfield.TextInputLayout
import com.jakewharton.rxbinding2.widget.RxTextView
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.observers.DisposableObserver
import io.reactivex.observers.ResourceSingleObserver
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit

class EditPaymentCardFragment() : FeaturedFragment() {
    private lateinit var mContext: Context
    private lateinit var mCardsGroup: LinearLayout
    private lateinit var mEdtCvvLabel: TextInputLayout
    private lateinit var mEdtCvv: TextInputEditText
    private lateinit var mSaveBtn: Button
    private var mSelectedCardType: String? = null
    private var mOrderId: String? = null
    private var mPurchaseOrderId: String? = null
    private var mOrderData: WismoOrderResponse.Payload? = null
    private lateinit var mViewModel: AddEditPaymentCardViewModel
    private var mTitleText: String? = null
    private var mCardCvv: String? = null
    private var deviceChannel = ""
    private val mHandler = Handler()
    private var mOrderTotal: Double? = null
    private var mCardNumber: String? = null
    private var mPaymentCards: List<PaymentCards>? = null
    private var mCards: Array<PaymentCards>? = null
    private var mPaResReceived = false
    private var mPaRes: String? = null
    private var mPaymentCard: PaymentCards? = null
    private val mRadioButtons: ArrayList<Pair<LinearLayout, AppCompatRadioButton>> = ArrayList()
    private val threeDsDataSource = ThreeDsDataSource()
    var selectedId = -1

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        val view = inflater.inflate(R.layout.edit_payment_method, container, false)
        setupView(view)
        wireListeners(view)
        return view
    }

    override fun loadBundleData(bundle: Bundle?) {
        super.loadBundleData(bundle)
        if (bundle != null) {
            mOrderId = bundle.getString(KEY_ORDER_ID)
            mOrderData = bundle.getParcelable(KEY_ORDER_DATA)
            if (bundle.containsKey(ORDER_TOTAL)) {
                mOrderTotal = bundle.getDouble(ORDER_TOTAL)
            }
        }
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        mContext = context
        mTitleText = mContext.getString(R.string.payment_method)
        mViewModel = ViewModelProviders.of((requireActivity()))[AddEditPaymentCardViewModel::class.java]
    }

    fun setTitleText(titleText: String?) {
        mTitleText = titleText
    }

    private val cardsObservable: Observable<PaymentDetailsResponse>?
        get() {
            var observable: Observable<PaymentDetailsResponse>? = null
            if (SingleProfile.getProfileResponse() != null && SingleProfile.getProfileResponse()?.profileId != null) {
                val model = PaymentCardsViewModel(SingleProfile.getService())
                observable =
                    model.getPaymentCardsBasic(SingleProfile.getProfileResponse()?.profileId!!)
            }
            return observable
        }

    private fun setupView(view: View) {
        mCardsGroup = ViewUtil.findViewById<LinearLayout>(view, R.id.group)
        mSaveBtn = ViewUtil.findViewById<Button>(view, R.id.save_button)
        mSaveBtn.isEnabled = false
        if (mPaymentCards == null) {
            val getCardsObservable =
                cardsObservable
            if (getCardsObservable != null) {
                safeAdd(
                    getCardsObservable
                        .subscribeOn(Schedulers.io())
                        .observeOn(AndroidSchedulers.mainThread())
                        .subscribe(
                            { response: PaymentDetailsResponse ->
                                if (response.cards != null) {
                                    ViewUtil.findViewById<View>(
                                        view,
                                        R.id.loading
                                    ).setVisibility(View.GONE)
                                    mCards = response.cards
                                    populateCreditCards(response.cards, mOrderData, view)
                                    wireListeners(view)
                                }
                            },
                            { throwable: Throwable? ->
                                if (throwable is RxFailure) {
                                    val error: RxFailure = throwable
                                    BaseAsdaDialogHelper.displayErrorDialog(
                                        error.asdaResponse,
                                        error.integer,
                                        getActivity(),
                                        SCREEN_NAME
                                    )
                                } else {
                                    try {
                                        BaseAsdaDialogHelper.displayErrorDialog(
                                            null,
                                            -1,
                                            getActivity(),
                                            SCREEN_NAME
                                        )
                                    } catch (ignore: Exception) {
                                        // Do nothing
                                    }
                                }
                            }
                        )
                )
            }
        } else {
            ViewUtil.findViewById<View>(view, R.id.loading).visibility =
                View.GONE
            populateCreditCards(mCards, mOrderData, view)
        }
    }

    private fun setupCvvObservable(
        view: View,
        cvvEdt: TextInputEditText,
        cvvLabel: TextInputLayout
    ) {
        val mEditCvvObservable = RxTextView.textChanges(cvvEdt)
            .skip(3)
            .debounce(300, TimeUnit.MILLISECONDS, AndroidSchedulers.mainThread())
        mEditCvvObservable.subscribe(
            object : DisposableObserver<CharSequence?>() {
                override fun onNext(cvv: CharSequence) {
                    if (Utils.validateCsc(cvv.toString(), mSelectedCardType)) {
                        mSaveBtn.isEnabled = true
                        hideKeyboard(view)
                        cvvLabel.error = null
                        mCardCvv = cvv.toString()
                    } else {
                        mSaveBtn.isEnabled = false
                        cvvLabel.error = getString(R.string.enter_valid_csc)
                        ViewUtil.findViewById<View>(view, R.id.edtCvv_text).visibility =
                            View.GONE
                    }
                    if (CreditCardTypeConstants.TYPE_AMEX == mSelectedCardType) {
                        cvvEdt.filters = arrayOf<InputFilter>(LengthFilter(4))
                        cvvEdt.filters = arrayOf<InputFilter>(LengthFilter(4))
                    } else {
                        cvvEdt.filters = arrayOf<InputFilter>(LengthFilter(3))
                    }
                }

                override fun onError(throwable: Throwable) {
                    // No op
                }

                override fun onComplete() {
                    // No op
                }
            })
    }

    private fun wireListeners(view: View) {
        mSaveBtn.setOnClickListener(
            object : OnSingleClickListener(this) {
                override fun onSingleClick(v: View) {
                    showDialog(DIALOG_AUTHORISING)
                    if (mPaymentCard != null && mPaymentCard?.cardBrand != null) {
                        if (isThreeDs2XEnabled(
                                mContext,
                                mPaymentCard?.cardBrand!!
                            )
                        ) {
                            createJwtCXO(view)
                        } else {
                            editPaymentCardCXO(
                                view,
                                false,
                                MODIFY_ORDER_CXO_ID1
                            )
                        }
                    }
                }
            })
        ViewUtil.findViewById<View>(view, R.id.add_label)
            .setOnClickListener(
                object : OnSingleClickListener(this) {
                    override fun onSingleClick(v: View) {
                        push(AddPaymentCardFragment.newInstanceCXO(mOrderId, mOrderTotal))
                    }
                })
        ViewUtil.findViewById<View>(view, R.id.cancel_button).setOnClickListener(
            { v: View? -> pop() })
    }

    private fun createJwtCXO(view: View) {
        if (mPaymentCard != null) {
            if (mPaymentCard?.expired == true) {
                DialogHelper.displayErrorDialog(
                    mContext.getString(R.string.selected_card_expired), mContext, SCREEN_NAME
                )
            } else {
                val createJwtPaymentDetails = CreateJwtPaymentDetails()
                createJwtPaymentDetails.creditCardNickName = mPaymentCard?.cardId
                val generateJwtRequest = GenerateJwtRequest(
                    CreateJwtDataRequest(mOrderId, mOrderTotal, createJwtPaymentDetails)
                )
                mViewModel
                    .generateJwtCXO(generateJwtRequest)
                    ?.subscribe(
                        object : ResourceSingleObserver<GenerateJwtResponse?>() {
                            override fun onSuccess(generateJwtResponse: GenerateJwtResponse) {
                                if (generateJwtResponse.data.jwt != null &&
                                    !TextUtils.isEmpty(generateJwtResponse.data.jwt)
                                ) {
                                    threeDsDataSource.initializeCardinal(
                                        generateJwtResponse.data.jwt!!, Utils.isDebug
                                    )
                                }
                                threeDsDataSource
                                    .getInitCardinalLiveData()
                                    .observe(
                                        viewLifecycleOwner,
                                        Observer { value: Boolean? ->
                                            editPaymentCardCXO(
                                                view,
                                                false,
                                                MODIFY_ORDER_CXO_ID1
                                            )
                                        }
                                    )
                            }

                            override fun onError(e: Throwable) {
                                editPaymentCardCXO(
                                    view, false, MODIFY_ORDER_CXO_ID1
                                )
                            }
                        })
            }
        }
    }

    private fun editPaymentCardCXO(view: View?, confirm3ds: Boolean, type: String) {
        showDialog(DIALOG_AUTHORISING)
        if (ThreeDsUtils.isCardinalInitSuccess) {
            deviceChannel = ThreeDsUtils.DEVICE_CHANNEL
        }
        val data = mPaymentCard?.let {
            mCardCvv?.let { it1 ->
                ModifyOrderCXOData(
                    it, it1, false, false, mOrderId, mPurchaseOrderId, null,
                    null,
                    null,
                    null,
                    null,
                    Authentication.getInstance().csrfToken,
                    mPaRes,
                    mOrderTotal,
                    mContext,
                    it.cardId,
                    MODIFY_ORDER_EDIT_CARD,
                    type
                )
            }
        }
        if (data != null) {
            mViewModel.makeCXOModifyOrderCall(data, confirm3ds)
                .subscribe(getModifyOrderObserver(view))
        }
    }

    private fun getModifyOrderObserver(view: View?): ResourceSingleObserver<CheckoutResponse?> {
        return object : ResourceSingleObserver<CheckoutResponse?>() {
            override fun onSuccess(checkoutResponse: CheckoutResponse) {
                if ((
                    checkoutResponse.data != null
                    ) && (
                        checkoutResponse.data?.purchaseOrder != null
                        ) && (
                        checkoutResponse.data?.purchaseOrder?.threeDsVerification != null
                        ) && mViewModel.paymentHelper.is3DSRequired(checkoutResponse)
                ) {
                    val cxoCartManager: CartManager = instance
                    if ((cxoCartManager.orderId != null) && (cxoCartManager.authentication.singleProfileId != null) && threeDsDataSource.isThreeDsSDKVersion(
                            checkoutResponse
                                .data!!
                                .purchaseOrder!!
                                .threeDsVerification!!
                                .version
                        )
                    ) {
                        trackThreeDSEventCXO(
                            checkoutResponse
                                .data!!
                                .purchaseOrder!!
                                .threeDsVerification!!
                                .version,
                            TAG,
                            cxoCartManager.orderId,
                            cxoCartManager.authentication.singleProfileId!!,
                            cxoCartManager.cartId
                        )
                    }
                    mPurchaseOrderId = checkoutResponse.data?.purchaseOrder?.purchaseOrderId
                    val cardType: String? = checkoutResponse
                        .data!!
                        .purchaseOrder!!
                        .paymentDetails
                        ?.get(0)
                        ?.cardType
                    threeDsDataSource.doThreeDsSecureCXOCheckout(
                        (mContext),
                        checkoutResponse.data!!.purchaseOrder!!.threeDsVerification,
                        cardType,
                        isThreeDs2XEnabled((mContext), (cardType)!!)
                    )
                    threeDsDataSource
                        .getThreeDsLiveData()
                        .observe(
                            viewLifecycleOwner,
                            Observer { threeDsResponse: ThreeDsResult ->
                                if (debug) {
                                    Log.i(
                                        TAG,
                                        "Modify Order CXO" + threeDsResponse.status
                                    )
                                }
                                if ((
                                    threeDsResponse
                                        .status
                                        == mContext.getString(R.string.threeDsSuccess)
                                    )
                                ) {
                                    if ((
                                        threeDsResponse.paRes != null &&
                                            !TextUtils.isEmpty(threeDsResponse.paRes)
                                        )
                                    ) {
                                        // showDialog(DIALOG_AUTHORISING);
                                        if (debug) {
                                            Log.i(
                                                TAG,
                                                "Edit payment card CXO PaRes received"
                                            )
                                        }
                                        mPaRes = threeDsResponse.paRes
                                    }
                                    mPaResReceived = true
                                } else if ((
                                    threeDsResponse
                                        .status
                                        == mContext.getString(R.string.threeDsFailed)
                                    )
                                ) {
                                    if ((
                                        debug &&
                                            (
                                                threeDsResponse.errorNumber != null
                                                ) && (threeDsResponse.errorDescription != null)
                                        )
                                    ) {
                                        Log.i(
                                            TAG,
                                            (
                                                "ErrorNumber: " +
                                                    threeDsResponse.errorNumber +
                                                    " errorDescription: " +
                                                    threeDsResponse.errorDescription
                                                )
                                        )
                                    }
                                    mHandler.postDelayed(
                                        {
                                            if (view != null) hideKeyboard((view))

                                            showDialog(DIALOG_AUTHORISING_FAILED)
                                        },
                                        DELAY_SECONDS.toLong()
                                    )
                                    if ((
                                        threeDsResponse.errorResponse != null
                                        ) && (
                                            threeDsResponse.errorResponse?.firstErrorMessage != null
                                            )
                                    ) {
                                        AsdaDialogHelper.displayErrorDialog(
                                            threeDsResponse.errorResponse,
                                            1000,
                                            mContext,
                                            SCREEN_NAME
                                        )
                                    } else {
                                        showDialog(DIALOG_AUTHORISING_FAILED)
                                    }
                                }
                            }
                        )
                } else {
                    try {
                        dismissDialog(DIALOG_AUTHORISING, true)
                        showDialog(DIALOG_AUTHORISING_SUCCESS)
                    } catch (ignored: Exception) {
                        // Do nothing
                    }
                }
            }

            override fun onError(throwable: Throwable) {
                try {
                    dismissDialog(DIALOG_AUTHORISING, true)
                } catch (ignored: Exception) {
                    // Do nothing
                }
                if (throwable is RxFailure) {
                    val error: RxFailure = throwable
                    if (error.integer != -2) {
                        AsdaDialogHelper.displayErrorDialog(
                            error.asdaResponse, error.integer, activity, SCREEN_NAME
                        )
                    }
                } else {
                    try {
                        showDialog(AddPaymentCardFragment.DIALOG_AUTHORISING_FAILED)
                    } catch (ignore: Throwable) {
                        // Do nothing
                    }
                }
            }
        }
        // TODO: Implement Modify Order CXO
    }

    private fun populateCreditCards(
        paymentCards: Array<PaymentCards>?,
        order: WismoOrderResponse.Payload?,
        view: View
    ) {
        mCardsGroup.removeAllViews()
        mRadioButtons.clear()
        val primaryCards: MutableList<PaymentCards> = ArrayList()
        val secondaryCards: MutableList<PaymentCards> = ArrayList()
        if (order?.payment != null && !TextUtils.isEmpty(order.payment?.orderCardNickName) &&
            order.payment?.orderCardNickName?.contains("temp") == true
        ) {
            val paymentCard = PaymentCards()
            paymentCard.card = order.payment?.orderCardNickName
            paymentCard.expired = true
            paymentCard.cardBrand = order.payment?.cardType
            primaryCards.add(paymentCard)
        }
        if (paymentCards != null) {
            for (paymentCard: PaymentCards in paymentCards) {
                if (paymentCard.associatedToOrder) {
                    primaryCards.add(paymentCard)
                } else {
                    secondaryCards.add(paymentCard)
                }
            }
        }
        primaryCards.addAll(secondaryCards)
        mPaymentCards = primaryCards
        var id = 0
        for (card: PaymentCards in primaryCards) {
            if (!TextUtils.isEmpty(card.getCreditCardNumber())) {
                val container = ViewUtil.inflate<LinearLayout>(
                    view.context, R.layout.custom_radio_button, view as ViewGroup, false
                )
                val radioButton = ViewUtil.findViewById<AppCompatRadioButton>(container, R.id.radio)
                val cardText: CharSequence = card.getCreditCardNumber()
                mCardNumber = cardText.toString()
                radioButton.id = primaryCards.indexOf(card)
                radioButton.tag = card
                radioButton.isEnabled = !card.expired
                val type = Utils.getCardIssuer(card.creditCardType)
                mSelectedCardType = card.creditCardType
                val cardIcon = ContextCompat.getDrawable(mContext, type.cardIcon)
                if (cardIcon != null) {
                    val wrappedDrawable = WrappedDrawable(
                        cardIcon
                    )
                    wrappedDrawable.setBounds(
                        0,
                        0,
                        mContext.resources.getDimensionPixelSize(R.dimen.credit_card_width),
                        mContext.resources.getDimensionPixelSize(R.dimen.credit_card_height)
                    )
                    radioButton.setCompoundDrawablesWithIntrinsicBounds(
                        wrappedDrawable,
                        null,
                        null,
                        null
                    )
                }
                radioButton.text = cardText.toString()
                radioButton.compoundDrawablePadding = 24
                radioButton.setPadding(24, 0, 0, 0)
                container.setPadding(8, 8, 8, 8)
                mRadioButtons.add(Pair(container, radioButton))
                mCardsGroup.addView(
                    container,
                    LinearLayout.LayoutParams.MATCH_PARENT,
                    LinearLayout.LayoutParams.WRAP_CONTENT
                )
                if (mOrderData != null && mOrderData?.payment != null && mOrderData?.payment?.cardType != null && mOrderData?.payment?.maskedCardNumber != null) {
                    if (mOrderData?.payment?.cardType == card.mCreditCardType && mOrderData?.payment?.maskedCardNumber == card.last4Digits) {
                        selectedId = id++
                    } else {
                        id++
                    }
                }
            }
        }
        for (pair: Pair<LinearLayout, AppCompatRadioButton> in mRadioButtons) {
            pair.second?.setOnCheckedChangeListener { buttonView: CompoundButton, isChecked: Boolean ->
                if (isChecked) {
                    for (rb: Pair<LinearLayout, AppCompatRadioButton> in mRadioButtons) {
                        rb.second?.isChecked = false
                        if (rb.first != null) {
                            ViewUtil.findViewById<View>(
                                (rb.first!!),
                                R.id.edtCvv_label
                            ).visibility = View.GONE
                            ViewUtil.findViewById<View>(
                                (rb.first!!),
                                R.id.edtCvv_text
                            ).visibility = View.GONE
                        }
                    }
                    buttonView.setChecked(true)
                    mPaymentCard = pair.second?.tag as PaymentCards?
                    mSelectedCardType =
                        pair.second?.getId()?.let {
                            (mPaymentCards as MutableList<PaymentCards>)[it].creditCardType
                        }
                    mCardNumber = getString(
                        R.string.edit_payment_card_number,
                        pair.second?.getId()?.let {
                            (mPaymentCards as MutableList<PaymentCards>)
                                .get(it)
                                .getCreditCardNumber()
                                .substring(
                                    (
                                        (mPaymentCards as MutableList<PaymentCards>).get(it)
                                            .getCreditCardNumber().length -
                                            4
                                        )
                                )
                        }
                    )
                    if (pair.first != null && pair.second != null && pair.second?.getId() != selectedId) {
                        mEdtCvvLabel =
                            ViewUtil.findViewById<TextInputLayout>(
                                (pair.first)!!,
                                R.id.edtCvv_label
                            )
                        mEdtCvv =
                            ViewUtil.findViewById<TextInputEditText>(
                                (pair.first)!!,
                                R.id.edtCvv
                            )
                        setupCvvObservable(view, mEdtCvv, mEdtCvvLabel)
                        mEdtCvvLabel.setVisibility(View.VISIBLE)
                        mEdtCvv.requestFocus()
                        ViewUtil.showKeyboard(mEdtCvv)
                    } else {
                        hideKeyboard(view)
                        mSaveBtn.isEnabled = false
                    }
                }
            }
        }
        if (mOrderData != null && mOrderData?.payment != null && !TextUtils.isEmpty(mOrderData?.payment?.orderCardNickName) &&
            mOrderData?.payment?.orderCardNickName?.contains("temp") == true
        ) {
            selectedId = 0
        }
        if (selectedId >= 0 && selectedId <= mRadioButtons.size && mRadioButtons[selectedId].first != null) {
            mRadioButtons.get(selectedId).second?.isChecked = true
            ViewUtil.findViewById<View>(
                (mRadioButtons.get(selectedId).first)!!,
                R.id.edtCvv_text
            ).visibility =
                View.GONE
            ViewUtil.findViewById<View>(
                (mRadioButtons.get(selectedId).first)!!,
                R.id.edtCvv_label
            ).visibility =
                View.GONE
            hideKeyboard(view)
        } else {
            if (mRadioButtons.isNotEmpty() && mRadioButtons.get(0).first != null && mRadioButtons.get(0).second != null) {
                mRadioButtons.get(0).second?.isChecked = true
                ViewUtil.findViewById<View>(
                    (mRadioButtons.get(0).first)!!,
                    R.id.edtCvv_text
                ).visibility =
                    View.GONE
                ViewUtil.findViewById<View>(
                    (mRadioButtons.get(0).first)!!,
                    R.id.edtCvv_label
                ).visibility =
                    View.GONE
                hideKeyboard(view)
            }
        }
        mCardsGroup.visibility = View.VISIBLE
    }

    override fun onCreateDialog(dialog: Int): Dialog? {
        when (dialog) {
            DIALOG_AUTHORISING -> {
                val builder = AlertDialog.Builder(
                    mContext
                )
                builder.setTitle(R.string.authorising_card)
                builder.setMessage(getString(R.string.please_wait_while_card, mCardNumber))
                builder.setCancelable(false)
                return builder.create()
            }
            DIALOG_AUTHORISING_SUCCESS -> {
                val successBuilder = AlertDialog.Builder(
                    mContext
                )
                successBuilder.setTitle(R.string.authorising_success)
                successBuilder.setMessage(getString(R.string.success_authorize_text, mCardNumber))
                successBuilder.setPositiveButton(
                    R.string.done
                ) { dialogInterface, i ->
                    dismissDialog(DIALOG_AUTHORISING_SUCCESS)
                    pop()
                }
                return successBuilder.create()
            }
            DIALOG_AUTHORISING_FAILED -> {
                val failedBuilder = AlertDialog.Builder(
                    (mContext)
                )
                val customTitle = TextView(mContext)
                customTitle.setText(R.string.authorising_failed)
                customTitle.textSize = 20f
                customTitle.setPadding(55, 42, 0, 0)
                customTitle.gravity = Gravity.LEFT
                customTitle.setTextColor(
                    ContextCompat.getColor(
                        (mContext),
                        R.color.sa_savings_red
                    )
                )
                failedBuilder.setCustomTitle(customTitle)
                failedBuilder.setMessage(getString(R.string.authorise_failed_text, mCardNumber))
                failedBuilder.setPositiveButton(
                    R.string.edit_details
                ) { dialogInterface, i -> dismissDialog(DIALOG_AUTHORISING_FAILED) }
                failedBuilder.setNegativeButton(
                    R.string.close,
                    (
                        { dialogInterface, i ->
                            dismissDialog(DIALOG_AUTHORISING_FAILED)
                            pop()
                        }
                        )
                )
                return failedBuilder.create()
            }
            else -> return super.onCreateDialog(dialog)
        }
    }

    override fun onResume() {
        super.onResume()
        mHandler.postDelayed(
            {
                if (mPaResReceived) {
                    view?.let { hideKeyboard(it) }
                    editPaymentCardCXO(
                        view,
                        true,
                        MODIFY_ORDER_CXO_ID2
                    )
                }
            },
            DELAY_SECONDS.toLong()
        )
    }

    override fun hasActionBar(): Boolean {
        return true
    }

    override fun getActionBarTitle(): String? {
        return mTitleText
    }

    companion object {
        private val SCREEN_NAME = "Payment Card Edit"
        val TAG = EditPaymentCardFragment::class.java.simpleName
        private const val KEY_ORDER_ID = "order_id"
        private const val KEY_ORDER_DATA = "order_data"
        private const val ORDER_TOTAL = "order_total"
        private const val DIALOG_AUTHORISING = 1
        const val DIALOG_AUTHORISING_SUCCESS = 2
        const val DIALOG_AUTHORISING_FAILED = 4
        private const val DELAY_SECONDS = 100

        @JvmStatic
        fun newInstanceCXO(
            orderId: String?,
            orderTotal: Double?,
            order: WismoOrderResponse.Payload? = null
        ): EditPaymentCardFragment {
            val fragment = EditPaymentCardFragment()
            val args = Bundle()
            args.putString(KEY_ORDER_ID, orderId)
            if (orderTotal != null) {
                args.putDouble(ORDER_TOTAL, (orderTotal))
            }
            args.putParcelable(KEY_ORDER_DATA, order)
            fragment.arguments = args
            return fragment
        }
    }
}
