package com.asda.android.app.orders

import android.annotation.SuppressLint
import android.content.Context
import android.text.TextUtils
import android.util.Log
import android.view.View
import android.widget.TextView
import androidx.annotation.VisibleForTesting
import androidx.core.app.NotificationManagerCompat
import androidx.core.text.HtmlCompat
import androidx.databinding.ObservableBoolean
import androidx.databinding.ObservableField
import androidx.lifecycle.MediatorLiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Observer
import com.asda.android.R
import com.asda.android.analytics.AnalyticsConfig
import com.asda.android.analytics.AnalyticsConfig.cartId
import com.asda.android.analytics.AnalyticsConfig.customerId
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.constants.Anivia.REFUND_STATUS_BUTTON_CLICK_EVENT
import com.asda.android.analytics.events.LinkButtonClickEvent
import com.asda.android.analytics.events.OverlayEvent
import com.asda.android.analytics.events.RefundStatusButtonImpressionEvent
import com.asda.android.app.settings.FeatureSettingManager
import com.asda.android.base.core.constants.DateFormats
import com.asda.android.base.core.constants.TimeZoneFormat
import com.asda.android.base.core.convertToFormat
import com.asda.android.base.core.livedata.SingleEventMediator
import com.asda.android.base.core.response.BaseStateResponse
import com.asda.android.base.core.scheduler.SchedulerProvider
import com.asda.android.base.core.view.ViewUtil
import com.asda.android.cxo.CartManager
import com.asda.android.orders.constants.CONTACT_US_LINK
import com.asda.android.orders.constants.OrderConstants
import com.asda.android.orders.data.OrderRepository
import com.asda.android.orders.wismo.constants.WismoCardConstants
import com.asda.android.restapi.service.constants.ViewOrderConstants
import com.asda.android.restapi.service.data.AsdaResponse
import com.asda.android.restapi.service.data.ViewRefundsResponse
import com.asda.android.restapi.service.data.WismoOrderResponse
import com.asda.android.restapi.service.data.cart.Cart
import com.asda.android.restapi.service.data.refund.RefundEligibilityResponse
import com.asda.android.service.base.Authentication
import com.asda.android.utils.Utils.isDevBuild
import com.asda.android.utils.Utils.isLocationEnabled
import retrofit2.HttpException
import java.util.Arrays
import java.util.Locale

class OrderDetailsViewModel :
    Observer<BaseStateResponse<RefundEligibilityResponse?>?> {
    private var mOrder: WismoOrderResponse.Payload? = null

    @get:VisibleForTesting
    lateinit var context: Context
        private set
    val mIsAmending: ObservableBoolean = ObservableBoolean()
    val mIsModifiable: ObservableBoolean = ObservableBoolean()
    val mUseGreenReorder: ObservableBoolean = ObservableBoolean(true)
    @kotlin.jvm.JvmField
    val mIsCardEditAllowed: ObservableBoolean = ObservableBoolean(false)
    @kotlin.jvm.JvmField
    val mIsUnauthorised: ObservableBoolean = ObservableBoolean(false)
    val mShowMoreInfoLink: ObservableBoolean = ObservableBoolean(false)
    val mFeedbackButtonVisibility: ObservableBoolean = ObservableBoolean(false)
    val helpButtonVisibility = ObservableBoolean(false)
    val mRefundButtonText: ObservableField<CharSequence> = ObservableField("")
    val mRefundButtonVisibility: ObservableBoolean = ObservableBoolean(false)
    val mRefundGroupVisibility: ObservableBoolean = ObservableBoolean(false)
    @kotlin.jvm.JvmField
    val mErrorMessageVisibility: ObservableBoolean = ObservableBoolean(false)
    @kotlin.jvm.JvmField
    val orderNotSettledErrorVisibility: ObservableBoolean = ObservableBoolean(false)
    val mExtendedRefundButtonVisibility: ObservableBoolean = ObservableBoolean(false)
    val mCheckoutAmendsButtonText: ObservableField<CharSequence> = ObservableField("")
    val mRefundText: ObservableField<CharSequence> = ObservableField("")
    val mPreDeliveryText: ObservableField<CharSequence> = ObservableField("")
    val mRefundTitleText: ObservableField<CharSequence> = ObservableField("")
    @kotlin.jvm.JvmField
    val mProgressBarStatus: ObservableField<CharSequence> = ObservableField("")
    @kotlin.jvm.JvmField
    val mPreDeliveryVisibility: ObservableBoolean = ObservableBoolean(false)
    @kotlin.jvm.JvmField
    val mOrderStatus: ObservableField<CharSequence> = ObservableField("Order Status")
    val mOrderNumber: ObservableField<CharSequence> = ObservableField("")
    val mCancelButtonVisibility: ObservableBoolean = ObservableBoolean()
    @kotlin.jvm.JvmField
    var eligibilityLiveData = MediatorLiveData<BaseStateResponse<RefundEligibilityResponse?>?>()
    @kotlin.jvm.JvmField
    var refundClickLiveData = MutableLiveData<Boolean?>()
    @kotlin.jvm.JvmField
    var returnsLiveData = SingleEventMediator<Boolean>()
    val returnsRefundButtonVisibility = ObservableBoolean(false)
    @kotlin.jvm.JvmField
    var reOrderClickLiveData = MutableLiveData<Boolean?>()
    private var mRefundData: ViewRefundsResponse? = null
    private var mMaskedCardNumber: String? = null
    var orderId: String? = null
        private set
    private var mCXOCart: Cart? = null
    private val repository: OrderRepository

    var cXOCart: Cart?
        get() {
            if ((mCXOCart == null) && (CartManager.instance.cartResponse != null) && (CartManager.instance.cartResponse?.data != null)) {
                mCXOCart = CartManager.instance.cartResponse?.data?.cart
            }
            return mCXOCart
        }
        set(cxoCart) {
            mCXOCart = cxoCart
        }

    fun setOrder(order: WismoOrderResponse.Payload) {
        mOrder = order
        mIsAmending.set(cXOCartAmendingStatus)
        mOrder?.isModifiable?.let { mIsModifiable.set(it) }
        mOrder?.isCancellable?.let { mCancelButtonVisibility.set(it) }
        mCheckoutAmendsButtonText.set(
            if (mIsAmending.get()) context.getString(R.string.button_check_out_amends) else context.getString(
                R.string.continue_shopping
            )
        )
        mShowMoreInfoLink.set(isUnauthorizedPayment(order))
        updateReorderButton()
    }

    fun onAttach(context: Context) {
        this.context = context
        if (mOrder != null) {
            mIsAmending.set(cXOCartAmendingStatus)
            mOrder?.isModifiable?.let { mIsModifiable.set(it) }
            mOrder?.isCancellable?.let { mCancelButtonVisibility.set(it) }
            mCheckoutAmendsButtonText.set(
                if (mIsAmending.get()) context.getString(R.string.button_check_out_amends) else context.getString(
                    R.string.continue_shopping
                )
            )
            updateReorderButton()
        }
    }

    @get:VisibleForTesting
    val cXOCartAmendingStatus: Boolean
        get() = (
            cXOCart != null
            ) && (
            cXOCart?.cartMetadata != null
            ) && cXOCart?.cartMetadata?.amendedOrderId == mOrder?.webOrderID && cXOCart?.cartMetadata?.amendedCart == true

    private fun updateReorderButton() {
        mUseGreenReorder.set(!mIsAmending.get() && !mIsModifiable.get())
    }

    fun onClickReorder(view: View?) {
        if (mOrder != null) {
            reOrderClickLiveData.postValue(true)
        }
    }

    fun setUpPreDeliveryText(payload: WismoOrderResponse.Payload) {
        val preDeliveryText = HtmlCompat.fromHtml(getPreDeliveryText(payload), HtmlCompat.FROM_HTML_MODE_LEGACY)
        mPreDeliveryText.set(preDeliveryText)
        mPreDeliveryVisibility.set(true)
    }

    @VisibleForTesting
    fun getPreDeliveryText(payload: WismoOrderResponse.Payload): String {
        return if (payload.thirdPartyDeliveryOrder) {
            if (WismoCardConstants.ORDER_LOADED_TO_VAN == payload.status) {
                val deliveryProvider = payload.thirdPartyDeliveryProvider
                context.getString(R.string.xyz_pre_delivery_text, deliveryProvider)
            } else {
                context.getString(R.string.third_party_pre_delivery_text)
            }
        } else {
            context.getString(R.string.pre_delivery_text)
        }
    }

    fun setPostDeliveryText(refundAvailabilityPeriod: String?) {
        mRefundButtonVisibility.set(false)
        mRefundGroupVisibility.set(false)
        helpButtonVisibility.set(false)
        val text: String = context.getString(R.string.contact_us_text, refundAvailabilityPeriod).toString()
        mPreDeliveryText.set(text)
        mPreDeliveryVisibility.set(true)
    }

    fun setupRMSRefund(order: WismoOrderResponse.Payload, orderId: String) {
        this.orderId = orderId
        if (order.refundAvailable.equals("true", ignoreCase = true)) {
            mRefundGroupVisibility.set(true)
            val refundPeriod: String? =
                if (order.refundAvailabilityPeriod != null) order.refundAvailabilityPeriod else ""
            val message: String =
                context.resources.getString(R.string.refund_title_text, refundPeriod).toString()
            mRefundTitleText.set(message)
            helpButtonVisibility.set(true)
            if (order.isReturnAvailable) {
                if (FeatureSettingManager.isYourReturnsPageEnabled(context)) {
                    helpButtonVisibility.set(false)
                } else {
                    mRefundButtonText.set(context.getString(R.string.refund_status))
                }
            } else {
                mRefundButtonText.set(context.getString(R.string.not_received_order))
            }
        } else {
            setPostDeliveryText(order.refundAvailabilityPeriod)
        }
        if (((WismoCardConstants.ORDER_NOT_FULFILLED_SETTLEMENT_EXCEPTION == order.status) || (WismoCardConstants.ORDER_NOT_SETTLED_SETTLEMENT_EXCEPTION == order.status))) {
            mRefundButtonVisibility.set(false)
            mRefundGroupVisibility.set(false)
            helpButtonVisibility.set(false)
        }
    }

    @VisibleForTesting
    private fun isRefundInProgress(order: WismoOrderResponse.Payload): Boolean {
        val refundList: ArrayList<WismoOrderResponse.RefundItem> = ArrayList()
        refundList.addAll(Arrays.asList(*order.refundItemList?.approved))
        refundList.addAll(Arrays.asList(*order.refundItemList?.cancelled))
        refundList.addAll(Arrays.asList(*order.refundItemList?.pending))
        return !refundList.isEmpty()
    }

    fun setupRefund(order: WismoOrderResponse.Payload, data: ViewRefundsResponse?) {
        // For ASDAA-13471:In delivered state, refund button will be replaced with Edit payment button.
        // check the ticket for more info.
        // TODO Once RMS is rolled out 100%, remove this absurd logic.
        if (ViewOrderConstants.ORDER_STATUS_V4_DELIVERED.equals(order.orderStatus, ignoreCase = true)) {
            if ((data == null || (OrderDetailsPresenter.AUTHORIZE_FAILED == order.payment?.paymentStatus && order.isPayAppMigrated && order.allowCardEdit))) {
                mRefundButtonVisibility.set(false)
            } else {
                setupRefundButtons(order, data)
            }
        } else {
            if ((
                (
                    data == null
                    ) || ViewOrderConstants.ORDER_STATUS_DELIVERED != order.orderStatus ||
                    (
                        ViewOrderConstants.ORDER_STATUS_DELIVERED == order.orderStatus && (
                            (OrderDetailsPresenter.AUTHORIZE_FAILED == order.payment?.paymentStatus) && order.isPayAppMigrated &&
                                order.allowCardEdit
                            )
                        )
                )
            ) {
                mRefundButtonVisibility.set(false)
            } else {
                setupRefundButtons(order, data)
            }
        }
    }

    fun setupRefundButtons(
        order: WismoOrderResponse.Payload,
        data: ViewRefundsResponse?
    ) {
        mRefundData = data
        mRefundButtonVisibility.set(true)
        if ((
            (data != null) && !data.hasErrorCode("ASDAGWS_SELFSERVICE_ORDER_NOT_ELIGIBLE") && (data.selfservice != null) && !TextUtils.isEmpty(data.selfservice?.orderId) &&
                !TextUtils.isEmpty(data.selfservice?.deliveryDate) &&
                (data.selfservice?.items != null) && (data.selfservice?.items?.size!! > 0)
            )
        ) {
            var hasRefund = false
            for (refundItem: ViewRefundsResponse.RefundItem? in data.selfservice?.items!!) {
                if (refundItem != null && ("true" == refundItem.refundFlag)) {
                    hasRefund = true
                    break
                }
            }
            if (hasRefund) {
                mRefundButtonText.set(context.getString(R.string.refund_status))
                val availability: Int = getRefundAvailability(order.refundAvailabilityPeriod)
                mRefundText.set(
                    context.resources?.let {
                        HtmlCompat.fromHtml(
                            it
                                .getQuantityString(
                                    R.plurals.refund_availability_copy, availability, availability
                                ),
                            HtmlCompat.FROM_HTML_MODE_LEGACY
                        )
                    }
                )
            } else if ("true".equals(order.refundAvailable, ignoreCase = true)) {
                mRefundButtonText.set(context.getString(R.string.request_refund))
                val availability: Int = getRefundAvailability(order.refundAvailabilityPeriod)
                mRefundText.set(
                    context.resources?.let {
                        HtmlCompat.fromHtml(
                            it
                                .getQuantityString(
                                    R.plurals.refund_availability_copy, availability, availability
                                ),
                            HtmlCompat.FROM_HTML_MODE_LEGACY
                        )
                    }
                )
            } else {
                showExternalRefundLink()
            }
        } else {
            showExternalRefundLink()
        }
    }

    private fun showExternalRefundLink() {
        mRefundText.set(context.getString(R.string.request_refund))
        mRefundButtonVisibility.set(false)
        mExtendedRefundButtonVisibility.set(true)
    }

    fun setOrderNumber(orderNumber: String) {
        mOrderNumber.set(orderNumber)
    }

    fun setMaskedCardNumber(maskedCardNumber: String?) {
        mMaskedCardNumber = maskedCardNumber
    }

    override fun onChanged(eligibilityBaseStateResponse: BaseStateResponse<RefundEligibilityResponse?>?) {
        eligibilityLiveData.postValue(eligibilityBaseStateResponse)
    }

    @get:SuppressLint("CheckResult")
    val refundEligibility: Unit
        get() {
            repository.getRefundEligibility((orderId.toString()))
        }

    fun onClickRefundStatusButton(view: View) {
        if (isRefundInProgress(view)) {
            refundClickLiveData.setValue(true)
            Tracker.get()
                .trackEvent(
                    AsdaAnalyticsEvent(Anivia.BUTTON_TAP_EVENT)
                        .putString(Anivia.BUTTON_NAME_KEY, REFUND_STATUS_BUTTON_NAME)
                        .putString(Anivia.LOCATION_KEY, LOCATION)
                )
        } else {
            sendRefundClickEvent(OrderConstants.ORDER_NOT_RECEIVED_LINK_TEXT)
            ViewUtil.openWebPage(
                CONTACT_US_LINK,
                context, context.getString(R.string.not_received_order)
            )
        }
    }

    fun setUpRefundReturnButton(payload: WismoOrderResponse.Payload) {
        if (payload.isReturnAvailable && FeatureSettingManager.isYourReturnsPageEnabled(context)) {
            Tracker.get()?.trackEvent(
                RefundStatusButtonImpressionEvent(
                    Anivia.ORDER_DETAIL,
                    Anivia.ORDERS,
                    true
                )
            )
            returnsRefundButtonVisibility.set(true)
        }
    }

    fun onClickReturnsButton(view: View) {
        Tracker.get()?.trackEvent(AsdaAnalyticsEvent(REFUND_STATUS_BUTTON_CLICK_EVENT))
        returnsLiveData.postValue(true)
    }

    private fun isRefundInProgress(view: View): Boolean {
        var isRefundInProgress = false
        if (view is TextView) {
            isRefundInProgress = context.getString(R.string.refund_status)
                .equals(view.text.toString().toLowerCase(Locale.ROOT), ignoreCase = true)
        }
        return isRefundInProgress
    }

    fun sendRefundClickEvent(linkText: String?) {
        if (mOrder != null) {
            customerId = Authentication.getInstance().profileId
            cartId = CartManager.instance.getBasketId()
            AnalyticsConfig.isLocationEnabled = isLocationEnabled((context))
            var formattedDate: String? = OrderConstants.EMPTY
            if (mOrder?.orderDate != null) {
                formattedDate = mOrder?.orderDate?.convertToFormat(
                    DateFormats.YYYY_MM_DD_T_KK_MM_SS_SSS_SSSS,
                    DateFormats.yyyy_MM_dd_HH_mm_ss,
                    TimeZoneFormat.UTC,
                    TimeZoneFormat.UK
                )
            }
            val event = LinkButtonClickEvent(
                orderId,
                formattedDate,
                mOrder?.orderStatus,
                isRefundInProgress(mOrder!!),
                linkText.toString(),
                NotificationManagerCompat.from((context)).areNotificationsEnabled(),
                null,
                Anivia.ORDERS,
                null,
                Anivia.ORDER_DETAIL
            )
            Tracker.get().trackEvent(event)
        }
    }

    @VisibleForTesting
    fun sendSSRMaxLimitEvent(message: String?) {
        if (mOrder != null) {
            customerId = profileId
            cartId = CartManager.instance.getBasketId()
            AnalyticsConfig.isLocationEnabled = isLocationEnabled
            val event = orderId?.let {
                OverlayEvent(
                    SSR_MAX_LIMIT_REACHED,
                    message,
                    Anivia.ORDER_DETAIL,
                    it,
                    isNotificationEnabled,
                    Anivia.ORDERS
                )
            }
            event?.let { tracker.trackEvent(it) }
        }
    }

    fun responseBodyContainsKnownErrorCode(e: Throwable): Boolean {
        val responseBody: String
        try {
            responseBody = (e as HttpException).response()?.errorBody()?.string().toString()
        } catch (ex: Exception) {
            return false
        }
        return (
            responseBody.contains(CART_INVALID_PROFILE_ORDER) ||
                responseBody.contains(CART_ORDER_NOT_FOUND) ||
                responseBody.contains(CART_NON_REBOOKABLE_ORDER)
            )
    }

    fun getErrorResponse(message: String?): AsdaResponse {
        val response = AsdaResponse()
        val error: AsdaResponse.Error = AsdaResponse.Error()
        error.errorMessage = message
        response.errors = arrayOf(error)
        return response
    }

    @get:VisibleForTesting
    val isNotificationEnabled: Boolean
        get() {
            return NotificationManagerCompat.from(context).areNotificationsEnabled()
        }

    @get:VisibleForTesting
    val tracker: Tracker
        get() {
            return Tracker.get()
        }

    @get:VisibleForTesting
    val profileId: String?
        get() {
            return Authentication.getInstance().profileId
        }

    @get:VisibleForTesting
    val isLocationEnabled: Boolean
        get() {
            return isLocationEnabled((context))
        }

    companion object {
        private val TAG: String = "OrderDetailsViewModel"
        private val LOCATION: String = "order details"
        private val REFUND_STATUS_BUTTON_NAME: String = "Refund Status"
        private val SSR_MAX_LIMIT_REACHED: String = "Refund limit exceeded"
        @kotlin.jvm.JvmField
        val CART_INVALID_PROFILE_ORDER: String = "CART_INVALID_PROFILE_ORDER"
        @kotlin.jvm.JvmField
        val CART_ORDER_NOT_FOUND: String = "CART_ORDER_NOT_FOUND"
        @kotlin.jvm.JvmField
        val CART_NON_REBOOKABLE_ORDER: String = "CART_NON_REBOOKABLE_ORDER"
        @kotlin.jvm.JvmField
        val REORDER_ERROR_MESSAGE: String = "Sorry, you cannot book this order again."
        private fun isUnauthorizedPayment(order: WismoOrderResponse.Payload): Boolean {
            return ((ViewOrderConstants.ORDER_STATUS_UNAUTHORIZED == order.orderStatus) || (OrderDetailsPresenter.AUTHORIZATION_FAILURE == order.orderAuthStatus))
        }

        @VisibleForTesting
        fun getRefundAvailability(refundAvailabilityPeriod: String?): Int {
            var refundAvailability: Int = -1
            if (!TextUtils.isEmpty(refundAvailabilityPeriod)) {
                try {
                    refundAvailability = Integer.valueOf(refundAvailabilityPeriod.toString())
                } catch (exception: NumberFormatException) {
                    if (isDevBuild) {
                        Log.e(TAG, "Exception", exception)
                    }
                }
            }
            return refundAvailability
        }
    }

    init {
        repository = OrderRepository(SchedulerProvider())
        eligibilityLiveData.addSource(repository.eligibilityLiveData, this)
    }
}
