package com.asda.android.app.orders

import android.content.Context
import android.os.Bundle
import android.text.TextUtils
import android.view.View
import androidx.lifecycle.Observer
import com.asda.android.app.main.view.AsdaActivity
import com.asda.android.app.settings.featureflag.FeatureSettings.isNewOrdersModuleEnabled
import com.asda.android.base.core.constants.FragmentConfigConstants
import com.asda.android.base.core.utils.orZero
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.orders.orderdetails.view.fragment.OrderDetailsFragment
import com.asda.android.orders.orders.view.fragment.OrderListFragmentNew
import com.asda.android.orders.utils.OrdersConfig
import com.asda.android.service.base.OrderManager

class OrderFragment : FeaturedFragment() {

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        val orderId = getOrderIdParameter()
        val isLegacyChecking = OrderManager.get().isLegacyCheckinOrder(orderId)

        if (isNewOrdersModuleEnabled(activity as Context)) {
            if (TextUtils.isEmpty(orderId)) {
                pushNoAnimation(OrderListFragmentNew.newInstance(getFilterDpOrdersValue()), false)
            } else {
                val newOrderDetailsFragment =
                    OrderDetailsFragment.newInstance(orderId.orEmpty())
                pushNoAnimation(newOrderDetailsFragment, false)
            }
        } else {
            if (TextUtils.isEmpty(orderId)) {
                pushNoAnimation(OrderListFragment.newInstance(getFilterDpOrdersValue()), false)
            } else {
                val fragment = OrderDetailsPresenter.newInstance(orderId, !isLegacyChecking)
                pushNoAnimation(fragment, false)
            }
        }

        OrdersConfig.reOrderLiveData.observe(
            viewLifecycleOwner,
            Observer { isReOrderSuccess ->
                if (isReOrderSuccess) {
                    (activity as AsdaActivity).getFragmentSwitcher()?.switchToFragment(FragmentConfigConstants.TAG_CART, null)
                }
            }
        )
    }

    override fun onBackPressed(): Boolean {
        val currentFragManager = getCurrentFragmentManager()
        if (currentFragManager?.backStackEntryCount.orZero() > 0) {
            try {
                currentFragManager?.popBackStackImmediate()
                return true
            } catch (e: IllegalStateException) {
                // no op
            }
        }
        return false
    }

    override fun isParentFragment(): Boolean = true

    private fun getFilterDpOrdersValue(): Boolean =
        dynamicArguments?.getBoolean(FILTER_DP_ORDERS_KEY) ?: false

    override fun getInternalTag(): String = TAG

    private fun getOrderIdParameter(): String? =
        dynamicArguments?.getString(ORDER_ID_KEY)

    companion object {
        const val FILTER_DP_ORDERS_KEY = "filter_dp_orders_key"
        const val ORDER_ID_KEY = "order_id_key"
        const val IS_LEGACY_CHECKIN_ORDER = "is_legacy_order"
        const val ORDER_FRAGMENT_TAG = "OrderFragment"
        const val TAG = "OrderFragment"
    }
}
