package com.asda.android.app.orders

import com.asda.android.restapi.service.data.CreateJwtPaymentDetails
import com.asda.android.restapi.service.data.PaymentDetailsResponse
import com.asda.android.restapi.service.data.PaymentInfo
import com.asda.android.restapi.service.data.checkout.CheckoutResponse
import com.asda.android.restapi.singleprofile.ProfileResponse
import pangea.EncryptCreditCardService

class PaymentHelper {

    fun updatePaymentCXO(
        paymentInfo: PaymentInfo,
        encryptedCc: EncryptCreditCardService.EncryptedCc,
        paRes: String?
    ): PaymentInfo {
        paymentInfo.integrityCheck = encryptedCc.integrityCheck
        paymentInfo.keyId = encryptedCc.keyId
        paymentInfo.phase = encryptedCc.phase
        paymentInfo.encryptedCvv = encryptedCc.encryptedCvv
        paymentInfo.pares = paRes
        paymentInfo.encryptedPan = encryptedCc.encryptedCc
        return paymentInfo
    }

    fun getCardIndex(
        paymentCard: PaymentDetailsResponse.PaymentCards,
        paymentDetailsResponse: PaymentDetailsResponse
    ): Int {
        val isMainCard = "y" == paymentCard.mainCard
        var cardIndex = 0
        if (!isMainCard &&
            paymentDetailsResponse.cards != null && paymentDetailsResponse.cards?.size!! > 1
        ) {
            cardIndex = 1
        }
        return cardIndex
    }

    fun getCardIndex(
        paymentCard: PaymentDetailsResponse.PaymentCards,
        paymentCards: Array<PaymentDetailsResponse.PaymentCards>?
    ): Int {
        val isMainCard = "y" == paymentCard.mainCard
        var cardIndex = 0
        if (!isMainCard && paymentCards != null && paymentCards.size > 1) {
            cardIndex = 1
        }
        return cardIndex
    }

    fun hasProfileCard(profileResponse: ProfileResponse?): Boolean {
        return profileResponse?.cards != null
    }

    fun isProfileCardsInValid(profileCards: Array<PaymentDetailsResponse.PaymentCards>?, cardIndex: Int): Boolean {
        return profileCards == null || profileCards.isEmpty() || cardIndex >= profileCards.size
    }

    fun is3DSRequired(checkoutResponse: CheckoutResponse?): Boolean {
        return checkoutResponse?.data != null && checkoutResponse.data?.purchaseOrder != null && checkoutResponse.data?.purchaseOrder?.threeDsVerification != null && (
            "true"
                ==
                    checkoutResponse
                        .data?.purchaseOrder?.threeDsVerification?.threeDsAuthRequired
            ) && checkoutResponse.data?.purchaseOrder?.paymentDetails != null &&
            checkoutResponse.data?.purchaseOrder?.paymentDetails?.isNotEmpty() == true
    }

    fun setCardEncyptedValues(
        paymentCard: PaymentDetailsResponse.PaymentCards,
        encryptedCc: EncryptCreditCardService.EncryptedCc
    ) {
        paymentCard.card = encryptedCc.encryptedCc
        paymentCard.integrityCheck = encryptedCc.integrityCheck
        paymentCard.ppEncryptedCC = encryptedCc.encryptedCc
        paymentCard.ppIntegrityCheck = encryptedCc.integrityCheck
        paymentCard.keyId = encryptedCc.keyId
        paymentCard.phase = encryptedCc.phase
        paymentCard.isDefault = true
        paymentCard.encryptedCvv = encryptedCc.encryptedCvv
        paymentCard.ppEncryptedCvv = encryptedCc.encryptedCvv
    }

    fun setCardJwtEncyptedValues(
        encryptedCc: EncryptCreditCardService.EncryptedCc,
        cardType: String
    ): CreateJwtPaymentDetails {
        val createJwtPaymentDetails = CreateJwtPaymentDetails()
        createJwtPaymentDetails.integrityCheck = encryptedCc.integrityCheck
        createJwtPaymentDetails.encryptedCvv = encryptedCc.encryptedCvv
        createJwtPaymentDetails.encryptedPan = encryptedCc.encryptedCc
        createJwtPaymentDetails.keyId = encryptedCc.keyId
        createJwtPaymentDetails.phase = encryptedCc.phase
        createJwtPaymentDetails.cardTypeName = cardType
        return createJwtPaymentDetails
    }
}
