package com.asda.android.app.recurringslot

import android.app.Application
import android.content.Context
import android.view.View
import androidx.databinding.ObservableBoolean
import androidx.databinding.ObservableField
import androidx.lifecycle.AndroidViewModel
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.interfaces.ITracker
import com.asda.android.payment.threeds.ThreeDsUtils
import com.asda.android.restapi.service.data.recurringslots.SlotSubscription
import com.asda.android.restapi.service.data.recurringslots.SubscriptionInfo
import com.asda.android.restapi.service.data.recurringslots.SubscriptionsResponse
import com.asda.android.slots.AsdaSlotsConfig
import com.asda.android.slots.R
import com.asda.android.slots.bookslot.adapter.SubscriptionAdapter
import com.asda.android.slots.bookslot.bookslotv3.view.SubscriptionAdapterListener
import com.asda.android.slots.utils.SlotUtils

class RecurringSlotAccountViewModel(application: Application) : AndroidViewModel(application) {
    val mFrequency = ObservableField("")
    val showPeakSeasonMessage = ObservableBoolean(false)
    val showPeakSeasonLink = ObservableBoolean(false)
    val peakSeasonMessage = ObservableField("")
    var peakSeasonFaqUrl = ""

    val mIsLoading = ObservableBoolean(true)
    private var mFragment: RecurringSlotDetailsFragment? = null
    var id: String? = null
    var subscriptionInfo: SubscriptionInfo? = null
    var subscriptionAdapter: SubscriptionAdapter? = null

    fun attach(fragment: RecurringSlotDetailsFragment) {
        mFragment = fragment
    }

    fun setData(subscriptionsResponse: SubscriptionsResponse?): Boolean {
        subscriptionsResponse?.data?.subscriptions?.let {
            if (it.isEmpty()) {
                return false
            }
            subscriptionInfo = it[0].subscription_info
            val context = AsdaSlotsConfig.application.applicationContext
            id = subscriptionInfo?.subscription_id

            setRecyclerAdapter(it)

            mFrequency.set(
                context?.getString(
                    R.string.rs_frequency_header_label,
                    SlotUtils.getFrequencyLabel(context, subscriptionInfo?.frequency)
                )
            )

            context?.let { _context ->
                SlotUtils.getPeakSeasonConfig(_context)?.let { _config ->
                    _config.message?.let { _message ->
                        if (isValid(_message)) {
                            showPeakSeasonMessage.set(true)
                            peakSeasonMessage.set(_message)

                            _config.faqUrl?.let { _faqUrl ->
                                if (isValid(_faqUrl)) {
                                    showPeakSeasonLink.set(true)
                                    peakSeasonFaqUrl = _faqUrl
                                }
                            }
                        }
                    }
                }
            }
        }

        mIsLoading.set(false)
        return true
    }

    fun isValid(input: CharSequence): Boolean {
        return input.isNotEmpty() && input.isNotBlank()
    }

    private fun setRecyclerAdapter(subscriptions: List<SlotSubscription>?) {
        mFragment?.context?.let { _context: Context ->
            subscriptions?.let { _subscriptions ->
                if (_subscriptions.isNotEmpty() && _subscriptions.first().subscription_info?.subscription_id?.isNotEmpty() == true &&
                    _subscriptions.first().subscription_info?.subscription_bookings?.isNotEmpty() == true
                ) {
                    subscriptionAdapter = SubscriptionAdapter(
                        _context,
                        _subscriptions.first().subscription_info,
                        _subscriptions.first().subscription_info?.subscription_bookings,
                        object : SubscriptionAdapterListener {
                            override fun onSubscriptionSkip(
                                subscriptionId: String,
                                bookingId: String?,
                                bookingStartDate: String
                            ) {
                                mFragment?.onClickSkip(
                                    subscriptionId, bookingId,
                                    _context.getString(
                                        R.string.rs_item_delete_date, bookingStartDate
                                    )
                                )
                            }
                        }
                    )
                    mFragment?.setAdapter(subscriptionAdapter)
                }
            }
        }
    }

    fun onClickCancel(view: View) {
        mFragment?.onClickCancel()
    }

    fun onClickFAQ(view: View) {
        mFragment?.onClickFAQ()
    }

    fun onClickLearnMore(view: View) {
        mFragment?.onClickLearnMore(peakSeasonFaqUrl)
    }

    fun triggerRsEvent(
        iTracker: ITracker,
        event: AsdaAnalyticsEvent // skipRecurringSlot or cancelRecurringSlot
    ) {
        event.apply {
            putString(Anivia.PAGE_NAME_KEY, Anivia.PV_RECURRING_SLOT_DETAILS)
            putString(Anivia.PREVIOUS_PAGE_NAME, Anivia.SECTION_BOOK_SLOT)
            putString(Anivia.CUSTOMER_ID_KEY, AsdaSlotsConfig.authentication.profileId ?: "")
            if (AsdaSlotsConfig.activeDeliveryPass != null) {
                putString(Anivia.DELIVERY_PASS_STATE_KEY, true.toString())
                putString(Anivia.DELIVERY_PASS_TYPE, AsdaSlotsConfig.activeDeliveryPass?.dpCardType)
            } else {
                putString(Anivia.DELIVERY_PASS_STATE_KEY, false.toString())
            }
            putString(Anivia.THREE_DS_DOWNGRADED, ThreeDsUtils.isDownGrade)
            putString(Anivia.THREE_DS_VERSION_KEY, ThreeDsUtils.getThreeDsVersionTriggered(ThreeDsUtils.THREE_DS_SDK_VERSION))
        }
        iTracker.trackEvent(event)
    }
}
