package com.asda.android.app.shop

import androidx.annotation.VisibleForTesting
import androidx.lifecycle.ViewModel
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.events.RelatedSearchClickEvent
import com.asda.android.analytics.events.RelatedSearchImpressionEvent
import com.asda.android.base.core.data.SearchInfo
import com.asda.android.base.core.data.SearchInfo.REFINED
import com.asda.android.base.core.data.SearchInfo.RELATED
import com.asda.android.search.model.SearchTermDTO
import com.asda.android.service.base.Authentication

class ShelfListFragmentViewModel : ViewModel() {

    fun sendImpressionAnalytics(searchTerm: String, itemList: List<SearchTermDTO>, @SearchInfo.SearchType searchType: String) {
        itemList.take(8).forEachIndexed { i, searchTermDTO ->
            val event = RelatedSearchImpressionEvent(searchTerm, searchType, getCustomerId(), searchTermDTO.searchTerm, (i + 1).toString())
            getTracker().trackEvent(event)
        }
    }

    fun sendClickAnalytics(searchTerm: String, clickedTerm: String, position: Int, searchType: Int) {
        val event = RelatedSearchClickEvent(searchTerm, getSearchType(searchType), getCustomerId(), clickedTerm, position.toString())
        getTracker().trackEvent(event)
    }

    fun getSearchType(searchType: Int): String {
        return if (searchType == 0) REFINED else RELATED
    }

    @VisibleForTesting
    fun getCustomerId(): String {
        return Authentication.getInstance().profileId.orEmpty()
    }

    @VisibleForTesting
    fun getTracker(): Tracker {
        return Tracker.get()
    }
}
