package com.asda.android.app.shop.productdetail.view

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.os.Handler
import android.util.Log
import android.view.View
import android.widget.TextView
import androidx.appcompat.app.AppCompatActivity
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentManager
import com.asda.android.R
import com.asda.android.analytics.info.CartItemAnalyticsInfo
import com.asda.android.analytics.info.FavoritesInfo
import com.asda.android.analytics.info.OffersForYouInfo
import com.asda.android.app.direct.DirectActivity
import com.asda.android.app.main.view.AsdaActivity
import com.asda.android.app.mobileAppPreview.view.MobileAppPreviewActivity
import com.asda.android.app.shop.WriteReviewActivity
import com.asda.android.app.shop.view.CXOHelper
import com.asda.android.app.shop.view.MerchandisingHandler
import com.asda.android.app.shoppinglists.ShoppingListActivity
import com.asda.android.base.core.constants.BottomNavConstants.TAG_SHOP
import com.asda.android.base.core.constants.BottomNavConstants.TAG_SHOP_OLD
import com.asda.android.base.core.constants.DeepLinkConstants
import com.asda.android.base.core.parser.MobileAppPreviewParser
import com.asda.android.base.core.utils.AsdaBaseCoreConfig
import com.asda.android.base.core.utils.orFalse
import com.asda.android.base.core.utils.tryCast
import com.asda.android.base.core.view.FragmentUtils
import com.asda.android.base.core.view.adapter.IGroupAdapterData
import com.asda.android.base.core.view.fragments.BaseFragment
import com.asda.android.base.core.view.ui.DialogHelper
import com.asda.android.base.core.view.ui.ItemQuantityController
import com.asda.android.base.interfaces.*
import com.asda.android.cxo.view.fragments.CXOCartFragment
import com.asda.android.cxo.view.fragments.CXOCartPagerFragment
import com.asda.android.cxo.view.fragments.RecipesTrolleyFragment
import com.asda.android.designlibrary.model.PromoTypeInfoModel
import com.asda.android.favourites.features.favorites.view.FavoritesFragmentNew
import com.asda.android.home.view.HomeFragmentNew
import com.asda.android.products.ui.AsdaProductsConfig
import com.asda.android.products.ui.detail.constants.PdpConstants.SOURCE_SCREEN_NAME
import com.asda.android.products.ui.detail.model.ProductDetailsExtraInfo
import com.asda.android.products.ui.detail.view.ItemAvailabilityHelper
import com.asda.android.products.ui.detail.view.ProductDetailsActivityNew
import com.asda.android.products.ui.filters.validator.FilterValidator
import com.asda.android.products.ui.filters.view.FiltersActivity
import com.asda.android.products.ui.product.constants.AvailabilityTypes
import com.asda.android.products.ui.product.datasource.PersonalisedSamplesRepository
import com.asda.android.products.ui.product.formatter.PromotionInfoFormatter
import com.asda.android.products.ui.product.helper.ProductHelper
import com.asda.android.products.ui.product.helper.ProductViewProviderHelper
import com.asda.android.products.ui.product.model.ProductViewInfoModel
import com.asda.android.products.ui.product.model.PromoViewInfoModel
import com.asda.android.products.ui.product.tile.view.OnIconographyIconClickListener
import com.asda.android.products.ui.product.tile.view.ProductTileGroupAdapter
import com.asda.android.products.ui.product.validator.AvailabilityInfoValidator
import com.asda.android.products.ui.utils.ProductUtils
import com.asda.android.restapi.app.product.model.FilterGroup
import com.asda.android.restapi.app.product.model.QuantityControllerViewInfoModel
import com.asda.android.restapi.app.product.model.SearchInfoModel
import com.asda.android.restapi.app.shop.model.ShelfListItem
import com.asda.android.restapi.cms.model.PersonalisedSampleResponse
import com.asda.android.restapi.cms.model.Zone
import com.asda.android.restapi.constants.EventConstants
import com.asda.android.restapi.service.Callback
import com.asda.android.restapi.service.RestUtils
import com.asda.android.restapi.service.base.IAsdaService
import com.asda.android.restapi.service.base.Iconography
import com.asda.android.restapi.service.data.Cart
import com.asda.android.restapi.service.data.IconographyResponse
import com.asda.android.restapi.service.data.ProductDetails
import com.asda.android.restapi.service.data.cart.Items
import com.asda.android.restapi.service.data.catalog.IroProductDetailsPdp
import com.asda.android.restapi.service.data.catalog.IroProductDetailsPlp
import com.asda.android.restapi.service.data.direct.ReviewsResponse
import com.asda.android.search.view.SearchResultsFragment
import com.asda.android.service.constants.CartManagerConstants
import com.asda.android.taxonomy.features.linksave.view.LinkSaveFragment
import com.asda.android.taxonomy.features.offersForYou.view.OffersForYouFragment
import com.asda.android.taxonomy.features.shelf.view.ShelfFragment
import com.asda.android.taxonomy.features.topOffers.view.TopOffersFragment
import com.asda.android.taxonomy.features.viewall.view.ViewAllTaxonomyFragment
import com.asda.android.utils.view.startOnTrolleyModify
import io.reactivex.Observable
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.subjects.BehaviorSubject
import java.math.BigDecimal
import java.util.*
import kotlin.collections.HashMap

const val TAG = "ProductManager"

var filterGroups: ArrayList<FilterGroup> = arrayListOf()

object ProductManager : IProductManager {

    override fun getLinkSaveFragment(
        context: Context,
        promotionId: String?,
        promotionText: String?
    ): BaseFragment {
        return LinkSaveFragment.newInstance(promotionId, promotionText)
    }

    override fun launchLinkSave(
        context: Context,
        currentFragmentManager: FragmentManager?,
        transitionListener: TransitionListener?,
        promotionId: String?,
        promotionText: String?
    ) {
        val fragment = getLinkSaveFragment(context, promotionId, promotionText)
        FragmentUtils.push(
            currentFragmentManager,
            transitionListener,
            fragment,
            addToBackStack = true,
            animate = true,
            breadcrumb = promotionText,
            replace = true
        )
    }

    override fun launchLinkSave(
        context: Context,
        sourceFragment: Fragment?,
        transitionListener: TransitionListener?,
        promotionId: String?,
        promotionText: String?
    ) {
        sourceFragment.tryCast<BaseFragment> {
            val fragment = getLinkSaveFragment(context, promotionId, promotionText)
            val shopFragment = launchShopBottomTab(this)
            shopFragment?.tryCast<BaseFragment> {
                push(
                    fragment,
                    getString(R.string.product_details),
                    isReplace = true,
                    addToBackStack = true
                )
            }
        }
    }

    override fun onTrolleyModify(
        context: Context,
        listingName: String,
        itemId: String?,
        itemQtyForCartAdd: String?,
        sourcePageId: String?
    ) {
        if (context is Activity)
            startOnTrolleyModify(context, listingName, itemId, itemQtyForCartAdd, sourcePageId)
    }

    private val cxoHelper = CXOHelper()
    private val productHelper = ProductHelper()
    private val productViewProviderHelper = ProductViewProviderHelper()
    override fun getCartIcon(view: View?): View? {
        return view?.findViewById(R.id.cart_total_price)
    }

    override fun onItemUpdated(
        context: Context,
        statusCode: Int,
        title: String?,
        screenName: String?
    ) {
        if (statusCode == CartManagerConstants.STATUS_CODE_ERROR) {
            com.asda.android.cxo.CartManager.instance.cartResponse?.errors?.toMutableList()?.let {
                cxoHelper.showErrorDialog(context, it)
            }
            return
        }

        if (statusCode == CartManagerConstants.STATUS_CODE_MAX_QTY_EXCEEDED) {
            DialogHelper.displayErrorDialog(
                context.getString(R.string.max_qty_error, title),
                context,
                screenName
            )
        }

        if (com.asda.android.cxo.CartManager.instance.cartResponse?.data?.cart?.fulfillmentDetails?.slotInfo?.expressItemLimit ?: 0 > 0 &&
            com.asda.android.cxo.CartManager.instance.cartResponse?.data?.cart?.cartItemSummary?.totalQuantity ?: 0 > 0 &&
            com.asda.android.cxo.CartManager.instance.cartResponse?.data?.cart?.fulfillmentDetails?.slotInfo?.expressItemLimit ==
            com.asda.android.cxo.CartManager.instance.cartResponse?.data?.cart?.cartItemSummary?.totalQuantity
        ) {
            handleAlert(context, CartManagerConstants.PRODUCT_SLOT_LIMIT_REACHED)
        }
    }

    override fun getIconographyResponse(
        context: Context,
        iconId: String?,
        callback: Callback<Iconography>,
        notifyCallbackIfInitialized: Boolean,
        asdaService: IAsdaService
    ): IconographyResponse.Icon? {
        iconId?.let {
            return Iconography.getInstance(
                context,
                callback as Callback<Iconography?>,
                notifyCallbackIfInitialized,
                asdaService
            )?.getIcon(context, iconId)
        }
        return null
    }

    override fun getPromoMissingCount(promotionId: String?, productId: String?): Int {
        return ProductUtils().getMissingCount(promotionId, productId)
    }

    override fun launchProductDetail(
        sourceFragment: Fragment?,
        context: Context,
        item: IroProductDetailsPlp.Items,
        isBigIcon: Boolean,
        specialOffers: Boolean,
        specialOffersTagIsClickable: Boolean,
        favoriteMode: Boolean,
        bannerCampaignId: String?,
        searchQuery: String?,
        previousSearchModule: String?
    ) {
        item.itemId?.let { itemId ->
            sourceFragment?.let {
                it.tryCast<BaseFragment> {
                    val fragment = com.asda.android.products.ui.detail.view.ProductDetailsFragment
                        .newInstance(
                            productDetailsExtraInfo = ProductDetailsExtraInfo(
                                itemId = itemId,
                                specialOffers = specialOffers,
                                specialOffersTagIsClickable = specialOffersTagIsClickable,
                                bannerCampaignId = bannerCampaignId,
                                searchQuery = searchQuery,
                                availability = item.inventory?.availabilityInfo?.let { availability ->
                                    ItemAvailabilityHelper().isAvailable(
                                        availability
                                    )
                                },
                                price = RestUtils.poundStringToBigDecimal(item.price?.priceInfo?.price)
                                    .toString(),
                                previousSearchModule = previousSearchModule,
                                isFeaturedFavorite = item.isFeatured.orFalse()
                            )
                        )
                    val shopFragment = launchShopBottomTab(sourceFragment)
                    shopFragment?.tryCast<BaseFragment> {
                        push(
                            fragment,
                            getString(R.string.product_details),
                            isReplace = shouldReplacePDPFragment(sourceFragment)
                        )
                    } ?: run {
                        this.push(
                            fragment,
                            getString(R.string.product_details),
                            isReplace = shouldReplacePDPFragment(sourceFragment)
                        )
                    }
                }
            } ?: kotlin.run {
                ProductDetailsActivityNew.launch(
                    context,
                    ProductDetailsExtraInfo(
                        itemId = itemId,
                        specialOffers = specialOffers,
                        specialOffersTagIsClickable = specialOffersTagIsClickable,
                        bannerCampaignId = bannerCampaignId,
                        searchQuery = searchQuery,
                        availability = item.inventory?.availabilityInfo?.let { availability ->
                            ItemAvailabilityHelper().isAvailable(
                                availability
                            )
                        },
                        price = RestUtils.poundStringToBigDecimal(item.price?.priceInfo?.price)
                            .toString(),
                        isFeaturedFavorite = item.isFeatured.orFalse()
                    )
                )
            }
        }
    }

    fun launchShopBottomTab(sourceFragment: Fragment?): Fragment? {
        if (sourceFragment is HomeFragmentNew || sourceFragment is CXOCartPagerFragment || sourceFragment is FavoritesFragmentNew || sourceFragment is CXOCartFragment || sourceFragment is RecipesTrolleyFragment) {
            sourceFragment.activity?.tryCast<AsdaActivity> {
                if (AsdaBaseCoreConfig.featureSettingManager.isNewTaxonomyEnabled(this.applicationContext)) {
                    this.getFragmentSwitcher()?.switchToFragment(TAG_SHOP)
                } else {
                    this.getFragmentSwitcher()?.switchToFragment(TAG_SHOP_OLD)
                }
                return this.getFragmentSwitcher()?.getTopFragment()
            }
        }
        return null
    }

    override fun launchProductDetail(
        sourceFragment: Fragment?,
        context: Context,
        itemId: String,
        isBigIcon: Boolean
    ) {
        sourceFragment?.let {
            it.tryCast<BaseFragment> {
                val fragment = com.asda.android.products.ui.detail.view.ProductDetailsFragment
                    .newInstance(
                        productDetailsExtraInfo = ProductDetailsExtraInfo(
                            itemId = itemId,
                            specialOffers = false,
                            specialOffersTagIsClickable = false,
                            bannerCampaignId = null
                        )
                    )
                if (this is ShelfFragment)
                    this.push(fragment, getString(R.string.product_details), isReplace = false)
                else
                    this.push(fragment, getString(R.string.product_details))
            }
        } ?: kotlin.run {
            ProductDetailsActivityNew.launch(
                context,
                ProductDetailsExtraInfo(
                    itemId = itemId,
                    specialOffers = false,
                    specialOffersTagIsClickable = false,
                    bannerCampaignId = null
                )
            )
        }
    }

    override fun launchProductDetail(
        sourceFragment: Fragment?,
        context: Context,
        item: ShelfListItem,
        isBigIcon: Boolean,
        specialOffers: Boolean,
        specialOffersTagIsClickable: Boolean,
        favoriteMode: Boolean,
        bannerCampaignId: String?,
        searchQuery: String?,
        previousSearchModule: String?
    ) {
        item.id?.let { itemId ->
            sourceFragment?.let {
                it.tryCast<BaseFragment> {
                    val fragment = com.asda.android.products.ui.detail.view.ProductDetailsFragment
                        .newInstance(
                            productDetailsExtraInfo = ProductDetailsExtraInfo(
                                itemId = itemId,
                                specialOffers = specialOffers,
                                specialOffersTagIsClickable = specialOffersTagIsClickable,
                                bannerCampaignId = bannerCampaignId,
                                searchQuery = searchQuery,
                                previousSearchModule = previousSearchModule
                            )
                        )
                    val shopFragment = launchShopBottomTab(sourceFragment)
                    shopFragment?.tryCast<BaseFragment> {
                        push(
                            fragment,
                            getString(R.string.product_details),
                            isReplace = shouldReplacePDPFragment(sourceFragment)
                        )
                    } ?: run {
                        this.push(
                            fragment,
                            getString(R.string.product_details),
                            isReplace = shouldReplacePDPFragment(sourceFragment)
                        )
                    }
                }
            } ?: kotlin.run {
                ProductDetailsActivityNew.launch(
                    context,
                    ProductDetailsExtraInfo(
                        itemId = itemId,
                        specialOffers = specialOffers,
                        specialOffersTagIsClickable = specialOffersTagIsClickable,
                        bannerCampaignId = bannerCampaignId
                    )
                )
            }
        }
    }

    override fun launchProductReviews(
        context: Context,
        item: IroProductDetailsPdp.Items,
        imageUrl: String,
        isShowWriteReview: Boolean,
        reviewStatistics: ReviewsResponse.Includes.Product.ReviewStatistics?,
        view: View
    ) {
        context.tryCast<AppCompatActivity> {
            DirectActivity.launchProductReviews(
                this,
                true,
                item.itemId,
                item.item?.itemName,
                item.price?.priceInfo?.price,
                imageUrl,
                reviewStatistics,
                item.item?.cin,
                isShowWriteReview,
                view
            )
        }
    }

    override fun launchProductDetail(
        sourceFragment: Fragment?,
        context: Context,
        cxoItem: com.asda.android.restapi.service.data.cart.Cart.CartItems,
        itemId: String
    ) {
        ProductDetailsActivityNew.launch(context, ProductDetailsExtraInfo(itemId = itemId))
    }

    override fun launchProductDetailFromBanner(
        context: Context,
        fragmentManager: FragmentManager?,
        itemId: String,
        specialOffers: Boolean,
        specialOffersTagIsClickable: Boolean,
        bannerCampaignId: String?
    ) {
        ProductDetailsActivityNew.launch(
            context,
            ProductDetailsExtraInfo(
                itemId = itemId,
                specialOffers = specialOffers,
                specialOffersTagIsClickable = specialOffersTagIsClickable,
                bannerCampaignId = bannerCampaignId
            )
        )
    }

    private fun getItemQuantityController(
        context: Context,
        shelfListItem: ShelfListItem,
        viewInfoModel: QuantityControllerViewInfoModel,
        view: View
    ): ItemQuantityController {
        var cartItem: Cart.CartItem? = null
        var cxoCartItem: com.asda.android.restapi.service.data.cart.Cart.CartItems? = null
        val itemQuantityController: ItemQuantityController
        cartItem = getCartItemFrom(
            context,
            shelfListItem,
            viewInfoModel.mode,
            viewInfoModel.isShoppingList
        )
        if (viewInfoModel.isShoppingList) {
            cxoCartItem = cxoHelper.getCxoItemFromShelfItem(shelfListItem, true, cartItem)
        } else {
            cxoCartItem = cxoHelper.getItemInCartCXO(shelfListItem, false)
        }
        itemQuantityController = cxoHelper.initCXOItemQuantityController(
            cxoCartItem,
            shelfListItem,
            viewInfoModel.alwaysShowPopup,
            viewInfoModel.isShoppingList
        )

        viewInfoModel.isValidForQuantityController =
            cartItem != null || cxoCartItem != null || viewInfoModel.alwaysShowPopup
        when {
            viewInfoModel.isValidForQuantityController -> {
                when {
                    cxoCartItem != null -> itemQuantityController.update(cxoCartItem)
                    cartItem != null -> itemQuantityController.update(cartItem)
                }
                when {
                    viewInfoModel.listType == ShoppingListActivity.ListType.USER_GENERATED -> itemQuantityController.showQtyPopup(
                        context,
                        view,
                        shelfListItem.allowPartialQty
                    )
                    viewInfoModel.listType == ShoppingListActivity.ListType.SYSTEM_GENERATED -> {
                        itemQuantityController.setAllowZeroQuantity(false)
                        itemQuantityController.showQtyPopup(
                            context,
                            view,
                            shelfListItem.allowPartialQty,
                            false
                        )
                    }
                }
            }
        }
        return itemQuantityController
    }

    override fun launchWriteProductReview(
        context: Context,
        productId: String?,
        name: String?,
        price: String?,
        imageUrl: String?
    ) {
        WriteReviewActivity.start(context, productId, name, price, imageUrl)
    }

    override fun getItemQuantityController(
        context: Context,
        item: IroProductDetailsPlp.Items,
        viewInfoModel: QuantityControllerViewInfoModel,
        view: View
    ): ItemQuantityController {
        val shelfListItem = ShelfListItem.buildShelfObject(
            item,
            viewInfoModel.specialOffersTagIsClickable,
            viewInfoModel.isBigIcon
        )
        return getItemQuantityController(context, shelfListItem, viewInfoModel, view)
    }

    override fun getItemQuantityController(
        context: Context,
        item: IroProductDetailsPdp.Items,
        viewInfoModel: QuantityControllerViewInfoModel,
        view: View
    ): ItemQuantityController {
        val shelfListItem =
            ShelfListItem.buildShelfObjectPdp(item, viewInfoModel.specialOffersTagIsClickable)
        return getItemQuantityController(context, shelfListItem, viewInfoModel, view)
    }

    override fun getCartItemFrom(
        context: Context,
        item: ShelfListItem,
        mode: Int,
        isShoppingList: Boolean
    ): Cart.CartItem? {
        return productHelper.getCartItemFrom(item, mode, isShoppingList)
    }

    override fun validateQuantity(
        context: Context,
        item: IroProductDetailsPlp.Items,
        specialOffersTagIsClickable: Boolean,
        isBigIcon: Boolean,
        mode: Int,
        shopContext: ShopContext,
        quantityController: IQuantityController
    ): Pair<BigDecimal, BigDecimal> {
        return productHelper.validateQuantity(
            item,
            specialOffersTagIsClickable,
            isBigIcon,
            mode,
            shopContext,
            quantityController
        )
    }

    override fun getQuantityInCart(
        context: Context,
        item: IroProductDetailsPdp.Items,
        specialOffersTagIsClickable: Boolean,
        mode: Int,
        shopContext: ShopContext
    ): BigDecimal {
        return productHelper.getQuantityInCart(item, specialOffersTagIsClickable, mode, shopContext)
    }

    override fun getOnIconographyIconClickListener(
        description: String,
        auxPromoIconOverlay: TextView,
        hideIconographyOverlayCallbacks: WeakHashMap<View, Runnable>,
        handler: Handler
    ): View.OnClickListener {
        return OnIconographyIconClickListener(
            description,
            auxPromoIconOverlay,
            hideIconographyOverlayCallbacks,
            handler
        )
    }

    override fun registerHookLogic(
        context: Context,
        zone: Zone,
        viewId: String,
        additionalInfo: Map<String, Any?>?,
        viewProvider: IViewProvider,
        isCachingEnabled: Boolean
    ): Boolean {
        return productViewProviderHelper.registerHookLogic(
            context,
            zone,
            viewId,
            additionalInfo,
            viewProvider,
            isCachingEnabled
        )
    }

    override fun validateQuantity(
        context: Context,
        item: IroProductDetailsPdp.Items,
        specialOffersTagIsClickable: Boolean,
        isBigIcon: Boolean,
        mode: Int,
        shopContext: ShopContext,
        quantityController: IQuantityController
    ): Pair<BigDecimal, BigDecimal> {
        return productHelper.validateQuantity(
            item,
            specialOffersTagIsClickable,
            mode,
            shopContext,
            quantityController
        )
    }

    override fun getCartAnalytics(
        context: Context,
        position: Int,
        activeSort: Int,
        searchInfoModel: SearchInfoModel,
        firstCartAdd: Boolean,
        shelfListItem: ShelfListItem,
        pageName: String?,
        categoryMerchandising: String?,
        offersForYouInfo: OffersForYouInfo?,
        favoriteInfo: FavoritesInfo?
    ): CartItemAnalyticsInfo {
        return productHelper.getCartAnalytics(
            context,
            position,
            activeSort,
            searchInfoModel,
            firstCartAdd,
            shelfListItem,
            pageName,
            categoryMerchandising,
            offersForYouInfo,
            favoriteInfo
        )
    }

    override fun getQuantityText(
        context: Context,
        item: IroProductDetailsPlp.Items,
        viewInfoModel: QuantityControllerViewInfoModel,
        isAvailable: Boolean
    ): String? {
        val shelfListItem = ShelfListItem.buildShelfObject(
            item,
            viewInfoModel.specialOffersTagIsClickable,
            viewInfoModel.isBigIcon
        )
        return getQuantityText(context, shelfListItem, viewInfoModel, isAvailable)
    }

    override fun getQuantityText(
        context: Context,
        item: IroProductDetailsPdp.Items,
        viewInfoModel: QuantityControllerViewInfoModel,
        isAvailable: Boolean
    ): String? {
        val shelfListItem =
            ShelfListItem.buildShelfObjectPdp(item, viewInfoModel.specialOffersTagIsClickable)
        return getQuantityText(context, shelfListItem, viewInfoModel, isAvailable)
    }

    private fun getQuantityText(
        context: Context,
        shelfListItem: ShelfListItem,
        viewInfoModel: QuantityControllerViewInfoModel,
        isAvailable: Boolean
    ): String? {
        var cartItem: Cart.CartItem? = null
        var cxoCartItem: com.asda.android.restapi.service.data.cart.Cart.CartItems? = null
        cartItem = getCartItemFrom(
            context,
            shelfListItem,
            viewInfoModel.mode,
            viewInfoModel.isShoppingList
        )
        if (viewInfoModel.isShoppingList) {
            cxoCartItem = cxoHelper.getCxoItemFromShelfItem(shelfListItem, true, cartItem)
        } else {
            cxoCartItem = cxoHelper.getItemInCartCXO(shelfListItem, false)
        }

        var quantityText = ""
        if (isAvailable || viewInfoModel.isShoppingList) {
            if (cxoCartItem?.quantity != null) {
                quantityText =
                    RestUtils.getCXOItemQuantity(cxoCartItem, viewInfoModel.isShoppingList, false)
            } else {
                cartItem?.let {
                    quantityText = cartItem.quantityString
                }
            }
            if (quantityText.isEmpty().not() && ProductDetails.PRICE_PER_WEIGHT_WEIGHTED.equals(
                    shelfListItem.pricePerWt,
                    ignoreCase = true
                )
            ) {
                quantityText += " kg"
            }
        } else {
            quantityText = context.getString(R.string.add)
        }
        if (quantityText.isEmpty()) {
            quantityText = context.getString(R.string.add)
        }
        return quantityText
    }

    override fun getFilterGroups(): ArrayList<FilterGroup> {
        return filterGroups
    }

    override fun setFilterGroups(filters: MutableList<FilterGroup>) {
        getFilterGroups().clear()
        getFilterGroups().addAll(filters)
    }

    override fun launchFilter(
        sourceFragment: Fragment?,
        requestCode: Int,
        sourceScreenName: String?
    ) {
        if (filterGroups.isNotEmpty()) {
            sourceFragment?.let {
                val intent = Intent(it.context, FiltersActivity::class.java)
                intent.putExtra(SOURCE_SCREEN_NAME, sourceScreenName)
                it.startActivityForResult(intent, requestCode)
            }
        } else {
            Log.e(TAG, "Please set filterGroups before launching FiltersActivity")
        }
    }

    override fun handleAlert(context: Context, code: Int) {
        if (code == CartManagerConstants.PRODUCT_SLOT_LIMIT_REACHED) {
            cxoHelper.showQuantityCheckDialog(context)
        }
    }

    override fun getItemInCartCXOFromItemId(itemId: String): com.asda.android.restapi.service.data.cart.Cart.CartItems? {
        return cxoHelper.getItemInCartCXOFromItemId(itemId)
    }

    private fun shouldReplacePDPFragment(sourceFragment: Fragment) = when (sourceFragment) {
        is ShelfFragment -> false
        is SearchResultsFragment -> true
        is HomeFragmentNew -> false
        is ViewAllTaxonomyFragment -> false
        is TopOffersFragment -> false
        is OffersForYouFragment -> false
        else -> true
    }

    override fun launchTopOffers(sourceFragment: Fragment, context: Context, id: String) {
        sourceFragment.tryCast<BaseFragment> {
            val fragment = AsdaProductsConfig.taxonomyManager.getTopOffersFragment(
                context as Activity,
                id,
                "",
                DeepLinkConstants.TYPE_TOP_OFFERS,
                null,
                null,
                null
            ) as BaseFragment
            val shopFragment = launchShopBottomTab(this)
            shopFragment?.tryCast<BaseFragment> {
                push(
                    fragment,
                    DeepLinkConstants.TYPE_TOP_OFFERS,
                    isReplace = shouldReplacePDPFragment(sourceFragment),
                    addToBackStack = true
                )
            } ?: run {
                MerchandisingHandler.handleCmsMerchandisingEvent(
                    context as Activity,
                    this.getCurrentFragmentManager(),
                    sourceFragment,
                    DeepLinkConstants.TYPE_TOP_OFFERS,
                    id,
                    null,
                    false,
                    null
                )
            }
        }
    }

    override fun isAvailable(availabilityInfo: IroProductDetailsPlp.AvailabilityInfo?): Boolean {
        return availabilityInfo?.availability != null && AvailabilityInfoValidator().getAvailability(
            availabilityInfo
        ) == AvailabilityTypes.AVAILABLE
    }

    override fun getProductTileGroupAdapter(
        sourceFragment: Fragment?,
        context: Context,
        width: Int,
        data: List<*>,
        additionalInfo: MutableMap<String, Any?>
    ): ProductTileGroupAdapter? {
        data.tryCast<IGroupAdapterData> {

            val infoModel = ProductViewInfoModel().apply {
                additionalInfo[EventConstants.SCREEN_NAME].tryCast<String> {
                    screenName = this
                }

                additionalInfo[EventConstants.SOURCE_PAGE].tryCast<String> {
                    sourcePage = this
                }

                additionalInfo[EventConstants.SOURCE_PAGE].tryCast<Int> {
                    maxPage = this
                }

                additionalInfo[EventConstants.CATEGORY_MERCHANDISING].tryCast<String> {
                    categoryMerchandising = this
                }
            }

            return ProductTileGroupAdapter(
                sourceFragment,
                context,
                width,
                toList(),
                infoModel,
                additionalInfo
            )
        }
        return null
    }

    override fun launchOffersForYou(sourceFragment: Fragment, context: Context, id: String) {
        sourceFragment.tryCast<BaseFragment> {
            val fragment = AsdaProductsConfig.taxonomyManager.getOffersForYouFragment(
                context as Activity,
                id,
                "",
                DeepLinkConstants.TYPE_OFFERS_FOR_YOU,
                null,
                null,
                null
            ) as BaseFragment
            val shopFragment = launchShopBottomTab(this)
            shopFragment?.tryCast<BaseFragment> {
                push(
                    fragment,
                    DeepLinkConstants.TYPE_OFFERS_FOR_YOU,
                    isReplace = shouldReplacePDPFragment(sourceFragment),
                    addToBackStack = true
                )
            } ?: run {
                MerchandisingHandler.handleCmsMerchandisingEvent(
                    context,
                    this.getCurrentFragmentManager(),
                    sourceFragment,
                    DeepLinkConstants.TYPE_OFFERS_FOR_YOU,
                    id,
                    null,
                    false,
                    null
                )
            }
        }
    }

    override fun areFiltersGroupsSame(
        currentFilters: ArrayList<FilterGroup>,
        lastSelected: List<FilterGroup>
    ): Boolean {
        val filterValidator = FilterValidator()
        return filterValidator.validate(listOf(currentFilters, lastSelected))
    }

    override fun getHorizontalTileWidth(context: Context) =
        context.resources.getDimensionPixelSize(com.asda.android.base.core.R.dimen.dp_160)

    override fun parseProductPromoInfo(items: IroProductDetailsPlp.Items): List<PromoTypeInfoModel> {
        return PromotionInfoFormatter().format(
            PromoViewInfoModel(
                items.promotionInfo,
                items.price,
                items.isBundle
            )
        )
    }

    override fun launchMobilePreview(activity: Activity, params: HashMap<String, String>) {
        val intent = Intent(activity, MobileAppPreviewActivity::class.java).apply {
            putExtra(MobileAppPreviewParser.PAGE_TYPE, params[MobileAppPreviewParser.PAGE_TYPE])
            putExtra(
                MobileAppPreviewParser.PREVIEW_DATE,
                params[MobileAppPreviewParser.PREVIEW_DATE]
            )
            putExtra(MobileAppPreviewParser.PAGE_ID, params[MobileAppPreviewParser.PAGE_ID])
            // need to add more params based on requirements
        }
        activity.startActivity(intent)
    }

    override fun isSample(context: Context, itemId: String): Observable<Boolean> {
        val result: BehaviorSubject<Boolean> = BehaviorSubject.create()
        val disposable = CompositeDisposable()
        disposable.add(
            PersonalisedSamplesRepository.getPersonalisedSamples(context).subscribe { response ->
                val items = response.data?.personalisedSamples?.products?.items
                if (items == null) {
                    result.onNext(false)
                    return@subscribe
                }
                result.onNext(items.any { item -> itemId == item.itemId })
                disposable.clear()
            }
        )
        return result
    }

    override fun isSample(
        data: PersonalisedSampleResponse,
        itemIds: List<Items>
    ): HashMap<String, Boolean> {
        val result = HashMap<String, Boolean>()
        itemIds.forEach {
            result[it.sku_id] = (data.data?.personalisedSamples?.products?.items?.any { item -> item.itemId == it.sku_id } == true)
        }
        return result
    }

    override fun getPersonalisedSamples(context: Context): Observable<PersonalisedSampleResponse> {
        return PersonalisedSamplesRepository.getPersonalisedSamples(context)
    }
}
