package com.asda.android.app.shop.taxonomy.view

import android.app.Activity
import androidx.fragment.app.Fragment
import com.asda.android.R
import com.asda.android.analytics.AsdaAnalyticsEvent
import com.asda.android.app.main.AsdaApplication
import com.asda.android.app.main.view.AsdaActivity
import com.asda.android.app.shop.ShelfRecipesTabbedFragment
import com.asda.android.base.interfaces.ISearchManager
import com.asda.android.restapi.search.model.AsdaSearchData
import com.asda.android.taxonomy.features.taxonomy.formatter.TaxonomyPathEndFormatter

object SearchManager : ISearchManager {

    private val taxonomyPathEndFormatter = TaxonomyPathEndFormatter()

    override fun onSearchRequested(
        searchData: AsdaSearchData,
        taxonomyPath: String
    ): Pair<Fragment, String> {

        val fragmentBase =
            ShelfRecipesTabbedFragment.newInstance(
                searchData,
                false
            )
        fragmentBase.setHeaderText(
            AsdaApplication.getComponent().application.getString(
                R.string.search_title,
                searchData.searchText
            )
        )
        fragmentBase.setTaxonomy(getTaxonomy(taxonomyPath))
        return Pair(fragmentBase, getBreadCrumbTitle(searchData))
    }

    private fun getTaxonomy(taxonomyPath: String): String? {
        return when (val searchPage = "search") {
            taxonomyPathEndFormatter.format(taxonomyPath) -> taxonomyPath
            else -> AsdaAnalyticsEvent.concatStringArgs(taxonomyPath, searchPage)
        }
    }

    private fun getBreadCrumbTitle(searchData: AsdaSearchData): String {
        return when {
            searchData.isBarcodeScanSearch -> AsdaApplication.getComponent().application
                .getString(R.string.scan_barcode_shelf_header_text)
            else -> AsdaApplication.getComponent().application
                .getString(R.string.search_title, searchData.searchText)
        }
    }

    override fun isInMainActivity(activity: Activity?): Boolean {
        return activity is AsdaActivity
    }
}
