package com.asda.android.app.shop.taxonomy.view

import android.app.Activity
import android.content.Context
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentManager
import com.asda.android.analytics.info.OffersForYouInfo
import com.asda.android.app.shop.productdetail.view.ProductManager.launchShopBottomTab
import com.asda.android.app.shop.view.MerchandisingHandler
import com.asda.android.base.core.constants.DeepLinkConstants
import com.asda.android.base.core.constants.DeepLinkConstants.TYPE_OFFERS_FOR_YOU
import com.asda.android.base.core.model.DeepLinkUrlParserInputModel
import com.asda.android.base.core.parser.DeepLinkUrlIdParser
import com.asda.android.base.core.parser.DeepLinkUrlParser
import com.asda.android.base.core.utils.AsdaBaseCoreConfig
import com.asda.android.base.core.utils.tryCast
import com.asda.android.base.core.view.fragments.BaseFragment
import com.asda.android.base.core.view.fragments.FeaturedFragment
import com.asda.android.base.interfaces.ITaxonomyManager
import com.asda.android.restapi.cms.model.Refinement
import com.asda.android.restapi.cms.model.Zone
import com.asda.android.restapi.constants.EventConstants
import com.asda.android.restapi.constants.EventConstants.IS_SPECIAL_OFFER
import com.asda.android.restapi.service.data.cms.Refinement.TYPE_SPECIAL_OFFERS
import com.asda.android.restapi.service.data.cms.Refinement.TYPE_TOP_OFFERS
import com.asda.android.taxonomy.AsdaTaxonomyConfig
import com.asda.android.taxonomy.features.shop.view.OldShopFragment
import com.asda.android.taxonomy.features.shop.view.ShopFragment
import com.asda.android.taxonomy.features.taxonomy.constants.TaxonomyConstants.CATEGORY_ID
import com.asda.android.taxonomy.features.taxonomy.constants.TaxonomyConstants.EVENT_PAGE_URL
import com.asda.android.taxonomy.features.taxonomy.constants.TaxonomyConstants.LINK_TYPE_SPECIAL_OFFERS
import com.asda.android.taxonomy.features.taxonomy.formatter.OffersForYouInfoFromZonesFormatter
import com.asda.android.taxonomy.features.taxonomy.helper.view.TaxonomyViewHelper
import com.asda.android.taxonomy.features.taxonomy.model.TaxonomyViewInfoModel

object TaxonomyManager : ITaxonomyManager {
    private val deepLinkUrlParser = DeepLinkUrlParser()
    private val deepLinkUrlIdParser = DeepLinkUrlIdParser()

    override fun handleClickEvent(activity: Activity?, fragmentManager: FragmentManager?, sourceFragment: Fragment, additionalInfo: MutableMap<String, Any?>) {
        if (activity == null || fragmentManager == null) {
            return
        }
        var type: String? = null
        var id: String? = null
        var campaignId: String? = null
        var isTempoUrl = false
        additionalInfo[EventConstants.PAGE_ID].tryCast<String> { type = this }
        additionalInfo[EventConstants.DIMENSION_ID].tryCast<String> { id = this }
        additionalInfo[EventConstants.CAMPAIGN_ID].tryCast<String> { campaignId = this }
        additionalInfo[EventConstants.IS_TEMPO_URL].tryCast<Boolean> { isTempoUrl = this }

        MerchandisingHandler.handleCmsMerchandisingEvent(activity, fragmentManager, sourceFragment, type, id, campaignId, isTempoUrl, null)
    }

    override fun getTopOffersFragment(
        context: Context,
        pageId: String?,
        queryString: String?,
        taxonomy: String,
        source: Fragment?,
        refinements: ArrayList<Refinement>?,
        additionalInfo: Map<String, String?>?
    ): BaseFragment? {
        var taxonomyRefinements = refinements
        if (taxonomyRefinements == null) {
            taxonomyRefinements = ArrayList()
            taxonomyRefinements.add(Refinement(type = TYPE_TOP_OFFERS))
        }
        return getTaxonomyFragmentDetails(context, taxonomyRefinements, additionalInfo)?.second
    }

    override fun processArguments(fragment: Fragment?) {
        fragment.tryCast<FeaturedFragment> {
            MerchandisingHandler.processArguments(this)
        }
    }

    override fun launchTaxonomy(source: Fragment?, refinements: ArrayList<Refinement>?, additionalInfo: Map<String, String?>?) {

        if (refinements?.get(0)?.name?.equals(DeepLinkConstants.TYPE_RECIPES, true) == true && source != null && source.activity != null) {
            MerchandisingHandler.handleCmsMerchandisingEvent(source.activity, null, null, DeepLinkConstants.TYPE_RECIPES, null, null, false, null)
            return
        }

        val fragment: BaseFragment?
        val title: String?

        val taxonomyFragmentDetails = getTaxonomyFragmentDetails(source?.context, refinements, additionalInfo)

        title = taxonomyFragmentDetails?.first
        fragment = taxonomyFragmentDetails?.second

        source.tryCast<BaseFragment> {
            if (fragment != null) {
                val shopFragment = launchShopBottomTab(source)
                shopFragment?.tryCast<BaseFragment> {
                    push(fragment, title)
                } ?: run {
                    this.push(fragment, title)
                }
            } else {
                val url = refinements?.last()?.url
                val type = deepLinkUrlParser.parse(DeepLinkUrlParserInputModel(url, null))
                val urlDetails = deepLinkUrlIdParser.parse(DeepLinkUrlParserInputModel(url, null))
                val id = urlDetails.first
                val infoMap = mutableMapOf<String, Any?>()
                infoMap[EventConstants.PAGE_ID] = type
                infoMap[EventConstants.DIMENSION_ID] = id
                infoMap[EventConstants.CAMPAIGN_ID] = additionalInfo?.get(EventConstants.CAMPAIGN_ID)
                infoMap[EventConstants.IS_TEMPO_URL] = urlDetails.second
                handleClickEvent(this.activity, this.getCurrentFragmentManager(), this, infoMap)
            }
        }
    }

    override fun launchShop(source: Fragment?) {
        source.tryCast<BaseFragment> {
            this.context?.let {
                if (AsdaBaseCoreConfig.featureSettingManager.isNewTaxonomyEnabled(it)) {
                    this.pushNoAnimation(ShopFragment(), false)
                } else {
                    this.pushNoAnimation(OldShopFragment(), false)
                }
            }
        }
    }

    fun getTaxonomyFragmentDetails(context: Context?, refinements: ArrayList<Refinement>?, additionalInfo: Map<String, String?>?): Pair<String?, BaseFragment?>? {
        refinements?.last()?.let {
            return it.name to getTaxonomyFragment(context, it, refinements, additionalInfo)
        }
        return null
    }

    private fun getTaxonomyFragment(context: Context?, refinement: Refinement, refinements: ArrayList<Refinement>?, additionalInfo: Map<String, String?>?): BaseFragment? {
        val hierarchyId = refinement.hierarchyId
        val taxonomyId = refinement.taxonomyId
        val storeId = AsdaTaxonomyConfig.asdaService.lastStoreIdFromSessionMetadata
        val categoryId = additionalInfo?.get(CATEGORY_ID)
        val taxonomyPath = getTaxonomyPathFromRefinements(refinements)
        val campaignId = additionalInfo?.get(EventConstants.CAMPAIGN_ID)
        val specialOffer = additionalInfo?.get(IS_SPECIAL_OFFER).orEmpty()
        var eventPageUrl = additionalInfo?.get(EVENT_PAGE_URL)
        if (refinement.linkType == DeepLinkConstants.TYPE_EVENT && !refinement.url.isNullOrEmpty() && eventPageUrl.isNullOrEmpty()) {
            eventPageUrl = refinement.url
        }
        val taxonomyViewInfoModel = TaxonomyViewInfoModel(
            hierarchyId = hierarchyId,
            taxonomyId = taxonomyId,
            title = refinement.name,
            storeId = storeId,
            taxonomyPath = taxonomyPath,
            bannerCampaignId = campaignId,
            categoryId = categoryId,
            isSpecialOffers = specialOffer.toBoolean() || refinement.linkType?.equals(LINK_TYPE_SPECIAL_OFFERS, true) == true,
            eventPageUrl = eventPageUrl,
            refinements = refinements,
            pageName = additionalInfo?.get(EventConstants.PAGE_NAME),
            linkType = refinement.linkType,
            childCount = refinement.childCount,
            pageType = additionalInfo?.get(EventConstants.PAGE_TYPE)
        )
        val type = if (refinement.name?.toLowerCase() == SPECIAL_OFFERS.toLowerCase()) TYPE_SPECIAL_OFFERS else refinement.type
        return TaxonomyViewHelper().getFragment(taxonomyViewInfoModel, type)
    }

    private fun getTaxonomyPathFromRefinements(refinements: ArrayList<Refinement>?): String? {
        val taxonomyPath = refinements?.map { it.name }?.joinToString(separator = " > ")
        return taxonomyPath?.let {
            return "shop > $it"
        }
    }

    private const val SPECIAL_OFFERS = "Special Offers"

    override fun getOffersForYouFragment(
        context: Context,
        pageId: String?,
        queryString: String?,
        taxonomy: String,
        source: Fragment?,
        refinements: ArrayList<Refinement>?,
        additionalInfo: Map<String, String?>?
    ): BaseFragment? {
        var taxonomyRefinements = refinements
        if (taxonomyRefinements == null) {
            taxonomyRefinements = ArrayList()
            taxonomyRefinements.add(Refinement(type = TYPE_OFFERS_FOR_YOU))
        }
        return getTaxonomyFragmentDetails(context, taxonomyRefinements, additionalInfo)?.second
    }

    override fun getOffersForYouStateFromZones(zones: List<Zone>?): OffersForYouInfo? {
        return getOffersForYouInfoFromZonesFormatter().format(zones, null)
    }

    private fun getOffersForYouInfoFromZonesFormatter(): OffersForYouInfoFromZonesFormatter {
        return OffersForYouInfoFromZonesFormatter()
    }
}
