package com.asda.android.app.shoppinglists

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.util.Log
import android.view.MenuItem
import android.view.View
import androidx.annotation.IntDef
import com.asda.android.BuildConfig
import com.asda.android.R
import com.asda.android.app.shop.constants.ShopContextModule
import com.asda.android.base.core.constants.ActivityRequestCodes
import com.asda.android.base.core.view.actitivies.AsdaAppCompatActivity
import com.asda.android.base.interfaces.ShopContext
import com.asda.android.products.ui.AsdaProductsConfig
import com.asda.android.utils.view.ViewUtils
import java.lang.annotation.Retention
import java.lang.annotation.RetentionPolicy

class ShoppingListActivity : AsdaAppCompatActivity() {
    private var mShopContext: ShopContext? = null

    @IntDef(ListType.USER_GENERATED, ListType.SYSTEM_GENERATED)
    @Retention(RetentionPolicy.SOURCE)
    annotation class ListType {
        companion object {
            const val USER_GENERATED = 1
            const val SYSTEM_GENERATED = 2
        }
    }

    public override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.container_fragment)
        mShopContext = AsdaProductsConfig.shopContext(true)
        ShopContextModule.setShopContext(mShopContext)
        if (savedInstanceState == null) {
            processIntent(intent)
        }
    }

    override fun onResume() {
        super.onResume()
        ShopContextModule.setShopContext(mShopContext)
    }

    override fun onPause() {
        super.onPause()
        ShopContextModule.setShopContext(null)
    }

    public override fun onNewIntent(intent: Intent) {
        super.onNewIntent(intent)
        processIntent(intent)
    }

    private fun processIntent(intent: Intent) {
        val mAction = intent.getStringExtra(EXTRA_ACTION)
        val listId = intent.getStringExtra(EXTRA_LIST_ID)
        val listName = intent.getStringExtra(EXTRA_LIST_NAME)
        val listType = intent.getIntExtra(EXTRA_LIST_TYPE, ListType.USER_GENERATED)
        if (VIEW_LIST == mAction) {
            mShopContext?.setSelectedListId(listId)
            push(ShoppingListViewFragment.newInstance(listId, listName, listType))
            showHeaderButtons()
        } else if (EDIT_LISTS == mAction) {
            push(EditShoppingListsFragment())
        } else {
            if (BuildConfig.DEBUG) {
                Log.e(TAG, "No known action specified")
            }
            finish()
        }
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item != null && item.itemId == android.R.id.home) {
            onBackPressed()
            return true
        }
        return super.onOptionsItemSelected(item)
    }

    override fun getFragmentContainer(): Int {
        return R.id.fragment_container
    }

    companion object {
        private val TAG = "ShoppingListActivity"

        private val EXTRA_ACTION = "extra_action"
        private val EXTRA_LIST_ID = "extra_list_id"
        private val EXTRA_LIST_NAME = "extra_list_name"
        private val EXTRA_LIST_TYPE = "extra_list_type"
        private val VIEW_LIST = "view_list"
        private val EDIT_LISTS = "edit_lists"

        fun launchViewList(
            activity: Activity,
            listId: String,
            listName: String,
            view: View?,
            @ListType listType: Int
        ) {
            val intent = Intent(activity, ShoppingListActivity::class.java)
            intent.putExtra(ShoppingListActivity.EXTRA_ACTION, VIEW_LIST)
            intent.putExtra(ShoppingListActivity.EXTRA_LIST_ID, listId)
            intent.putExtra(ShoppingListActivity.EXTRA_LIST_NAME, listName)
            intent.putExtra(ShoppingListActivity.EXTRA_LIST_TYPE, listType)
            ViewUtils.startActivityForResult(
                activity, intent, ActivityRequestCodes.REQUEST_CODE_REORDER, view
            )
        }

        fun launchListsEditor(activity: Activity, view: View?) {
            val intent = Intent(activity, ShoppingListActivity::class.java)
            intent.putExtra(EXTRA_ACTION, EDIT_LISTS)
            ViewUtils.startActivity(activity, intent, view)
        }
    }
}
