package com.asda.android.app.storelocator

import androidx.annotation.VisibleForTesting
import java.lang.ref.WeakReference
import java.util.*

open class FilterManager() {
    var mEditor: HashMap<String, Boolean>? = null
    val mPrefs = HashMap<String, Boolean>()
    val mActiveFilters = ArrayList<String>()
    var mListenerRef: WeakReference<Listener>? = null
    fun setListener(listener: Listener) {
        mListenerRef = WeakReference(listener)
    }

    open fun startEditing() {
        mEditor = HashMap()
    }

    open fun cancelEditing() {
        mEditor = null
    }

    open fun stopEditingAndSave() {
        mEditor?.let {
            var filterChanged = false
            for ((key, value) in it) {
                if (mPrefs.containsKey(key) && value != mPrefs[key]) {
                    filterChanged = true
                    mPrefs[key] = value
                }
            }
            updateActiveFilter()
            if (filterChanged) {
                getListener()?.onFilterSettingsUpdated()
            }
        }
    }

    @VisibleForTesting
    fun getListener(): Listener? {
        return mListenerRef?.get()
    }

    @VisibleForTesting
    fun updateActiveFilter() {
        mActiveFilters.clear()
    }

    open fun clearAll() {
        val hadFilters = mActiveFilters.isEmpty()
        mActiveFilters.clear()
        mPrefs.clear()
        if (hadFilters) {
            getListener()?.onFilterSettingsUpdated()
        }
    }

    open fun setServiceCheck(i: Int, check: Boolean) {
        // Implementation not required
    }

    interface Listener {
        fun onFilterSettingsUpdated()
    }

    open fun getActiveFilters(): ArrayList<String> {
        return mActiveFilters
    }

    open fun getServicesNames(): Array<String?> {
        return arrayOfNulls(0)
    }

    open fun isFilterActive(): Boolean {
        return mActiveFilters.isNotEmpty()
    }

    open fun getServicesChecks(): BooleanArray {
        return BooleanArray(0)
    }
}
