package com.asda.android.app.storelocator

import androidx.annotation.VisibleForTesting
import com.asda.android.app.storelocator.model.StoreData
import com.asda.android.base.core.utils.orZero
import com.google.android.gms.maps.model.LatLng
import java.util.*

class StoreManager(val mFilterManager: FilterManager?) {
    var mStoreList: ArrayList<StoreData>? = null
    var mFilteredStoreList: ArrayList<StoreData>? = null
    var mUserLocation: LatLng? = null
    private fun init() {
        mStoreList = ArrayList()
        mFilteredStoreList = ArrayList()
    }

    val isFilterActive: Boolean
        get() = mFilterManager != null && mFilterManager.isFilterActive()

    fun clear() {
        mStoreList?.clear()
        mFilteredStoreList?.clear()
    }

    fun size(): Int {
        return if (mFilterManager != null && mFilterManager.isFilterActive()) {
            mFilteredStoreList?.size.orZero()
        } else {
            mStoreList?.size.orZero()
        }
    }

    fun setStores(stores: Array<StoreData>) {
        mStoreList?.clear()
        addStores(stores)
    }

    fun addStores(stores: Array<StoreData>) {
        mFilteredStoreList?.clear()
        for (store1 in stores) {
            if (mUserLocation != null) {
                store1.setSourceCoordinates(mUserLocation?.latitude.orZero(), mUserLocation?.longitude.orZero())
            }
        }
        for (store in stores) {
            mStoreList?.add(store)
            if (mFilterManager != null) {
                val activeFilter: List<String> = mFilterManager.getActiveFilters()
                addIfPassFilter(store, activeFilter)
            }
        }
    }

    fun getStore(i: Int): StoreData? {
        return if (mFilterManager != null && mFilterManager.isFilterActive()) {
            if (i < 0 || i >= mFilteredStoreList?.size.orZero()) null else mFilteredStoreList?.get(i)
        } else {
            if (i < 0 || i >= mStoreList?.size.orZero()) null else mStoreList?.get(i)
        }
    }

    val closestStore: StoreData?
        get() = if (mFilterManager != null && mFilterManager.isFilterActive()) {
            getClosestStore(mFilteredStoreList)
        } else {
            getClosestStore(mStoreList)
        }

    @VisibleForTesting
    fun getClosestStore(storeList: ArrayList<StoreData>?): StoreData? {
        var distance = Int.MAX_VALUE.toDouble()
        var store: StoreData? = null
        storeList?.forEach { tmpStore ->
            if (tmpStore.distanceFromSource < distance) {
                distance = tmpStore.distanceFromSource
                store = tmpStore
            }
        }
        return store
    }

    fun updateFilter() {
        if (mFilterManager != null) {
            val filters: List<String> = mFilterManager.getActiveFilters()
            mFilteredStoreList?.clear()
            mStoreList?.forEach { store ->
                addIfPassFilter(store, filters)
            }
        }
    }

    @VisibleForTesting
    fun addIfPassFilter(store: StoreData, filters: List<String>) {
        var hasAllServices = true
        for (filter in filters) {
            if (!store.hasService(filter)) {
                hasAllServices = false
                break
            }
        }
        if (hasAllServices) {
            mFilteredStoreList?.add(store)
        }
    }

    fun setUserLocation(currentUserLocation: LatLng?) {
        mUserLocation = currentUserLocation
    }

    init {
        init()
    }
}
