package com.asda.android.app.storelocator

import android.util.Log
import java.util.*
import java.util.regex.Pattern

class TimeOfDay(hour: Int, minute: Int, amPm: Int) {
    private val mHour: Int
    private val mMinute: Int
    private val mAmPm: Int
    fun isEarlierThan(compare: TimeOfDay): Boolean {
        if (mAmPm == compare.mAmPm) {
            return (
                mHour < compare.mHour ||
                    mHour == compare.mHour && mMinute < compare.mMinute
                )
        } else if (mAmPm == Calendar.AM && compare.mAmPm == Calendar.PM) {
            return true
        }
        return false
    }

    override fun toString(): String {
        var hour = mHour
        var minute = mMinute
        var amPm = mAmPm
        if (hour == 11 && minute == 59 && amPm == Calendar.PM) {
            hour = 0
            minute = 0
            amPm = Calendar.AM
        }
        return if (minute != 0) {
            String.format(
                Locale.US,
                "%d.%d%s",
                if (hour == 0) 12 else hour,
                minute,
                if (amPm == Calendar.AM) "am" else "pm"
            ) // UK locale doesn't use AM/PM
        } else {
            String.format(
                Locale.US, "%d%s", if (hour == 0) 12 else hour, if (amPm == Calendar.AM) "am" else "pm"
            )
        }
    }

    companion object {
        // 8am
        // 8.30am
        // 8:30am
        private const val REG_EXP_TIME = "^(\\d+)[.:]?(\\d*)([ap]m)$"
        private val TIME_PATTERN = Pattern.compile(REG_EXP_TIME)
        private const val GROUP_HOURS = 1
        private const val GROUP_MINUTES = 2
        private const val GROUP_AM_PM = 3
        fun createFromString(inputTime: String): TimeOfDay? {
            var time = inputTime
            var hour: Int
            var minute: Int
            var amPm: Int
            try {
                if (time.equals("Midnight", ignoreCase = true)) {
                    time = "11.59pm"
                }
                val matcher = TIME_PATTERN.matcher(time)
                if (matcher.matches()) {
                    hour = matcher.group(GROUP_HOURS).toInt()
                    minute = if (matcher.group(GROUP_MINUTES).isNotEmpty()) {
                        matcher.group(GROUP_MINUTES).toInt()
                    } else {
                        0
                    }
                    amPm = if (matcher.group(GROUP_AM_PM).startsWith("a")) {
                        Calendar.AM
                    } else {
                        Calendar.PM
                    }

                    // 12.00 am -> 11.59 pm (since 12am is actually the day after)
                    if (hour == 12 && minute == 0 && amPm == Calendar.AM) {
                        hour = 11
                        minute = 59
                        amPm = Calendar.PM
                    }
                    return TimeOfDay(hour, minute, amPm)
                }
            } catch (e: NumberFormatException) {
                Log.w("TimeOfDay", e)
                return null
            }
            return null
        }
    }

    /**
     * @param hour 1-12
     * @param minute 0-59
     * @param amPm Calendar.AM or Calendar.PM
     */
    init {
        var hour = hour
        if (hour == 12) {
            hour = 0
        }
        mHour = hour
        mMinute = minute
        mAmPm = amPm
    }
}
