package com.asda.android.app.view

import android.app.Activity
import android.content.Context
import android.widget.ArrayAdapter
import androidx.fragment.app.Fragment
import com.asda.android.app.checkout.OrderApplyItemsActivity
import com.asda.android.app.home.view.RegisterHelper
import com.asda.android.app.main.view.AsdaActivity
import com.asda.android.app.shoppinglists.ShoppingListActivity
import com.asda.android.base.core.utils.tryCast
import com.asda.android.base.core.view.fragments.BaseFragment
import com.asda.android.base.interfaces.IAsdaViewHelper
import com.asda.android.base.interfaces.RegisterStateCallback
import com.asda.android.restapi.deliverypass.GenericCardCheckoutModel
import com.asda.android.restapi.service.RestUtils
import com.asda.android.restapi.service.data.CheckoutDeliveryPassResponse
import com.asda.android.restapi.service.data.PaymentDetailsResponse
import com.asda.android.restapi.service.data.PostcodeCheckResponse
import com.asda.android.restapi.service.data.PostcodeCheckResponse.AddressSuggestion
import com.asda.android.service.EventUtils
import com.asda.android.slots.deliverypass.model.DPCardCheckoutModel
import io.reactivex.Observable
import io.reactivex.functions.BiFunction

class AsdaViewHelper : IAsdaViewHelper {
    override fun onDiscardSelected(activity: Activity?, fragment: Fragment) {
        if (activity is AsdaActivity && RestUtils.isTablet(activity)) {
            val fragmentSwitcher = activity.getFragmentSwitcher()
            if (fragmentSwitcher != null && fragmentSwitcher.hasPending()) {
                fragmentSwitcher.switchToPending()
            }
        }
        if (activity is OrderApplyItemsActivity) {
            activity.popPresenter()
        }
        (activity as? ShoppingListActivity)?.finish()
        fragment.tryCast<BaseFragment> {
            this.pop()
        }
    }

    override fun getExpiryMaxFutureYears(): Int {
        return io.card.payment.CreditCard.EXPIRY_MAX_FUTURE_YEARS
    }

    override fun showEnterPostCodeDialog(context: Context?, title: String?, message: String?, buttonResId: Int, onPostcodeCheckPassed: RegisterStateCallback) {
        RegisterHelper.showEnterPostCodeDialog(context, title, message, buttonResId, onPostcodeCheckPassed)
    }

    override fun dontDeliver(data: PostcodeCheckResponse?): Boolean {
        return RegisterHelper.dontDeliver(data)
    }
    override fun broadcast(context: Context?, action: String) {
        EventUtils.broadcast(context, action)
    }
    override fun createAddressSpinnerAdapter(
        context: Context?,
        addressSuggestions: Array<AddressSuggestion?>?
    ): ArrayAdapter<CharSequence?>? {
        return RegisterHelper.createAddressSpinnerAdapter(context, addressSuggestions)
    }

    override fun chainLoadCreditCardsDP(creditCardObservable: Observable<PaymentDetailsResponse?>?, checkoutObservable: Observable<CheckoutDeliveryPassResponse?>?): Observable<GenericCardCheckoutModel?> {
        return Observable.zip(
            creditCardObservable,
            checkoutObservable,
            BiFunction<PaymentDetailsResponse?, CheckoutDeliveryPassResponse?, GenericCardCheckoutModel?> { creditCardResponse: PaymentDetailsResponse?, checkoutResponse: CheckoutDeliveryPassResponse? ->
                DPCardCheckoutModel(creditCardResponse, null, checkoutResponse)
            }
        )
    }
}
