package com.asda.android.app.view

import android.content.Context
import android.content.res.TypedArray
import android.util.Log
import android.util.TypedValue
import android.view.Gravity
import android.view.View
import android.view.ViewGroup.MarginLayoutParams
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.appcompat.app.ActionBar
import androidx.appcompat.app.AppCompatActivity
import androidx.drawerlayout.widget.DrawerLayout
import androidx.fragment.app.FragmentManager
import com.asda.android.R
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.events.PageViewEvent
import com.asda.android.analytics.events.exception.CaughtExceptionEvent
import com.asda.android.app.shop.view.CXOHelper
import com.asda.android.base.core.utils.A11yUtil
import com.asda.android.base.core.utils.AsdaBaseCoreConfig
import com.asda.android.base.core.utils.tryCast
import com.asda.android.base.core.view.CartAnimationView
import com.asda.android.base.interfaces.IDrawerManager
import com.asda.android.base.interfaces.IDrawerManager.CartStateListener
import com.asda.android.base.interfaces.TransitionListener
import com.asda.android.cxo.view.fragments.CXOCartFragment
import com.asda.android.cxo.view.fragments.CXOCartPagerFragment
import com.asda.android.cxo.view.fragments.CartFragmentListener
import com.asda.android.utils.view.ViewUtil
import java.lang.reflect.Field

object DrawerManager : IDrawerManager {
    override fun getAppName(context: Context?): Int {
        return R.string.app_name
    }

    private var mCXOCartFragment: CXOCartFragment? = null
    private var mCartView: View? = null

    private var mCartCountViewInTrolley: TextView? = null
    private var mCartPriceViewInTrolley: TextView? = null
    private var mCartUpdateAnimationView: CartAnimationView? = null
    private var mCartInfoContainerInTrolley: LinearLayout? = null
    private var mCartCloseButton: ImageView? = null
    private var mToolbar: ActionBar? = null
    override fun initializeCartView(mContext: AppCompatActivity?, mDrawerLayout: DrawerLayout?, transitionListener: TransitionListener, drawerInfoChangeListener: IDrawerManager.DrawerInfoChangeListener) {
        mCartView = mDrawerLayout?.findViewById(R.id.cxo_right_drawer)
        val cxoCartPagerFragment = mContext?.supportFragmentManager?.findFragmentById(R.id.cxo_right_drawer) as CXOCartPagerFragment
        cxoCartPagerFragment.setCartFragmentListener(object : CartFragmentListener {
            override fun onCartFragmentCreated(fragment: CXOCartFragment) {
                mCXOCartFragment = fragment
                mCXOCartFragment?.transitionListener = transitionListener
            }
        })

        mCartView?.let {
            mCartInfoContainerInTrolley = it.findViewById(R.id.trolley_signed_in)
            mCartCountViewInTrolley = it.findViewById(R.id.cart_count_in_trolley)
            mCartPriceViewInTrolley = it.findViewById(R.id.cart_total_price_in_trolley)
            mCartUpdateAnimationView = it.findViewById(R.id.cart_animation_in_drawer)
            mCartCloseButton = it.findViewById(R.id.cartCloseButton)
            mCartCloseButton?.setOnClickListener { toggleCart(drawerInfoChangeListener) }
        }
    }

    override fun toggleCart(drawerInfoChangeListener: IDrawerManager.DrawerInfoChangeListener) {
        if (!drawerInfoChangeListener.getCartEnabled()) {
            return
        }
        if (drawerInfoChangeListener.getCartLock()) {
            drawerInfoChangeListener.getOnCartChangeListener().setCartLocked(false)
        } else if (drawerInfoChangeListener.getCartLockInitialState() == true && mCartView != null && drawerInfoChangeListener.getDrawerLayout().isDrawerVisible(mCartView!!)) {
            drawerInfoChangeListener.getOnCartChangeListener().setCartLocked(true)
            drawerInfoChangeListener.getOnSyncListener().onSync()
        } else if (mCartView != null && drawerInfoChangeListener.getDrawerLayout().isDrawerVisible(mCartView!!)) {
            drawerInfoChangeListener.getOnCartChangeListener().closeCart()
        } else {
            drawerInfoChangeListener.getOnCartChangeListener().openCart()
        }
    }

    override fun setUpActionBar(mContext: AppCompatActivity?, mDrawerLayout: DrawerLayout?) {
        val toolbar = mContext?.supportActionBar
        toolbar?.let {
            toolbar.setDisplayShowCustomEnabled(true)
            toolbar.setCustomView(
                mContext.layoutInflater.inflate(R.layout.asda_header, mDrawerLayout, false),
                ActionBar.LayoutParams(
                    ActionBar.LayoutParams.WRAP_CONTENT,
                    ActionBar.LayoutParams.MATCH_PARENT,
                    Gravity.END
                )
            )
        }
        mToolbar = toolbar
    }

    override fun onDrawerClosed(view: View?) {
        CXOHelper().toggleCartFragment(false)
    }

    override fun onDrawerOpened(view: View?) {
        CXOHelper().toggleCartFragment(true)
    }

    override fun isCartView(view: View?): Boolean {
        return view == mCartView
    }

    override fun setCartInfo(context: Context?, value: String?, count: String?) {
        ViewUtil.setValueWithCurrency(R.id.cart_total_price_in_trolley, mCartView, value, R.string.total_trolley_price)
        mCartCountViewInTrolley!!.text = count
        mCartInfoContainerInTrolley!!.contentDescription = context?.let { A11yUtil().combineTrollyInfo(it, value!!, count!!) }
    }

    override fun getCartView(): View? {
        return mCartView
    }

    override fun getCartCountViewInTrolley(): TextView? {
        return mCartCountViewInTrolley
    }

    override fun getCartPriceViewInTrolley(): TextView? {
        return mCartPriceViewInTrolley
    }

    override fun getCartInfoContainerInTrolley(): LinearLayout? {
        return mCartInfoContainerInTrolley
    }

    override fun getCartCloseButton(): ImageView? {
        return mCartCloseButton
    }

    override fun getToolbar(): ActionBar? {
        return mToolbar
    }

    override fun getFragmentManager(): FragmentManager? {
        if (mCXOCartFragment?.onBackPressed() == true) {
            return mCXOCartFragment?.getCurrentFragmentManager()
        }
        return null
    }

    override fun openCart(mCartEnabled: Boolean, mCartLock: Boolean, mDrawerLayout: DrawerLayout) {
        if (!mCartEnabled) {
            return
        }
        if (mCartLock) {
            return
        }
        mCartView?.let {
            if (!mDrawerLayout.isDrawerVisible(it)) {
                mDrawerLayout.openDrawer(it)
                AsdaBaseCoreConfig.tracker?.trackPageView(PageViewEvent(Anivia.PV_TROLLEY, Anivia.SECTION_TROLLEY, Anivia.SECTION_TROLLEY))
            }
        }
    }

    override fun closeCart(mCartLock: Boolean, mCartEnabled: Boolean, mDrawerLayout: DrawerLayout, animate: Boolean, onSyncListener: IDrawerManager.OnSyncListener): Boolean {
        if (!mCartEnabled) {
            return false
        }
        if (mCartLock) {
            return false
        }
        mCartView?.let {
            if (mDrawerLayout.isDrawerVisible(it)) {
                if (animate) {
                    mDrawerLayout.closeDrawer(it)
                } else {
                    val cartParams = mCartView!!.layoutParams as DrawerLayout.LayoutParams
                    setClose(cartParams, onSyncListener)
                    mCartView!!.layoutParams = cartParams
                }
                return true
            }
        }

        return false
    }

    private fun setClose(param: DrawerLayout.LayoutParams, onSyncListener: IDrawerManager.OnSyncListener) {
        val c: Class<*> = param.javaClass
        var field: Field
        try {
            field = c.getDeclaredField("onScreen")
            field.isAccessible = true
            field[param] = 0f
            field = c.getDeclaredField("openState")
            field.isAccessible = true
            field[param] = 0x4
        } catch (e: Exception) {
            Log.e("AsdaDrawerToggle", "setClose fail", e)
            AsdaBaseCoreConfig.tracker?.trackException(CaughtExceptionEvent(e))
        }
        onSyncListener.onSync()
    }

    override fun setCartLocked(lock: Boolean, mMainView: View, mBreadcrumbs: View?, mDrawerLayout: DrawerLayout, mContext: AppCompatActivity, mCartEnabled: Boolean) {
        var cartParams = mCartView?.layoutParams as DrawerLayout.LayoutParams
        val mainParams = mMainView.layoutParams as MarginLayoutParams
        var breadcrumbsParams: MarginLayoutParams? = null
        mBreadcrumbs?.layoutParams?.tryCast<MarginLayoutParams> { breadcrumbsParams = this }
        if (lock) {
            if (cartParams.gravity != Gravity.NO_GRAVITY && mCartEnabled) {
                cartParams = DrawerLayout.LayoutParams(cartParams.width, cartParams.height) // NO_GRAVITY, locked layout
                var toolbarHeight = mToolbar!!.height
                if (toolbarHeight <= 0) {
                    val value = TypedValue()
                    mContext.resources.getValue(R.dimen.toolbar_height, value, true)
                    val attributes: TypedArray = mContext.getTheme().obtainStyledAttributes(intArrayOf(value.data))
                    toolbarHeight = attributes.getDimensionPixelOffset(0, ViewUtil.dpToPixels(64, mContext))
                    attributes.recycle()
                }
                cartParams.topMargin = toolbarHeight
                mCartView?.layoutParams = cartParams
                mCartInfoContainerInTrolley!!.visibility = View.GONE
            }
        } else {
            if (cartParams.gravity == Gravity.NO_GRAVITY) {
                cartParams.topMargin = 0
                cartParams.gravity = Gravity.END
                mCartView?.layoutParams = cartParams
                mCartInfoContainerInTrolley!!.visibility = View.VISIBLE
            }
        }
        if (mCartEnabled) {
            if (cartParams.gravity == Gravity.NO_GRAVITY) {
                mCartView?.visibility = View.VISIBLE
                mainParams.rightMargin = mContext.resources.getDimension(R.dimen.cart_width).toInt()
                mMainView.layoutParams = mainParams
                breadcrumbsParams?.rightMargin = mContext.resources.getDimension(R.dimen.cart_width).toInt()
                mBreadcrumbs?.layoutParams = breadcrumbsParams
            } else {
                mCartView?.let { mDrawerLayout.setDrawerLockMode(DrawerLayout.LOCK_MODE_UNLOCKED, it) }
                mainParams.rightMargin = 0
                mMainView.layoutParams = mainParams
                breadcrumbsParams?.rightMargin = 0
                mBreadcrumbs?.layoutParams = breadcrumbsParams
            }
        } else {
            if (cartParams.gravity == Gravity.NO_GRAVITY) {
                mCartView?.visibility = View.GONE
            } else {
                mCartView?.let { mDrawerLayout.setDrawerLockMode(DrawerLayout.LOCK_MODE_LOCKED_CLOSED, it) }
            }
            mainParams.rightMargin = 0
            mMainView.layoutParams = mainParams
            breadcrumbsParams?.rightMargin = 0
            mBreadcrumbs?.layoutParams = breadcrumbsParams
        }
    }

    override fun getCartStateListener(): CartStateListener {
        return mCartStateListener
    }

    private val mCartStateListener: CartStateListener = object : CartStateListener {
        override fun onStateChange(updating: Boolean) {
            if (mCartUpdateAnimationView != null) {
                mCartUpdateAnimationView!!.setActive(updating)
            }
        }
    }
}
