package com.asda.android.utils

import android.content.Context
import com.asda.android.admonetization.criteo.constants.AdConstants
import com.asda.android.admonetization.criteo.constants.AdConstants.CCD_ACCOUNT
import com.asda.android.admonetization.criteo.constants.AdConstants.CUSTOMER_ID
import com.asda.android.admonetization.criteo.constants.AdConstants.ENDCA_USER_SEGMENTS
import com.asda.android.admonetization.criteo.constants.AdConstants.NO_PARAM
import com.asda.android.admonetization.criteo.constants.AdConstants.NRPP
import com.asda.android.admonetization.criteo.constants.AdConstants.RETAILIER_VISITOR_ID
import com.asda.android.admonetization.criteo.constants.AdConstants.SHIP_ON_DATE
import com.asda.android.analytics.anivia.AniviaTracker.getVisitorIdFromDisk
import com.asda.android.app.shop.constants.PageId
import com.asda.android.base.core.constants.*
import com.asda.android.cmsprovider.AsdaCMSConfig
import com.asda.android.service.base.AsdaServiceFactory
import com.asda.android.service.base.Authentication
import com.asda.android.utils.Utils.isQAServer
import java.util.*

object CMSUtils {

    const val CRITEO_TAXONOMY_PAGE_ID = "viewCategoryAppAndroid"
    const val HOME_PAGE_ID = "viewHomeAppAndroid"

    /**
     * Map pageId to corresponding personalization service query parameter.
     * [ContentManager.mDimensionId] will be the value of this param.
     * @param pageId
     * @return query parameter for p13n service
     */
    fun getId(pageId: String?): String {
        if (pageId == null) {
            return PageId.NONE
        }
        when (pageId) {
            "/pages/mobileapp/browse/cat" -> return PageId.CATEGORY_ID
            "/pages/mobileapp/browse/dept" -> return PageId.DEPARTMENT_ID
            "/pages/mobileapp/browse/aisle" -> return PageId.AISLE_ID
            "/pages/mobileapp/browse/shelf" -> return PageId.SHELF_ID
            "/pages/mobileapp/browse/product" -> return PageId.SKU_ID
            else -> return PageId.NONE
        }
    }

    fun setBasicQueryParams(context: Context): HashMap<String, String?> {
        val data = HashMap<String, String?>()
        data[RETAILIER_VISITOR_ID] = getVisitorIdFromDisk(context)
        data["retail-shop"] = AsdaServiceFactory.get().storeId
        data["regionId"] = AsdaServiceFactory.get().storeId
        if ((Authentication.getInstance() as Authentication).isLoggedIn) {
            data[CUSTOMER_ID] = AsdaCMSConfig.authentication.identifierAndProfileId?.second
        } else {
            data[CUSTOMER_ID] = " "
        }
        return data
    }

    fun addCriteoApiQueryParameters(context: Context): HashMap<String, String?> {
        val data = setBasicQueryParams(context)
        if (isQAServer(context))
            data[CCD_ACCOUNT] = AdConstants.TRACKING_ASDA_DEV_APP
        else
            data[CCD_ACCOUNT] = AdConstants.TRACKING_ASDA_APP

        data["${AdConstants.AD_PAGE_ID}$PAGE_TYPE_CATEGORY"] = CRITEO_TAXONOMY_PAGE_ID
        data["${AdConstants.AD_PAGE_ID}$PAGE_TYPE_DEPARTMENT"] = CRITEO_TAXONOMY_PAGE_ID
        data["${AdConstants.AD_PAGE_ID}$PAGE_TYPE_AISLE"] = CRITEO_TAXONOMY_PAGE_ID
        data["${AdConstants.AD_PAGE_ID}$PAGE_TYPE_SHELF"] = CRITEO_TAXONOMY_PAGE_ID
        data["${AdConstants.AD_PAGE_ID}$PAGE_TYPE_HOME"] = HOME_PAGE_ID

        return data
    }

    fun monetizationCmsQueryParameters(): HashMap<String, String?> {
        val data = HashMap<String, String?>()
        data[ENDCA_USER_SEGMENTS] = AsdaServiceFactory.get().userSegments
        data[SHIP_ON_DATE] = AsdaServiceFactory.get().shipDate
        data[NRPP] = "60"
        data[NO_PARAM] = "0"
        data["store_id"] = AsdaServiceFactory.get().storeId
        return data
    }
}
