package com.asda.android.utils

import android.location.Location
import androidx.annotation.VisibleForTesting

object LocationMath {
    /**
     * Compute the distance between two lat/long pairs, expresses in either meters, miles or
     * kilometers
     *
     * @param unit 'M' for miles, 'K' for kilometers, 'D' degrees
     * @return the distance between the two lat/long pairs, in miles or kilometers
     */
    @JvmStatic
    fun computeDistance(
        lat1: Double,
        lon1: Double,
        lat2: Double,
        lon2: Double,
        unit: Char
    ): Double {
        val results = FloatArray(1)
        Location.distanceBetween(lat1, lon1, lat2, lon2, results)
        return convertDistance(results[0].toDouble(), unit)
    }

    /**
     * converts the distance into expected unit, default expectation is in meters
     *
     * @param dist
     * @param unit 'M' for miles, 'K' for kilometers
     * @return
     */
    @VisibleForTesting
    fun convertDistance(dist: Double, unit: Char): Double {
        var dist = dist
        when (unit) {
            'm' -> { // meters
            }
            'M' -> { // miles
                dist /= 1609.34
            }
            'K' -> { // kilometers
                dist /= 1000
            }
        }
        return dist
    }
}
