package com.asda.android.utils

import android.content.Context
import android.location.Address
import android.location.Geocoder
import android.location.Location
import android.util.Log
import androidx.annotation.VisibleForTesting
import com.asda.android.app.model.AddressSearchResult
import com.google.android.gms.maps.model.LatLng
import java.io.IOException
import java.util.*

object LocationUtils {
    @JvmStatic
    fun locationToCoordinates(location: Location): LatLng {
        return LatLng(location.latitude, location.longitude)
    }

    @JvmStatic
    fun getRoundedCoordinates(
        coordinates: LatLng,
        latModulo: Double,
        longModulo: Double
    ): LatLng {
        val roundedCoordinates: LatLng
        var latitude = coordinates.latitude
        var longitude = coordinates.longitude
        latitude -= latitude % latModulo
        longitude -= longitude % longModulo
        roundedCoordinates = LatLng(latitude, longitude)
        return roundedCoordinates
    }

    @JvmStatic
    fun getAddress(context: Context, searchString: String?): AddressSearchResult {
        val geoCoder = Geocoder(context, Locale.UK)
        val addressList = getAddress(geoCoder, searchString)
        return AddressSearchResult(context, addressList)
    }

    @VisibleForTesting
    fun getAddress(geoCoder: Geocoder, searchString: String?): List<Address?>? {
        val addressList: List<Address?>?
        addressList = try {
            geoCoder.getFromLocationName(searchString, 5)
        } catch (e: IOException) {
            Log.w("LocationUtils", e)
            null
        }
        return addressList
    }
}
