package com.asda.android.utils

import android.net.Uri
import com.asda.android.products.ui.filters.constants.DynamicFilterConstants.VIEW_TYPE_SORT_ITEM
import com.asda.android.products.ui.filters.model.AsdaFilter
import com.asda.android.restapi.service.data.ViewListResponse
import com.asda.android.restapi.service.data.cms.Refinement
import java.util.*

object ProductPageUtils {
    fun createDynamicFilter(refinement: Refinement?, type: Int): AsdaFilter {
        val asdaFilter = AsdaFilter(refinement)
        asdaFilter.type = type
        return asdaFilter
    }

    @JvmStatic
    fun createCustomRefinement(
        label: String?,
        contentPath: String?,
        dimensionId: String?,
        categoryName: String?,
        isSelected: Boolean
    ): Refinement {
        val refinement = Refinement()
        refinement.label = label
        refinement.contentPath = contentPath
        refinement.dimensionId = dimensionId
        refinement.category = categoryName
        refinement.selected = isSelected
        return refinement
    }

    @JvmStatic
    fun createFacetParam(facets: Array<AsdaFilter>?): String? {
        val builder = StringBuilder()
        if (facets != null && facets.isNotEmpty()) {
            for (facet in facets) {
                if (builder.isNotEmpty()) {
                    builder.append("::")
                }
                builder.append(facet.category).append(":").append(facet.singleDimensionId)
            }
        }
        val arrayList = ArrayList<AsdaFilter>()
        arrayList.toTypedArray()
        return if (builder.isNotEmpty()) Uri.encode(builder.toString()) else null
    }

    @JvmStatic
    fun createFacetArray(filters: Array<AsdaFilter>?): Array<ViewListResponse.Facet> {
        val facets = ArrayList<ViewListResponse.Facet>()
        if (filters != null && filters.isNotEmpty()) {
            for (filter in filters) {
                val facet = ViewListResponse.Facet()
                facet.displayName = filter.name
                facet.facetName = filter.category
                facet.isSortOption = filter.type == VIEW_TYPE_SORT_ITEM
                facet.isSpecialOffer = filter.isSpecialOffer
                if (filter.refinement != null) {
                    facet.itemCount = filter.refinement!!.count
                }
                facets.add(facet)
            }
        }
        return facets.toTypedArray()
    }
}
