package com.asda.android.utils.view

import android.app.Notification
import android.app.NotificationChannel
import android.app.NotificationManager
import android.content.Context
import android.os.Build
import android.util.Log
import androidx.annotation.RequiresApi
import com.asda.android.R
import com.asda.android.analytics.Tracker
import com.asda.android.analytics.constants.Anivia
import com.asda.android.analytics.events.ErrorEvent
import com.asda.android.designlibrary.view.dialog.AsdaNotification

object NotificationUtils {
    private const val TAG = "NotificationUtils"
    private const val ASDA = "ASDA"

    // used in the service onCreate call notification.
    const val ASDA_NOTIFICATION_ID = 1

    @JvmStatic
    fun buildTheNotificationForService(context: Context, tag: String): Notification? {
        return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            val notification = buildNotification(context)
            if (notification == null) {
                Log.d(tag, "Create notification failed for service.")
                Tracker.get()
                    .trackEvent(
                        ErrorEvent(
                            Anivia.GEOFENCE_ERROR_EVENT,
                            "$tag The notification was null, hence service could not started."
                        )
                    )
            }
            notification
        } else {
            Notification()
        }
    }

    @RequiresApi(api = Build.VERSION_CODES.O)
    fun buildNotification(context: Context): Notification? {
        val notificationChannelId = "notify_geo_fence_check_in"
        setNotificationChannel(
            context, notificationChannelId, context.getString(R.string.geo_fence_channel_name)
        )
        return AsdaNotification()
            .newInstance(
                context,
                notificationChannelId,
                ASDA,
                NotificationManager.IMPORTANCE_MIN,
                true,
                Notification.CATEGORY_SERVICE
            )
    }

    @RequiresApi(api = Build.VERSION_CODES.O)
    fun setNotificationChannel(context: Context, channelId: String?, channelName: String) {
        val notificationManager = context.getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
        val channel = NotificationChannel(channelId, channelName, NotificationManager.IMPORTANCE_DEFAULT)
        try {
            notificationManager.createNotificationChannel(channel)
        } catch (e: Exception) {
            Log.e(TAG, "The Notification manager null for $channelName", e)
        }
    }
}
