package com.asda.android.utils.view

import android.Manifest
import android.annotation.TargetApi
import android.app.Activity
import android.content.ActivityNotFoundException
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.net.Uri
import android.os.Build
import android.provider.Settings
import androidx.core.content.ContextCompat

object Permissions {
    // request codes must be less than 256
    const val REQUEST_CODE_PERMISSIONS_REQUEST_LOCATION = 200
    const val REQUEST_CODE_PERMISSIONS_REQUEST_CAMERA = 201
    private const val PREFS_NAME = "permissions_installation_info"
    private const val PERMISSION_LOCATION = Manifest.permission.ACCESS_FINE_LOCATION
    private fun checkPermissionGranted(context: Context, permission: String): Boolean {
        return (
            ContextCompat.checkSelfPermission(context, permission)
                == PackageManager.PERMISSION_GRANTED
            )
    }

    @JvmStatic
    fun isLocationPermissionGranted(context: Context): Boolean {
        return checkPermissionGranted(context, PERMISSION_LOCATION)
    }

    @JvmStatic
    fun isCameraPermissionGranted(context: Context): Boolean {
        return checkPermissionGranted(context, Manifest.permission.CAMERA)
    }

    @JvmStatic
    fun isWakeLockPermissionGranted(context: Context): Boolean {
        return checkPermissionGranted(context, Manifest.permission.WAKE_LOCK)
    }

    @JvmStatic
    @TargetApi(Build.VERSION_CODES.M)
    fun requestLocationPermission(activity: Activity) {
        activity.requestPermissions(
            arrayOf(Manifest.permission.ACCESS_COARSE_LOCATION, PERMISSION_LOCATION),
            REQUEST_CODE_PERMISSIONS_REQUEST_LOCATION
        )
    }

    @JvmStatic
    @TargetApi(Build.VERSION_CODES.M)
    fun requestCameraPermission(activity: Activity) {
        activity.requestPermissions(
            arrayOf(Manifest.permission.CAMERA),
            REQUEST_CODE_PERMISSIONS_REQUEST_CAMERA
        )
    }

    fun openPermissionsWindow(activity: Activity) {
        try {
            val intent = Intent(Settings.ACTION_APPLICATION_DETAILS_SETTINGS)
            intent.addCategory(Intent.CATEGORY_DEFAULT)
            intent.data = Uri.parse("package:" + activity.packageName)
            activity.startActivity(intent)
        } catch (ex: ActivityNotFoundException) {
            val intent = Intent(Settings.ACTION_MANAGE_APPLICATIONS_SETTINGS)
            intent.addCategory(Intent.CATEGORY_DEFAULT)
            activity.startActivity(intent)
        }
    }

    @JvmStatic
    fun onLocationPermissionRequestReceived(context: Activity) {
        val sharedPrefs = context.getSharedPreferences(PREFS_NAME, Activity.MODE_PRIVATE)
        sharedPrefs.edit().putBoolean(PERMISSION_LOCATION, false).apply()
    }

    object LocationPermissionGrantedEvent {
        const val ACTION = "LocationPermissionGranted"
    }
}
